// ADOBE SYSTEMS INCORPORATED
// Copyright  1993 - 2002 Adobe Systems Incorporated
// All Rights Reserved
//
// NOTICE:  Adobe permits you to use, modify, and distribute this 
// file in accordance with the terms of the Adobe license agreement
// accompanying it.  If you have received this file from a source
// other than Adobe, then your use, modification, or distribution
// of it requires the prior written permission of Adobe.
//-------------------------------------------------------------------------------

#ifdef SCRIPTABLE

#include "AfhFMScripting.h"
#include "AfhFM.h"
#include "AfhFMcontext.h"

///FMcontext fmc;	//NEVER REFERENCED??? Huge waste of static RAM!!!!

int ScriptParamRead = 0;
int ScriptParam[] = {0,0,0,0,0,0,0,0};
int ScriptParamUsed[] = {true,true,true,true,true,true,true,true};



//-------------------------------------------------------------------------------
//
// ReadScriptParameters
//
// See if we were called by the Photoshop scripting system and return the value
// in displayDialog if the user wants to see our dialog.
// 
//-------------------------------------------------------------------------------
OSErr ReadScriptParameters(Boolean* displayDialog)
{
	OSErr err = noErr;
	PIReadDescriptor token = 0;//NULL;
	DescriptorKeyID key = 0;//NULL;
	DescriptorTypeID type = 0;//NULL;
	//DescriptorUnitID units;
	int32 flags = 0;
	//double percent;
	int value;
	//DescriptorEnumID disposition;
	//Boolean ignoreSelection;
	DescriptorKeyIDArray array = {'ctl0','ctl1','ctl2'}; //= { keyAmount, keyDisposition, 0 }; //NULL
	PIDescriptorParameters* descParams;
	ReadDescriptorProcs* readProcs;

    int iKey;
    char * pKey;
    char swap;
    

	if (displayDialog != NULL)
		*displayDialog = (unsigned char)gParams->queryForParameters;
	else
		return errMissingParameter;

	descParams = gStuff->descriptorParameters;
	if (descParams == NULL) return err;
	
	readProcs = gStuff->descriptorParameters->readDescriptorProcs;
	if (readProcs == NULL) return err;

    //Info ("%d", descParams->playInfo); //plugInDialogDisplay

	
	if (descParams->descriptor != NULL) // || gStuff->hostSig==0
	{
		token = readProcs->openReadDescriptorProc(descParams->descriptor, array);
		if (token != NULL)
		{
			
            //Info ("ReadScriptParameters2");
            
            while(readProcs->getKeyProc(token, &key, &type, &flags) && !err)
			{
			 
                pKey = (char *)&key;
                swap = *pKey;
                *pKey = *(pKey+3);
                *(pKey+3) = swap;
                swap = *(pKey+1);
                *(pKey+1) = *(pKey+2);
                *(pKey+2) = swap;
                iKey = atoi(pKey);
                
                //Info ("%s\n%d",(char *)&key,iKey);

                err = readProcs->getIntegerProc(token, &value);
                if (!err) 
                {
                    ScriptParam[iKey] = value;
                    ScriptParamUsed[iKey] = true;
                    ScriptParamRead++;
                }


                /*
                switch (key)
				{
					case '0000'://keyAmount: //'ctl0'
						err = readProcs->getIntegerProc(token, &value);//getUnitFloatProc(token, &units, &percent);
						if (!err)
							ScriptParam[0] = value;//gFmcp->pre_ctl[0] = value;//gParams->percent = (int16)percent;
						break;
					case '0001'://keyDisposition:
						err = readProcs->getIntegerProc(token, &value);//getEnumeratedProc(token, &disposition);
						if (!err)
						{
							//gParams->disposition = ScriptToDialog(disposition);
							//CopyColor(gData->color, 
							//	      gData->colorArray[gParams->disposition]);
							ScriptParam[1] = value;//gFmcp->pre_ctl[1] = value;//
						}
						break;
					case '0002': //keyIgnoreSelection:
						err = readProcs->getIntegerProc(token, &value);//getBooleanProc(token, &ignoreSelection);
						if (!err)
							ScriptParam[2] = value;//gFmcp->pre_ctl[2] = value;//gParams->ignoreSelection = ignoreSelection;
						break;

                    case '0003': 
						err = readProcs->getIntegerProc(token, &value);
						if (!err) ScriptParam[3] = value;//gFmcp->pre_ctl[3] = value;
						break;
                    case '0004': 
						err = readProcs->getIntegerProc(token, &value);
						if (!err) ScriptParam[4] = value;//gFmcp->pre_ctl[4] = value;
						break;
                    case '0005': 
						err = readProcs->getIntegerProc(token, &value);
						if (!err) ScriptParam[5] = value;//gFmcp->pre_ctl[5] = value;
						break;
                    case '0006': 
						err = readProcs->getIntegerProc(token, &value);
						if (!err) ScriptParam[6] = value;//gFmcp->pre_ctl[6] = value;
						break;
                    case '0007': 
						err = readProcs->getIntegerProc(token, &value);
						if (!err) ScriptParam[7] = value;//gFmcp->pre_ctl[7] = value;
						break;


                    case '0008': if (!readProcs->getIntegerProc(token, &value)) ScriptParam[8] = value; break;
                    case '0009': if (!readProcs->getIntegerProc(token, &value)) ScriptParam[9] = value; break;
                    case '0010': if (!readProcs->getIntegerProc(token, &value)) ScriptParam[10] = value; break;
                    
                    
					default:
						err = readErr;
						break;
				}
                */
			}
			err = readProcs->closeReadDescriptorProc(token);
			gStuff->handleProcs->disposeProc(descParams->descriptor);
			descParams->descriptor = NULL;

		}
		*displayDialog = descParams->playInfo == plugInDialogDisplay;
	}

    //For Fireworks we needs to use playInfo even if descriptor == NULL
    if (gStuff->hostSig==0) {
        *displayDialog = !(descParams->playInfo == plugInDialogDisplay); //Only display dialog on second run
    }

	return err;
}



//-------------------------------------------------------------------------------
//
// WriteScriptParameters
//
// Write our parameters to the Photoshop scripting system in case we are being
// recorded in the actions pallete.
// 
//-------------------------------------------------------------------------------
OSErr WriteScriptParameters(void)
{
	OSErr err = noErr;
	PIWriteDescriptor token = NULL;
	PIDescriptorHandle h;
	//const double percent = gParams->percent;
	PIDescriptorParameters*	descParams;
	WriteDescriptorProcs* writeProcs;
    
    char sNumber[5]; 
    char swap;
    unsigned long * lnum;
    int i;

    //Info ("WriteScriptParameters1");

	descParams = gStuff->descriptorParameters;
	if (descParams == NULL) return err;
	
	writeProcs = gStuff->descriptorParameters->writeDescriptorProcs;
	if (writeProcs == NULL) return err;

	token = writeProcs->openWriteDescriptorProc();
	if (token != NULL)
	{
	
        for (i=0;i<256;i++)
        { 
            if (ScriptParamUsed[i]){
                
                _itoa( i, sNumber, 10 );
                if (i<10){
                    sNumber[0] = sNumber[0];
                    sNumber[1] = 0x30;
                    sNumber[2] = 0x30;
                    sNumber[3] = 0x30;
                } else if (i<100) {
                    swap = sNumber[0];
                    sNumber[0] = sNumber[1];
                    sNumber[1] = swap;
                    sNumber[2] = 0x30;
                    sNumber[3] = 0x30;
                    
                } else {
                    swap = sNumber[0];
                    sNumber[0] = sNumber[2];
                    sNumber[1] = sNumber[1];
                    sNumber[2] = swap;
                    sNumber[3] = 0x30;
                }
                sNumber[4] = 0;
                
                lnum = (unsigned long *)&sNumber;

                //Info ("%d - %d - %d",i,*lnum, '0217');

                writeProcs->putIntegerProc(token,*lnum,ScriptParam[i]);
            }
        }
                

/*
                //Info ("WriteScriptParameters2");

        writeProcs->putIntegerProc(token, //putUnitFloatProc(token, 
			                         '0000',//keyAmount, //'ctl0'
									 //unitPercent, 
									 ScriptParam[0]);//&percent); //gFmcp->pre_ctl[0]
		writeProcs->putIntegerProc(token, //putEnumeratedProc(token, 
			                          '0001',//keyDisposition, 
									  //typeMood, 
									  ScriptParam[1]);//DialogToScript(gParams->disposition));
		//if (gParams->ignoreSelection)
		writeProcs->putIntegerProc(token, //putBooleanProc(token, 
			                           '0002',//keyIgnoreSelection, 
									   ScriptParam[2]);//gParams->ignoreSelection);

        writeProcs->putIntegerProc(token,'0003',ScriptParam[3]);
        writeProcs->putIntegerProc(token,'0004',ScriptParam[4]);
        writeProcs->putIntegerProc(token,'0005',ScriptParam[5]);
        writeProcs->putIntegerProc(token,'0006',ScriptParam[6]);
        writeProcs->putIntegerProc(token,'0007',ScriptParam[7]);



        writeProcs->putIntegerProc(token,'0008',ScriptParam[8]);
        writeProcs->putIntegerProc(token,'0009',ScriptParam[9]);
        writeProcs->putIntegerProc(token,'0010',ScriptParam[10]);
        
*/

		gStuff->handleProcs->disposeProc(descParams->descriptor);
		writeProcs->closeWriteDescriptorProc(token, &h);
		descParams->descriptor = h;
		descParams->recordInfo = plugInDialogOptional;
	}
	else
	{
		return errMissingParameter;
	}
	return err;
}


#endif
