/*
	File: AfhFMauthorize.c

	C source file for the FilterMeister Authorization/Registration routines.

    Copyright 2000 AFH Systems.

    26-May-2000 -- Extracted from AfhFMUI.c
	14JUL2013	A. Hunter	1. Modify for WIN64.

*/

#define VERBOSE 1

/*****************************************************************************/

#ifdef FULL

#include <stdio.h>
#include <time.h>

#include "AfhFM.h"
#include "resource.h"
#include "WinUtil.h"    // Message crackers and stuff

#include "md5global.h"      /* MD5 stuff */
#include "md5.h"            /* MD5 stuff */

/*****************************************************************************/

/* Global variables */
BOOL bAuthorized = FALSE;
DWORD dwAuthorized2;
DWORD dwAuthorized3;
DWORD RegCodeFingerPrint[4] = {0, 0, 0, 0};
int nFmVerMajor, nFmVerMinor, nFmVerBuild;

/* Locals for communication with dialog proc */
static char szFmUsername[256];
static char szFmOrganization[256];
static char szFmLicenseIsValid[256];
static char szFmRegCode[256];

/* Other local variables */
static char szFmVersion[32];
static char szFmVersionCheck[16];
static DWORD MachineFingerPrint[4];
static DWORD Key1[4];

/*****************************************************************************/

int WINAPI AuthorizeDialogProc(HWND hDlg, UINT wMsg, WPARAM wParam, LPARAM lParam)
{
    switch  (wMsg) {

        case  WM_INITDIALOG:
            {//scope
                // center the dialog on screen (if no parent);
                // else center in client area of parent.
                UINT flags = SWP_NOZORDER | SWP_NOACTIVATE | SWP_NOSIZE;
                HWND hParent;
                int  nHeight;
                int  nWidth;
                RECT rcDialog;
                RECT rcParent;
                int  x, y;
                int  xScreen;
                int  yScreen;

                GetWindowRect(hDlg, &rcDialog);
                nWidth  = rcDialog.right  - rcDialog.left;
                nHeight = rcDialog.bottom - rcDialog.top;

                if (!(hParent = GetParent(hDlg)))
                    hParent = GetDesktopWindow();

                GetClientRect(hParent, &rcParent);
                ClientToScreen(hParent, (LPPOINT)&rcParent.left);  // point(left,  top)
                ClientToScreen(hParent, (LPPOINT)&rcParent.right); // point(right, bottom)

                x = max(rcParent.right - rcParent.left - nWidth, 0) / 2
                    + rcParent.left;
                //Don't let it go off right edge of screen
                xScreen = GetSystemMetrics(SM_CXSCREEN);
                if (x + nWidth > xScreen)
                    x = max(0, xScreen - nWidth);
                //Don't let it go off left edge of screen
                x = max(x, 0);

	            y = max(rcParent.bottom - rcParent.top - nHeight, 0) / 2
                    + rcParent.top;
                //Don't let it go off top edge of screen
                y = max(y, 0);
                //Don't let it go off bottom edge of screen
                yScreen = GetSystemMetrics(SM_CYSCREEN);
                if (y + nHeight > yScreen)
                    y = max(0, yScreen - nHeight);

                SetWindowPos(hDlg, NULL,
                             x, y, nWidth, nHeight,
                             flags);
            }//scope
            return  TRUE; // TRUE says we did not set the focus to a control.

#if 0
        case WM_DESTROY:
            return FALSE;   // call defproc
#endif

        case WM_COMMAND:
            {//scope
                int idd = COMMANDID (wParam);
                ////int iCmd = COMMANDCMD(wParam, lParam);
                switch  (idd) {
                case IDOK:
                    GetDlgItemText(hDlg, IDC_EDIT1, szFmUsername, sizeof(szFmUsername));
                    GetDlgItemText(hDlg, IDC_EDIT2, szFmOrganization, sizeof(szFmOrganization));
                    GetDlgItemText(hDlg, IDC_EDIT3, szFmRegCode, sizeof(szFmRegCode));
                case IDCANCEL:
                    break;          // fall through to EndDialog()
                default:
                    return FALSE;   // this idd not handled
                } //switch idd
                EndDialog(hDlg, idd);   // close modal dialog
                return TRUE;    // do not call defProc
            }//scope

        default:
            return  FALSE;  // wMsg not handled
    } //switch wMsg

    return  TRUE;   // handled; do not call defProc

} /*AuthorizeDialogProc*/

/*****************************************************************************/

BOOL GetFmVersion(void)
{
    //Get current FM version number
    unsigned int nDigits, nFields, nChars;
    unsigned int nChecksum;
    char szChecksum[16];
    int i;

    //
    // Bare version number string is resource # IDS_FMVERSION.
    //
    if (LoadString(hDllInstance, IDS_FMVERSION, szFmVersion, sizeof szFmVersion) <= 0) {
        // Version number not found, or null.
#ifdef VERBOSE
        ErrorOk("FM version number resource string not found or corrupt.");
#endif
        return FALSE;
    }

    //
    // Check version against secret checksum (IDS_STRING4)
    // -- to make sure user hasn't tampered with it...
    //
    if (LoadString(hDllInstance, IDS_STRING4, szFmVersionCheck, sizeof szFmVersionCheck) <= 0) {
        // Version checksum number not found, or null.
#ifdef VERBOSE
        ErrorOk("FM version checksum string not found or corrupt.");
#endif
        return FALSE;
    }
    nChecksum = 0;
    nDigits = 0;
    for (i = 0; szFmVersion[i] != '\0'; i++) {
        if (isdigit(szFmVersion[i])) {
            nChecksum += szFmVersion[i] - '0';
            nDigits++;
        }
    }//for i
    wsprintf(szChecksum, "%u", nChecksum);
    if (nDigits < 3 || strcmp(szChecksum, szFmVersionCheck) != 0) {
#ifdef VERBOSE
        ErrorOk("FM version checksum failed.");
#endif
        return FALSE;
    }

    //
    // Scan FM version number for credibility, and parse its fields...
    //
    nFields = sscanf(szFmVersion, "%u.%u.%u%n",
                     &nFmVerMajor, &nFmVerMinor, &nFmVerBuild, &nChars);
    if (nFields != 3) {
#ifdef VERBOSE
        ErrorOk("FM version number incorrectly formatted (nFields = %u)", nFields);
#endif
        return FALSE;
    }
    if (nChars != strlen(szFmVersion)) {
#ifdef VERBOSE
        ErrorOk("FM version number incorrectly formatted (nChars = %u)", nChars);
#endif
        return FALSE;
    }

#if 0
    Info("FM version %s: major = %u, minor = %u, build = %u",
         szFmVersion, nFmVerMajor, nFmVerMinor, nFmVerBuild);
#endif
    return TRUE;
} /*GetFmVersion*/

/*****************************************************************************/

static void ComputeMachineFingerPrint(void)
{
    // Compute a unique fingerprint for this machine
    MD5_CTX context;
    char string[2048] = "{";
    char szRootPathName[MAX_PATH] = "C:\\";
    char szVolumeNameBuffer[MAX_PATH];
    DWORD dwVolumeSerialNumber;
    DWORD dwMaximumComponentLength;
    DWORD dwFileSystemFlags;
    char szFileSystemNameBuffer[MAX_PATH];
    char szTemp[256];
    BYTE szComputerName[256];

#if 0
    Info("CPUID version = %8.8x\n", X86_cpuid_version());
    Info("CPUID features = %8.8x\n", X86_cpuid_features());
    Info("CPUID vendor ID = %s\n", X86_cpuid_vendor_id());
#endif
    sprintf(szTemp, "%8.8x-%8.8x/%s/",
            X86_cpuid_version(), X86_cpuid_features(), X86_cpuid_vendor_id());
    strcat(string, szTemp);

    // Include the License Is Valid string (Key3)...
    strcat(string, szFmLicenseIsValid);

    RQV(ROK(HKEY_LOCAL_MACHINE,
        "System\\CurrentControlSet\\control\\ComputerName\\ComputerName"),
        "ComputerName", szComputerName, sizeof(szComputerName));

    strcat(string, "::");
    strcat(string, szComputerName);


    if (GetVolumeInformation(
        szRootPathName,	// address of root directory of the file system 
        szVolumeNameBuffer,	// address of name of the volume 
        sizeof(szVolumeNameBuffer),	// length of szVolumeNameBuffer 
        &dwVolumeSerialNumber,	// address of volume serial number 
        &dwMaximumComponentLength,	// address of system's maximum filename length
        &dwFileSystemFlags,	// address of file system flags 
        szFileSystemNameBuffer,	// address of name of file system 
        sizeof(szFileSystemNameBuffer) 	// length of szFileSystemNameBuffer 
    )) {
#if 0
        Info("Root path name = \"%s\"", szRootPathName);
        Info("Volume name = \"%s\"", szVolumeNameBuffer);
        Info("File system name = \"%s\"", szFileSystemNameBuffer);
        Info("File system flags = %8.8x", dwFileSystemFlags);
        Info("Volume serial number = %8.8x", dwVolumeSerialNumber);
        Info("Max component length = %d", dwMaximumComponentLength);
#endif
        strcat(string, "[");
        strcat(string, szRootPathName);
        strcat(string, "=");
        strcat(string, szVolumeNameBuffer);
        strcat(string, ";");
        strcat(string, szFileSystemNameBuffer);
        sprintf(szTemp, "(%d)", dwMaximumComponentLength);
        strcat(string, szTemp);
        sprintf(szTemp, "<%8.8x>", dwVolumeSerialNumber);
        strcat(string, szTemp);
        strcat(string, "]");
    }
    else {
        strcat(string, "(GetVolInfo failed)");
    }

    strcat(string, "}");
#if 0
    Info("MD5 (\"%s\")",string);
#endif

    // Use MD5 to compute the Master Machine Fingerprint.
    MD5Init(&context);
    MD5Update(&context, string, (int)strlen (string));

    // Include the Registration Code FIngerprint (Key2) as part of
    // the Master Machine Fingerprint...
    MD5Update(&context, (unsigned char *)RegCodeFingerPrint, sizeof(RegCodeFingerPrint));
    MD5Final((unsigned char *)MachineFingerPrint, &context);
} /*ComputeMachineFingerPrint*/

/*****************************************************************************/

static BOOL LicenseIsValid(void)
{
    time_t btime;
    char szTime[26];
    char szYear[5];
    unsigned int nFields, nChars;
    int nVerMajor, nVerMinor, nVerBuild;

    //Get current year...
    time(&btime);
    strcpy(szTime, ctime(&btime));
#if 0
    Info("Time is %s", szTime);
#endif
    strncpy(szYear, &szTime[20], 4);
    szYear[4] = '\0';


    if (strcmp(szFmLicenseIsValid, "forever") == 0) return TRUE;

    if (strcmp(szFmLicenseIsValid, "1999") == 0) {
        // Old style expiration date for original beta testers,
        // actually valid through end of year 2000.
        if (strcmp(szYear, "1999") == 0) return TRUE;
        if (strcmp(szYear, "2000") == 0) return TRUE;
        return FALSE;
    }

    // Try form: "through Vn.n.n"
    nFields = sscanf(szFmLicenseIsValid, "through V%u.%u.%u%n",
                     &nVerMajor, &nVerMinor, &nVerBuild, &nChars);
    if (nFields == 3 &&
        nChars == strlen(szFmLicenseIsValid)) {
        if (nFmVerMajor < nVerMajor) return TRUE;
        if (nFmVerMajor > nVerMajor) return FALSE;
        if (nFmVerMinor < nVerMinor) return TRUE;
        if (nFmVerMinor > nVerMinor) return FALSE;
        if (nFmVerBuild > nVerBuild) return FALSE;
        return TRUE;
    }

    // Try form: "until Vn.n.n"
    nFields = sscanf(szFmLicenseIsValid, "until V%u.%u.%u%n",
                     &nVerMajor, &nVerMinor, &nVerBuild, &nChars);
    if (nFields == 3 &&
        nChars == strlen(szFmLicenseIsValid)) {
        if (nFmVerMajor < nVerMajor) return TRUE;
        if (nFmVerMajor > nVerMajor) return FALSE;
        if (nFmVerMinor < nVerMinor) return TRUE;
        if (nFmVerMinor > nVerMinor) return FALSE;
        if (nFmVerBuild < nVerBuild) return TRUE;
        return FALSE;
    }

    // Unrecognized form:
    return FALSE;
} /*LicenseIsValid*/

/*****************************************************************************/

static BOOL TryToAuthorize(const char *szField1, const char *szField2,
                           const char *szIsValid)
{
    MD5_CTX context;
    static const char szSeparator1[] = "/*/";
    unsigned int len;
    unsigned char RegCodeDigest[16];
    char szTestRegCode[] = "aaaa-bbbb-cccc-dddd";
    static const char szSeparator2[] = "-#=";
    char szFingerPrint [sizeof(szFmUsername) +
                        sizeof(szSeparator1) +
                        sizeof(szFmOrganization) +
                        sizeof(szSeparator2) +
                        sizeof(szFmLicenseIsValid) - 5 + 1];

    // Set global License Is Valid string...
    strcpy(szFmLicenseIsValid, szIsValid);

    // FingerPrint = 'field1(username)' + '/*/' + 'field2(organization)' + '-#=' + 'valid'
    strcpy(szFingerPrint, szField1);
    strcat(szFingerPrint, szSeparator1);
    strcat(szFingerPrint, szField2);
    strcat(szFingerPrint, szSeparator2);
    strcat(szFingerPrint, szFmLicenseIsValid);
    len = (UINT)strlen(szFingerPrint);
#if 0
    Info("MD5 Fingerprint (\"%s\")", szFingerPrint);
#endif

    // Compute the digest of the fingerprint
    MD5Init (&context);
    MD5Update (&context, szFingerPrint, len);
    MD5Final (RegCodeDigest, &context);

    // Convert digest to a more human-friendly form
    // which is the "registration code"
    {//scope
        static char base64[64] = "0123456789ABCDEFGHIJKLMNOPQRSTUV"
                                 "WXYZabcdefghijklmnopqrstuvwxyzQW";
        int i;
        int j = 0;

        for (i = 0; i < 16; i++) {
            if (i && i % 4 == 0) j++;  //skip '-'
            szTestRegCode[j++] = base64[(RegCodeDigest[i] >> 1) & 0x3f];
        }
    }//scope

    // Compare reg code to what user entered
    if (strcmp(szTestRegCode, szFmRegCode) != 0) {
        // Registration code doesn't match...
#if 0
        Info("Incorrect registration code, should be %s", szTestRegCode);
#endif
        return FALSE;
    }

    // Test that license is still valid.
    if (LicenseIsValid()) {
        // Registration code is correct, and license is valid.
        // Save the full digest...
        memcpy(RegCodeFingerPrint, RegCodeDigest, sizeof(RegCodeFingerPrint));
        // Scramble the digest a bit for extra security
        // (but make sure it's reversible!)
        RegCodeFingerPrint[0] ^= 0x478F8A55;
        RegCodeFingerPrint[1] ^= 0x948FB7A7;
        RegCodeFingerPrint[2] ^= 0x06782FAC;
        RegCodeFingerPrint[3] ^= 0xFF561302;

        // Compute the master fingerprint for this machine
        ComputeMachineFingerPrint();

        // Try to write registration info to the registry (CURRENT USER):
		//
		// (Before Win/Vista, any user could write to HKLM, so
		// we registered FM globally, but in Vista and Win7 the
		// standard user can't write to HKLM -- except via "registry
		// virtualization", which doesn't work in all cases -- so
		// we now follow best practice and register FM under HKCU.)
		//
		// TODO: Any need to check the status of the next 3 RSB calls????
		//       They will probably all succeed or all fail, which we
		//		 ignore anyway...

        // Key1 is the Master Machine Fingerprint...
        RSB(RCK(RCK(HKEY_CURRENT_USER,
            "SOFTWARE\\AFH Systems\\FilterMeister"), szFmVersion),
            "Key1", MachineFingerPrint, sizeof(MachineFingerPrint));

        // Key2 is the Registration Code Fingerprint...
        RSB(RCK(RCK(HKEY_CURRENT_USER,
            "SOFTWARE\\AFH Systems\\FilterMeister"), szFmVersion),
            "Key2", RegCodeFingerPrint, sizeof(RegCodeFingerPrint));

        // Key3 is the License Is Valid string...
        RSB(RCK(RCK(HKEY_CURRENT_USER,
            "SOFTWARE\\AFH Systems\\FilterMeister"), szFmVersion),
            "Key3", (DWORD *)szFmLicenseIsValid, (DWORD)strlen(szFmLicenseIsValid));

        return TRUE;
    }

    return FALSE;

} /*TryToAuthorize*/

/*****************************************************************************/

static BOOL AlreadyRegistered(HKEY hRoot)
{
    if (RQV(ROK(ROK(hRoot, "SOFTWARE\\AFH Systems\\FilterMeister"),
        szFmVersion), "Key1", (LPBYTE)Key1, sizeof(Key1)) != ERROR_SUCCESS)
    {
        // Key1 (Master Machine Fingerprint) is missing...
        return FALSE;
    }

    if (RQV(ROK(ROK(hRoot, "SOFTWARE\\AFH Systems\\FilterMeister"),
        szFmVersion), "Key2", (LPBYTE)RegCodeFingerPrint, sizeof(RegCodeFingerPrint))
        != ERROR_SUCCESS)
    {
        // Key2 (Registration Code Fingerprint) is missing...
        return FALSE;
    }

    if (RQV(ROK(ROK(hRoot, "SOFTWARE\\AFH Systems\\FilterMeister"),
        szFmVersion), "Key3", szFmLicenseIsValid, sizeof(szFmLicenseIsValid))
        != ERROR_SUCCESS)
    {
        // Key3 (License Is Valid string) is missing...
        return FALSE;
    }

    // Compute the master fingerprint for this machine
    ComputeMachineFingerPrint();

    if (memcmp(MachineFingerPrint, Key1, sizeof(MachineFingerPrint)) != 0)
    {
        // Key1 does not match the current Master Machine Fingerprint...
        return FALSE;
    }

    if (!LicenseIsValid()) {
        // License has expired, or not valid for this version of FM...
        return FALSE;
    }

    return TRUE;
} /*AlreadyRegistered*/

/*****************************************************************************/

#define ROT13(c) ( (c)>='A' && (c)<='Z' ? ((c)-'A'+13)%26+'A' : \
                   (c)>='a' && (c)<='z' ? ((c)-'a'+13)%26+'a' : (c) )

#define XROT 0x15
#define ROTX(c) ( (c)&0xe0 ? (c)^(XROT) : (c) )

#if 0
#define rr ROT13
#else
#define rr ROTX
#endif

#if 0

static char *rot13(const char *sz)
{
    static char sz1[256];
    int i = 0;
#if 0
    Info("rot13 in = \"%s\"", sz);
#endif
    while (i < 255 && *sz) {
        sz1[i++] = ROT13(*sz);
        sz++;
    }
    sz1[i] = '\0';
#if 0
    Info("rot13 out = \"%s\"", sz1);
#endif
    return sz1;
} /*rot13*/

#else

static char *rotx(const char *sz)
{
    static char sz1[256];
    int i = 0;
#if 0
    Info("rotx in = \"%s\"", sz);
#endif
    while (i < 255 && *sz) {
        sz1[i++] = ROTX(*sz);
        sz++;
    }
    sz1[i] = '\0';
#if 0
    Info("rotx out = \"%s\"", sz1);
#endif
    return sz1;
} /*rotx*/

#endif

// ROT13 or ROTX code for "ShareIt! Special Pre-release Offer"...
static const char szSpecial1[] = {
    rr('S'),rr('h'),rr('a'),rr('r'),rr('e'),rr('I'),rr('t'),rr('!'),rr(' '),
    rr('S'),rr('p'),rr('e'),rr('c'),rr('i'),rr('a'),rr('l'),rr(' '),rr('P'),
    rr('r'),rr('e'),rr('-'),rr('r'),rr('e'),rr('l'),rr('e'),rr('a'),rr('s'),
    rr('e'),rr(' '),rr('O'),rr('f'),rr('f'),rr('e'),rr('r'),'\0'
};

/*****************************************************************************/

BOOL DoAuthorize(BOOL forceReauthorize)
{
    int nResult = IDCANCEL;

#if 0   //always recheck for now (for extra security)...
    if (bAuthorized && !forceReauthorize) {
        // Already authorized...
        return TRUE;
    }
#endif

    szFmUsername[0] = '\0';
    szFmOrganization[0] = '\0';
    szFmLicenseIsValid[0] = '\0';
    szFmRegCode[0] = '\0';
    szFmVersion[0] = '\0';

    bAuthorized = FALSE;
    dwAuthorized2 = 0;
    dwAuthorized3 = 0;
    RegCodeFingerPrint[0] = 0;
    RegCodeFingerPrint[1] = 0;
    RegCodeFingerPrint[2] = 0;
    RegCodeFingerPrint[3] = 0;

    //Get current FM version number
    if (!GetFmVersion()) {
        ErrorOk("Not registered (FM version number is corrupt).");
        return FALSE;
    }


	// Check to see if we are already registered under HKCU or HKLM,
	// and we are not forcing a reauthorization...
	//
	// Historical note: Prior to Win/Vista, FM registered itself
	// globally under HKLM, but the standard user no longer has power
	// to write to HKLM, so FM is now registered under HKCU instead.
	// But we still check for a legacy registration under HKLM if
	// the user is not already authorized under HKCU.

    if (!forceReauthorize && 
		(AlreadyRegistered(HKEY_CURRENT_USER) || 
		 AlreadyRegistered(HKEY_LOCAL_MACHINE)
		)
	   ) 
	{
        // We are already registered!
        nResult = IDOK;
    }

    else do {
		// Keep asking user for registration info until
		// authorization succeeds or user aborts the process...
        nResult = (int)DialogBoxParam(hDllInstance,
                                 (LPSTR)"AUTHORIZE",
                                 (HWND)MyHDlg,
                                 (DLGPROC)AuthorizeDialogProc,
                                 (LPARAM)0);
        if (nResult == IDOK) {
            if (
				// Try various registration schemes against this info...
                TryToAuthorize(szFmUsername, szFmOrganization, "forever") ||
                TryToAuthorize(szFmUsername, szFmOrganization, "1999") ||
                TryToAuthorize(szFmUsername, rotx(szSpecial1), "until V2.0.0") ||
                TryToAuthorize(szFmOrganization, rotx(szSpecial1), "until V2.0.0")
                )
            {
                // Valid authorization!
                nResult = IDOK;     //redundant
            }
            else {
                // Registration failed (incorrect or expired).
                nResult = IDNO;
            }
        }
        if (nResult == IDNO) {
            //insert a small delay after invalid authorization
            //to slow down macro-driven crackers...
            Sleep(100);
        }
    } while (nResult == IDNO && YesNo("Registration failed or expired.  Retry?") == IDYES);

#if 0
    Info("nResult = %d\n"
         "Username = \"%s\"\n"
         "Organization = \"%s\"\n"
         "License is valid = \"%s\"\n"
         "Reg code = \"%s\"",
         nResult, szFmUsername, szFmOrganization, szFmLicenseIsValid, szFmRegCode);
#endif

    if (nResult == IDOK) {
        bAuthorized = TRUE;
        dwAuthorized2 = RegCodeFingerPrint[2];
        dwAuthorized3 = RegCodeFingerPrint[3];
        return TRUE;
    }
    return FALSE;
} /*DoAuthorize*/

/*****************************************************************************/

#endif //FULL
