/***************************** AfhFMsymtab.h ********************************/
/*                                                                          */
/*  FM Symbol Table (Object) definitions.                                   */
/*                                                                          */
/*  Copyright 1999-2014 AFH Systems.                                        */
/*                                                                          */
/*  FilterMeister is free software. You can redistribute and modify it      */
/*  under the terms of the MPL2 License.                                    */
/*  See: https://www.mozilla.org/en-US/MPL/2.0/                             */
/*  Any changes to the FilterMeister files have to be published or sent     */
/*  to harry@thepluginsite.com. The filter plugin code itself does not      */
/*  need to be disclosed.                                                   */
/*                                                                          */
/*  06Jan99  Alex Hunter  1. Initial version.                               */
/*  01Sep12  Alex Hunter  1. Add minimal support for X64.					*/
/*  27Aug13  Alex Hunter  1. Add stHALF (16-bit real), amRIP (RIP-relative).*/
/*  17Sep13  Alex Hunter  1. Add block # field.								*/
/*	03Oct13  Alex Hunter  1. Add INT_PTR and UINT_PTR.						*/
/*	10Jun14	 Alex Hunter  1. Add fmbNONNEG for nonnegative types.			*/
/*	11Jun14  Alex Hunter  1. Add predefined 'const' types.					*/
/*                                                                          */
/*--------------------------------------------------------------------------*/

////// Do uses of pList overlap??????

/****** FM type/symbol object-representation definitions ******/

typedef enum {fmUNDEF, fmSCALAR, fmPOINTER, fmARRAY, fmSTRUCT, fmUNION,
              fmENUM, fmFUNCTION, fmTYPEDEF, fmVAR, fmMEMBER, fmLABEL} fm_WHAT;

typedef enum {btUNDEF, btVOID, btINTEGRAL, btREAL} fm_BASETYPE;

typedef enum {stUNDEF, stVOID, stBOOL, stCHAR, stBYTE, stSHORT, stWORD,
              stINT, stDWORD, stLONG,
#ifdef X64
			  stINT64, stQWORD,
			  stINT_PTR, stUINT_PTR,
#endif
			  stHALF, stFLOAT, stDOUBLE, stTREAL} fm_SUBTYPE;

// Allocation methods
typedef enum {
    amUNDEF,    //undefined
    amSTATIC,   //static -- [ebx].offset -- thread-local??
    amSTACK,    //stack  -- [ebp].offset
    amHEAP,     //heap
    amBASED,    //based  -- [offset]
    amREG,      //register
	amRIP,		//RIP-relative (static) -- [rip].($-offset)
	amTLS,		//thread-local storage (non-FMC)
} fm_ALLOCMETHOD;

// An FM symbol table object
typedef struct fm_OBJ {
    fm_WHAT what;
    fm_BASETYPE basetype;   //when what=fmSCALAR
    fm_SUBTYPE subtype;     //when what=fmSCALAR
    fm_ALLOCMETHOD am;      //when what=fmVAR
    const char *name;
    struct fm_OBJ *pType;   //var type, func ret type, composite type, etc.
    struct fm_OBJ *pList;   //hash link, param list, member list, etc.		//DO THESE USES OVERLAP???
    int flags;
    int nCount;				//array dimension, list count, etc.
    int offset;
    int size;				//sizeof this object
} fm_OBJ, *pfm_OBJ;

// Symbol/type flags
enum {/* type specifiers */
      fmbSIGNED, fmbUNSIGNED, fmbSHORT, fmbLONG, fmbSATURATED,
      /* type qualifiers */
      fmbCONST, fmbVOLATILE,
      /* storage class specifiers */
      fmbTYPEDEF, fmbEXTERN, fmbSTATIC, fmbAUTO, fmbREGISTER, fmbSTACK, fmbHEAP,
      /* other attributes */
      fmbOPENDIM, fmbGLOBAL, fmbNONNEG};
#define fmfSIGNED       (1 << fmbSIGNED)
#define fmfUNSIGNED     (1 << fmbUNSIGNED)
#define fmfSHORT        (1 << fmbSHORT)
#define fmfLONG         (1 << fmbLONG)
#define fmfSATURATED    (1 << fmbSATURATED)
#define fmfCONST        (1 << fmbCONST)
#define fmfVOLATILE     (1 << fmbVOLATILE)
#define fmfTYPEDEF      (1 << fmbTYPEDEF)
#define fmfEXTERN       (1 << fmbEXTERN)
#define fmfSTATIC       (1 << fmbSTATIC)
#define fmfAUTO         (1 << fmbAUTO)
#define fmfREGISTER     (1 << fmbREGISTER)
#define fmfSTACK        (1 << fmbSTACK)
#define fmfHEAP         (1 << fmbHEAP)
#define fmfOPENDIM      (1 << fmbOPENDIM)
#define fmfGLOBAL       (1 << fmbGLOBAL)
#define fmfNONNEG		(1 << fmbNONNEG)

#define fmfTYPE_SPECIFIER_FLAGS (fmfSIGNED|fmfUNSIGNED|fmfSHORT|fmfLONG|fmfSATURATED|fmfNONNEG)
#define fmfTYPE_QUALIFIER_FLAGS (fmfCONST|fmfVOLATILE)
#define fmfSTORAGE_CLASS_FLAGS  (fmfTYPEDEF|fmfEXTERN|fmfSTATIC|fmfAUTO|fmfREGISTER|fmfSTACK|fmfHEAP)

/****** Global prototypes ******/

void reset_symbol_table(void);
void init_symbol_table(void);
void enter_scope(void);
int leave_scope(void);
pfm_OBJ enter_member(const char *name, pfm_OBJ pSym);
pfm_OBJ enter_symbol(const char *name, fm_WHAT what, pfm_OBJ pType);
pfm_OBJ lookup_member(const char *name, pfm_OBJ pSym);
pfm_OBJ lookup_symbol(const char *name);
pfm_OBJ compose_type(pfm_OBJ typeObj, int type, int dim);
pfm_OBJ finalize_type(pfm_OBJ type);
pfm_OBJ newUNDEFtype(void);
pfm_OBJ create_scalar_type(fm_BASETYPE basetype, fm_SUBTYPE subtype,
                           int size, const char *name, int flags);
void display_flags(int flags);
void display_name(const char *name);
void display_size(int size);
void display_list(pfm_OBJ list);
void display_OBJ(pfm_OBJ t);

/****** Predefined fundamental types ******/

pfm_OBJ pfmVOID;
pfm_OBJ pfmBOOL;
pfm_OBJ pfmCHAR;
pfm_OBJ pfmBYTE;
pfm_OBJ pfmSHORT;
pfm_OBJ pfmWORD;
pfm_OBJ pfmINT;
pfm_OBJ pfmDWORD;
pfm_OBJ pfmLONG;
#ifdef X64
pfm_OBJ pfmINT64;
pfm_OBJ pfmQWORD;
pfm_OBJ pfmINT_PTR;
pfm_OBJ pfmUINT_PTR;
#endif //X64
pfm_OBJ pfmHALF;
pfm_OBJ pfmFLOAT;
pfm_OBJ pfmDOUBLE;
pfm_OBJ pfmTREAL;

#ifndef DONT_USE_PREDEFINED_CONST_TYPES
//We also need predefined 'const' types.
pfm_OBJ pfmCONST_VOID;
pfm_OBJ pfmCONST_BOOL;
pfm_OBJ pfmCONST_CHAR;
pfm_OBJ pfmCONST_BYTE;
pfm_OBJ pfmCONST_SHORT;
pfm_OBJ pfmCONST_WORD;
pfm_OBJ pfmCONST_INT;
pfm_OBJ pfmCONST_DWORD;
pfm_OBJ pfmCONST_LONG;
#ifdef X64
pfm_OBJ pfmCONST_INT64;
pfm_OBJ pfmCONST_QWORD;
pfm_OBJ pfmCONST_INT_PTR;
pfm_OBJ pfmCONST_UINT_PTR;
#endif //X64
pfm_OBJ pfmCONST_HALF;
pfm_OBJ pfmCONST_FLOAT;
pfm_OBJ pfmCONST_DOUBLE;
pfm_OBJ pfmCONST_TREAL;
#endif //CONST_TYPES

/**************** end of AfhFMsymtab.h ********************/
