#ifndef FMDIB_H
#define FMDIB_H
/*
 *   fmdib.h
 *
 *   Copyright 1997-2014 AFH Systems
 *
 *   FilterMeister is free software. You can redistribute and modify it
 *   under the terms of the MPL2 License.
 *   See: https://www.mozilla.org/en-US/MPL/2.0/
 *   Any changes to the FilterMeister files have to be published or sent
 *   to harry@thepluginsite.com. The filter plugin code itself does not
 *   need to be disclosed.
 *
 */
//
//	09AUG2013	A. Hunter	1. Changes for Win64
//

#include <stdio.h>
#include "fmmath.h"
#include "fmdib_core.h"
#include "fmdib_bitmap.h"
#include "fmdib_antialias.h"
#include "fmdib_text.h"

#ifndef GRADIENT_FILL_RECT_H
// Following typedefs are missing in older versions of WinGDI.h
// Add them here so we can compile with VSC 5.0
typedef USHORT COLOR16;

typedef struct _TRIVERTEX
{
    LONG    x;
    LONG    y;
    COLOR16 Red;
    COLOR16 Green;
    COLOR16 Blue;
    COLOR16 Alpha;
}TRIVERTEX,*PTRIVERTEX,*LPTRIVERTEX;

typedef struct _GRADIENT_RECT
{
    ULONG UpperLeft;
    ULONG LowerRight;
}GRADIENT_RECT,*PGRADIENT_RECT,*LPGRADIENT_RECT;
//
// gradient drawing modes
//
#define GRADIENT_FILL_RECT_H    0x00000000
#define GRADIENT_FILL_RECT_V    0x00000001
#define GRADIENT_FILL_TRIANGLE  0x00000002
#define GRADIENT_FILL_OP_FLAG   0x000000ff
#endif	//GRADIENT_FILL_RECT_H

typedef HRESULT (STDAPICALLTYPE *LPGRADIENTFILL) (HDC, PTRIVERTEX, ULONG, PVOID, ULONG, ULONG);
LPGRADIENTFILL lpGradientFill = NULL;

/* FUTURE
	Copy to clipboard/paste from clipboard
	Hardware scaling?
	Buffered canvas with FSAA
		internal:
			GDIstartBuffer(hwnd, width, height);
			GDIendBuffer(hwnd, width, height);
	Pen style                        
		constants
	Brush style
		constants
	More primitives
		triangle -> polygon engine
		bezier
		b-spline
	Native AA algorithms
		Martin's AA line code
*/

//---------------------------------------------------------------------------

// Pen state

HPEN GDIoldPen = NULL;
HPEN GDInewPen = NULL;

UINT GDIcurrentPenColor = 0;

#define GDIpenMode_unassigned	0
#define GDIpenMode_clear		1
#define GDIpenMode_color		2
BYTE GDIpenMode = GDIpenMode_unassigned;

int GDIcurrentPenWidth	= -999;	//THREAD-SAFE???
int GDIpenWidth    		= 1;
static INT_PTR fm_setPenWidth(INT_PTR width)
{
	GDIpenWidth = max(1, (int)width);
	return true;
}

int GDIcurrentPenEndCaps	= -999;	//THREAD-SAFE???
int GDIpenEndCaps			= PS_ENDCAP_ROUND;
static INT_PTR fm_setEndCaps(INT_PTR e)
{
	if (e == PS_ENDCAP_ROUND || e == PS_ENDCAP_SQUARE || e == PS_ENDCAP_FLAT) {
		GDIpenEndCaps	= (int)e;
	}

	return true;
}

int GDIcurrentPenJoin	= -999;				//Should get re-inited???
int GDIpenJoin			= PS_JOIN_ROUND;	//Should get re-inited???
static INT_PTR fm_setJoin(INT_PTR j)
{
	if (j == PS_JOIN_ROUND || j == PS_JOIN_BEVEL || j == PS_JOIN_MITER) {
		GDIpenJoin	= (int)j;
	}

	return true;
}

int GDIcurrentPenInsideFrame	= -999;	//Should get re-inited???
int GDIpenInsideFrame	 		= 0;	//Should get re-inited???
static INT_PTR fm_setInsideFrame(INT_PTR e)
{
	GDIpenInsideFrame	= (e? PS_INSIDEFRAME : 0);

	return true;
}

static void GDI_setPen(UINT color)
{
	if (GDInewPen) {
		if (GDIpenMode  		!= GDIpenMode_color
		 || color       		!= GDIcurrentPenColor
		 || GDIpenWidth 		!= GDIcurrentPenWidth
		 || GDIpenInsideFrame 	!= GDIcurrentPenInsideFrame
		 || GDIpenJoin 			!= GDIcurrentPenJoin
		 || GDIpenEndCaps 		!= GDIcurrentPenEndCaps) {
			
			LOGBRUSH lb;
			
			SelectObject(hdcGDI, GDIoldPen);
			DeleteObject(GDInewPen);

			lb.lbStyle = BS_SOLID;
			lb.lbColor = color;
			lb.lbHatch = 0;
			GDInewPen = ExtCreatePen(PS_GEOMETRIC | PS_SOLID | GDIpenEndCaps
				| GDIpenInsideFrame | GDIpenJoin
				,GDIpenWidth, &lb, 0, NULL);

			GDIoldPen = SelectObject(hdcGDI, GDInewPen);
		}
	} else {
		LOGBRUSH lb;
		lb.lbStyle = BS_SOLID;
		lb.lbColor = color;
		lb.lbHatch = 0;
		GDInewPen = ExtCreatePen(PS_GEOMETRIC | PS_SOLID | GDIpenEndCaps
				| GDIpenInsideFrame | GDIpenJoin
			,GDIpenWidth, &lb, 0, NULL);
		GDIoldPen = SelectObject(hdcGDI, GDInewPen);
	}

	GDIcurrentPenColor 			= color;
	GDIcurrentPenWidth			= GDIpenWidth;
	GDIcurrentPenInsideFrame	= GDIpenInsideFrame;
	GDIcurrentPenEndCaps		= GDIpenEndCaps;
	GDIcurrentPenJoin			= GDIpenJoin;
	GDIpenMode	   				= GDIpenMode_color;
}

static void GDI_clearPen()
{
	if (GDInewPen) {
		if (GDIpenMode  != GDIpenMode_clear) {
			SelectObject(hdcGDI, GDIoldPen);
			DeleteObject(GDInewPen);

			GDInewPen = CreatePen(PS_NULL, 0, 0);
			GDIoldPen = SelectObject(hdcGDI, GDInewPen);
		}
	} else {
		GDInewPen = CreatePen(PS_NULL, 0, 0);
		GDIoldPen = SelectObject(hdcGDI, GDInewPen);
	}

	GDIpenMode	   = GDIpenMode_color;
}

static void GDI_destroyPen()
{
	if (GDInewPen) {
		SelectObject(hdcGDI, GDIoldPen);
		DeleteObject(GDInewPen);
		GDInewPen = NULL;
	}
}

//---------------------------------------------------------------------------
// Brush state

HBRUSH GDIoldBrush = NULL;
HBRUSH GDInewBrush = NULL;
UINT GDInewBrushColor = 0;
LOGBRUSH GDIlogBrush;

#define GDIbrushMode_unassigned	0
#define GDIbrushMode_clear		1
#define GDIbrushMode_color		2
BYTE GDIbrushMode = GDIbrushMode_unassigned;

static void GDI_setBrush(UINT color)
{
	if (GDInewBrush) {
		if (GDIbrushMode	!= GDIbrushMode_color
		 || color			!= GDInewBrushColor) {
			SelectObject(hdcGDI, GDIoldBrush);
			DeleteObject(GDInewBrush);

			GDInewBrush = CreateSolidBrush(color);
			GDIoldBrush = SelectObject(hdcGDI, GDInewBrush);
		}
	} else {
		GDInewBrush = CreateSolidBrush(color);
		GDIoldBrush = SelectObject(hdcGDI, GDInewBrush);
	}

	GDInewBrushColor = color;
	GDIbrushMode	 = GDIbrushMode_color;
}

static void GDI_clearBrush()
{
	if (GDInewBrush) {
		if (GDIbrushMode != GDIbrushMode_clear) {
			SelectObject(hdcGDI, GDIoldBrush);
			DeleteObject(GDInewBrush);

			GDIlogBrush.lbStyle = BS_NULL;
			GDInewBrush = CreateBrushIndirect(&GDIlogBrush);
			GDIoldBrush = SelectObject(hdcGDI, GDInewBrush);
		}
	} else {
		GDIlogBrush.lbStyle = BS_NULL;
		GDInewBrush = CreateBrushIndirect(&GDIlogBrush);
		GDIoldBrush = SelectObject(hdcGDI, GDInewBrush);
	}

	GDIbrushMode	 = GDIbrushMode_clear;
}

static void GDI_destroyBrush()
{
	if (GDInewBrush) {
		SelectObject(hdcGDI, GDIoldBrush);
		DeleteObject(GDInewBrush);
		GDInewBrush = NULL;
	}
}

//---------------------------------------------------------------------------

static INT_PTR fm_setRectFill(INT_PTR left, INT_PTR top, INT_PTR right, INT_PTR bottom, UINT_PTR color)
{
	if (hbmDIB && hbmGDI) {
		RECT rect = { (int)left, (int)top, (int)right, (int)bottom };

		if (!modeGDI) {
            modeGDI = true;
            BitBlt(hdcGDI, 0, 0, GDIcanvasWidth, GDIcanvasHeight, hdcDIB, 0, 0, SRCCOPY);
        }
        
		GDI_setBrush((UINT)color);
		
        return FillRect(hdcGDI, &rect, GDInewBrush);
    }

	return false;
};

//---------------------------------------------------------------------------

//requires msimg32.lib! Works on Win 98+ and WinNT 2000+
static INT_PTR fm_setRectGradient(INT_PTR left, INT_PTR top, INT_PTR right, INT_PTR bottom,
								  UINT_PTR color_TL, UINT_PTR color_BR, INT_PTR horizontal)
{
	if (hbmDIB && hbmGDI) {	
		TRIVERTEX vertex[2];
		GRADIENT_RECT gRect;

		vertex[0].x     = (int)left;
		vertex[0].y     = (int)top;
		vertex[0].Red   = GetRValue(color_TL) << 8;
		vertex[0].Green = GetGValue(color_TL) << 8;
		vertex[0].Blue  = GetBValue(color_TL) << 8;
		vertex[0].Alpha = 0x0000;

		vertex[1].x     = (int)right;
		vertex[1].y     = (int)bottom; 
		vertex[1].Red   = GetRValue(color_BR) << 8;
		vertex[1].Green = GetGValue(color_BR) << 8;
		vertex[1].Blue  = GetBValue(color_BR) << 8;
		vertex[1].Alpha = 0x0000;

		gRect.UpperLeft  = 0;
		gRect.LowerRight = 1;

		// load gradientfill function
		if (!lpGradientFill) {
			if (!hInst_msimg32) {
				hInst_msimg32 = LoadLibrary("msimg32.dll");	//check for error???
			}
			lpGradientFill = ((LPGRADIENTFILL) GetProcAddress(hInst_msimg32, "GradientFill"));
		}

		if (lpGradientFill) {
			return lpGradientFill(modeGDI? hdcGDI : hdcDIB, vertex, 2, &gRect, 1,
								  horizontal? GRADIENT_FILL_RECT_H : GRADIENT_FILL_RECT_V);
		}//else error???
		//TODO Else: coded gradient
    }

	return false;
};

//---------------------------------------------------------------------------

static INT_PTR fm_setFill(UINT_PTR color)
{
	return fm_setRectFill(0, 0, GDIcanvasWidth, GDIcanvasHeight, color);
};

//---------------------------------------------------------------------------

static INT_PTR fm_setRectFrame(INT_PTR left, INT_PTR top, INT_PTR right, INT_PTR bottom, UINT_PTR color)
{
	if (hbmDIB && hbmGDI) {

		RECT rect = { (int)left, (int)top, (int)right, (int)bottom };

		if (!modeGDI) {
			modeGDI = true;
			BitBlt(hdcGDI, 0, 0, GDIcanvasWidth, GDIcanvasHeight, hdcDIB, 0, 0, SRCCOPY);
		}

		GDI_setBrush((UINT)color);
		return FrameRect(hdcGDI, &rect, GDInewBrush);
	}

	return false;
};

//---------------------------------------------------------------------------

static INT_PTR fm_setRectangle(INT_PTR left, INT_PTR top, INT_PTR right, INT_PTR bottom, UINT_PTR color)
{
	if (hbmDIB && hbmGDI) {
		if (!modeGDI) {
			modeGDI = true;
			BitBlt(hdcGDI, 0, 0, GDIcanvasWidth, GDIcanvasHeight, hdcDIB, 0, 0, SRCCOPY);
		}

		GDI_setPen((UINT)color);
		return Rectangle(hdcGDI, (int)left, (int)top, (int)right, (int)bottom);
	}

	return false;
};

//---------------------------------------------------------------------------

static INT_PTR fm_setEllipseFill(INT_PTR left, INT_PTR top, INT_PTR right, INT_PTR bottom, UINT_PTR color)
{
	if (hbmDIB && hbmGDI) {
        if (!modeGDI) {
            modeGDI = true;
			BitBlt(hdcGDI, 0, 0, GDIcanvasWidth, GDIcanvasHeight, hdcDIB, 0, 0, SRCCOPY);
		}

		GDI_setBrush((UINT)color);
		GDI_clearPen();

		return Ellipse(hdcGDI, (int)min(left, right),	  (int)min(top, bottom),
							   (int)max(left, right) + 1, (int)max(top, bottom) + 1);
    }

    return false;
};

//---------------------------------------------------------------------------

static INT_PTR fm_setAngleArcFill(INT_PTR x, INT_PTR y, UINT_PTR radius,
								  double a, double b, UINT_PTR color)
{
	if (hbmDIB && hbmGDI) {
		if (!modeGDI) {
			modeGDI = true;
			BitBlt(hdcGDI, 0, 0, GDIcanvasWidth, GDIcanvasHeight, hdcDIB, 0, 0, SRCCOPY);
		}

		GDI_setBrush((UINT)color);
		GDI_clearPen();

		return Pie(hdcGDI, (int)(x - radius), (int)(y - radius), 
						   (int)(x + radius), (int)(y + radius),
						   (int)( (INT_PTR)fm_fr2x(b, (double)radius) + x),
						   (int)(-(INT_PTR)fm_fr2y(b, (double)radius) + y),
						   (int)( (INT_PTR)fm_fr2x(a, (double)radius) + x),
						   (int)(-(INT_PTR)fm_fr2y(a, (double)radius) + y) );
	}

	return false;
};

//---------------------------------------------------------------------------

static INT_PTR fm_setAngleArc(INT_PTR x, INT_PTR y, UINT_PTR radius,
							  double a, double b, UINT_PTR color)
{
	if (hbmDIB && hbmGDI) {
		if (!modeGDI) {
			modeGDI = true;
			BitBlt(hdcGDI, 0, 0, GDIcanvasWidth, GDIcanvasHeight, hdcDIB, 0, 0, SRCCOPY);
		}

		GDI_clearBrush();
		GDI_setPen((UINT)color);

		return Pie(hdcGDI, (int)(x - radius), (int)(y - radius), 
						   (int)(x + radius), (int)(y + radius),
						   (int)( (INT_PTR)fm_fr2x(b, (double)radius) + x),
						   (int)(-(INT_PTR)fm_fr2y(b, (double)radius) + y),
						   (int)( (INT_PTR)fm_fr2x(a, (double)radius) + x),
						   (int)(-(INT_PTR)fm_fr2y(a, (double)radius) + y) );
	}

	return false;
};

//---------------------------------------------------------------------------

static INT_PTR fm_setEllipse(INT_PTR left, INT_PTR top, INT_PTR right, INT_PTR bottom, UINT_PTR color)
{
	if (hbmDIB && hbmGDI) {
		if (!modeGDI) {
            modeGDI = true;
            BitBlt(hdcGDI, 0, 0, GDIcanvasWidth, GDIcanvasHeight, hdcDIB, 0, 0, SRCCOPY);
        }

		GDI_clearBrush();
		GDI_setPen((UINT)color);

        return Ellipse(hdcGDI, (int)left, (int)top, (int)right, (int)bottom);		
	}

    return false;
};

//---------------------------------------------------------------------------

static INT_PTR fm_setLine(INT_PTR start_x, INT_PTR start_y, INT_PTR end_x, INT_PTR end_y, UINT_PTR color)
{
	if (hbmDIB && hbmGDI) {
        if (!modeGDI) {
            modeGDI = true;
			BitBlt(hdcGDI, 0, 0, GDIcanvasWidth, GDIcanvasHeight, hdcDIB, 0, 0, SRCCOPY);
		}

		GDI_setPen((UINT)color);

		MoveToEx(hdcGDI, (int)start_x, (int)start_y, NULL);
		return LineTo(hdcGDI, (int)end_x, (int)end_y);
	}

	return false;
};

//---------------------------------------------------------------------------

static void GDI_destroy ()
{
	GDI_destroyFont();
	GDI_destroyPen();
	GDI_destroyBrush();

	if (hbmGDI && hdcGDI && hbmGDIOld) {
		SelectObject (hdcGDI, hbmGDIOld);
	}
	DeleteObject (hbmGDI);
	hbmGDI = NULL;
	DeleteDC (hdcGDI);
	hbmGDI = NULL;
	hdcGDI = NULL;
	hbmGDIOld = NULL;

	if (hbmDIB && hdcDIB && hbmDIBOld) {
		SelectObject (hdcDIB, hbmDIBOld);
	}
	DeleteObject (hbmDIB);
	DeleteDC (hdcDIB);
	hbmDIB = NULL;
	hdcDIB = NULL;
	hbmDIBOld = NULL;
}

//---------------------------------------------------------------------------


#ifdef BCB
static int fm_startSetPixelSS (HWND hwnd, int factor)    //FM
#else
static INT_PTR fm_startSetPixelSS (INT_PTR n, INT_PTR factor)    //FM
#endif
{
#ifndef BCB
	HWND hwnd;
#endif
	RECT rcCtl;
	BITMAPINFO bi;

	GDI_destroy();

#ifndef BCB
	if (n==CTL_PREVIEW)
		hwnd = GetDlgItem(fmc.hDlg, 101);
	else
		hwnd = (n >= 0? gParams->ctl[n].hCtl : fmc.hDlg); //FM
#endif

	GetWindowRect(hwnd, &rcCtl);

	GDIsuperSampling = (int)min(max(1, factor), 8);
	GDIctlWidth      = (rcCtl.right - rcCtl.left);
	GDIctlHeight     = (rcCtl.bottom - rcCtl.top);
	GDIcanvasWidth   = GDIctlWidth * GDIsuperSampling;
	GDIcanvasHeight  = GDIctlHeight * GDIsuperSampling;
	GDIcanvasSize    = GDIcanvasWidth * GDIcanvasHeight;

	ZeroMemory(&bi, sizeof(BITMAPINFO));
	bi.bmiHeader.biSize = sizeof(BITMAPINFO);
	bi.bmiHeader.biWidth = GDIcanvasWidth;
	bi.bmiHeader.biHeight = -GDIcanvasHeight;
	bi.bmiHeader.biPlanes = 1;
	bi.bmiHeader.biBitCount = 32;

	hdcSource = GetDC(hwnd);

	hdcDIB = CreateCompatibleDC(hdcSource);
	hbmDIB = CreateDIBSection(hdcSource, (BITMAPINFO *)&bi,
			DIB_RGB_COLORS, (void **)&dibPixels, NULL, 0);
	hbmDIBOld = SelectObject(hdcDIB, hbmDIB);

	hdcGDI = CreateCompatibleDC(hdcSource);
	hbmGDI = CreateCompatibleBitmap(hdcSource, GDIcanvasWidth, GDIcanvasHeight);
	hbmGDIOld = SelectObject(hdcGDI, hbmGDI);
	if (hbmGDI) {
		BitBlt(hdcGDI, 0, 0, GDIcanvasWidth, GDIcanvasHeight, hdcSource, 0, 0, SRCCOPY);
	}

	// Initialization
	modeGDI = true;

	return (hbmDIB && hbmGDI);
}

//---------------------------------------------------------------------------

#ifdef BCB
static int fm_startSetPixel (HWND hwnd)    //FM
#else
static INT_PTR fm_startSetPixel (INT_PTR n)    //FM
#endif
{
#ifdef BCB
	return fm_startSetPixelSS(hwnd, 1);
#else
	return fm_startSetPixelSS(n, 1);
#endif
}

//---------------------------------------------------------------------------

#ifdef BCB
static int fm_endSetPixel (HWND hwnd)    //FM
#else
static INT_PTR fm_endSetPixel (INT_PTR n)    //FM
#endif
{
	if (hbmDIB && hbmGDI) {
#ifndef BCB
		HWND hwnd;
		if (n==CTL_PREVIEW)
			hwnd = GetDlgItem(fmc.hDlg, 101);
		else
			hwnd = (n >= 0? gParams->ctl[n].hCtl : fmc.hDlg); //FM
#endif
		if (GDIsuperSampling <= 1) {
			// normal/non-FSAA mode
			BitBlt(hdcSource, 0, 0, GDIcanvasWidth, GDIcanvasHeight,
					modeGDI? hdcGDI : hdcDIB, 0, 0, SRCCOPY);
		} else {
			SetStretchBltMode(hdcSource, HALFTONE);
			//TODO	I don't quite know how SetBrushOrgEx works, but it's required.
			//		Might be doing something wrong though, especially regarding 95/98/ME
			//		tracking of brushes, as described in MSDN docs.
			SetBrushOrgEx(hdcSource, 0, 0, NULL);

			StretchBlt(hdcSource, 0, 0, GDIctlWidth, GDIctlHeight,
						modeGDI? hdcGDI : hdcDIB, 0, 0, GDIcanvasWidth, GDIcanvasHeight,
						SRCCOPY);
		}

		GDI_destroy();
		ReleaseDC(hwnd, hdcSource);

		return true;
	}

	return false;
}

//---------------------------------------------------------------------------

static INT_PTR fm_setPixel (INT_PTR x, INT_PTR y, UINT_PTR color)
{
	if (hbmDIB && hbmGDI) {
		INT_PTR i = x + (y * GDIcanvasWidth);

		if (i >= 0 && i < GDIcanvasSize) {
			if (modeGDI) {
				BitBlt(hdcDIB, 0, 0, GDIcanvasWidth, GDIcanvasHeight, hdcGDI, 0, 0, SRCCOPY);
				GdiFlush();
				modeGDI = false;
			}

			dibPixels[i] = (UINT)( ((color >> 16) & 0x000000FF) |
								   ( color        & 0x0000FF00) |
								   ((color << 16) & 0x00FF0000) );
		}

		return true;
	}

	return false;
}

//---------------------------------------------------------------------------

static UINT_PTR fm_getPixel (INT_PTR x, INT_PTR y)
{
	if (hbmDIB && hbmGDI) {
		int i = (int)x + ((int)y * GDIcanvasWidth);
		UINT value;

		if (i >= 0 && i < GDIcanvasSize) {
			if (modeGDI) {
				BitBlt(hdcDIB, 0, 0, GDIcanvasWidth, GDIcanvasHeight, hdcGDI, 0, 0, SRCCOPY);
				GdiFlush();	//
				modeGDI = false;
			}

			value = dibPixels[i];
			return (value >> 16) | (value & 0x0000FF00) | ( (value << 16) & 0x00FF0000);
		}
	}

	return 0;
}

//---------------------------------------------------------------------------

static INT_PTR fm_startSetPixelBuffer (INT_PTR width, INT_PTR height)
{
    BITMAPINFO bi;

	GDI_destroy(); // When recompiling in FM, old stuff may not have been destroyed resulting in GDI trashing when recompiling enough times.

    GDIcanvasWidth = (int)width;
	GDIcanvasHeight = (int)height;
    GDIcanvasSize = GDIcanvasWidth * GDIcanvasHeight;

    ZeroMemory(&bi, sizeof(BITMAPINFO));
    bi.bmiHeader.biSize = sizeof(BITMAPINFO);
	bi.bmiHeader.biWidth = GDIcanvasWidth;
	bi.bmiHeader.biHeight = -GDIcanvasHeight;
	bi.bmiHeader.biPlanes = 1;
	bi.bmiHeader.biBitCount = 32;

	hdcSource = CreateDC(TEXT("DISPLAY"), NULL, NULL, NULL);

	hdcDIB = CreateCompatibleDC(hdcSource);
	hbmDIB = CreateDIBSection(hdcSource, (BITMAPINFO *)&bi,
			DIB_RGB_COLORS, (void **)&dibPixels, NULL, 0);
	SelectObject(hdcDIB, hbmDIB);

	hdcGDI = CreateCompatibleDC(hdcSource);
	hbmGDI = CreateCompatibleBitmap(hdcSource, GDIcanvasWidth, GDIcanvasHeight);
	SelectObject(hdcGDI, hbmGDI);
	if (hbmGDI) {
		BitBlt(hdcGDI, 0, 0, GDIcanvasWidth, GDIcanvasHeight, hdcSource, 0, 0, SRCCOPY);
	}//else error???

	modeGDI = true;

	return (hbmDIB && hbmGDI);
}

//---------------------------------------------------------------------------

static INT_PTR fm_endSetPixelBuffer (void)
{
	if (hbmDIB && hbmGDI) {
		GDI_destroy();

		DeleteDC(hdcSource);

		return true;
	}

	return false;
}

//---------------------------------------------------------------------------

static INT_PTR fm_getSetPixelWidth (void) {
	return GDIcanvasWidth;
}

//---------------------------------------------------------------------------

static INT_PTR fm_getSetPixelHeight (void) {
	return GDIcanvasHeight;
}

//---------------------------------------------------------------------------

#endif
// FMDIB_H

