   /**MEMBER**=NONE
    **CMS**=RMX_MD86VAX
   */
/****************************** MD386PROTO.H *****************************/
/*                                                                       */
/*  Intermodule prototypes for MD386.                                    */
/*                                                                       */
/*  Copyright 1996-2014 AFH Systems                                      */
/*                                                                       */
/*  FilterMeister is free software. You can redistribute and modify it   */
/*  under the terms of the MPL2 License.                                 */
/*  See: https://www.mozilla.org/en-US/MPL/2.0/                          */
/*  Any changes to the FilterMeister files have to be published or sent  */
/*  to harry@thepluginsite.com. The filter plugin code itself does not   */
/*  need to be disclosed.                                                */
/*                                                                       */
/*    20Oct96  Alex Hunter  1. Translated to C using PLMC32.             */
/*    23Oct96  Alex Hunter  1. Param to Mdput is by-reference, not       */
/*                             by-value!!!                               */
/*    12Apr97  Alex Hunter  1. Add SET/SHOW LANGUAGE.                    */
/*	  16JUL13  Alex Hunter	1. Modify for Win64.						 */
/*                                                                       */
/*************************************************************************/

#define MAX_PARAMS          200

 typedef struct
         {
             UINT4          commandcode;
             UINT4          commandsubcode;
             UINT4          commandflags;
             UINT4          params[MAX_PARAMS];
         }  SEMANTICLISTSTRUCTURE;


 /* Floating point format conversion routines. */

 void cvt_ieee_to_vms_flt(const void *sorcp, float *destp, const UINT2 *nlongp);
 void cvt_vms_to_ieee_flt(const float *sorcp, void *destp, const UINT2 *nlongp);
 void cvt_ieee_to_vms_dbl(const void *sorcp, double *destp, const UINT2 *nquadp);
 void cvt_vms_to_ieee_dbl(const double *sorcp, void *destp, const UINT2 *nquadp);
 double Cvtra86vx(const void *tbytep);
 void Cvtravx86(const void *vaxr8p, void *tempreal86p);


 /* MDD dictionary routines. */

 void Mdd_add_leaf(UINT1 leaftype, UINT1 leafen, INT4 leafvalue);
 void *Mdd_allocate(UINT4 nbytes);
 void Mdd_count_dictionary(void);
 BLKDEFSTRUCTURE *Mdd_enter_blkdef(BLOCKSTRUCTURE *parentp, UINT1 *namep,
                                   UINT4 offset, UINT2 frame,
                                   UINT4 blocklength, UINT2 ldtsel,
                                   UINT4 groupindex, UINT4 typeindex,
                                   UINT1 procflag, UINT1 l,
                                   UINT4 returnaddressoffset);
 DEBSYMSTRUCTURE *Mdd_enter_debsym(BLOCKSTRUCTURE *parentp, UINT1 *namep,
                                   UINT4 offset, UINT2 frame, UINT2 ldtsel,
                                   UINT4 groupindex, UINT4 typeindex,
                                   UINT1 frameinfo);
 GRPDEFSTRUCTURE *Mdd_enter_grpdef(BLOCKSTRUCTURE *parentp,
                                   UINT4 groupnameindex, UINT4 grouplength,
                                   UINT4 offset, UINT2 frame);
 LINNUMSTRUCTURE *Mdd_enter_linnum(MODDEFSTRUCTURE *parentp, UINT4 linenumber,
                                   UINT4 offset, UINT2 frame, UINT2 ldtsel,
                                   UINT4 groupindex);
 MODDEFSTRUCTURE *Mdd_enter_module(PRGDEFSTRUCTURE *parentp,
                                   UINT1 *modulenamep, UINT1 *translatornamep);
 LNAMESTRUCTURE *Mdd_enter_name(const UINT1 *namep);
 PRGDEFSTRUCTURE *Mdd_enter_program(const UINT1 *namep, UINT1 programtype,
                                    UINT4 staticsize, UINT4 dynamicsize);
 PUBDEFSTRUCTURE *Mdd_enter_pubdef(BLOCKSTRUCTURE *parentp, UINT1 *namep,
                                   UINT4 publicindex, UINT4 offset, UINT2 frame,
                                   UINT4 typeindex);
 UINT4 Mdd_enter_scalar_type(UINT1 type, UINT4 nbytes);
 SEGDEFSTRUCTURE *Mdd_enter_segdef(BLOCKSTRUCTURE *parentp,
                                   UINT4 segmentnameindex, 
                                   UINT4 classnameindex,
                                   UINT4 overlaynameindex, 
                                   UINT1 a, UINT1 c, UINT1 pageflag, UINT1 g, 
                                   UINT4 segmentlength, UINT4 offset, 
                                   UINT2 frame);
 TYPDEFSTRUCTURE *Mdd_enter_typdef(UINT4 typeindex);
 void *Mdd_find_module_line(QUALIFIERSTRUCTURE *qual_p, UINT4 line_number,
                            UINT1 nextlow);
 void Mdd_finish_dictionary(void);
 UINT4 Mdd_initialize(void);
 LINNUMSTRUCTURE *Mdd_lookup_linnum(UINT4 linenumber);
 LNAMESTRUCTURE *Mdd_lookup_name(UINT1 *namep);
 void Mdd_reset_dictionary(void);
 UINT4 Mdd_terminate(void);


 /* MDE error reporting routines. */

 void Mde_abort_command(UINT4 msgid, UINT_PTR p1, UINT_PTR p2, UINT_PTR p3);
 void Mde_ctrl_c_abort(void);
 void Mde_error(UINT4 msgid, UINT_PTR p1, UINT_PTR p2, UINT_PTR p3);
 void Mde_fatal(UINT4 msgid, UINT_PTR p1, UINT_PTR p2, UINT_PTR p3);
 UINT4 Mde_initialize(void);
 void Mde_log(UINT4 msgid, UINT_PTR p1, UINT_PTR p2, UINT_PTR p3);
 UINT4 Mde_terminate(void);
 void Mde_warn(UINT4 msgid, UINT_PTR p1, UINT_PTR p2, UINT_PTR p3);


 /* MDPUT formatted output routines. */

 void Mdput(const char *cp);
 void Mdputnl(void);
 void Mdputnl_crt(void);
 void Mdputcr(void);
 void Mdputcr_crt(void);
 void Mdputnocr(void);
 void Mdputbl(void);
 void Mdputc(const char *textcp);
 void Mdputan(const char *textp, const UINT4 *np);
 void Mdputz(const char *textzp);
 void Mdputs(const STRINGDESCRIPTOR *textdp);
 void Mdputs_crt(const STRINGDESCRIPTOR *textdp);
 void Mdputsnb(const STRINGDESCRIPTOR *textdp);
 void Mdputcv(const UINT4 *cvp);
 void Mdputhdw(const UINT4 *hdwp);
 void Mdputhw(const UINT2 *hwp);
 void Mdputhb(const UINT1 *hbp);
 void Mdputhd(const UINT1 *hdp);
 void Mdputd(const INT4 *decp);
 void Mdputu(const UINT4 *decp);
 void Mdputr4(const float *r4p);
 void Mdputr8(const double *r8p);
 UINT2 Mdgets(const STRINGDESCRIPTOR *textdp);
 void Mdputr4wd(const float *r4p, const UINT2 *wp, const UINT2 *dp);
 void Mdputr4wdb(const float *r4p, const UINT2 *wp, const UINT2 *dp);
 void Mdputi4wdb(const INT4 *i4p, const UINT2 *wp, const UINT2 *dp);


 /* MDH help routines. */

 void Mdh_help(SEMANTICLISTSTRUCTURE *semanticlistp);


 /* MDI command input routines. */

 void Mdi_cancel_input(void);
 UINT4 Mdi_get_command_line(const STRINGDESCRIPTOR *commanddp);
 UINT4 Mdi_get_input(const STRINGDESCRIPTOR *getstrdp, 
                     const STRINGDESCRIPTOR *promptdp, UINT2 *outlenp);
 UINT4 Mdi_initialize(void);
 UINT4 Mdi_pop_command_file(void);
 UINT4 Mdi_push_command_file(const STRINGDESCRIPTOR *filespecdp);
 UINT4 Mdi_rewind(void);
 UINT4 Mdi_terminate(void);


 /* MDL object module loader routines. */

 void Mdl_abort(UINT4 msgid, UINT4 p1, UINT4 p2, UINT4 p3);
 UINT4 Mdl_initialize(void);
 void Mdl_load_object_module(void *filenamedp, UINT1 loadflag, UINT1 debugflag);
 UINT4 Mdl_terminate(void);


 /* MDM macro routines. */

 UINT4 Mdm_define_macro(void *iddp, void *valuedp);
 UINT4 Mdm_expand_macros(const STRINGDESCRIPTOR *inputlinedp,
                         const STRINGDESCRIPTOR *outputlinedp);
 UINT4 Mdm_initialize(void);
 UINT4 Mdm_terminate(void);


 /* MDO console/logfile output routines. */

 void Mdo_close_logfile(void);
 UINT4 Mdo_initialize(void);
 UINT4 Mdo_log(STRINGDESCRIPTOR *stringdp);
 void Mdo_log_command(STRINGDESCRIPTOR *stringdp);
 void Mdo_open_logfile(void);
 void Mdo_put_msg(UINT4 msgid, UINT4 msgflags, UINT_PTR p1, UINT_PTR p2, 
                  UINT_PTR p3);
 UINT4 Mdo_put_output(void *textdp);
 void Mdo_reset_line_count(void);
 UINT4 Mdo_terminate(void);


 /* MDP command parser routines. */

 UINT4 Mdp_initialize(void);
 UINT4 Mdp_parse_command(STRINGDESCRIPTOR *commanddp, 
                         SEMANTICLISTSTRUCTURE **semanticlistpp);
 UINT4 Mdp_terminate(void);


 /* MDQ i/o routines. */

 UINT4 Mdq_close(void *unitp);
 void  Mdq_control_c_abort_enable(void);
 UINT4 Mdq_initialize(void);
 UINT4 Mdq_open(const void *unitp, const void *filenamedp, const void *defaultnamedp);
 UINT4 Mdq_read(void *unitp, const void *bufferdp, void *countp);
 UINT4 Mdq_terminate(void);


 /* MDS source file routines. */

 void Mds_get_source_line(CONNECTION conn, UINT4 lineindex,
                          const STRINGDESCRIPTOR *bufferdp, 
                          UINT4 *actualcountp);
 UINT4 Mds_initialize(void);
 void Mds_set_source(SEMANTICLISTSTRUCTURE *semanticlistp);
 void Mds_show_source_at(UINT2 frame, UINT4 offset);
 void Mds_show_source_line(CONNECTION conn, UINT4 lineindex);
 UINT4 Mds_terminate(void);
 void Mds_type(SEMANTICLISTSTRUCTURE *semanticlistp);


 /* MDV execution vehicle routines. */

 UINT4 Mdv_allocate_memory(UINT4 nbytes, UINT2 *selp);
 void Mdv_allow_breaks(void);
 void Mdv_break_ast(UINT4 node, UINT4 port);
 void Mdv_cache_gdt(UINT1 turncacheon);
 UINT4 Mdv_cancel_all(void);
 UINT4 Mdv_cancel_all_breakpoints(void);
 UINT4 Mdv_cancel_all_watchpoints(void);
 UINT4 Mdv_cancel_breakpoint(UINT4 addr, UINT4 nrtasks, UINT4 tasktokensp[]);
 UINT4 Mdv_cancel_profile(UINT2 tasktoken);
 UINT4 Mdv_cancel_watchpoint(UINT4 addr, UINT4 nrtasks, UINT4 tasktokensp[]);
 UINT4 Mdv_change_access_rights(UINT2 sel, UINT1 rights);
 void Mdv_check_for_task_break(void);
 UINT1 Mdv_code_is_32_bits(UINT2 frame);
 UINT4 Mdv_connect_to_remote(void);
 UINT4 Mdv_delete_user_job(UINT2 jobtoken, UINT2 tasktoken);
 UINT4 Mdv_delete_user_jobs(void);
 UINT4 Mdv_deposit_linear(UINT4 linearaddress, UINT4 nbytes, 
                          void *bufferp);
 UINT4 Mdv_deposit_memory(UINT2 frame, UINT4 offset, UINT4 nbytes, 
                          void *bufferp);
 void Mdv_disallow_breaks(void);
 UINT4 Mdv_examine_linear(UINT4 linearaddress, UINT4 nbytes, 
                          void *bufferp);
 UINT4 Mdv_examine_memory(UINT2 frame, UINT4 offset, UINT4 nbytes, 
                          void *bufferp);
 UINT4 Mdv_execute(UINT2 maxsteps, UINT4 atomicset, UINT4 locuslow, 
                   UINT4 locushigh);
 UINT4 Mdv_finish_loading(void);
 UINT4 Mdv_initialize(void);
 UINT4 Mdv_input(UINT2 port, UINT2 iosize, void *valuep);
 UINT4 Mdv_output(UINT2 port, UINT2 iosize, void *valuep);
 UINT4 Mdv_prepare_to_load(void);
 UINT4 Mdv_receive(void *msgp, UINT2 count, UINT2 *actualcountp);
 void Mdv_reset(void);
 UINT4 Mdv_segmented_to_linear(UINT2 frame, UINT4 offset);
 UINT4 Mdv_send(UINT1 portid, void *msgp, UINT2 count);
 UINT4 Mdv_send_login(void *msgp, UINT2 count);
 UINT4 Mdv_send_supervisor(void *msgp, UINT2 count);
 UINT4 Mdv_set_breakpoint(UINT4 addr, UINT2 aftercount, UINT4 nrtasks, 
                          UINT4 tasktokensp[]);
 UINT4 Mdv_set_profile(UINT4 startaddr, UINT4 endaddr, UINT2 tasktoken);
 UINT4 Mdv_set_watchpoint(UINT4 addr, UINT4 len, UINT4 rw, 
                          UINT2 aftercount, UINT4 nrtasks, 
                          UINT4 tasktokensp[]);
 void Mdv_show_break(void);
 void Mdv_show_profile(UINT2 tasktoken);
 void Mdv_show_users(void);
 void Mdv_show_watch(void);
 UINT4 Mdv_terminate(void);


 /* MDW window management routines. */

 void Mdw_cancel_cursor_line(void);
 void Mdw_cancel_window(void);
 void Mdw_display_source_line(CONNECTION conn, UINT4 lineindex, UINT1 hiliteflag);
 UINT4 Mdw_initialize(void);
 UINT4 Mdw_initialize_crt(void);
 void Mdw_set_window(SEMANTICLISTSTRUCTURE *semanticlistp);
 UINT4 Mdw_terminate(void);


 /* MDX command execution routines. */

 void Mdx_cancel(SEMANTICLISTSTRUCTURE *semanticlistp);
 void Mdx_cancel_break(SEMANTICLISTSTRUCTURE *semanticlistp);
 void Mdx_cancel_mode(void);
 void Mdx_cancel_scope(void);
 void Mdx_cancel_type(SEMANTICLISTSTRUCTURE *semanticlistp);
 void Mdx_cancel_watch(SEMANTICLISTSTRUCTURE *semanticlistp);
 void Mdx_check_norun(void);
 void Mdx_check_simulator(void);
 void Mdx_deposit(SEMANTICLISTSTRUCTURE *semanticlistp);
 void Mdx_difference(SEMANTICLISTSTRUCTURE *semanticlistp);
 void Mdx_evaluate(SEMANTICLISTSTRUCTURE *semanticlistp);
 void Mdx_examine(SEMANTICLISTSTRUCTURE *semanticlistp);
 void Mdx_go(SEMANTICLISTSTRUCTURE *semanticlistp);
 UINT4 Mdx_initialize_topcpu(void);
 void Mdx_input(SEMANTICLISTSTRUCTURE *semanticlistp);
 void Mdx_load(const STRINGDESCRIPTOR *filenamedp, UINT4 flags);
 void Mdx_output(SEMANTICLISTSTRUCTURE *semanticlistp);
 void Mdx_run(SEMANTICLISTSTRUCTURE *semanticlistp);
 void Mdx_run_log(void *fnamedp);
 void Mdx_set(SEMANTICLISTSTRUCTURE *semanticlistp);
 void Mdx_set_break(SEMANTICLISTSTRUCTURE *semanticlistp);
 void Mdx_set_language(SEMANTICLISTSTRUCTURE *semanticlistp);
 void Mdx_set_log(SEMANTICLISTSTRUCTURE *semanticlistp);
 void Mdx_set_mode(SEMANTICLISTSTRUCTURE *semanticlistp);
 UINT2 Mdx_set_options(UINT2 oldoptions, UINT4 optionflags);
 void Mdx_set_output(SEMANTICLISTSTRUCTURE *semanticlistp);
 void Mdx_set_profile(SEMANTICLISTSTRUCTURE *semanticlistp);
 void Mdx_set_prompt(SEMANTICLISTSTRUCTURE *semanticlistp);
 void Mdx_set_scope(SEMANTICLISTSTRUCTURE *semanticlistp);
 void Mdx_set_step(SEMANTICLISTSTRUCTURE *semanticlistp);
 void Mdx_set_type(SEMANTICLISTSTRUCTURE *semanticlistp);
 void Mdx_set_watch(SEMANTICLISTSTRUCTURE *semanticlistp);
 void Mdx_show(SEMANTICLISTSTRUCTURE *semanticlistp);
 void Mdx_show_calls(SEMANTICLISTSTRUCTURE *semanticlistp);
 void Mdx_show_instruction_break(UINT1 breaktype, UINT2 tasktoken, 
                                 UINT1 opcode, UINT4 watchaddr, 
                                 TASKREGS *taskregsp,
                                 TASKNDP *ndpstuffp);
 void Mdx_show_job(SEMANTICLISTSTRUCTURE *semanticlistp);
 void Mdx_show_language(void);
 void Mdx_show_log(void);
 void Mdx_show_mode(void);
 void Mdx_show_module(SEMANTICLISTSTRUCTURE *semanticlistp);
 void Mdx_show_ndp(SEMANTICLISTSTRUCTURE *semanticlistp);
 void Mdx_show_object(SEMANTICLISTSTRUCTURE *semanticlistp);
 void Mdx_show_output(void);
 void Mdx_show_reg(SEMANTICLISTSTRUCTURE *semanticlistp);
 void Mdx_show_scope(void);
 void Mdx_show_step(void);
 void Mdx_show_symbol(SEMANTICLISTSTRUCTURE *semanticlistp);
 void Mdx_show_system(SEMANTICLISTSTRUCTURE *semanticlistp);
 void Mdx_show_task(SEMANTICLISTSTRUCTURE *semanticlistp);
 void Mdx_show_topcpu(SEMANTICLISTSTRUCTURE *semanticlistp);
 void Mdx_show_type(SEMANTICLISTSTRUCTURE *semanticlistp);
 void Mdx_show_typedef(UINT4 typeindex);
 void Mdx_step(SEMANTICLISTSTRUCTURE *semanticlistp);
 void Mdx_use(SEMANTICLISTSTRUCTURE *semanticlistp);


 /* MDWILD wildcard parsing routines. */

 UINT1 Mdx_wildcards_present(ASCIC *patternp);
 UINT1 Mdx_wildcard_match(ASCIC *targetp);


 /* MDC semantic descriptor list evaluation routines. */

 void *Mdc_evaluate_sdl(void *sdlp, int compile_flag);


 /* MDZ symbolic address routines. */

 void Mdz_count_symbols(void);
 UINT4 Mdz_expand_elist(void *elistp, void *stringdp, void *countp);
 UINT4 Mdz_expand_symbolic_address(UINT2 frame, UINT4 offset, 
                                   void *stringdp, void *countp);
 void *Mdz_fully_qualified_name(void *currentblockp);
 void Mdz_get_scalar_type_and_length(UINT4 typeindex, void *scalartypep, 
                                     void *scalarlengthp);
 UINT4 Mdz_initialize(void);
 void Mdz_insert_symbol(UINT4 offset, UINT2 frame, UINT1 priority, void *blockp);
 void *Mdz_line_number(UINT2 frame, UINT4 offset, void *upperoffsetp);
 void Mdz_physical_to_segmented(UINT4 a24, void *framep, void *offsetp);
 UINT4 Mdz_proc_containing(UINT2 frame, UINT4 offset, void *blockpp);
 void Mdz_show_elist(void *elistp);
 void Mdz_show_fully_qualified_name(void *currentblockp);
 void Mdz_show_local_value(UINT4 typeindex, void *valuep);
 void Mdz_show_scalar_value(UINT4 scalartype, UINT4 scalarlength, void *valuep);
 UINT4 Mdz_show_simple_address(UINT2 frame, UINT4 offset);
 void Mdz_show_simple_name(void *blockp);
 void Mdz_show_symbolic_address(UINT2 frame, UINT4 offset);
 void Mdz_sort_symbols(void);
 void *Mdz_symbolic_address(UINT2 frame, UINT4 offset);
 UINT4 Mdz_terminate(void);


 /********* Other MD386 Routines **************/

 void qiksortu(const UINT4 *num_elts_p, void *array_p, const UINT2 *elt_siz_p, 
               const UINT2 *key_off_p, const UINT2 *key_siz_p);
 void Udiabort(UINT2 status);

/************************** end of MD386PROTO.H ***************************/
