/* **MEMBER**=NONE
 * **CMS**=RMX_MD86VAX
 *
 * MDA386TB.H contains 80386 disassembler tables included by
 * MDA.C.
 *
 *
 *  Copyright 1996-2014 AFH Systems
 *
 *  FilterMeister is free software. You can redistribute and modify it
 *  under the terms of the MPL2 License.
 *  See: https://www.mozilla.org/en-US/MPL/2.0/
 *  Any changes to the FilterMeister files have to be published or sent
 *  to harry@thepluginsite.com. The filter plugin code itself does not
 *  need to be disclosed.
 *
 * 07Dec82 TEG  Written.
 * 17Nov88 ADG  Updated for 80386.
 * 05Dec91 TEG  Rebuilt.
 * 04May97 AFH  Converted to C.
 * 13May97 AFH  1. Fix incorrect opcodes and decoding for CMPXCHG.
 *              2. Operands for BSF, BSR were reversed.
 *              3. Operand order was wrong for SHLD/SHRD r/m,reg,CL.
 *              4. In Shift Group 2 (DCG03), the /6 subopcode should
 *                 be UNK, not SHL.
 *              5. Add Pentium/Pentium Pro/Pentium II instructions.
 *              6. Add MMX instructions.
 *              7. Display SGDTW/D, SIDTW/D, LGDTW/D, LIDTW/D depending
 *                 on operand size. WHY??? WHY NOT USE FBYTE/PWORD/TBYTE PTR 
 *                 FOR USE16 (5 bytes), USE32 (6 bytes) and USE64 (10 bytes), resp.????????
 *              8. Fix incorrect operand decoding for FST (DD/Dx).
 *              9. ASM386 doesn't recognize the syntax FADD ST(i), so
 *                 force md386 to emit FADD ST,ST(i); ditto for FMUL,
 *                 FSUB, FSUBR, FDIV, FDIVR.
 *             10. Similarly, ASM386 doesn't recognize the syntax FADDP ST(i)
 *                 for FADDP ST(i),ST; ditto for FMULP, FSUBP, FSUBRP,
 *                 FDIVP, FDIVRP.
 *             11. Reinstate FENI, FDISI, FSETPM in case they are encountered
 *                 in legacy code.
 *  03Aug12 AFH 1. Add (minimal) X64 support.
 *	31May14 AFH 1. Add more comments to document the mda data structures.
 *	01Jun14 AFH	1. Add kludge for ARPL/MOVSXD x64 change.
 *
 */

#define DCD_OPSIZX   0x0020001F  /* index into OPSIZ (discontiguous!). */
#define DCD_NOPRNDS  0x00000060  /* # of operands. */
#define DCD_DESTREG  0x00000080  /* destination is reg. */
#define DCD_MODRM    0x00000100  /* modrm expected. */
#define DCD_MODRM2   DCD_MODRM
#define DCD_SCNDOP   0x00000200  /* secondary opcode. */
#define DCD_OFFNOMOD 0x00000400  /* offset without modrm. */
#define DCD_IMPEAX   0x00000800  /* implicit EAX/AX/AL. */
#define DCD_OPREG    0x00001000  /* opcode gives reg #. */
#define DCD_REGSEG   0x00002000  /* reg is segreg. */
#define DCD_REGSPCL  0x00004000  /* reg is special. */
#define DCD_MNEMD    0x00008000  /* oprndsiz affects mnem. */
#define DCD_SHIFT1   0x00010000  /* shift by 1 bit. */
#define DCD_SHIFTCL  0x00020000  /* shift by CL bits. */
#define DCD_IBMOD32  0x00040000  /* immediate byte mod 32. */	//mod 64 for X64 ???
#define DCD_IBZX     0x00080000  /* zero-extend immed byte. */
#define DCD_PFHEX    0x00100000  /* prefer hex (immed). */
#define DCD_00200000 0x00200000  /* Part of DCD_OPSIZX!!! */
#define DCD_00400000 0x00400000  /* UNUSED. */
#define DCD_ESC      0x00800000  /* escape (80387). */
#define DCD_REGNT    0x07000000  /* seg/special reg #/type, */
                                 /* or string op type. */
#define DCD_DCWX     0xF8000000  /* index into DECODW. */

#define DCW_FAR          0x0001  /* operand is far. */
#define DCW_JMPCALL      0x0002  /* jmp/call, not ret/iret. */
#define DCW_JMPCOND      0x0004  /* conditional jmp. */
#define DCW_RET          0x0004  /* ret. */
#define DCW_IRET         0x0008  /* iret. */
#define DCW_ENTER        0x0010  /* enter. */
#define DCW_STRING       0x0020  /* one-byte string op. */
#define DCW_STRENE       0x0040  /* string op sees e/ne. */
#define DCW_WAIT         0x0080  /* wait. */
#define DCW_SHIFT        0x0100  /* shift. */
#define DCW_INOUT        0x0200  /* in/out. */
#define DCW_LEA          0x0400  /* lea. */
#define DCW_AAMAAD       0x0800  /* aam/aad. */		//X64 invalid ???
#define DCW_XLAT         0x1000  /* xlat. */
#define DCW_PREFIX       0xE000  /* prefix or two-byte op. */
#define DCPRF_SEG        0x2000  /* prefix: seg override. */
#define DCPRF_OSIZ       0x4000  /* prefix: opsize toggle. */
#define DCPRF_ASIZ       0x6000  /* prefix: adrsize toggle. */
#define DCPRF_LOCK       0x8000  /* prefix: lock. */
#define DCPRF_REPNE      0xA000  /* prefix: repne. */
#define DCPRF_REPE       0xC000  /* prefix: repe. */
#define DCPRF_TWOBYT     0xE000  /* two-byte opcode. */
/* Following value doesn't fit in the DCW "word" (see mda.c) */
#define DCPRF_REX		0x10000  /* "phantom" value for prefix: rex */

#define DCG01  1
#define DCG02  2
#define DCG03  3
#define DCG04  4
#define DCG05  5
#define DCG06  6
#define DCG07  7
#define DCG08  8
#define DCG09  9
#define DCG10 10
#define DCG11 11
#define DCG12 12
#define DCG13 13
#define DCG14 14
#define DCG15 15

#define DFG01  1
#define DFG02  2
#define DFG03  3
#define DFG04  4
#define DFG05  5
#define DFG06  6
#define DFG07  7
#define DFG08  8
#define DFG09  9

/*
 *  The DCD_xxxx masks define bits in DECOD, DECODT, DECODS, DECODF, and
 *  DECODFR.  The DCW_xxxx masks define bits in DECODW.
 *  OPMNE, OPMNET, OPMNES, OPMNEF, OPMNEFR, and OPMNEFS give indices into
 *   MNEOP or secondary opcode group #'s.
 *  DECOD and OPMNE are for 1-byte opcodes.
 *  DECODT and OPMNET are for 2-byte opcodes.
 *  DECODS and OPMNES are for secondary opcodes.
 *  DECODF and OPMNEF are for 80387 opcodes referencing memory (M2 < 3).
 *  DECODFR and OPMNEFR are for 80387 opcodes not referencing memory
 *   (M2 = 3).
 *  OPMNEFS is for 80387 secondary opcodes (M2 = 3 only).
 *  MNEOP contains ascii mnemonics of opcodes;  MNERG, MNESG contain
 *   ascii names of registers, etc.
 *
 *  Secondary opcode groups are:
 *       80-81,83:     DCG01 (Intel Group 1 - Immediate)
 *       8F:           DCG02 (POP Ev)
 *       C0-C1,D0-D3:  DCG03 (Intel Group 2 - Shifts)
 *       C6-C7:        DCG04 (MOV Eb,Ib/MOV Ev,Iv)
 *       F6:           DCG05 (Intel Group 3a - Unary)
 *       F7:           DCG06 (Intel Group 3b - Unary)
 *       FE:           DCG07 (Intel Group 4)
 *       FF:           DCG08 (Intel Group 5)
 *       000F:         DCG09 (Intel Group 6)
 *       010F:         DCG10 (Intel Group 7)
 *       BA0F:         DCG11 (Intel Group 8)
 *       C70F:         DCG12 (Intel Group 9 - CMPXCGH8B)
 *       610F:         DCG13 (Intel Group A1 - PSHIMW)
 *       620F:         DCG14 (Intel Group A2 - PSHIMD)
 *       630F:         DCG15 (Intel Group A3 - PSHIMQ)
 *
 *  Secondary floating-point opcode groups are:
 *       D0D9:         DFG01
 *       D9DE:         DFG02
 *       E0D9:         DFG03
 *       E2DB:         DFG04
 *       E0DF:         DFG05
 *       E8D9:         DFG06
 *       E9DA:         DFG07
 *       F0D9:         DFG08
 *       F8D9:         DFG09
 *
 *  Escape (80387) primary opcodes are six bits:  the high-order three
 *  bits are bits 5-3 of the modrm byte, and the low-order three bits are
 *  the low-order three bits of the opcode byte (D8+x).
 *
 *  String instructions are typed (DCD_REGNT) as follows:
 *       6C-6D INS   1              AA-AB STOS  5
 *       6E-6F OUTS  2              AC-AD LODS  6
 *       A4-A5 MOVS  3              AE-AF SCAS  7
 *       A6-A7 CMPS  4
 *
 */

/*
 *	OPSIZ[] is indexed by the DCD_OPSIZX field of DECOD*[].
 *
 *  In each dword in OPSIZ,			          USE32       USE16
 *   we use six nibbles (hi ... lo) thus:  imm reg rm  imm reg rm
 *	where
 *		imm is the size of the immediate field in bytes:
 *			0 = no imm field, 1 = imm8, 2 = imm16, 3 = ????,
 *			4 = imm32, 6 = ???.
 *			
 *		reg is the size of the type of the reg field:
 *			0 = no reg, 1 = byte (r8), 2 = word (r16), 4 = dword (r32),
 *			6 = pword (mmx) 8 = qword r64, 0xA = tbyte reg (fpr)
 *
 *		rm is the size of the type of the r/m field:
 *			0 = no r/m field, 1 = byte, 2 = word, 4 = dword,
 *			6 = pword, 8 = qword, 0xA = tbyte.
 */
      static const UINT4 OPSIZ[64] = {
            0x000000, 0x011011, 0x044022, 0x022022,   /* 00 */
            0x041021, 0x042022, 0x046024, 0x048024,   /* 04 */
            0x044044, 0x021021, 0x084042, 0x024022,   /* 08 */
            0x066066, 0x042021, 0x012012, 0x088088,   /* 0C */
            0x111111, 0x444222, 0x144122, 0x122122,   /* 10 */
            0x100100, 0x300300, 0x400200, 0x600400,   /* 14 */
            0x200200, 0x112112, 0x142122, 0x084084,   /* 18 */
            0x0A20A2, 0x0A40A4, 0x0A80A8, 0x0AA0AA,   /* 1C */
            0x108108,        0,        0,        0,   /* 20 */
                   0,        0,        0,        0,   /* 24 */
                   0,        0,        0,        0,   /* 28 */
                   0,        0,        0,        0,   /* 2C */
                   0,        0,        0,        0,   /* 30 */
                   0,        0,        0,        0,   /* 34 */
                   0,        0,        0,        0,   /* 38 */
                   0,        0,        0,        0    /* 3C */
      };

      static const UINT2 DECODW[32] = {
             0x0000, 0x0001, 0x0002, 0x0003,   /* 00 */
             0x0006, 0x0004, 0x0005, 0x0009,   /* 04 */
             0x0010, 0x0020, 0x0060, 0x0080,   /* 08 */
             0x0100, 0x0200, 0x0220, 0x0400,   /* 0C */
             0x0800, 0x1000,      0,      0,   /* 10 */
                  0,      0,      0,      0,   /* 14 */
                  0, 0x2000, 0x4000, 0x6000,   /* 18 */
             0x8000, 0xA000, 0xC000, 0xE000    /* 1C */
      };


      static const UINT4 DECOD[256] = {
          0x000141,   0x000142,   0x0001C1,   0x0001C2, /* 00 */
          0x0008D0,   0x0008D1,  0x000302B,  0x00030AB, /* 04 */
          0x000141,   0x000142,   0x0001C1,   0x0001C2, /* 08 */
          0x1008D0,   0x1008D1,  0x100302B, 0xF8000000, /* 0C */
          0x000141,   0x000142,   0x0001C1,   0x0001C2, /* 10 */
          0x0008D0,   0x0008D1,  0x200302B,  0x20030AB, /* 14 */
          0x000141,   0x000142,   0x0001C1,   0x0001C2, /* 18 */
          0x0008D0,   0x0008D1,  0x300302B,  0x30030AB, /* 1C */
          0x000141,   0x000142,   0x0001C1,   0x0001C2, /* 20 */
          0x1008D0,   0x1008D1, 0xC8003000,   0x000881, /* 24 */
          0x000141,   0x000142,   0x0001C1,   0x0001C2, /* 28 */
          0x0008D0,   0x0008D1, 0xC9003000,   0x000881, /* 2C */
          0x000141,   0x000142,   0x0001C1,   0x0001C2, /* 30 */
          0x1008D0,   0x1008D1, 0xCA003000,   0x000883, /* 34 */
          0x000141,   0x000142,   0x0001C1,   0x0001C2, /* 38 */
          0x0008D0,   0x0008D1, 0xCB003000,   0x000883, /* 3C */
         0x00010A2,  0x10010A2,  0x20010A2,  0x30010A2, /* 40 */
         0x40010A2,  0x50010A2,  0x60010A2,  0x70010A2, /* 44 */
         0x00010A2,  0x10010A2,  0x20010A2,  0x30010A2, /* 48 */
         0x40010A2,  0x50010A2,  0x60010A2,  0x70010A2, /* 4C */
         0x0001022,  0x1001022,  0x2001022,  0x3001022, /* 50 */
         0x4001022,  0x5001022,  0x6001022,  0x7001022, /* 54 */
         0x00010A2,  0x10010A2,  0x20010A2,  0x30010A2, /* 58 */
         0x40010A2,  0x50010A2,  0x60010A2,  0x70010A2, /* 5C */
          0x008002,   0x008082,   0x0001C7,   0x000143, /* 60 *///63=143 for ARPL 
																//   100 = MODRM
																//	  40 = 2 opnds
																// DCD_DESTREG=0 
																//	   3 = opszv index = WORD (inv?)
																//	OPSIZ[3] = 022 022 (r/m16,r16)
																//63=xxx for MOVSDX
																//	 100 = MODRM
																//	  40 = 2 opnds
																// DCD_DESTREG=1 
																//	   8 = opszv index = QWORD
																//	OPSIZ[x] = 044 000 (r32,r/m32)

        0xCC003000, 0xCD003000, 0xD0000000, 0xD8000000, /* 64 */
          0x000031,   0x0001F1,   0x000030,   0x0001F2, /* 68 */
        0x71000049, 0x7100804B, 0x720000C9, 0x720080CB, /* 6C */
        0x20100034, 0x20100034, 0x20100034, 0x20100034, /* 70 */
        0x20100034, 0x20100034, 0x20100034, 0x20100034, /* 74 */
        0x20100034, 0x20100034, 0x20100034, 0x20100034, /* 78 */
        0x20100034, 0x20100034, 0x20100034, 0x20100034, /* 7C */
          0x000350,   0x000351,          0,   0x000352, /* 80 */
          0x0001C1,   0x0001C2,   0x0001C1,   0x0001C2, /* 84 */
          0x000141,   0x000142,   0x0001C1,   0x0001C2, /* 88 */
          0x002143, 0x780001C2,   0x0021C3,   0x000322, /* 8C */
         0x0001882,  0x10018C2,  0x20018C2,  0x30018C2, /* 90 */
         0x40018C2,  0x50018C2,  0x60018C2,  0x70018C2, /* 94 */
          0x00888D,   0x00888A, 0x18000037, 0x58000000, /* 98 */																
          0x008002,   0x008082,   0x000801,   0x000881, /* 9C */
          0x000CC1,   0x000CC2,   0x000C41,   0x000C42, /* A0 */
        0x4B0000C1, 0x4B0080C2, 0x54000041, 0x54008042, /* A4 */
          0x1008D0,   0x1008D1, 0x4D000821, 0x4D008822, /* A8 */
        0x4E0008A1, 0x4E0088A2, 0x57000821, 0x57008822, /* AC */
         0x00010D0,  0x10010D0,  0x20010D0,  0x30010D0, /* B0 */
         0x40010D0,  0x50010D0,  0x60010D0,  0x70010D0, /* B4 */
         0x00010D1,  0x10010D1,  0x20010D1,  0x30010D1, /* B8 */
         0x40010D1,  0x50010D1,  0x60010D1,  0x70010D1, /* BC */
        0x601C0350, 0x601C0352, 0x28000038, 0x28000000, /* C0 */
        0x080001C6, 0x080001C6,   0x000350,   0x000351, /* C4 */
        0x400C0055,   0x000082, 0x30000038, 0x30000000, /* C8 */
          0x000000,   0x000034,   0x000000, 0x38008000, /* CC */
        0x60010341, 0x60010342, 0x60020341, 0x60020342, /* D0 */
        0x800008B3, 0x800008B3,          0, 0x8B0018C1, /* D4 */
          0x800100,   0x800100,   0x800100,   0x800100, /* D8 */
          0x800100,   0x800100,   0x800100,   0x800100, /* DC */
        0x20000034, 0x20000034, 0x20000034, 0x20008034, /* E0 */
        0x681808D9, 0x681808DA, 0x68180859, 0x6818085A, /* E4 */
        0x10100036, 0x10100036, 0x18000037, 0x10100034, /* E8 */
        0x6A0018CE, 0x6A0018C5, 0x6A00184E, 0x6A001845, /* EC */
        0xE0000000,          0, 0xE8000000, 0xF0000000, /* F0 */
          0x000000,   0x000000,   0x000300,   0x000300, /* F4 */
          0x000000,   0x000000,   0x000000,   0x000000, /* F8 */
          0x000000,   0x000000,   0x000321,   0x000300  /* FC */
      };

	  /* DECODT[] is indexed by 2nd byte of a 2-byte opcode */
      static const UINT4 DECODT[256] = {
          0x000323,   0x000300,   0x0001C5,   0x0001C5, /* 00 */
                 0,          0,   0x000000,          0, /* 04 */
          0x000000,   0x000000,          0,   0x000000, /* 08 */
                 0,          0,          0,          0, /* 0C */
                 0,          0,          0,          0, /* 10 */
                 0,          0,          0,          0, /* 14 */
                 0,          0,          0,          0, /* 18 */
                 0,          0,          0,          0, /* 1C */
         0x1004148,  0x2004148,  0x10041C8,  0x20041C8, /* 20 */
         0x3004148,          0,  0x30041C8,          0, /* 24 */
                 0,          0,          0,          0, /* 28 */
                 0,          0,          0,          0, /* 2C */
          0x000000,   0x000000,   0x000000,   0x000000, /* 30 */
                 0,          0,          0,          0, /* 34 */
                 0,          0,          0,          0, /* 38 */
                 0,          0,          0,          0, /* 3C */
          0x0001C2,   0x0001C2,   0x0001C2,   0x0001C2, /* 40 */
          0x0001C2,   0x0001C2,   0x0001C2,   0x0001C2, /* 44 */
          0x0001C2,   0x0001C2,   0x0001C2,   0x0001C2, /* 48 */
          0x0001C2,   0x0001C2,   0x0001C2,   0x0001C2, /* 4C */
                 0,          0,          0,          0, /* 50 */
                 0,          0,          0,          0, /* 54 */
                 0,          0,          0,          0, /* 58 */
                 0,          0,          0,          0, /* 5C */
          0x0001CF,   0x0001CF,   0x0001CF,   0x0001CF, /* 60 */
          0x0001CF,   0x0001CF,   0x0001CF,   0x0001CF, /* 64 */
          0x0001CF,   0x0001CF,   0x0001CF,   0x0001CF, /* 68 */
                 0,          0,   0x0001DB,   0x0001CF, /* 6C */
                 0,   0x200340,   0x200340,   0x200340, /* 70 */
          0x0001CF,   0x0001CF,   0x0001CF,   0x000000, /* 74 */
                 0,          0,          0,          0, /* 78 */
                 0,          0,   0x00015B,   0x00014F, /* 7C */
        0x20100036, 0x20100036, 0x20100036, 0x20100036, /* 80 */
        0x20100036, 0x20100036, 0x20100036, 0x20100036, /* 84 */
        0x20100036, 0x20100036, 0x20100036, 0x20100036, /* 88 */
        0x20100036, 0x20100036, 0x20100036, 0x20100036, /* 8C */
          0x000121,   0x000121,   0x000121,   0x000121, /* 90 */
          0x000121,   0x000121,   0x000121,   0x000121, /* 94 */
          0x000121,   0x000121,   0x000121,   0x000121, /* 98 */
          0x000121,   0x000121,   0x000121,   0x000121, /* 9C */
         0x400302B,  0x40030AB,   0x000000,   0x000142, /* A0 */
        0x600C0172, 0x60020162,          0,          0, /* A4 */
         0x500302B,  0x50030AB,   0x000000,   0x000142, /* A8 */
        0x600C0172, 0x60020162,          0,   0x0001C2, /* AC */
          0x000141,   0x000142, 0x080001C6,   0x000142, /* B0 */
        0x080001C6, 0x080001C6,   0x0001C4,   0x0001C5, /* B4 */
                 0,          0,   0x000352,   0x000142, /* B8 */
          0x0001C2,   0x0001C2,   0x0001C4,   0x0001C5, /* BC */
																//BE=MOVSXB (flgl=0001C4)
																// 100 = DCD_MODRM
																//  80 = DCD_DESTREG
																//DCD_NOPRNDS = 2
																//OPSIZ index = 4
																//OPSIZ[4] = 041 r32,r/m8 (use32)
																//           021 r16,r/m8 (use16)
																//--------------------------------
																//BF=MOVSXW (flgl=0001C5)
																// 100 = DCD_MODRM
																//  80 = DCD_DESTREG
																//DCD_NOPRNDS = 2
																//OPSIZ index = 5
																//OPSIZ[5] = 042 r32,r/m16 (use32)
																//           022 r16,r/m16 (use16,not used?)
																//---------------------------------
																//63=MOVSXD (flgl=0001Cx)
																// 100 = DCD_MODRM
																//	80 = DCD_DESTREG
																//DCD_NOPRNDS = 2 (40)
																//OPSIZ index = x
																//OPSIZ[x] = 084 r64,r/m32 (use64)
																//			 044 r32,r/m32 (use32)
																//			 022 r16,r/m32	(use16???)
																//OPSIZ[x] = 044022 or 044xxx ??
																//            x = 2 or  x = 8
																//==> flgl = 0001C2 or 0001C8
																//for 0001C2:
																// op = 03 (add Gv,Ev)
																// op = 0B (or Gv,Ev)
																// op = 13 (adc Gv,Ev)...etc.
																//for 0001C8: none?
																//
          0x000141,   0x000142,          0,          0, /* C0 */
                 0,          0,          0,   0x00032F, /* C4 */
         0x00010A8,  0x10010A8,  0x20010A8,  0x30010A8, /* C8 */
         0x40010A8,  0x50010A8,  0x60010A8,  0x70010A8, /* CC */
                 0,   0x0001CF,   0x0001CF,   0x0001CF, /* D0 */
                 0,   0x0001CF,          0,          0, /* D4 */
          0x0001CF,   0x0001CF,          0,   0x0001CF, /* D8 */
          0x0001CF,   0x0001CF,          0,   0x0001CF, /* DC */
                 0,   0x0001CF,   0x0001CF,          0, /* E0 */
                 0,   0x0001CF,          0,          0, /* E4 */
          0x0001CF,   0x0001CF,          0,   0x0001CF, /* E8 */
          0x0001CF,   0x0001CF,          0,   0x0001CF, /* EC */
                 0,   0x0001CF,   0x0001CF,   0x0001CF, /* F0 */
                 0,   0x0001CF,          0,          0, /* F4 */
          0x0001CF,   0x0001CF,   0x0001CF,          0, /* F8 */
          0x0001CF,   0x0001CF,   0x0001CF,          0  /* FC */
        };

      static const UINT4 DECODS[DCG15][8] = {
          0x000000,   0x100000,   0x000000,   0x000000, /* 00  1 */
          0x100000,   0x000000,   0x100000,   0x000000, /* 04    */
          0x000000,   0x000000,   0x000000,   0x000000, /* 00  2 */
          0x000000,   0x000000,   0x000000,   0x000000, /* 04    */
        0x60000000, 0x60000000, 0x60000000, 0x60000000, /* 00  3 */
        0x60000000, 0x60000000, 0x60000000, 0x60000000, /* 04    */
          0x000000,   0x000000,   0x000000,   0x000000, /* 00  4 */
          0x000000,   0x000000,   0x000000,   0x000000, /* 04    */
          0x100350,          0,   0x000321,   0x000321, /* 00  5 */
          0x000BA9,   0x000BA9,   0x000BA9,   0x000BA9, /* 04    */
          0x100351,          0,   0x000322,   0x000322, /* 00  6 */
          0x000BAA,   0x000BAA,   0x000BAA,   0x000BAA, /* 04    */
          0x000321,   0x000321,          0,          0, /* 00  7 */
                 0,          0,          0,          0, /* 04    */
          0x000322,   0x000322, 0x10000322, 0x18000326, /* 00  8 */
        0x10000322, 0x18000326,   0x000322,          0, /* 04    */
          0x000323,   0x000323,   0x0003A3,   0x0003A3, /* 00  9 */
          0x000323,   0x000323,          0,          0, /* 04    */
          0x00832C,   0x00832C,   0x0083AC,   0x0083AC, /* 00 10 */
          0x000323,          0,   0x0003A3, 0x78000321, /* 04    */
                 0,          0,          0,          0, /* 00 11 */
          0x010352,   0x010352,   0x010352,   0x010352, /* 04    */ /* Why is DCD_SHIFT1 set??? */
                 0,   0x00032F,          0,          0, /* 00 12 */
                 0,          0,          0,          0, /* 04    */
                 0,          0,   0x280340,          0, /* 00 13 */
          0x280340,          0,   0x280340,          0, /* 04    */
                 0,          0,   0x280340,          0, /* 00 14 */
          0x280340,          0,   0x280340,          0, /* 04    */
                 0,          0,   0x280340,          0, /* 00 15 */
                 0,          0,   0x280340,          0  /* 04    */
      };


      static const UINT4 DECODF[64] = {
          0x01BD, 0x01BD, 0x01BD, 0x01BD, /* 00 */
          0x01BE, 0x01BE, 0x01BC, 0x01BC, /* 04 */
          0x01BD,      0, 0x01BD,      0, /* 08 */
          0x01BE,      0, 0x01BC,      0, /* 0C */
          0x01BD, 0x013D, 0x01BD, 0x013D, /* 10 */
          0x01BE, 0x013E, 0x01BC, 0x013C, /* 14 */
          0x01BD, 0x013D, 0x01BD, 0x013D, /* 18 */
          0x01BE, 0x013E, 0x01BC, 0x013C, /* 1C */
          0x01BD, 0x01A2, 0x01BD,      0, /* 20 */
          0x01BE, 0x01A2, 0x01BC, 0x01BF, /* 24 */
          0x01BD, 0x01A3, 0x01BD, 0x01BF, /* 28 */
          0x01BE,      0, 0x01BC, 0x01BE, /* 2C */
          0x01BD, 0x0122, 0x01BD,      0, /* 30 */
          0x01BE, 0x0122, 0x01BC, 0x013F, /* 34 */
          0x01BD, 0x0123, 0x01BD, 0x013F, /* 38 */
          0x01BE, 0x0123, 0x01BC, 0x013E  /* 3C */
      };

      static const UINT4 DECODFR[64] = {
          0x00DF, 0x00BF, 0x00BF, 0x00BF, /* 00 */
          0x005F, 0x003F, 0x005F,      0, /* 04 */
          0x00DF, 0x00BF, 0x00BF, 0x00BF, /* 08 */
          0x005F,      0, 0x005F,      0, /* 0C */
          0x00BF, 0x0200, 0x00BF, 0x00BF, /* 10 */
               0, 0x003F,      0,      0, /* 14 */
          0x00BF,      0, 0x00BF, 0x00BF, /* 18 */
               0, 0x003F, 0x0200,      0, /* 1C */
          0x00DF, 0x0200,      0, 0x0200, /* 20 */
          0x005F, 0x00BF, 0x005F, 0x1223, /* 24 */
          0x00DF, 0x0200, 0x0200, 0x00BF, /* 28 */
          0x005F, 0x00BF, 0x005F, 0x00BF, /* 2C */
          0x00DF, 0x0200,      0, 0x00BF, /* 30 */
          0x005F,      0, 0x005F, 0x00BF, /* 34 */
          0x00DF, 0x0200,      0,      0, /* 38 */
          0x005F,      0, 0x005F,      0  /* 3C */
      };


      static const UINT1 OPMNE[256] = {
           6,    6,    6,    6,    6,    6,  114,  109,   /* 00 */
         104,  104,  104,  104,  104,  104,  114,    0,   /* 08 */
           5,    5,    5,    5,    5,    5,  114,  109,   /* 10 */
         129,  129,  129,  129,  129,  129,  114,  109,   /* 18 */
           7,    7,    7,    7,    7,    7,  133,   32,   /* 20 */
         166,  166,  166,  166,  166,  166,  133,   33,   /* 28 */
         175,  175,  175,  175,  175,  175,  133,    1,   /* 30 */
          25,   25,   25,   25,   25,   25,  133,    4,   /* 38 */
          42,   42,   42,   42,   42,   42,   42,   42,   /* 40 */
          34,   34,   34,   34,   34,   34,   34,   34,   /* 48 */
         114,  114,  114,  114,  114,  114,  114,  114,   /* 50 */
         109,  109,  109,  109,  109,  109,  109,  109,   /* 58 */
         115,  110,    9,    8,  133,  133,  156,  156,   /* 60 */
         114,   40,  114,   40,   43,   44,  106,  107,   /* 68 */
          68,   66,   55,   54,   59,   65,   56,   53,   /* 70 */
          71,   67,   69,   70,   62,   61,   63,   60,   /* 78 */
       DCG01,DCG01,    0,DCG01,  167,  167,  173,  173,   /* 80 */
          94,   94,   94,   94,   94,   75,   94,DCG02,   /* 88 */
         102,  173,  173,  173,  173,  173,  173,  173,   /* 90 */
          18,   30,   17,  170,  117,  112,  127,   72,   /* 98 */
          94,   94,   94,   94,   95,   96,   26,   27,   /* A0 */
         167,  167,  162,  163,   85,   86,  130,  131,   /* A8 */
          94,   94,   94,   94,   94,   94,   94,   94,   /* B0 */
          94,   94,   94,   94,   94,   94,   94,   94,   /* B8 */
       DCG03,DCG03,  124,  124,   77,   74,DCG04,DCG04,   /* C0 */
          36,   76,  124,  124,   47,   46,   48,   51,   /* C8 */
       DCG03,DCG03,DCG03,DCG03,    3,    2,    0,  174,   /* D0 */
          37,   37,   37,   37,   37,   37,   37,   37,   /* D8 */
          90,   89,   88,   57,   41,   41,  105,  105,   /* E0 */
          17,   64,   64,   64,   41,   41,  105,  105,   /* E8 */
          84,    0,  122,  123,   38,   24,DCG05,DCG06,   /* F0 */
          20,  159,   22,  161,   21,  160,DCG07,DCG08    /* F8 */
      };

      static const UINT2 OPMNET[256] = {
       DCG09,DCG10,   73,   91,    0,    0,   23,    0,   /* 00 */
          49,  171,    0,  198,    0,    0,    0,    0,   /* 08 */
           0,    0,    0,    0,    0,    0,    0,    0,   /* 10 */
           0,    0,    0,    0,    0,    0,    0,    0,   /* 18 */
          94,   94,   94,   94,   94,    0,   94,    0,   /* 20 */
           0,    0,    0,    0,    0,    0,    0,    0,   /* 28 */
         199,  196,  194,  195,    0,    0,    0,    0,   /* 30 */
           0,    0,    0,    0,    0,    0,    0,    0,   /* 38 */
         188,  186,  178,  177,  180,  185,  179,  176,   /* 40 */
         191,  187,  189,  190,  183,  182,  184,  181,   /* 48 */
           0,    0,    0,    0,    0,    0,    0,    0,   /* 50 */
           0,    0,    0,    0,    0,    0,    0,    0,   /* 58 */
         343,  345,  344,  304,  318,  320,  319,  305,   /* 60 */
         340,  342,  341,  303,    0,    0,  301,  302,   /* 68 */
           0,DCG13,DCG14,DCG15,  315,  317,  316,  300,   /* 70 */
           0,    0,    0,    0,    0,    0,  301,  302,   /* 78 */
          68,   66,   55,   54,   59,   65,   56,   53,   /* 80 */
          71,   67,   69,   70,   62,   61,   63,   60,   /* 88 */
         146,  144,  136,  135,  138,  143,  137,  134,   /* 90 */
         149,  145,  147,  148,  141,  140,  142,  139,   /* 98 */
         114,  109,  193,   13,  152,  152,    0,    0,   /* A0 */
         114,  109,  197,   16,  154,  154,    0,   40,   /* A8 */
          29,   29,   92,   15,   78,   80,   99,   99,   /* B0 */
           0,    0,DCG11,   14,   10,   11,   98,   98,   /* B8 */
         172,  172,    0,    0,    0,    0,    0,DCG12,   /* C0 */
          12,   12,   12,   12,   12,   12,   12,   12,   /* C8 */
           0,  332,  330,  331,    0,  323,    0,    0,   /* D0 */
         337,  338,    0,  313,  310,  311,    0,  314,   /* D8 */
           0,  329,  328,    0,    0,  322,    0,    0,   /* E0 */
         335,  336,    0,  324,  308,  309,    0,  346,   /* E8 */
           0,  327,  325,  326,    0,  321,    0,    0,   /* F0 */
         333,  339,  334,    0,  306,  312,  307,    0    /* F8 */
          };

      static const UINT2 OPMNES[DCG15][8] = {
           6,  104,    5,  129,    7,  166,  175,   25,   /*  1 */
         109,    0,    0,    0,    0,    0,    0,    0,   /*  2 */
         125,  126,  119,  120,  151,  153,    0,  128,   /*  3 */
          94,    0,    0,    0,    0,    0,    0,    0,   /*  4 */
         167,    0,  103,  101,  100,   40,   35,   39,   /*  5 */
         167,    0,  103,  101,  100,   40,   35,   39,   /*  6 */
          42,   34,    0,    0,    0,    0,    0,    0,   /*  7 */
          42,   34,   17,   17,   64,   64,  114,    0,   /*  8 */
         157,  165,   82,   93,  168,  169,    0,    0,   /*  9 */
         355,  357,  350,  352,  158,    0,   83,   50,   /* 10 */
           0,    0,    0,    0,   13,   16,   15,   14,   /* 11 */
           0,  192,    0,    0,    0,    0,    0,    0,   /* 12 */
           0,    0,  332,    0,  329,    0,  327,    0,   /* 13 */
           0,    0,  330,    0,  328,    0,  325,    0,   /* 14 */
           0,    0,  331,    0,    0,    0,  326,    0    /* 15 */
      };


      static const UINT1 OPMNEF[64] = {
           3,   34,   21,   26,    3,   34,   21,   26,   /* 00 */
          44,    0,   27,    0,   44,    0,   27,    0,   /* 08 */
           9,   59,   22,   30,    9,   59,   22,   30,   /* 10 */
          10,   62,   23,   31,   10,   62,   23,   31,   /* 18 */
          64,   37,   32,    0,   64,   52,   32,    5,   /* 20 */
          66,   36,   33,   34,   66,    0,   33,   26,   /* 28 */
          15,   61,   24,    0,   15,   53,   24,    6,   /* 30 */
          17,   60,   25,   62,   17,   63,   25,   31    /* 38 */
      };

      static const UINT1 OPMNEFR[64] = {
           3,   34,   77,   80,    3,   20,    4,    0,   /* 00 */
          44,   73,   79,   82,   44,    0,   45,    0,   /* 08 */
           9,DFG01,   78,   81,    0,   59,    0,    0,   /* 10 */
          10,    0,   84,   83,    0,   62,DFG02,    0,   /* 18 */
          64,DFG03,    0,DFG04,   66,   69,   67,DFG05,   /* 20 */
          66,DFG06,DFG07,   87,   64,   70,   65,   88,   /* 28 */
          15,DFG08,    0,   85,   17,    0,   18,   86,   /* 30 */
          17,DFG09,    0,    0,   15,    0,   16,    0    /* 38 */
      };

      static const UINT1 OPMNEFS[DFG09][8] = {
          46,    0,    0,    0,    0,    0,    0,    0,   /*  1 */
           0,   11,    0,    0,    0,    0,    0,    0,   /*  2 */
           7,    2,    0,    0,   68,   72,    0,    0,   /*  3 */
          19,   14,    8,   29,   55,    0,    0,    0,   /*  4 */
          63,    0,    0,    0,    0,    0,    0,    0,   /*  5 */
          35,   39,   38,   42,   40,   41,   43,    0,   /*  6 */
           0,   71,    0,    0,    0,    0,    0,    0,   /*  7 */
           1,   75,   50,   47,   74,   49,   13,   28,   /*  8 */
          48,   76,   58,   57,   51,   54,   56,   12    /*  9 */
      };


	  // Default seg reg for a given base reg
      static const UINT1 ISRTV[2][8] = {	//UPDATE X64!!
		 //AX CX DX BX SP BP SI DI
            3, 3, 2, 2, 3, 3, 2, 3,		/*USE16*/
            3, 3, 3, 3, 2, 2, 3, 3,		/*USE32*/
      };

	  // Map byte size to 1-based index into ptr type-name (MNEPT[]) 
	  // or reg name (MNERG[]) array.
      static const UINT1 OPSZV[17] = {
            0,		/* 0 is invalid */
			1,		/* 1 -> BYTE */
			2,		/* 2 -> WORD */
			2,		/* 3 -> WORD */		// ??? should be invalid ???
			3,		/* 4 -> DWORD */
			0,		/* 5 is invalid */
			4,		/* 6 -> PWORD */
			0,		/* 7 is invalid */
			5,		/* 8 -> QWORD */
			0,		/* 9 is invalid */
			6,		/* 10 -> TBYTE */
			0,		/* 11 is invalid */
			0,		/* 12 is invalid */
			0,		/* 13 is invalid */
			0,		/* 14 is invalid */
			0,		/* 15 is invalid */
			7,		/* 16 -> OWORD */
	  };


#define MNEBASF 200  /* index of first floating-point mnemonic (FUNK) */

      static const char MNEOP[360][9] = {
        "UNK      ","AAA      ","AAD      ","AAM      ","AAS      ", /*   0 */
        "ADC      ","ADD      ","AND      ","ARPL     ","BOUND    ", /*   5 */
        "BSF      ","BSR      ","BSWAP    ","BT       ","BTC      ", /*  10 */
        "BTR      ","BTS      ","CALL     ","CBW      ","CWDE     ", /*  15 */
        "CLC      ","CLD      ","CLI      ","CLTS     ","CMC      ", /*  20 */
        "CMP      ","CMPSB    ","CMPSW    ","CMPSD    ","CMPXCHG  ", /*  25 */
        "CWD      ","CDQ      ","DAA      ","DAS      ","DEC      ", /*  30 */
        "DIV      ","ENTER    ","ESC      ","HLT      ","IDIV     ", /*  35 */
        "IMUL     ","IN       ","INC      ","INSB     ","INSW     ", /*  40 */
        "INSD     ","INT      ","INT3     ","INTO     ","INVD     ", /*  45 */
        "INVLPG   ","IRET     ","IRETD    ","JA       ","JAE      ", /*  50 */
        "JB       ","JBE      ","JCXZ     ","JECXZ    ","JE       ", /*  55 */
        "JG       ","JGE      ","JL       ","JLE      ","JMP      ", /*  60 */
        "JNE      ","JNO      ","JNS      ","JO       ","JPE      ", /*  65 */
        "JPO      ","JS       ","LAHF     ","LAR      ","LDS      ", /*  70 */
        "LEA      ","LEAVE    ","LES      ","LFS      ","LGDT     ", /*  75 */
        "LGS      ","LIDT     ","LLDT     ","LMSW     ","LOCK     ", /*  80 */
        "LODSB    ","LODSW    ","LODSD    ","LOOP     ","LOOPE    ", /*  85 */
        "LOOPNE   ","LSL      ","LSS      ","LTR      ","MOV      ", /*  90 */
        "MOVSB    ","MOVSW    ","MOVSD    ","MOVSX    ","MOVZX    ", /*  95 */
        "MUL      ","NEG      ","NOP      ","NOT      ","OR       ", /* 100 */
        "OUT      ","OUTSB    ","OUTSW    ","OUTSD    ","POP      ", /* 105 */
        "POPA     ","POPAD    ","POPF     ","POPFD    ","PUSH     ", /* 110 */
        "PUSHA    ","PUSHAD   ","PUSHF    ","PUSHFD   ","RCL      ", /* 115 */
        "RCR      ","REP      ","REPNE    ","REPE     ","RET      ", /* 120 */
        "ROL      ","ROR      ","SAHF     ","SAR      ","SBB      ", /* 125 */
        "SCASB    ","SCASW    ","SCASD    ","SEG      ","SETA     ", /* 130 */
        "SETAE    ","SETB     ","SETBE    ","SETE     ","SETG     ", /* 135 */
        "SETGE    ","SETL     ","SETLE    ","SETNE    ","SETNO    ", /* 140 */
        "SETNS    ","SETO     ","SETPE    ","SETPO    ","SETS     ", /* 145 */
        "SGDT     ","SHL      ","SHLD     ","SHR      ","SHRD     ", /* 150 */
        "SIDT     ","SIZ      ","SLDT     ","SMSW     ","STC      ", /* 155 */
        "STD      ","STI      ","STOSB    ","STOSW    ","STOSD    ", /* 160 */
        "STR      ","SUB      ","TEST     ","VERR     ","VERW     ", /* 165 */
        "WAIT     ","WBINVD   ","XADD     ","XCHG     ","XLATB    ", /* 170 */
        "XOR      ","CMOVA    ","CMOVAE   ","CMOVB    ","CMOVBE   ", /* 175 */
        "CMOVE    ","CMOVG    ","CMOVGE   ","CMOVL    ","CMOVLE   ", /* 180 */
        "CMOVNE   ","CMOVNO   ","CMOVNS   ","CMOVO    ","CMOVPE   ", /* 185 */
        "CMOVPO   ","CMOVS    ","CMPXCHG8B","CPUID    ","RDMSR    ", /* 190 */
        "RDPMC    ","RDTSC    ","RSM      ","UD2      ","WRMSR    ", /* 195 */

        "FUNK     ","F2XM1    ","FABS     ","FADD     ","FADDP    ", /* 200 */
        "FBLD     ","FBSTP    ","FCHS     ","FCLEX    ","FCOM     ", /* 205 */
        "FCOMP    ","FCOMPP   ","FCOS     ","FDECSTP  ","FDISI    ", /* 210 */
        "FDIV     ","FDIVP    ","FDIVR    ","FDIVRP   ","FENI     ", /* 215 */
        "FFREE    ","FIADD    ","FICOM    ","FICOMP   ","FIDIV    ", /* 220 */
        "FIDIVR   ","FILD     ","FIMUL    ","FINCSTP  ","FINIT    ", /* 225 */
        "FIST     ","FISTP    ","FISUB    ","FISUBR   ","FLD      ", /* 230 */
        "FLD1     ","FLDCW    ","FLDENV   ","FLDL2E   ","FLDL2T   ", /* 235 */
        "FLDLG2   ","FLDLN2   ","FLDPI    ","FLDZ     ","FMUL     ", /* 240 */
        "FMULP    ","FNOP     ","FPATAN   ","FPREM    ","FPREM1   ", /* 245 */
        "FPTAN    ","FRNDINT  ","FRSTOR   ","FSAVE    ","FSCALE   ", /* 250 */
        "FSETPM   ","FSIN     ","FSINCOS  ","FSQRT    ","FST      ", /* 255 */
        "FSTCW    ","FSTENV   ","FSTP     ","FSTSW    ","FSUB     ", /* 260 */
        "FSUBP    ","FSUBR    ","FSUBRP   ","FTST     ","FUCOM    ", /* 265 */
        "FUCOMP   ","FUCOMPP  ","FXAM     ","FXCH     ","FXTRACT  ", /* 270 */
        "FYL2X    ","FYL2XP1  ","FCMOVB   ","FCMOVBE  ","FCMOVE   ", /* 275 */
        "FCMOVNB  ","FCMOVNBE ","FCMOVNE  ","FCMOVNU  ","FCMOVU   ", /* 280 */
        "FCOMI    ","FCOMIP   ","FUCOMI   ","FUCOMIP  ","         ", /* 285 */
        "         ","         ","         ","         ","         ", /* 290 */
        "         ","         ","         ","         ","MOVSXD   ", /* 295 */
#define MNE_MOVSXD 299	/* Kludge for X64 MOVSXD instruction */
        "EMMS     ","MOVD     ","MOVQ     ","PACKSSDW ","PACKSSWB ", /* 300 */
        "PACKUSWB ","PADDB    ","PADDD    ","PADDSB   ","PADDSW   ", /* 305 */
        "PADDUSB  ","PADDUSW  ","PADDW    ","PAND     ","PANDN    ", /* 310 */
        "PCMPEQB  ","PCMPEQD  ","PCMPEQW  ","PCMPGTB  ","PCMPGTD  ", /* 315 */
        "PCMPGTW  ","PMADDWD  ","PMULHW   ","PMULLW   ","POR      ", /* 320 */
        "PSLLD    ","PSLLQ    ","PSLLW    ","PSRAD    ","PSRAW    ", /* 325 */
        "PSRLD    ","PSRLQ    ","PSRLW    ","PSUBB    ","PSUBD    ", /* 330 */
        "PSUBSB   ","PSUBSW   ","PSUBUSB  ","PSUBUSW  ","PSUBW    ", /* 335 */
        "PUNPCKHBW","PUNPCKHDQ","PUNPCKHWD","PUNPCKLBW","PUNPCKLDQ", /* 340 */
        "PUNPCKLWD","PXOR     ","         ","         ","         ", /* 345 */

        "LGDTW    ","LGDTD    ","LIDTW    ","LIDTD    ","         ", /* 350 */
        "SGDTW    ","SGDTD    ","SIDTW    ","SIDTD    ","         "  /* 355 */
      };

	  /* Register mnemonics indexed by [regsize][reg#][char#] */
	  /* where regsize is actually a zero-based index into    */
	  /* the MNEPT pointer-type name array.                   */
      static const char MNERG[8][16][5] = {
		   /* 0: BYTE regs */
           "AL   ","CL   ","DL   ","BL   ","AH   ","CH   ","DH   ","BH   ",
		   "R8B  ","R9B  ","R10B ","R11B ","R12B ","R13B ","R14B ","R15B ",
		   /* 1: WORD regs */
           "AX   ","CX   ","DX   ","BX   ","SP   ","BP   ","SI   ","DI   ",
		   "R8W  ","R9W  ","R10W ","R11W ","R12W ","R13W ","R14W ","R15W ",
		   /* 2 : DWORD regs */
           "EAX  ","ECX  ","EDX  ","EBX  ","ESP  ","EBP  ","ESI  ","EDI  ",
		   "R8D  ","R9D  ","R10D ","R11D ","R12D ","R13D ","R14D ","R15D ",
		   /* 3: PWORD regs (used for MMX regs) */
           "MMX0 ","MMX1 ","MMX2 ","MMX3 ","MMX4 ","MMX5 ","MMX6 ","MMX7 ",
           "?60  ","?61  ","?62  ","?63  ","?64  ","?65  ","?66  ","?67  ",
		   /* 4: QWORD regs */
           "RAX  ","RCX  ","RDX  ","RBX  ","RSP  ","RBP  ","RSI  ","RDI  ",
		   "R8   ","R9   ","R10  ","R11  ","R12  ","R13  ","R14  ","R15  ",
		   /* 5: TBYTE regs */
		   "FPR0 ","FPR1 ","FPR2 ","FPR3 ","FPR4 ","FPR5 ","FPR6 ","FPR7 ",
           "?A0  ","?A1  ","?A2  ","?A3  ","?A4  ","?A5  ","?A6  ","?A7  ",
		   /* 6: OWORD regs */
		   "XMM0 ","XMM1 ","XMM2 ","XMM3 ","XMM4 ","XMM5 ","XMM6 ","XMM7 ",
		   "XMM8 ","XMM9 ","XMM10","XMM11","XMM12","XMM13","XMM14","XMM15",
		   /* 7: X64 BYTE regs */
           "AL   ","CL   ","DL   ","BL   ","SPL  ","BPL  ","SIL  ","DIL  ",
		   "R8B  ","R9B  ","R10B ","R11B ","R12B ","R13B ","R14B ","R15B ",
      };

	  /* Segment Register/Override Mnemonics */
      static const char MNESG[8][3] = {
          "ES:","CS:","SS:","DS:","FS:","GS:","?S:","?S:"
      };

	  /* Special Register Mnemonics */
      static const char MNESPRG[3][16][4] = {
		   /* 0: Control Registers */
           "CR0 ","CR1 ","CR2 ","CR3 ","CR4 ","CR5 ","CR6 ","CR7 ",
           "CR8 ","CR9 ","CR10","CR11","CR12","CR13","CR14","CR15",
		   /* 1: Debug Registers */
           "DR0 ","DR1 ","DR2 ","DR3 ","DR4 ","DR5 ","DR6 ","DR7 ",
           "DR8 ","DR9 ","DR10","DR11","DR12","DR13","DR14","DR15",
		   /* 2: Test Registers */
           "TR0 ","TR1 ","TR2 ","TR3 ","TR4 ","TR5 ","TR6 ","TR7 ",
           "TR8 ","TR9 ","TR10","TR11","TR12","TR13","TR14","TR15",
      };

	  /* 16-bit Addressing ModRM Mnemonics */
      static const char MNEXR[8][5] = {
           "BX+SI","BX+DI","BP+SI","BP+DI","SI   ","DI   ","BP   ","BX   "
      };

	  /* SIB Scale Factor Mnemonics */
      static const char MNESSS[4][2] = {
          "  ","*2","*4","*8"
      };

	  /* X87 Stack Register Mnemonics */
      static const char MNEST[8][5] = {
           "ST(0)","ST(1)","ST(2)","ST(3)","ST(4)","ST(5)","ST(6)","ST(7)"
      };

	  /* Pointer Type Mnemonics */
      static const char MNEPT[7][5] = {
           "BYTE ","WORD ","DWORD","PWORD","QWORD","TBYTE","OWORD"
      };

#ifdef X64
	  /* X64 REX prefix byte definitions */
#define REX		0x40	// the bare REX prefix byte
#define REX_W	0x08	// REX.W flag => operand size = 64 bits
#define REX_R	0x04	// REX.R flag => use extended GPR or XMM in reg field
#define REX_X	0x02	// REX.X flag => use extended GPR or XMM in index reg field
#define REX_B	0x01	// REX.B flag => use extended GPR or XMM in base reg field
#endif //X64
