/***************************  MDY.H  *******************************/
/*                                                                 */
/*  Copyright 1997-2014 AFH Systems                                */
/*                                                                 */
/*  FilterMeister is free software. You can redistribute and       */
/*  modify it under the terms of the MPL2 License.                 */
/*  See: https://www.mozilla.org/en-US/MPL/2.0/                    */
/*  Any changes to the FilterMeister files have to be published or */
/*  sent to harry@thepluginsite.com. The filter plugin code itself */
/*  does not need to be disclosed.                                 */
/*                                                                 */
/*    10DEC90  K. Krauter   1. upgrade to md386.                   */
/*    23Oct96  A. Hunter    1. Translated to C using PLMC32.       */
/*    29Oct96  Alex Hunter  1. Fix bug: VAROBJECT had grown larger */
/*                             than GENERALOBJECT, so Estack       */
/*                             entries were overlapping.           */
/*    05JAN99  Alex Hunter  1. Add DECL objects.                   */
/*	  15JUL13  Alex Hunter	1. Modify for WIN64.				   */
/*	  09MAY14  Alex Hunter	1. Give symbolic names to the 'magic'  */
/*							   typeindex values.				   */
/*							2. Add definition for ANYNARGS.		   */
/*	  12MAY14  Alex Hunter	1. Add nbytes field to VAROBJECT.	   */
/*	  10Jun14  Alex Hunter  1. Delete indexcount and derivedp      */
/*							   fields from VAROBJECT.			   */
/*							2. Add pSym and pType fields to		   */
/*							   VAROBECT.						   */
/*	  12Jun14  Alex Hunter	1. Add namep and flags field to		   */
/*							   VAROBJECT.						   */
/*	  24Jun14  Alex Hunter	1. Fix faulty definition of ANYNARGS.  */
/*                                                                 */
/*******************************************************************/

/* Structure of evaluation stack objects */

#define VAROBJECTKIND     0
#define IDOBJECTKIND      1
#define LINOBJECTKIND     2
#define LITOBJECTKIND     3
#define REGOBJECTKIND     4
#define SEGOBJECTKIND     5
#define GRPOBJECTKIND     6
#define FILOBJECTKIND     7
#define MARKEROBJECTKIND  8
#define DECLOBJECTKIND    9
#define TYPOBJECTKIND    10

 typedef struct
         {
             UINT1          kind;
             UINT4          typeindex;
             UINT4          offset;
             UINT2          frame;
             const char     *namep;
             pfm_OBJ        pSym; //pointer to symbol in user symboltable
             pfm_OBJ        pType; //pointer to type object
             UINT4          flags;	//flags from pSym
             INT4           extradata;	//nargs
			 INT4			nbytes;	//negated indicates unsigned type
         } VAROBJECT;

 /* The 'magic' typeindex values for FM are: */
#define VARTYPE_INT			 0
#define VARTYPE_REAL		 126
#define VARTYPE_INT_FUNC	 1234
#define VARTYPE_REAL_FUNC	 1237
#define VARTYPE_RGN_FUNC	 1238
#define VARTYPE_CHAR_ARRAY	 4321
#define VARTYPE_INT_PTR_FUNC 2468

 /* 'Magic' values for extradata in FM: */
#define ANYNARGS (-12345)	// magic number meaning any number of args is okay 

 typedef struct
         {
             UINT1          kind;
             UINT4          *lnamep;
             void           *qualp;
#ifdef FML
             //room to create a string descriptor...
             UINT4          nbytes;
             char           *charp;
#endif
         } IDOBJECT;

 typedef struct
         {
             UINT1          kind;
             UINT4          linenumber;
             void           *qualp;
         } LINOBJECT;

 typedef struct
         {
             UINT1          kind;
             UINT1          type;
             UINT2          nbytes;
             UINT1          *valuep;
#ifdef FML
             INT4           ivalue;
             double         rvalue;
#endif
         } LITOBJECT;

 typedef struct
         {
             UINT1          kind;
             UINT4          typeindex;
             UINT1          regindex;
         } REGOBJECT;

 typedef struct
         {
             UINT1          kind;
             LNAMESTRUCTURE *lnamep;
             void           *qualp;
         } SEGOBJECT;

 typedef struct
         {
             UINT1          kind;
             LNAMESTRUCTURE *lnamep;
             void           *qualp;
         } GRPOBJECT;

 typedef struct
         {
             UINT1          kind;
             UINT4          len;
             void           *ptr;
         } FILOBJECT;

 typedef struct
         {
             UINT1          kind;
         } MARKEROBJECT;

 typedef struct
         {
             UINT1          kind;
             UINT1          type;
             UINT4          dimension; //for arrays
         } DECLOBJECT;

 typedef struct
         {
             UINT1          kind;
             pfm_OBJ        pType; //pointer to type object
         } TYPOBJECT;

#define LITTYPEINTEGER    1
#define LITTYPELOGICAL    2
#define LITTYPEREAL       3
#define LITTYPECHAR       4
#define LITTYPEREGION     5

#define REGTYPEBYTE       1
#define REGTYPEWORD       2
#define REGTYPEDWORD      4

 typedef union
         {
             UINT1          kind;
             VAROBJECT      var;
             IDOBJECT       id;
             LINOBJECT      lin;
             LITOBJECT      lit;
             REGOBJECT      reg;
             SEGOBJECT      seg;
             GRPOBJECT      grp;
             FILOBJECT      fil;
             MARKEROBJECT   marker;
             DECLOBJECT     decl;
         } GENERALOBJECT;

 extern GENERALOBJECT  Estack[MAX_TEMPS];

 typedef struct
         {
             void           *link;
             LNAMESTRUCTURE *lnamep;
         } QUALIFIERSTRUCTURE;

/******************** end of MDY.H ********************************/
