#if defined(VAXC) || defined(__DECC)
/* Following pragma packs data structure members on byte  */
/* boundaries while aligning the base of the structure on */
/* a longword (as required, e.g., for SLCNET buffers).    */
#pragma nomember_alignment LONGWORD
/* Following pragma kills some spurious error messages    */
/* caused by the preceding pragma.                        */
#pragma message disable (NONMULTALIGN)
#endif

#if 1
//emit the following ops for FM...
#define EMIT_POSTINC 1
#define EMIT_POSTDEC 1
#define EMIT_PREINC 1
#define EMIT_PREDEC 1
#endif

/***************************** AfhFMparser.c *****************************/
/*                                                                       */
/*  Copyright 1997-2014 AFH Systems                                      */
/*                                                                       */
/*  FilterMeister is free software. You can redistribute and modify it   */
/*  under the terms of the MPL2 License.                                 */
/*  See: https://www.mozilla.org/en-US/MPL/2.0/                          */
/*  Any changes to the FilterMeister files have to be published or sent  */
/*  to harry@thepluginsite.com. The filter plugin code itself does not   */
/*  need to be disclosed.                                                */
/*                                                                       */
/*************************************************************************/
/*                                                                       */
/*              R E V I S I O N   H I S T O R Y                          */
/*                                                                       */
/*  15Dec97  Alex Hunter  1. Derived from mdp.c.                         */
/*  16Dec97  Alex Hunter  1. Treat C- and C++-style comments as white    */
/*                           space.                                      */
/*                        2. Add a "quit" command for stand-alone mode.  */
/*                        3. Delete old MD386 commands.                  */
/*                        4. Disallow digit as first char of identifier. */
/*  02Jan98  Alex Hunter  1. Allow f().a where f() is a function call    */
/*                           returning a struct or union.                */
/*                        2. Parse <statement> at top level.             */
/*  06Jan98  Alex Hunter  1. First stab at handling preprocessor         */
/*                           directives.                                 */
/*  08Jan98  Alex Hunter  1. Start bringing syntax into agreement with   */
/*                           draft 1 of the FML specification.           */
/*  10Jan98  Alex Hunter  1. Add OP_BREAKF to replace OP_NOT/OP_THEN/    */
/*                           OP_BREAK/OP_THEN sequence.                  */
/*  12Jan98  Alex Hunter  1. Move SDL definitions to mdp.h.              */
/*  25Aug07  Alex Hunter  1. Initial implementation of asm_statement.    */
/*  01Sep12	 Alex Hunter  1. Add minimal X64 support.					 */
/*  22Oct12  Alex Hunter  1. Add OP_ARGLIST operator for implementation  */
/*                           of argument lists pushed left-to-right.     */
/*	26Apr13	 Alex Hunter  1. Update control-style documentation comments.*/
/*  10Jul13  Alex Hunter  1. Fix some /Wp64 warnings.					 */
/*  15JUL13	 Alex Hunter  1. More changes for WIN64.					 */
/*	02AUG13	 Alex Hunter  1. Even more WIN64 changes (string descriptors,*/
/*							 etc.)										 */
/*	26AUG13	 Alex Hunter  1. Added N_ARRAYS.							 */
/*  03OCT13  Alex Hunter  1. Add DCL_HALF, DCL_INT_PTR, DCL_UINT_PTR.	 */
/*						  2. Add OP_HALF, OP_INT_PTR, OP_UINT_PTR.		 */
/*						  3. Change BOOL type to 1 byte.				 */
/*	21OCT13	 Alex Hunter  1. Change 'treal' to 'tempreal'.				 */
/*  05NOV13  Alex Hunter  1. Fixed typo in a comment.					 */
/*						  2. Fixed a WIN64 bug in error_token.			 */
/*  15NOV13  Alex Hunter  1. Parse 64-bit signed/unsigned integer		 */
/*							 constants and 64-bit character constants.	 */
/*						  2. Slight speedup to mdp_hex_digits & mdp_hex. */
/*                        3. Fix named constant N_ARRAYS.				 */
/*						  4. Add LIT_UINT32, LIT_UINT64.				 */
/*						  5. Add <binary_constant>.						 */
/*  19NOV13  Alex Hunter  1. Add warnings about constant overflow, etc.	 */
/*  20NOV13  Alex Hunter  1. Implement "highwater" mark for warnings to  */
/*                           eliminate redundant warning messages (see   */
/*                           solution (1) in note below about "Issuing   */
/*                           Diagnostic Messages".                       */
/*						  2. Add new extended BNF forms (<xxx>(n-m) etc.)*/
/*						  3. Add Unicode string syntax (NYI).			 */
/*	21NOV13  Alex Hunter  1. Add mdp_warnv, mdp_warn_constant_overflow.  */
/*  22NOV13  Alex Hunter  1. Add more warnings (wide chars, etc.)        */
/*  23NOV13  Alex Hunter  1. Increase max string size to 65535+NUL (same */
/*							 as the MSVC limit; ANSI C requires 509).	 */
/*						  2. Warn if C-style comment is terminated by    */
/*							 EOS (end-of-file).							 */
/*  24NOV13  Alex Hunter  1. A few more warnings (char constant overflow */
/*							 etc.).										 */
/*	29JAN14  Alex Hunter  1. Set filterTitleText and filterCategoryText  */
/*							 at parse time as well as filter-init time.  */
/*	30JAN14	 Alex Hunter  1. Also set filterFilenameText at parse time.  */
/*	19FEB14	 Alex Hunter  1. Set filter*Text variables only the first    */
/*							 time parsed, since they may also be parsed  */
/*							 again later if they are set dynamically.    */
/*							(This entire kludge is just to support cross */
/*							 compilations, in which case the init code   */
/*							 cannot be executed on the host platform.)   */
/*	03MAR14  Alex Hunter  1. Parse optional revision number after prog   */
/*							 type, and pass it along as new SD_PROG item.*/
/*						  2. Implement all integral types as 'int' and	 */
/*							 all real types as 'double' when rev < 2.0.	 */
/*						  3. Rearrange scan for type keywords in rough   */
/*							 order of occurrence (a tiny speedup).		 */
/*	04MAR14  Alex Hunter  1. Don't assert() in Mdp_dumpSDL when bc==0.	 */
/*	07MAR14	 Alex Hunter  1. Fix bug which gave incorrect value for      */
/*							 sizeof(INT_PTR) and sizeof(UNIT_PTR) during */
/*							 first compilation.							 */
/*	17MAR14	 Alex Hunter  1. Allow 'HORIZ*ONTAL' in addition to 'HORZ'.	 */
/*						  2. Allow 'VERT*ICAL' in addition to 'VERT'.	 */
/*	23MAR14	 Alex Hunter  1. Don't in-line cnvX and cnvY for X64GEN,	 */
/*							 until fixed later.							 */
/*	01APR14	 Alex Hunter  1. Change fixup value of bad escape sequence	 */
/*							 "\x" from "x" to "\\x" (which is a more	 */
/*							 useful default).							 */
/*	16MAY14	 Alex Hunter  1. isprint(char c) was asserting if c was < -1.*/
/*	24MAY14	 Alex Hunter  1. Temp workaround in push_number and in		 */
/*							 push_value to keep the compiler happy until */
/*							 push_param64 is implemented.				 */
/*						  2. DON'T FORGET to implement push_param64 when */
/*							 64-bit integer constants are needed!!		 */
/*  06JUN14	 Alex Hunter  1. Add OP_TEMPREAL and OP_BCD.				 */
/*						  2. Add OP_VOID.								 */
/*	28JUN14  Alex Hunter  1. Add missing error codes from errno.h.		 */
/*                                                                       */
/* NEEDS WORK:                                                           */
/*                                                                       */
/*  - should warn user if /*-comment is terminated by EOS. -DONE		 */
/*  - check Mdp_dumpSDL for correct printing of LIT_CHAR and SD_IDLIT.	 */
/*	- implement push_param64 for 64-bit values.							 */
/*                                                                       */
/*************************************************************************/

///////////////////////////////////////////////////////////////////////////
//
//  Extended BNF conventions:
//
// 1. "Xyz" -- characters in double quotes (") must match exactly 
//             (case-sensitive literal string)
// 2. 'Ctl' -- characters in single quotes (') are case-insensitive
//             (e.g., 'Ctl' will match ctl, Ctl, CTL, etc.)
// 3. <xxx>(n-m) -- where n and m are decimal constants, means at least
//				n but no more than m sequential occurrences
// 4. <xxx>(n) -- means exactly n occurrences ( <xxx>(n-n) )
// 5. <xxx>(n+) -- means n or more occurrences ( <xxx>(n-infinity) ) 
// 6. [<xxx>] -- means <xxx> is optional ( <xxx>(0-1) )
// 7. <xxx>* -- means 0 or more occurrences of <xxx> ( <xxx>(0+) )
// 8. <xxx>+ -- means 1 or more occurrences of <xxx> ( <xxx>(1+) )
// 9. <a> <b> <c> -- means <a>, <b>, <c> must occur in sequence
// 10. <a>|<b> -- means <a> or <b> (but not both)
// 11. [<a>|<b>] -- means <a> or <b> or neither
// 12. The only significance of {}'s is for grouping:
//              e.g., {<a><b>}|{<c><d>} means {<a> followed by <b>}
//              or {<c> followed by <d>}
// 13. A BNF syntax char (such as < or |) should be enclosed in single
//              or double quotes ("<" or "|") if it is meant as a literal
//              character to be matched from the input string.  Even if a
//              punctuation char is NOT a special BNF syntax char, it is
//              advisable to enclose it in quotes to clearly signify its
//              use as a character to be matched from the input string (e.g.,
//              it is good practice to quote a comma as ',' or ",")
////////////////////////////////////////////////////////////////////////////

//////// A Note about issuing Diagnostic Messages if the Parser Backs Up //////////
//
// CAVEAT: The parser may back up and reparse, and so may issue bogus or
// multiple diagnostics.  In general, we try to output warnings only where
// the parse MUST succeed the first time through! However, this is not always
// possible. <assignment_expression> bifurcates to parse either <assignment_stmt>
// or <conditional_expression>, and both can parse a <hexadecial_constant>.
// If the first parse (of <assignment_stmt>) then does not find a following
// <assignment_operator>, it parses for a <conditional_expression>, which 
// again may parse a <hexadecimal_constant>.  Ergo, if the constant is bad,
// two redundant warnings get issued.
// So either:
// (1) Find a way to suppress (or back out of) the redundant warnings.
//   One hack to do this is to keep a highwater mark of inp, where the
//   last warning was issued.  Then suppress any subsequent warnings
//   issued when inp <= inp_high_water_warn.  This probably doesn't
//   handle the general case, but may be adequate to suppress simple
//   annoying repetitions. Or,
// (2) find another syntax definition which does not bifurcate in this
//   fashion, or
// (3) pipeline the warning messages, with backup.  We could do this by
//   placing warnings in the SDL, where they will automatically get backed
//   out whenever the parser backs up. Then the SDL evaluator will issue
//   the remaining warnings at the appropriate time.
// Probably the most general and correct solution is (3), but the simple
// hack in (1) will likely suffice for now.  And solution (2) may be
// infeasible in many cases.
////////////////////////////////////////////////////////////////////////////////


/* Define STAND_ALONE_PARSER to build the stand-alone version */
/* of the parser (used to debug the parser routines).         */

/**** #define STAND_ALONE_PARSER 1  ****/

#if defined(STAND_ALONE_PARSER) || defined(MDP_TEST)

/* Include files for stand-alone version. */
#include "plmcvax.h"
#include <stdio.h>
#include <math.h>                       /* (named constants)                  */
#include <errno.h>						/* (named error codes)				  */
#include <stdlib.h>                     /* atof,                              */
#include <string.h>                     /* strchr, memcpy.                    */
#include <setjmp.h>                     /* setjmp, jmp_buf.                   */
#include <assert.h>                     /* assert.                            */
#include "md386def.hc"                  /* MD386__ message codes.             */
#include "mdp.h"                        /* MD386 parser definitions.          */
#include <windows.h>                    /* ALTERNATE, WINDING.                */
#include "AfhFM.h"                      /* msgBox                             */
#include "AfhFMcontext.h"				/* gParams, filterTitleText, etc.	  */
#include <time.h>                       /* CLOCKS_PER_SEC                     */
#include <locale.h>

#else

/* Include files for FM version. */
#include "plmcvax.h"
#include <stdlib.h>                     /* atof,                              */
#include <string.h>                     /* strchr, memcpy.                    */
#include <setjmp.h>                     /* setjmp, jmp_buf.                   */
#include <assert.h>                     /* assert.                            */
#include "md386def.hc"                  /* MD386__ message codes.             */
#include "md386lit.h"                   /* MD386 common definitions.          */
#include "mdg.h"                        /* MD386 global data definitions.     */
#include "mdd.h"                        /* MD386 dictionary data structures.  */
#include "mdp.h"                        /* MD386 parser definitions.          */
#include "mdy.h"                        /* QUALIFIERSTRUCTURE.                */
#include "md386proto.h"                 /* MD386 intermodule prototypes.      */
#include <windows.h>                    /* ALTERNATE, WINDING.                */

#endif

//From newer version of math.h
#ifndef M_E
	#define M_E        2.71828182845904523536
	#define M_LOG2E    1.44269504088896340736
	#define M_LOG10E   0.434294481903251827651
	#define M_LN2      0.693147180559945309417
	#define M_LN10     2.30258509299404568402
	#define M_PI       3.14159265358979323846
	#define M_PI_2     1.57079632679489661923
	#define M_PI_4     0.785398163397448309616
	#define M_1_PI     0.318309886183790671538
	#define M_2_PI     0.636619772367581343076
	#define M_2_SQRTPI 1.12837916709551257390
	#define M_SQRT2    1.41421356237309504880
	#define M_SQRT1_2  0.707106781186547524401	
#endif
#ifndef MAXFLOAT
	#define MAXFLOAT ((float)3.40282346638528860e+38)
	//#define MAXFLOAT  ((float)3.40282347e+38)
#endif


/*-----------------------------------------------------*/
/*                                                     */
/*                   MDP typedefs                      */
/*                                                     */
/*-----------------------------------------------------*/

typedef struct channel_node_tag {
    struct channel_node_tag *prev;
    int c;
} channel_node;

#ifdef STAND_ALONE_PARSER

#define PRINTF printf
#define PRINTF2 printf
#define PRINTF3 printf

#elif MDP_TEST && 0	//no longer needed here

void FMprint(const char *buff);

#define PRINTF(f) FMprint(f)

///BUFFER OVERRUNS POSSIBLE BELOW!!
#define PRINTF2(f,a) \
    do {\
        char buff[128];\
        sprintf(buff, f, a);\
        FMprint(buff);\
    } while(0)

#define PRINTF3(f,a,b) \
    do {\
        char buff[128];\
        sprintf(buff, f, a, b);\
        FMprint(buff);\
    } while(0)

#endif

#if defined(STAND_ALONE_PARSER)

void Mde_warn (UINT4 iss, UINT4 p1, UINT4 p2, UINT4 p3)
{
    PRINTF2 ("MDE_WARN: %8x\r\n", iss);
}

void Mde_abort_command (UINT4 iss, UINT4 p1, UINT4 p2, UINT4 p3)
{
    PRINTF2 ("MDE_ABORT_COMMAND: %8x\r\n", iss);
    exit(1);
}
#else

void Mde_abort_command (UINT4 iss, UINT4 p1, UINT4 p2, UINT4 p3);

#endif /* STAND_ALONE_PARSER */


/*
;  Symbolic names for flag bits.
*/

#define BIT0     0x00000001
#define BIT1     0x00000002
#define BIT2     0x00000004
#define BIT3     0x00000008
#define BIT4     0x00000010
#define BIT5     0x00000020
#define BIT6     0x00000040
#define BIT7     0x00000080
#define BIT8     0x00000100
#define BIT9     0x00000200
#define BIT10    0x00000400
#define BIT11    0x00000800
#define BIT12    0x00001000
#define BIT13    0x00002000
#define BIT14    0x00004000
#define BIT15    0x00008000
#define BIT16    0x00010000
#define BIT17    0x00020000
#define BIT18    0x00040000
#define BIT19    0x00080000
#define BIT20    0x00100000
#define BIT21    0x00200000
#define BIT22    0x00400000
#define BIT23    0x00800000
#define BIT24    0x01000000
#define BIT25    0x02000000
#define BIT26    0x04000000
#define BIT27    0x08000000
#define BIT28    0x10000000
#define BIT29    0x20000000
#define BIT30    0x40000000
#define BIT31    0x80000000
 
#define BIT(id)  (1 << (id))
#define FLAG(id) (1 << ((id) ^ 7))

/*
*  Local storage.
*/

/* PiPL 'fici' settings */
static FilterCaseInfo fici;
static int bitsetFiciCases;


/* Command flag bits. */
static UINT4 flags;

/* Command parameters. */
static UINT4 param1, param2, param3, param4,
             param5, param6, param7, param8;

/* Current operator. */
static UINT4 op;

/* Temporary storage for real*8 value. */
static double real_num;

/* Current string quote character. */
static char quote_char;

/* Current character string length. */
static UINT4 mdp_string_len;

/* Accumulator to hold a character string */
/* after embedded escape sequences are processed. */
#if 0 //for overflow testing
static char mdp_string_acc[16];	//must be a multiple of 4
#else
static char mdp_string_acc[65536];	//must be a multiple of 4 (this is the MSVC limit!)
#endif

/*
;  Common block for communication with MDX.
*/
static UINT4 More;             /* True (odd) if more to be parsed... */
static UINT4 *SDL;             /* Address of Semantic Descriptor List */
static UINT4 Nsdl;             /* Size of SDL array */
static UINT4 Ncount;           /* Number of words in the semantic descriptor list. */
static UINT4 mdp_flags;
#ifdef X64
static INT64  mdp_number;	/* integer constants can be 64-bits */
#else
static INT4  mdp_number;
#endif
static UINT4 mdp_number_unsigned;
static UINT4 mdp_number_bitlen;
static UINT4 mdp_number_overflow;
static double mdp_real_num;
static UINT4 mdp_tokencnt;
static char *mdp_tokenptr;
static struct
       {
        UINT4  len;
        char   *ptr;
       }     mdp_tokendsc;

/*************************************************************************/
/*                                                                       */
/*                            PARSER MACROS                              */
/*                                                                       */
/*************************************************************************/
#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

/* C-language expression parser routines are tri-state, */
/* returning 0 (failure) or one of the following values */
/* (note that all non-zero values appear to be TRUE to  */
/* C).                                                  */
#define RVALUE 1
#define LVALUE 2

#ifdef WIN32
/* inp is a WIN32 builtin??? */
#define inp inpxxx
#endif

#define STATE(state)  static int state(void)\
    { char *state_name = #state;

#define FSTATE(state) static int state(void);  /* forward prototype */

#define MARK \
    UINT4   ncount_save = Ncount;\
    UINT4   skip_whitespace_save = skip_whitespace;\
    char   *inp_save = inp;\
    mdp_skip_whitespace()

/* The do {...} while(0) artifice in the following macros is */
/* simply a dirty trick to make the macro body look like a   */
/* simple C statement that can always be followed by a ";".  */

#define ACCEPT do {skip_whitespace = skip_whitespace_save;\
    return TRUE;} while(0)

/* Return RVALUE or LVALUE from a C-language expression parser */
#define ACCEPTX(retval) do {skip_whitespace = skip_whitespace_save;\
    return (retval);} while(0)

#define REJECT do {Ncount = ncount_save;\
     skip_whitespace = skip_whitespace_save;\
     if (inp >= inp_high_water) {\
        inp_high_water = inp;\
        MDP_looking_for = state_name;\
     }\
     inp = inp_save;\
     return FALSE;} while(0)

/* Special version of REJECT for mdp_char*()... */
#define REJECT_CHAR do {Ncount = ncount_save;\
     skip_whitespace = skip_whitespace_save;\
     if (inp >= inp_high_water) {\
        inp_high_water = inp;\
        state_name[1] = c;\
        MDP_looking_for = state_name;\
     }\
     inp = inp_save;\
     return FALSE;} while(0)

/*************************************************************************/
/*                                                                       */
/*                        Forward Prototypes                             */
/*                                                                       */
/*************************************************************************/

static int mdp_keyword(char *kw);
static void mdp_skip_whitespace(void);
static int mdp_paragraph(void);

FSTATE(FM_program)
FSTATE(end_of_program)

// 3.1 Lexical Elements
FSTATE(error_token)
FSTATE(identifier)
FSTATE(constant)
FSTATE(floating_constant)
FSTATE(fractional_constant)
FSTATE(exponent_part)
FSTATE(digit_sequence)
FSTATE(floating_suffix)
FSTATE(integer_constant)
FSTATE(decimal_constant)
FSTATE(binary_constant)
FSTATE(octal_constant)
FSTATE(hexadecimal_constant)
FSTATE(integer_suffix)
FSTATE(unsigned_suffix)
FSTATE(long_suffix)
FSTATE(enumeration_constant)
FSTATE(boolean_constant)
FSTATE(named_constant)
FSTATE(named_floating_constant)
FSTATE(character_constant)
FSTATE(string_literal)
FSTATE(single_string_literal)
FSTATE(wide_string_literal)

// 3.1.5 Operators
FSTATE(indirect_component_selection_op)
FSTATE(increment_operator)
FSTATE(decrement_operator)
FSTATE(left_shift_operator)
FSTATE(right_shift_operator)
FSTATE(lt_operator)
FSTATE(gt_operator)
FSTATE(le_operator)
FSTATE(ge_operator)
FSTATE(eq_operator)
FSTATE(ne_operator)
FSTATE(logical_and_operator)
FSTATE(logical_or_operator)
FSTATE(and_operator)
FSTATE(or_operator)
FSTATE(compound_assignment_op)
FSTATE(assignment_operator)
FSTATE(concat_operator)
#ifndef NO_POWER_OPERATOR
FSTATE(power_operator)
#endif
FSTATE(any_operator)

// 3.1.6 Punctuators
FSTATE(ellipsis)
FSTATE(punctuator)

// 3.3 Expressions
FSTATE(C_language_expression)
FSTATE(primary_expression)
FSTATE(postfix_expression)
FSTATE(subscript_expression_list)
FSTATE(argument_expression_list)
FSTATE(unary_expression)
FSTATE(unary_operator)
FSTATE(type_name_in_parens)
FSTATE(restricted_type_name_in_parens)
FSTATE(sizeof_operand)
FSTATE(FFP_intrinsic_function)
FSTATE(FFP_inline_function)
FSTATE(argument_to_inline_function)
FSTATE(one_inline_argument)
FSTATE(two_inline_arguments)
FSTATE(three_inline_arguments)
FSTATE(seven_inline_arguments)

FSTATE(cast_expression)
FSTATE(power_expression)
FSTATE(multiplicative_expression)
FSTATE(additive_expression)
FSTATE(shift_expression)
FSTATE(relational_expression)

FSTATE(relational_operator)    // Dryrot, or TBD???

FSTATE(equality_expression)
FSTATE(equality_operator)    // Dryrot, or TBD???

FSTATE(AND_expression)
FSTATE(exclusive_OR_expression)
FSTATE(inclusive_OR_expression)
FSTATE(logical_AND_expression)
FSTATE(logical_OR_expression)
FSTATE(conditional_expression)

FSTATE(assignment_stmt)

FSTATE(assignment_expression)
FSTATE(expression)

// 3.4 Constant Expressions
FSTATE(constant_expression)
FSTATE(constant_integer_expression)

// 3.5 Declarations
FSTATE(declaration)
FSTATE(declaration_specifiers)
FSTATE(init_declarator_list)
FSTATE(init_declarator)
FSTATE(storage_class_specifier)
FSTATE(type_specifier)
FSTATE(struct_or_union_specifier)
FSTATE(struct_or_union)
FSTATE(struct_declaration_list)
FSTATE(struct_declaration)
FSTATE(specifier_qualifier_list)
FSTATE(struct_declarator_list)
FSTATE(struct_declarator)
FSTATE(enum_specifier)
FSTATE(enumerator_list)
FSTATE(enumerator)
FSTATE(type_qualifier)
FSTATE(declarator)
FSTATE(declarator1)
FSTATE(direct_declarator)
FSTATE(pointer)
FSTATE(type_qualifier_list)
FSTATE(array_dimension_list)
FSTATE(parameter_type_list)
#if 0
FSTATE(parameter_list)
#endif
FSTATE(parameter_declaration)
FSTATE(type_name)
FSTATE(abstract_declarator)
FSTATE(abstract_declarator1)
FSTATE(direct_abstract_declarator)
FSTATE(typedef_name)
FSTATE(initializer)
FSTATE(initializer_list)

// 3.6 Statements
FSTATE(statement)
FSTATE(labeled_statement)
FSTATE(compound_statement)
FSTATE(non_scoping_compound_statement)
#ifndef INTERMIX
FSTATE(declaration_list)
FSTATE(statement_list)
#endif
FSTATE(expression_statement)
FSTATE(selection_statement)
FSTATE(iteration_statement)
FSTATE(jump_statement)
FSTATE(asm_statement)
FSTATE(asm_instruction)
FSTATE(x86_instruction)
FSTATE(x86_opcode)
FSTATE(x86_operand)
FSTATE(x86_accumulator)
FSTATE(x86_reg32)
FSTATE(x86_reg16)
FSTATE(x86_reg8)
FSTATE(x86_segreg)
FSTATE(eol)
FSTATE(FFP_statement)
FSTATE(FFP_eval_specification)

// 3.7 Global Definitions
FSTATE(AFS_program)
FSTATE(FFW_program)
FSTATE(FFW_key_value_pair_but_not_FFP_key)
FSTATE(FFW_key_value_pair)
FSTATE(text_field)
FSTATE(text_paragraph)
FSTATE(bare_text_field)
FSTATE(bare_text_paragraph)
FSTATE(index)

// FFP grammar
FSTATE(FFP_program)
FSTATE(FFP_key_value_pair)
static int FFP_channel_list(channel_node *list);
FSTATE(organization_specification)
FSTATE(URL_specification)
FSTATE(version_specification)
#ifndef BETA
FSTATE(PIKIND_specification)
FSTATE(plugin_kind)
FSTATE(PIVersion_specification)
FSTATE(PIPriority_specification)
FSTATE(supported_modes_specification)
FSTATE(image_mode)
FSTATE(enable_info_specification)
FSTATE(required_host_specification)
FSTATE(filter_case_info_specification)
FSTATE(filter_case_spec)
FSTATE(filter_case_name)
FSTATE(filter_handling_item)
FSTATE(has_terminology_specification)
FSTATE(terminology_item)
FSTATE(PiMI_specification)
FSTATE(PiPL_specification)
#endif //ifndef BETA
FSTATE(event_handler_specification)
FSTATE(event_handler)
FSTATE(index_or_n)
FSTATE(dialog_specification)
FSTATE(dialog_subkey_item)
FSTATE(stretch_mode)
FSTATE(about_specification)
FSTATE(about_subkey_item)
FSTATE(embed_specification)
FSTATE(embed_subkey_item)
FSTATE(control_specification)
FSTATE(control_class_specification)
FSTATE(control_common_style)
FSTATE(control_standard_style)
FSTATE(control_scrollbar_style)
FSTATE(control_trackbar_style)
FSTATE(control_pushbutton_style)
FSTATE(control_checkbox_style)
FSTATE(control_radiobutton_style)
FSTATE(control_groupbox_style)
FSTATE(control_ownerdraw_style)
FSTATE(control_listbox_style)
FSTATE(control_combobox_style)
FSTATE(control_statictext_style)
FSTATE(control_edit_style)
FSTATE(control_tab_style)
FSTATE(control_frame_style)
FSTATE(control_rect_style)
FSTATE(control_image_style)
FSTATE(control_static_image_style)
FSTATE(control_bitmap_style)
FSTATE(control_icon_style)
FSTATE(control_metafile_style)
FSTATE(couple)
FSTATE(control_text_item)
FSTATE(control_subkey_item)
FSTATE(gradient)

// Region grammar
FSTATE(region_expression)
FSTATE(region_OR_expression)
FSTATE(region_XOR_expression)
FSTATE(region_AND_expression)
FSTATE(region_DIFF_expression)
FSTATE(region_primary_expression)
FSTATE(poly_fill_mode)
FSTATE(poly_points_list)
FSTATE(poly_point)

#ifndef BETA

// EnableInfo grammar
FSTATE(EI_booleanExpression)
FSTATE(EI_conjunction)
FSTATE(EI_relation)
FSTATE(EI_equality)
FSTATE(EI_simpleExpression)
FSTATE(EI_term)
FSTATE(EI_factor)
FSTATE(EI_intrinsic)
FSTATE(EI_constant)
FSTATE(EI_variable)

// PiMI grammar
FSTATE(PiMI_resource)
FSTATE(PiMI_resource_header)
FSTATE(optional_resource_name)
FSTATE(optional_resource_attribute_list)
FSTATE(resource_attribute)
FSTATE(PiMI_resource_body)
FSTATE(PiMI_version_property)
FSTATE(PiMI_subversion_property)
FSTATE(PiMI_priority_property)
FSTATE(PiMI_supported_modes_property)
FSTATE(PiMI_supported_mode)
FSTATE(PiMI_required_host_property)
FSTATE(PiMI_format_extension_field)
FSTATE(PiMI_type_creator_pair_list)
FSTATE(PiMI_supported_extensions_list)
FSTATE(PiMI_filter_case_properties)

// PiPL grammar
FSTATE(PiPL_resource)
FSTATE(PiPL_resource_header)
FSTATE(PiPL_resource_body)
FSTATE(PiPL_property_list)
FSTATE(PiPL_property)
FSTATE(PiPL_kind_property)
FSTATE(PiPL_kind_ID)
FSTATE(PiPL_version_property)
FSTATE(PiPL_version_clause)
FSTATE(PiPL_wired_version_ID)
FSTATE(PiPL_wired_version_ID_high)
FSTATE(PiPL_wired_version_ID_low)
FSTATE(PiPL_priority_property)
FSTATE(PiPL_required_host_property)
FSTATE(PiPL_name_property)
FSTATE(PiPL_category_property)
FSTATE(PiPL_68k_code_descriptor_property)
FSTATE(PiPL_68kFpu_code_descriptor_property)
FSTATE(PiPL_powerpc_code_descriptor_property)
FSTATE(PiPL_win32_x86_code_property)
FSTATE(PiPL_entry_point_name)
FSTATE(PiPL_bitmap_support)
FSTATE(PiPL_grayscale_support)
FSTATE(PiPL_indexed_support)
FSTATE(PiPL_RGB_support)
FSTATE(PiPL_CMYK_support)
FSTATE(PiPL_HSL_support)
FSTATE(PiPL_HSB_support)
FSTATE(PiPL_multichannel_support)
FSTATE(PiPL_duotone_support)
FSTATE(PiPL_LAB_support)
FSTATE(PiPL_Gray16_support)
FSTATE(PiPL_RGB48_support)

//Added by Harald Heim, December 6, 2002
FSTATE(PiPL_LAB48_support)
FSTATE(PiPL_CMYK64_support)
FSTATE(PiPL_deepmultichannel_support)
FSTATE(PiPL_duotone16_support)

FSTATE(PiPL_supported_modes_property)
FSTATE(PiPL_enable_info_property)
FSTATE(PiPL_filter_case_info_property)
FSTATE(PiPL_filter_info_case)
FSTATE(PiPL_input_matting)
FSTATE(PiPL_output_matting)
FSTATE(PiPL_layer_mask_flag)
FSTATE(PiPL_blank_data_flag)
FSTATE(PiPL_copy_source_flag)
FSTATE(PiPL_optional_writes_outside_selection_flag)
FSTATE(PiPL_type_creator_pair)
FSTATE(PiPL_format_file_type_property)
FSTATE(PiPL_type_creator_pair_list)
FSTATE(PiPL_read_types_property)
FSTATE(PiPL_write_types_property)
FSTATE(PiPL_filtered_types_property)
FSTATE(PiPL_ostype_list)
FSTATE(PiPL_read_extensions_property)
FSTATE(PiPL_write_extensions_property)
FSTATE(PiPL_filtered_extensions_property)
FSTATE(PiPL_saves_image_resources_flag)
FSTATE(PiPL_can_read_flag)
FSTATE(PiPL_can_write_flag)
FSTATE(PiPL_write_if_read_flag)
FSTATE(PiPL_format_flags_property)
FSTATE(PiPL_format_maximum_size_property)
FSTATE(PiPL_format_maximum_channels_property)
FSTATE(PiPL_parsable_types_property)
FSTATE(PiPL_parsable_extensions_property)
FSTATE(PiPL_filtered_parsable_types_property)
FSTATE(PiPL_filtered_parsable_extensions_property)
FSTATE(PiPL_parsable_clipboard_types_property)

#endif //ifndef BETA

// Color grammar
FSTATE(X11_color_name)
FSTATE(HTML_color_name)
FSTATE(JAVA_color_name)
FSTATE(NS_color_name)
FSTATE(Windows_static_color_name)
FSTATE(Windows_UI_color_name)
FSTATE(color)

#ifndef BETA
// FML grammar
FSTATE(FML_program)
FSTATE(global_declaration)
FSTATE(function_definition)
FSTATE(function_specifiers)
FSTATE(function_specifier)
FSTATE(function_declarator)
FSTATE(function_name)
FSTATE(parameter_list)
FSTATE(parameter_decl)
FSTATE(param_specifiers)
FSTATE(param_specifier)
FSTATE(param_declarator)
#endif


/*************************************************************************/
/*                                                                       */
/*                         PARSER VARIABLES                              */
/*                                                                       */
/*************************************************************************/
static char *inp;					/* Ptr to current char. */
static char *inp_start;				/* Ptr to first char. */
static char *inp_high_water;		/* Ptr to first char never matched. */
static char *inp_high_water_warn;	/* Where last warning was issued */
char *MDP_looking_for;
static UINT4 skip_whitespace = TRUE;
static UINT4 check_for_paragraph = FALSE;
static UINT4 processing_directive = FALSE;
static UINT4 processing_asm_statement = FALSE;
static int   pp_if_level = 0;
static UINT4 pp_skipping = FALSE;
static enum LANG current_language = LANG_C;
static enum PROG prog_type = PROG_UNKNOWN;
static prog_rev_major = INT_MAX;	//max available
static prog_rev_minor = INT_MAX;	//max available

/*************************************************************************/
/*                                                                       */
/*                           PARSER ROUTINES                             */
/*                                                                       */
/*************************************************************************/
static int mdp_line_from_inp(const char *inptr)
{
	int line = 1;
	char *p;
	for (p = inp_start; p < inptr; ++p) {
		if (*p == '\n') ++line;
	}
	return line;
} /*mdp_line_from_inp*/

static void mdp_warnv(const char *fmt,...)
{
    char szBuffer[1024];	//Truncates if not large enough!!
    va_list ap;

	if (inp > inp_high_water_warn) {
		inp_high_water_warn = inp;
		/* Interpret and interpolate the format string */
		va_start(ap, fmt);
		_vsnprintf(szBuffer, sizeof(szBuffer), fmt, ap);
		szBuffer[sizeof(szBuffer)-1] = 0;   //ensure NUL-terminated
		va_end(ap);
		strncpy(szBuffer, fm_formatString(szBuffer), sizeof(szBuffer));
		szBuffer[sizeof(szBuffer)-1] = 0;   //ensure NUL-terminated
		X86_warn(szBuffer);//FOR NOW
	}
} /*mdp_warnv*/

mdp_warn_constant_overflow(const char *whatKind, const char* inp_save)
{
	mdp_warnv("%s constant in line %d is too big: '%.*s'\r\n"
			  "Truncated to 0x%I64x",
			  whatKind,
			  mdp_line_from_inp(inp_save),
			  (int)(inp-inp_save), inp_save,
			  mdp_number);
} /*mdp_warn_constant_overflow*/

static char mdp_toupper(char c)
{
    if (c >= 'a' && c <= 'z') return c - ('a' - 'A');
    return c;
} /*mdp_toupper*/

static int mdp_isdigit(char c)
{
    if (c >= '0' && c <= '9') return TRUE;
    return FALSE;
} /*mdp_isdigit*/

static int mdp_isbinarydigit(char c)
{
    if (c >= '0' && c <= '1') return TRUE;
    return FALSE;
} /*mdp_isbinarydigit*/

static int mdp_isoctaldigit(char c)
{
    if (c >= '0' && c <= '7') return TRUE;
    return FALSE;
} /*mdp_isoctaldigit*/

static int mdp_ishexdigit(char c)
{
    if (c >= '0' && c <= '9') return TRUE;
    if (c >= 'A' && c <= 'F') return TRUE;
    if (c >= 'a' && c <= 'f') return TRUE;
    return FALSE;
} /*mdp_ishexdigit*/

static int mdp_iswhite(char c)
{   // I.E. returns 0xA0 for non-breaking space...
    if ((c & 0x7f) == ' ' || c == '\t') return TRUE;
    if (c == '\f' || c == '\v') return TRUE;
    if (processing_asm_statement) return FALSE; // cr and lf are not whitespace in asm_statement!
    if (processing_directive) return FALSE; // cr and lf are not whitespace to pp!
    if (c == '\n' || c == '\r') return TRUE;
    return FALSE;
} /*mdp_iswhite*/

/* Nonrecursive blank skipper for use within mdp_skip_whitespace(). */
static void mdp_skip_blanks(void)
{
#if 0
    // OOPS!  This needs to skip over comments too!
    // maybe just modify mdp_skip_whitespace to skip blanks and comments,
    // but not directives, when called recursively.
    while ((inp[0] & 0x7f) == ' ' || inp[0] == '\t')
        inp++;
#else
    assert(processing_directive);
    assert(skip_whitespace);
    mdp_skip_whitespace();
#endif
} /*mdp_skip_blanks*/

static void mdp_skip_whitespace(void)
{
    assert(!processing_directive || skip_whitespace);
    if (skip_whitespace) {
        while (mdp_iswhite(inp[0])
               ||
               (inp[0] == '/' && (inp[1] == '/' || inp[1] == '*'))
               ||
               (processing_asm_statement && inp[0] == ';')
#if 0  //turn off PP for now...
               ||
               (!processing_directive && inp[0] == '#') 
#endif
               )
        {
            if (inp[0] == '/') {
                inp++;
                if (inp[0] == '/')
                {   /* C++-style comment, terminates with CR, LF, or EOS. */
                    do inp++; while (inp[0] != 0 && inp[0] != '\n' && inp[0] != '\r');
                    if (inp[0] != 0)
                        inp++;  /* skip over CR or LF */
                }
                else
                {   /*  C-style comment, terminates with star-slash or EOS. */
					char *inp_save = inp;
                    do inp++; while (inp[0] != 0 && !(inp[0] == '*' && inp[1] == '/'));
					if (inp[0] != 0) {
                        inp += 2;   /* skip over star-slash */
					}
					else {
						//WARNING: C-style comment terminated by EOS...
						mdp_warnv("C-style comment starting in line %d\r\n"
								  "is terminated by end of source file.",
								  mdp_line_from_inp(inp_save));
					}
                }
            }
            else if (inp[0] == ';')
            {   /* asm-style comment within an asm_statement, terminates */
                /* with CR, LF, or EOS (but not '}')                     */
                do inp++; while (inp[0] != 0 && inp[0] != '\n' && inp[0] != '\r');
                /* do NOT skip over CR or LF!! */
            }
            else if (inp[0] == '#')
            {   /* possible preprocessor directive */
                //Should check that # is first non-blank after a new-line or BOS
                assert(!pp_skipping);
                assert(!processing_directive);
                processing_directive = TRUE;
                inp++;
                if (mdp_keyword("ifdef")) {
                }
                else if (mdp_keyword("ifndef")) {
                }
                else if (mdp_keyword("if")) {
                /* #if <condition> */
                    pp_if_level++;
                    //if (constant_expression()) {
                    if (mdp_keyword("true")) {
                        pp_skipping = FALSE; //redundant!!
                    }
                    else if (mdp_keyword("false")) {
                        pp_skipping = TRUE;
                        mdp_skip_blanks();
                        if (inp[0] != '\n' && inp[0] != '\r') {
                            //error
                            if (inp[0] == 0) {
                                //hit eos
                                assert(!"unexpected EOS");
                            }
                            assert(!"garbage after controlling expression");
                        }
                        while (pp_skipping) {
                            // look for new-line followed by #
                            do {
                                while (inp[0] != '\n' && inp[0] != '\r' && inp[0] != 0)
                                        inp++;
                                inp++;
                                mdp_skip_blanks();
                            } while (inp[0] != '#' && inp[0] != 0);
                            if (inp[0] == 0) {
                                //error -- EOS while skipping
                                assert(!"EOS while skipping (missing #endif)");
                            }
                            inp++;
                            if (mdp_keyword("ifdef") || mdp_keyword("ifndef") || mdp_keyword("if")) {
                                pp_if_level++;
                            }
                            else if (mdp_keyword("elif")) {
                                if (pp_if_level == 1) {
                                    // test to see if we should stop skipping
                                    if (mdp_keyword("true")) {
                                        pp_skipping = FALSE;
                                    }
                                }
                            }
                            else if (mdp_keyword("else")) {
                                if (pp_if_level == 1) {
                                    //stop skipping
                                    pp_skipping = FALSE;
                                }
                            }
                            else if (mdp_keyword("endif")) {
                                pp_if_level--;
                                if (pp_if_level == 0) {
                                    // time to stop skipping
                                    pp_skipping = FALSE;
                                }
                            }
                        } /*while skipping*/
                    }
                    else {
                        // bad expression in #if
                        assert(!"bad expression in #if");
                    }
                }
                else if (mdp_keyword("elif") || mdp_keyword("else")) {
                    assert(!pp_skipping);
                    if (pp_if_level <= 0) {
                        // #elif or #else without #if
                        assert(!"#else or #elif without #if");
                    }
                    pp_skipping = TRUE;
                    // skip to matching #endif (or another #else/elif)...
                }
                else if (mdp_keyword("endif")) {
                    assert(!pp_skipping);
                    pp_if_level--;
                    if (pp_if_level < 0) {
                        // #endif without #if
                        assert(!"#endif without #if");
                    }
                }
                else if (mdp_keyword("define")) {
                }
                else if (mdp_keyword("undef")) {
                }
                else if (mdp_keyword("pragma")) {
                }
                else if (mdp_keyword("error")) {
                }
                else {
                    // unknown directive
                    assert(!"unknown # directive");
                }
                // check rest of pp directive line...
                mdp_skip_blanks();
                if (inp[0] != '\n' && inp[0] != '\r' && inp[0] != 0) {
                    assert(!"garbage at end of # directive");
                }
                if (inp[0] != 0)
                    inp++;
                processing_directive = FALSE;
            }//inp[0] == '#'
            else if (check_for_paragraph && inp[0] == '\n')
            {   /* check for paragraph mark: <lf><lf> or <lf><cr> */
                if (inp[1] == '\r' || inp[1] == '\n') {
                    // it's a paragraph mark: return the leading <lf>
                    return;
                }
                inp++;  /* just ordinary whitespace */
            }
            else
            {   /* ordinary white space char */
                inp++;  /* skip over white space char */
            }
        }//while mdp_iswhite(inp[0]
    }//if skip_whitespace
} /*mdp_skip_whitespace*/

static int mdp_isidchar(char c)
{
    if (c >= 'A' && c <= 'Z') return TRUE;
    if (c >= 'a' && c <= 'z') return TRUE;
    if (c >= '0' && c <= '9') return TRUE;
    if (c == '_') return TRUE;
    return FALSE;
} /*mdp_idchar*/

static char state_name[] = "\'x\'";

static int mdp_char(char c)
{
    MARK;
    if (*inp == c) {
        inp++;
        ACCEPT;
    } else REJECT_CHAR;
} /*mdp_char*/

static int mdp_char_upper(char c)
{	/* NOTE: c must be uppercase or match will fail */
    MARK;
    if (mdp_toupper(*inp) == c) {
        inp++;
        ACCEPT;
    } else REJECT_CHAR;
} /*mdp_char_upper*/

static int mdp_any(void)
{
    char *state_name = "any character";
    MARK;
    if (*inp == 0) REJECT;
    inp++;
    ACCEPT;
} /*mdp_any*/

static int mdp_paragraph(void)
{
    static char state_name[] = "paragraph mark";
    MARK;
    if (*inp != '\n') REJECT;
    inp++;
    ACCEPT;
} /*mdp_paragraph*/

static int mdp_eos(void)
{
    char *state_name = "end of input";
    MARK;
    if (*inp == 0) ACCEPT; else REJECT;
} /*mdp_eos*/

static int mdp_keyword(char *kw)
{
    char *state_name = kw;
    MARK;
    /* Match mandatory head of keyword. */
    while (*kw != 0 && *kw != '*') {
        if (*inp != *kw) REJECT;
        kw++;
        inp++;
    }
    if (*kw == '*') kw++;
    /* Any remaining id chars must match optional tail of keyword. */
    while (mdp_isidchar(*inp)) {
        if (*inp != *kw) REJECT;
        kw++;
        inp++;
    }
    ACCEPT;
} /*mdp_keyword*/

static int mdp_keywordi(char *kw)
{   /* Case-insensitive version of mdp_keyword */
    char *state_name = kw;
    MARK;
    /* Match mandatory head of keyword. */
    while (*kw != 0 && *kw != '*') {
        if (mdp_toupper(*inp) != mdp_toupper(*kw)) REJECT;
        kw++;
        inp++;
    }
    if (*kw == '*') kw++;
    /* Any remaining id chars must match optional tail of keyword. */
    while (mdp_isidchar(*inp)) {
        if (mdp_toupper(*inp) != mdp_toupper(*kw)) REJECT;
        kw++;
        inp++;
    }
    ACCEPT;
} /*mdp_keywordi*/

static int mdp_decimal(void)	//NEED TO SET skip_whitespace = FALSE here and elsewhere???
{
    char *state_name = "decimal number";
    MARK;
    mdp_number = 0;
	mdp_number_overflow = false;
	mdp_number_unsigned = false;
	mdp_number_bitlen = 32;	//default
    if (!mdp_isdigit(*inp)) REJECT;
    while (mdp_isdigit(*inp)) {
		int iDigit = *inp++ - '0';
		//check for overflow...
		//Note that 9223372036854775807 == LLONG_MAX,
		//and 922337203685477580 == LLONG_MAX  / 10.
		if ((UINT64)mdp_number >  922337203685477580ULL ||
			(UINT64)mdp_number == 922337203685477580ULL && iDigit > 7) {
				mdp_number_overflow = true;
		}
		//accumulate next digit
        mdp_number = mdp_number * 10 + iDigit;
    }
	if (mdp_number_overflow) {
		mdp_warn_constant_overflow("Decimal", inp_save);
	}
    ACCEPT;
} /*mdp_decimal*/

static int mdp_octal_digits(void)
{
    char *state_name = "octal digit string";
    MARK;
    mdp_number = 0;
	mdp_number_overflow = false;
	mdp_number_unsigned = false;
	mdp_number_bitlen = 32;	//default
    if (!mdp_isoctaldigit(*inp)) REJECT;
    while (mdp_isoctaldigit(*inp)) {
		//check for overflow...
		mdp_number_overflow |= (mdp_number & 0xe000000000000000) != 0;
		//accumulate next digit
        mdp_number = (mdp_number << 3) + (*inp++ - '0');
    }
	if (mdp_number_overflow) {
		mdp_warn_constant_overflow("Octal", inp_save);
	}
    ACCEPT;
} /*mdp_octal_digits*/

#if 0 //not used
static int mdp_hex(void)
{
    char *state_name = "hexadecimal number";
    MARK;
    mdp_number = 0;
	mdp_number_overflow = false;
	mdp_number_unsigned = false;
	mdp_number_bitlen = 32;	//default
    if (!mdp_isdigit(*inp)) REJECT;	//Intel-style hex requires leading decimal digit
    while (mdp_ishexdigit(*inp)) {
		//check for overflow...
		mdp_number_overflow |= (mdp_number & 0xf000000000000000) != 0;
		//accumulate next digit
        mdp_number = (mdp_number << 4) | ( mdp_isdigit(*inp) ? *inp - '0' : (*inp & 7) + 9 );
        inp++;
    }
	if (mdp_number_overflow) {
		mdp_warn_constant_overflow("Hex", inp_save);
	}
    ACCEPT;
} /*mdp_hex*/
#endif

static int mdp_hex_digits(void)
{
    char *state_name = "hex digit string";
    MARK;
    mdp_number = 0;
	mdp_number_overflow = false;
	mdp_number_unsigned = false;
	mdp_number_bitlen = 32;	//default
    if (!mdp_ishexdigit(*inp)) REJECT;
    while (mdp_ishexdigit(*inp)) {
		//check for overflow...
		mdp_number_overflow |= (mdp_number & 0xf000000000000000) != 0;
		//accumulate next digit
        mdp_number = (mdp_number << 4) | ( mdp_isdigit(*inp) ? *inp - '0' : (*inp & 7) + 9 );
        inp++;
    }
	if (mdp_number_overflow) {
		mdp_warn_constant_overflow("Hexadecimal", inp_save);
	}
    ACCEPT;
} /*mdp_hex_digits*/

static int mdp_binary_digits(void)
{
    char *state_name = "binary digit string";
    MARK;
    mdp_number = 0;
	mdp_number_overflow = false;
	mdp_number_unsigned = false;
	mdp_number_bitlen = 32;	//default
    if (!mdp_isbinarydigit(*inp)) REJECT;
    while (mdp_isbinarydigit(*inp)) {
		//check for overflow...
		mdp_number_overflow |= (mdp_number & 0x8000000000000000) != 0;
		//accumulate next digit
        mdp_number = (mdp_number << 1) | (*inp++ - '0');
    }
	if (mdp_number_overflow) {
		mdp_warn_constant_overflow("Binary", inp_save);
	}
    ACCEPT;
} /*mdp_binary_digits*/

/*************************************************************************/
/*                                                                       */
/*                       MD386 PARSER ACTION ROUTINES                    */
/*                                                                       */
/*************************************************************************/

static int clear_flags_and_params(void)
{
    flags = 0;
    param1 = param2 = param3 = param4 = 0;
    param5 = param6 = param7 = param8 = 0;
    return TRUE;
}

static int push_constant(INT4 k)
{
    if (Ncount >= Nsdl) {
        Mde_abort_command (MD386__PARSEROVFL, 0, 0, 0);
        return FALSE;
    }
    SDL[Ncount++] = k;
    return TRUE;
} /*push_constant*/

/*
;  Push the value of the specified parameter (or any longword, for that
;  matter) onto the semantic stack.
*/
static int push_param(UINT4 p)
{
    if (Ncount >= Nsdl) {
        Mde_abort_command (MD386__PARSEROVFL, 0, 0, 0);
        return FALSE;
    }
    SDL[Ncount++] = p;
    return TRUE;
} /*push_param*/

/*
;  Pop a parameter OFF the semantic stack.
*/
static UINT4 pop_param(void)
{
    if (Ncount <= 0) {
        //Oops. Stack underflow.  (Call it overflow for now...)
        Mde_abort_command (MD386__PARSEROVFL, 0, 0, 0);
        return 0;
    }
    return SDL[Ncount--];	// Should this be --Ncount ???
} /*pop_param*/

/*
;  Push the flag bits onto the semantic stack.
*/
static int push_flags(void)
{
    if (Ncount >= Nsdl) {
        Mde_abort_command (MD386__PARSEROVFL, 0, 0, 0);
        return FALSE;
    }
    SDL[Ncount++] = flags;
    return TRUE;
} /*push_flags*/

/*
;  Push the value of the number just parsed onto the semantic stack.
*/
static int push_number(void)
{
    ////return push_param64(mdp_number);	//X64: push_param64()
    return push_param((UINT4)mdp_number);	//X64: push_param64() TEMP FIX FOR NOW!!
} /*push_number*/

#if 0	//UNUSED
/*
;  Push the descriptor of the string just parsed onto the semantic stack.
*/
static int push_descriptor(void)
{
    push_param(mdp_tokencnt);
    return push_param((UINT4) mdp_tokenptr);	//X64
} /*push_descriptor*/
#endif

/*
;  Change the semantic descriptor type to the specified type.
*/
static int change_sd(UINT4 newsd)        //UNUSED????
{
    if (Ncount < 3) return FALSE;  /* oops */
    SDL[Ncount-3] = newsd;
    return TRUE;
} /*change_sd*/

/*
;  Push the specified operator onto the semantic stack.
*/
static int push_op(UINT4 op)
{
    return push_param(op);
} /*push_op*/

/*
;  Push the specified descriptor and current number value onto the
;  semantic stack.
*/
static int push_value(UINT4 sd)
{
    push_param(sd);
    ////return push_param64(mdp_number);	//X64: push_param64()
    return push_param((UINT4)mdp_number);	//X64: push_param64() TEMP FIX FOR NOW!!
} /*push_value*/

/*
;  Initialize the string accumulator.
*/
static int init_string_acc(void)
{
    mdp_string_len = 0;
    return TRUE;
} /*init_string_acc*/

/*
;  Add a character to the string accumulator.
*/
static int add_char_to_acc(char c)
{
    if (mdp_string_len < sizeof(mdp_string_acc)) {
        mdp_string_acc[mdp_string_len++] = c;
    }
    else {
        // ERROR: String too long for accumulator.
		return FALSE;
    }
    return TRUE;
} /*add_char_to_acc*/

/*
;  Push a descriptor for the string accumulator onto the
;  semantic stack, padding stack with blanks to a dword boundary.
*/
static int push_char_sd(void)
{
    UINT4  padded_count;
    UINT4  i;
    UINT4  *p;

    push_param((mdp_string_len << 16) | LIT_CHAR);
    padded_count = (mdp_string_len + 3) & ~3;
    for (i = mdp_string_len;  i < padded_count;  i++) {
        /* pad string acc with blanks */
        mdp_string_acc[i] = ' ';
    }
    p = (UINT4 *) &mdp_string_acc;
    for (i = 1;  i <= padded_count/4;  i++) {
        push_param(*p++);
    }
    return TRUE;
} /*push_char_sd*/

/*
;   Convert a character constant string to an integer value.
;   (Overflow if more than 8 chars is ignored here, will be
;	diagnosed by caller.)
*/
static int convert_char_const_to_int(void)
{
    UINT4 i;

    mdp_number = 0;
	mdp_number_overflow = false;
	mdp_number_unsigned = false;		//correct???
	mdp_number_bitlen = 32;	//default
    for (i = 0;  i < mdp_string_len;  i++) {
		//Should we check for overflow (e.g. '123456789') here??
		//No, just truncate here to 8 least significant chars.
        mdp_number = (mdp_number << 8) | mdp_string_acc[i];
    }
    return TRUE;
} /*convert_char_const_to_int*/

/*
;   Push a descriptor for a literal ID onto the semantic stack,
;   padding the stack to a dword boundary.
*/
static int push_idlit_sd(const char *charp, UINT4 count)
{
    UINT4  padded_count;
    UINT4  i;
    UINT4  *p;
    static const UINT4 nulmask[4] = {
        0x00FFFFFF,
        0x00000000,
        0x000000FF,
        0x0000FFFF
        };

    count++;    //+1 for asciz NUL.
    push_constant((count << 16) | SD_IDLIT);
    padded_count = (count + 3) & ~3;
    p = (UINT4 *) charp;
    for (i = 1; i < padded_count/4; i++) {
        push_param(*p++);
    }
    //Push final word containing the ASCIZ NUL.
    push_param(*p & nulmask[count & 3]);
    return TRUE;
} /*push_idlit_sd*/

/************* Virtual Locale Routines ***************/
//
//  Should be moved to their own module!
//

#define MAX_LOCALE_NAME 1024 // For now...

//
//fm_save_locale:
//
static char * fm_save_locale (void)
{
	// get current locale name...
	const char * localep = setlocale(LC_ALL, NULL);
	if (localep) {
		int n = min((int)strlen(localep), MAX_LOCALE_NAME) + 1; //+1 for NUL
		char * copy = malloc(n);
		if (copy) {
			strncpy(copy, localep, n);
			copy[n-1] = 0;	//ensure NUL-terminated
		}
		return copy; //even if NULL; else must later be free-d.
	}
	return NULL; //setlocale returned NULL
}/*fm_save_locale*/

//
//fm_save_and_set_locale:
//
__inline static char * fm_save_and_set_locale (int category, const char * locale)
{
	char * saved_locale = fm_save_locale();
	setlocale(category, locale);
	return saved_locale;
}/*fm_save_and_set_locale*/

//
//fm_restore_saved_Locale:
//
__inline static void fm_restore_saved_locale (char * saved_locale)
{
	//restore a previously saved locale
	setlocale(LC_ALL, saved_locale);
	free (saved_locale);
}/*fm_restore_saved_locale*/

//
//fm_atof:
//
double fm_atof (const char * str)
{
	//long version
	//for internal FM use only; 
	//always uses "." for decimal point
	char * current_locale = fm_save_and_set_locale(LC_ALL, "C");
	double result = atof(str);
	fm_restore_saved_locale(current_locale);
	return result;
}/*fm_atof*/

//
//fm_strtod:
//
double fm_strtod (const char * str, char ** ptr)
{
	//long version
	//for internal FM use only; 
	//always uses "." for decimal point
	char * current_locale = fm_save_and_set_locale(LC_ALL, "C");
	double result = strtod(str, ptr);
	fm_restore_saved_locale(current_locale);
	return result;
}/*fm_strtod*/

/*
;  Convert the current token string to a real (D_floating) number.
*/
#if 0
static int cvt_t_d(void)
{
    //char *pInvalid;
    //mdp_real_num = strtod( mdp_tokenptr, &pInvalid);
    //sscanf (mdp_tokenptr, "%lf", &mdp_real_num);

    //Set Locale to English numbers
    setlocale( LC_NUMERIC, "English" );
    
    // atof() may fail in a locale for which the decimal point is not '.'!!!
    mdp_real_num = atof(mdp_tokenptr);

    //Reset locale setting
    setlocale( LC_NUMERIC, "" );

    return TRUE;
} /*cvt_t_d*/
#elif 1
static int cvt_t_d(void)
{
    // fm_atof() temporarily changes to "C" locale,
    // in which decimal point is always '.'
    mdp_real_num = fm_atof(mdp_tokenptr);
    return TRUE;
} /*cvt_t_d*/
#elif 0         // NFG! mdp_tokenptr is not nul-terminated (try again later)
static int cvt_t_d(void)
{
    // fm_strtod() temporarily changes to "C" locale,
    // in which decimal point is always '.'
    // N.B. strtod() provides more error checking
    // than atof(), and can potentially catch some
    // locale-specific problems...
    char *endp;
#if 1  // test error message by setting wrong locale...
    //Set Locale to German numbers
    setlocale( LC_NUMERIC, "German" );
#endif
    mdp_real_num = fm_strtod(mdp_tokenptr, &endp);
    if (endp[0] != '\0') {
        /* string was not fully parsed!  Probably */
        /* looking for wrong decimal point char?  */
        ErrorOk("Failed to parse floating-point number %s\n"
                "Possible problem with incorrect decimal point\n"
                "character in this locale?", mdp_tokenptr);
        return FALSE;
    }
    return TRUE;
} /*cvt_t_d*/
#else   //doesn't work!!!
static int cvt_t_d(void)
{
    // To make sure atof() works correctly here, we
    // need to set the locale-dependent decimal point
    // string temporarily to "."

    //Get address of locale convention structure
    struct lconv *lconvp = localeconv();

    //Save current decimal point string.
    char *current_decimal_point = lconvp->decimal_point;

    //Set decimal point temporarily to "."
    //WARNING: This is an abuse of the API -- the user is
    //         specifically forbidden to alter the contents
    //         of the lconv structure.  Nevertheless...
    lconvp->decimal_point = ".";

    // Call atof() with decimal point set to "."
    mdp_real_num = atof(mdp_tokenptr);

    //Reset the current decimal point string.
    lconvp->decimal_point = current_decimal_point;

    return TRUE;
} /*cvt_t_d*/
#endif

/*
;  Push the descriptor for a real (D_floating) number onto the 
;  semantic stack.
*/
static int push_real_sd(void)
{
    UINT4 *p = (UINT4 *) &mdp_real_num;

    push_constant(LIT_REAL);
    push_param(*p++);
    return push_param(*p);
} /*push_real_sd*/

/*
;   Push the address of a function onto the SDL prior to an OP_CALL.
;
*/
#if 1
//use new SD_IDLIT descriptor...
#define push_var_addr(var) \
        push_idlit_sd(#var, sizeof(#var)-1);\
        push_op(OP_LOOKUP)
#else
#define push_var_addr(var) \
        push_constant(SD_ID);\
        push_constant(sizeof(#var)-1);\
        push_param((UINT4)#var);\
        push_op(OP_LOOKUP)
#endif

#define push_proc_addr(proc) push_var_addr(proc); push_op(OP_ARGLIST)

/*************************************************************************/
/*                                                                       */
/*                     FILTERMEISTER PARSER STATES                       */
/*                                                                       */
/*************************************************************************/

/********** <Filter_Meister_program> ***********
;
; Parse a Filter Meister program.
;
;  <FM_program> :==   <AFS_program> |
;                     <FFW_program> |
;                     <FFP_program> |
;                     <FML_program> |
;                     <null_program>
;
;  <null_program> ::= <end_of_program>
;
;  <end_of_program> ::= ( '%%EOF' | <eos> )
;
*/
STATE(FM_program)
    int ok = TRUE;
    MARK;
    clear_flags_and_params();
    More = FALSE;                /* MORE=false in case of error... */
    prog_type = PROG_UNKNOWN;

#ifdef STAND_ALONE_PARSER
    if (mdp_keyword("quit")) {
        push_constant(-2);  /* special flag to quit standalone parser */
        push_constant(-1);
        ACCEPT;
    }
    else
#endif
    if (AFS_program()) {
    }
    else if (FFW_program()) {
    }
    else if (FFP_program()) {
    }
#ifndef BETA
    else if (FML_program()) {
    }
#endif
    else {
        /* Null program */
        prog_type = PROG_NULL;
        ok = end_of_program();
    }
    push_constant(-1);
    // Check for missing #endif...
    assert(pp_if_level == 0);
    if (ok) ACCEPT; else REJECT;
} /*FM_program*/

/* 
;
; Check for end of program.
;
*/
STATE(end_of_program)
    MARK;
    if (mdp_keywordi("%%EOF") || mdp_eos()) {
        More = FALSE;
        ACCEPT;
    }
    More = FALSE;
    REJECT;
} /*end_of_program*/



/************************* <error_token> ***************************
;
; Syntax:
;
;   <error_token> ::=
;       <identifier>    |
;       <constant>      |
;       <any_operator>  |
;       <punctuator>    |
;       <any_char>
;
; Semantic Descriptor:
;
;       +-------+-------+-------+-------+
;       |             8 |     0 |     1 |
;       +---------------+-------+-------+
;       |       # of chars in token     |
;       +-------------------------------+
;       |       ptr to first char       |
;       +-------------------------------+
;
*/
STATE(error_token)
    char *tokenptr;
    MARK;
    tokenptr = inp;
    if (identifier()) {
    }
    else if (constant()) {
    }
    else if (any_operator()) {
    }
    else if (punctuator()) {
    }
    else if (mdp_any()) {
    }
    Ncount = 0; //clear the SDL
    push_constant(SD_ID);
    push_param((UINT4)(inp - tokenptr));
    push_param((UINT4)(UINT_PTR)tokenptr);
#ifdef _WIN64
	//push high 4 bytes of 64-bit pointer
	push_param((UINT4)((UINT_PTR)tokenptr >> 32));
#endif
    ACCEPT;
} /*error_token*/


/************************** <identifier> ***************************
;
; Syntax:
;
;	<identifier> ::=
;		<nondigit> |
;		<identifier> <nondigit> |
;		<identifier> <digit>
;
;	<nondigit> ::=
;		_ | a | b | c | d | e | f | g | h | i | j | k | l | m |
;           n | o | p | q | r | s | t | u | v | w | x | y | z |
;		    A | B | C | D | E | F | G | H | I | J | K | L | M |
;           N | O | P | Q | R | S | T | U | V | W | X | Y | Z
;
;	<digit> ::=
;		0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9
;
; 
; Semantic Descriptor:
;
;       +-------+-------+-------+-------+
;       |             8 |     0 |     1 |
;       +---------------+-------+-------+
;       |       # of chars in id        |
;       +-------------------------------+
;       |       ptr to first char       |
;       +-------------------------------+
;
*/
STATE(identifier)
    MARK;
    mdp_tokencnt = 0;
    mdp_tokenptr = inp;
    if (mdp_isdigit(*inp)) REJECT;
    while (mdp_isidchar(*inp)) {
        mdp_tokencnt++;
        inp++;
    }
    if (mdp_tokencnt == 0) REJECT;
#if 1
    //use new SD_IDLIT instead of SD_ID...
    push_idlit_sd(mdp_tokenptr, mdp_tokencnt);
#else
    push_constant(SD_ID);
    push_param(mdp_tokencnt);
    push_param((UINT4) mdp_tokenptr);
#ifdef _WIN64
	//push high 4 bytes of 64-bit pointer
	push_param((UINT4) mdp_tokenptr + 4);
#endif
#endif
    ACCEPT;
} /*identifier*/


/******************************* <constant> ******************************
;
; Syntax:
;
;	<constant> ::=
;		<floating_constant>     |
;		<integer_constant>      |
;		<enumeration_constant>  |
;		<character_constant>    |
;		<boolean_constant>      |
;       <named_constant>		|
;       <named_floating_constant>
;
*/
STATE(constant)
    MARK;
    if (floating_constant()    ||
        integer_constant()     ||
#if 0
        // Causes problems for now ... disambiguate later!
        enumeration_constant() ||
#endif
        character_constant()   ||
        boolean_constant()     ||
        named_constant()       ||
		named_floating_constant()		)   ACCEPT;		
    REJECT;
} /*constant*/

/**************************** <floating_constant> ***************************
;
; Syntax:
;
;	<floating_constant> ::=
;		<named_floating_constant> |
;		<fractional_constant> [<exponent_part] [<floating_suffix>] |
;		<digit_sequence> <exponent_part> [<floating_suffix>]
;
;	<fractional_constant> ::=
;		[<digit_sequence>] "." <digit_sequence> |
;		<digit_sequence> "."
;
;	<exponent_part> ::=
;		{"e"|"E"} [<sign>] <digit_sequence>
;
;	<sign> ::= "+" | "-"
;
;	<digit_sequence> ::= {<digit>}+
;
;	<floating_suffix> ::= "f" | "l" | "F" | "L"
;
;
; Semantic Descriptor:
;
;       +-------+-------+-------+-------+
;       |             8 |     3 |     2 |
;       +---------------+-------+-------+
;       |                               |
;       |         real*8 value          |
;       |                               |
;       +-------------------------------+
;
*/
#if 1
STATE(floating_constant)
    MARK;
    if (named_floating_constant()) ACCEPT;
    skip_whitespace = FALSE;
    mdp_tokenptr = inp;
    if (fractional_constant()) {
        exponent_part();    //optional
    }
    else {
        if (!digit_sequence()) REJECT;
        if (!exponent_part()) REJECT;
    }
    cvt_t_d();
    floating_suffix();  //optional              <<== AFFECTS sizeof ??? sizeof(1.0F) == 4, not 8?
    push_real_sd();
    ACCEPT;
} /*floating_constant*/

STATE(fractional_constant)
    MARK;
    if (digit_sequence()) {
        if (!mdp_char('.')) REJECT;
        digit_sequence();   //optional
    }
    else {
        if (!mdp_char('.')) REJECT;
        if (!digit_sequence()) REJECT;
    }
    ACCEPT;
} /*fractional_constant*/

STATE(exponent_part)
    MARK;
    if (mdp_char('e') || mdp_char('E')) {
        mdp_char('+') || mdp_char('-'); //optional sign
        if (digit_sequence()) ACCEPT;
    }
    REJECT;
} /*exponent_part*/

STATE(digit_sequence)
    MARK;
    if (!mdp_isdigit(*inp)) REJECT;
    while (mdp_isdigit(*inp)) inp++;
    ACCEPT;
} /*digit_sequence*/

STATE(floating_suffix)
    MARK;
    if (mdp_char('f') || mdp_char('F') || mdp_char('l') || mdp_char('L')) ACCEPT;
    REJECT;
} /*floating_suffix*/
#else
STATE(floating_constant)
    MARK;
    if (!real_number_string()) REJECT;
    cvt_t_d();
    push_real_sd();
    ACCEPT;
} /*floating_constant*/

STATE(real_number_string)
    MARK;
    mdp_tokencnt = 0;
    mdp_tokenptr = inp;
    if (*inp == '.') {
        inp++;
        if (!decimal_sequence()) REJECT;
    }
    else if (decimal_sequence()) {
        if (*inp != '.') REJECT;
        inp++;
        decimal_sequence();
    }
    else REJECT;
    if (mdp_toupper(*inp) == 'E' || mdp_toupper(*inp) == 'D') {
        /* optional exponent */
        inp++;
        if (*inp == '+' || *inp == '-') inp++;
        if (!decimal_sequence()) REJECT;
    }
    mdp_tokencnt = inp - mdp_tokenptr;
    ACCEPT;
} /*real_number_string*/

static int decimal_sequence(void)
{
    if (!mdp_isdigit(*inp)) return FALSE;
    while (mdp_isdigit(*inp)) inp++;
    return TRUE;
} /*decimal_sequence*/
#endif

/************************** <integer_constant> *****************************
;
; Syntax:
;
;	<integer_constant> ::=
;		<decimal_constant> [<integer_suffix>]       |
;		<octal_constant> [<integer_suffix>]         |
;		<hexadecimal_constant> [<integer_suffix>]   |
;		<binary_constant> [<integer_suffix>]		|
;       <named_constant>
;
;	<decimal_constant> ::= <nonzero_digit> {<digit>}*
;
;	<octal_constant> ::= "0" {<octal_digit>}*
;
;	<hexadecimal_constant> ::= {"0x"|"0X"} {<hexadecimal_digit>}+
;
;	<binary_constant> ::= {"0b"|"0B"} {<binary_digit>}+
;
;	<binary_digit> ::= "0"|"1"
;
;	<nonzero_digit> ::=
;		"1"|"2"|"3"|"4"|"5"|"6"|"7"|"8"|"9"
;
;	<octal_digit> ::=
;		"0"|"1"|"2"|"3"|"4"|"5"|"6"|"7"
;
;	<hexadecimal_digit> ::=
;		"0"|"1"|"2"|"3"|"4"|"5"|"6"|"7"|"8"|"9"|
;		"a"|"b"|"c"|"d"|"e"|"f"|
;		"A"|"B"|"C"|"D"|"E"|"F"
;
;	<integer_suffix> ::=
;		<unsigned_suffix> [<long_suffix>] |
;		<long_suffix> [<unsigned_suffix>]
;
;	<unsigned_suffix> ::= "u" | "U"
;
;	<long_suffix> ::= "l" | "L" | "ll" | "LL" | "i64" | "I64"
;
; Semantic Descriptor:
;
;       +-------+-------+-------+-------+
;       |             4 |     1 |     2 |
;       +---------------+-------+-------+
;       |        32-bit value           |
;       +-------------------------------+
;
*/
STATE(integer_constant)
    MARK;
    skip_whitespace = FALSE;
    if (hexadecimal_constant() || binary_constant() ||
		      octal_constant() || decimal_constant()) {
        integer_suffix();   //optional
        ACCEPT;
    }
    else if (named_constant()) {
        ACCEPT;
    }
    REJECT;
} /*integer_constant*/

STATE(decimal_constant)
    MARK;
    if (mdp_decimal()) {
        push_value(LIT_INTEGER);
        ACCEPT;
    }
    REJECT;
} /*decimal_constant*/

STATE(octal_constant)
    MARK;
    skip_whitespace = FALSE;
    if (mdp_char('0')) {
        mdp_octal_digits(); //optional trailing octal digits
        push_value(LIT_INTEGER);
        ACCEPT;
    }
    REJECT;
} /*octal_constant*/

STATE(hexadecimal_constant)
    MARK;
    skip_whitespace = FALSE;
    if (mdp_char('0') && mdp_char_upper('X') && mdp_hex_digits()) {
        push_value(LIT_INTEGER);
        ACCEPT;
    }
    REJECT;
} /*hexadecimal_constant*/

STATE(binary_constant)
    MARK;
    skip_whitespace = FALSE;
    if (mdp_char('0') && mdp_char_upper('B') && mdp_binary_digits()) {
        push_value(LIT_INTEGER);
        ACCEPT;
    }
    REJECT;
} /*binary_constant*/

STATE(integer_suffix)
    MARK;
    if (unsigned_suffix()) long_suffix();
    else if (long_suffix()) unsigned_suffix();
    else REJECT;
    ACCEPT;
}

STATE(unsigned_suffix)
    MARK;
    if (mdp_char('u') || mdp_char('U')) {
		mdp_number_unsigned = true;
        ACCEPT;
    }
    REJECT;
}

STATE(long_suffix)
    MARK;
    if (mdp_char('l')) {
		if (mdp_char('l')) {
			// 'll' = long long = 64-bits
			mdp_number_bitlen = 64;
		}
        ACCEPT;
    }
    if (mdp_char('L')) {
		if (mdp_char('L')) {
			// 'LL' = long long = 64-bits
			mdp_number_bitlen = 64;
		}
        ACCEPT;
    }
	if (mdp_char_upper('I') && mdp_char('6') && mdp_char('4')) {
		// 'i64' or 'I64' = 64-bits
		mdp_number_bitlen = 64;
		ACCEPT;
	}
    REJECT;
}


/************************** <enumeration_constant> *****************************
;
; Syntax:
;
;	<enumeration_constant> ::= <identifier>
;
;
*/
STATE(enumeration_constant)
    MARK;
    if (identifier()) ACCEPT;
    REJECT;
} /*enumeration_constant*/


/************************** <boolean_constant> ************************
;
; Syntax:
;
;       <boolean_constant> ::= "false" |"true" 
;
;
; Semantic Descriptor:
;
;       +-------+-------+-------+-------+
;       |             4 |     2 |     2 |
;       +---------------+-------+-------+
;       |         32-bit value (0 or 1) |
;       +-------------------------------+
;
*/
STATE(boolean_constant)
    MARK;
	mdp_number_overflow = false;
	mdp_number_unsigned = false;
	mdp_number_bitlen = 32;	//default
    if (mdp_keyword("false")) mdp_number = 0;
    else if (mdp_keyword("true")) mdp_number = 1;
    else REJECT;
#if 0
    push_value(LIT_LOGICAL);
#else
    //temporary, until LOGICAL is fully implemented...
    push_value(LIT_INTEGER);
#endif
    ACCEPT;
} /*boolean_constant*/


/************************** <named_constant> ************************
;
; Syntax:
;
;       <named_constant> ::= "IDOK" | "IDCANCEL" | ... 
;
;
; Semantic Descriptor:
;
;       +-------+-------+-------+-------+
;       |             4 |     1 |     2 |
;       +---------------+-------+-------+
;       |           32-bit value        |
;       +-------------------------------+
;
*/
STATE(named_constant)
    MARK;
    //need a fast out to speed up the parser???  hash it, or switch on 1st char(s)?
    
    // Should switch first on initial char, then use an inner switch on first 3 chars (or
    // on chars 2 through 5??) -- since MSVC does not generate very efficient switch code 
    // for switch cases greater than 255 or so!

{//scope

	//can the following GP-fault if it accesses past NUL char at end of input buffer???
    int c3 = ((((BYTE)mdp_toupper(inp[0]) << 8) | 
                (BYTE)mdp_toupper(inp[1])) << 8) | 
                (BYTE)mdp_toupper(inp[2]);   //1st 3 chars as an uppercase character literal
    
	mdp_number_overflow = false;
	mdp_number_unsigned = false;
	mdp_number_bitlen = 32;	//default

	switch (c3) {

    case 'ALT':
        //from <Wingdi.h>
             if (mdp_keyword("ALTERNATE"))      mdp_number = ALTERNATE;
        else REJECT;
        break;

	case 'ANC':
             if (mdp_keyword("ANCHOR_LEFT"))     mdp_number = 1;
        else if (mdp_keyword("ANCHOR_RIGHT"))    mdp_number = 2;
        else if (mdp_keyword("ANCHOR_TOP"))      mdp_number = 4;
        else if (mdp_keyword("ANCHOR_BOTTOM"))   mdp_number = 8;
        else REJECT;
        break;

    case 'BIT':
        //from "PIGeneral.h"
             if (mdp_keyword("BitmapMode"))     mdp_number = plugInModeBitmap;
        else if (mdp_keyword("BITMAPMODE"))     mdp_number = plugInModeBitmap;
        else REJECT;
        break;

    case 'BLA':
        //from <Wingdi.h>
             if (mdp_keyword("BLACKONWHITE"))   mdp_number = BLACKONWHITE;
        else REJECT;
        break;

    case 'BN_':
        //from <Winuser.h>
             if (mdp_keyword("BN_CLICKED"))     mdp_number = BN_CLICKED;
        else if (mdp_keyword("BN_PAINT"))       mdp_number = BN_PAINT;
        else if (mdp_keyword("BN_HILITE"))      mdp_number = BN_HILITE;
        else if (mdp_keyword("BN_UNHILITE"))    mdp_number = BN_UNHILITE;
        else if (mdp_keyword("BN_DISABLE"))     mdp_number = BN_DISABLE;
        else if (mdp_keyword("BN_DBLCLK"))      mdp_number = BN_DBLCLK;
        else if (mdp_keyword("BN_SETFOCUS"))    mdp_number = BN_SETFOCUS;
        else if (mdp_keyword("BN_KILLFOCUS"))   mdp_number = BN_KILLFOCUS;
        else REJECT;
        break;

	case 'BS_':
        //from <Winuser.h>
             if (mdp_keyword("BS_PUSHBUTTON"))          mdp_number = BS_PUSHBUTTON;
        else if (mdp_keyword("BS_DEFPUSHBUTTON"))       mdp_number = BS_DEFPUSHBUTTON;
        else if (mdp_keyword("BS_CHECKBOX"))            mdp_number = BS_CHECKBOX;
        else if (mdp_keyword("BS_AUTOCHECKBOX"))        mdp_number = BS_AUTOCHECKBOX;
        else if (mdp_keyword("BS_RADIOBUTTON"))         mdp_number = BS_RADIOBUTTON;
        else if (mdp_keyword("BS_3STATE"))              mdp_number = BS_3STATE;
        else if (mdp_keyword("BS_AUTO3STATE"))          mdp_number = BS_AUTO3STATE;
        else if (mdp_keyword("BS_GROUPBOX"))            mdp_number = BS_GROUPBOX;
        else if (mdp_keyword("BS_USERBUTTON"))          mdp_number = BS_USERBUTTON;
        else if (mdp_keyword("BS_AUTORADIOBUTTON"))     mdp_number = BS_AUTORADIOBUTTON;
        else if (mdp_keyword("BS_OWNERDRAW"))           mdp_number = BS_OWNERDRAW;
        else if (mdp_keyword("BS_LEFTTEXT"))            mdp_number = BS_LEFTTEXT;
        else if (mdp_keyword("BS_TEXT"))                mdp_number = BS_TEXT;
        else if (mdp_keyword("BS_ICON"))                mdp_number = BS_ICON;
        else if (mdp_keyword("BS_BITMAP"))              mdp_number = BS_BITMAP;
        else if (mdp_keyword("BS_LEFT"))                mdp_number = BS_LEFT;
        else if (mdp_keyword("BS_RIGHT"))               mdp_number = BS_RIGHT;
        else if (mdp_keyword("BS_CENTER"))              mdp_number = BS_CENTER;
        else if (mdp_keyword("BS_TOP"))                 mdp_number = BS_TOP;
        else if (mdp_keyword("BS_BOTTOM"))              mdp_number = BS_BOTTOM;
        else if (mdp_keyword("BS_VCENTER"))             mdp_number = BS_VCENTER;
        else if (mdp_keyword("BS_PUSHLIKE"))            mdp_number = BS_PUSHLIKE;
        else if (mdp_keyword("BS_MULTILINE"))           mdp_number = BS_MULTILINE;
        else if (mdp_keyword("BS_NOTIFY"))              mdp_number = BS_NOTIFY;
        else if (mdp_keyword("BS_FLAT"))                mdp_number = BS_FLAT;
        else if (mdp_keyword("BS_RIGHTBUTTON"))         mdp_number = BS_RIGHTBUTTON;
        else REJECT;
        break;

    case 'CA_':
        //from "AfhFM.h"
             if (mdp_keyword("CA_NONE"))            mdp_number = CA_NONE;
        else if (mdp_keyword("CA_CANCEL"))          mdp_number = CA_CANCEL;
        else if (mdp_keyword("CA_APPLY"))           mdp_number = CA_APPLY;
        else if (mdp_keyword("CA_PREVIEW"))         mdp_number = CA_PREVIEW;
        else if (mdp_keyword("CA_EDIT"))            mdp_number = CA_EDIT;
        else if (mdp_keyword("CA_ABOUT"))           mdp_number = CA_ABOUT;
        else if (mdp_keyword("CA_RESET"))           mdp_number = CA_RESET;
        else if (mdp_keyword("CA_SIZE"))            mdp_number = CA_SIZE;
        else REJECT;
        break;
     
    case 'CBN':
        //from <Winuser.h>
             if (mdp_keyword("CBN_ERRSPACE"))       mdp_number = CBN_ERRSPACE;
        else if (mdp_keyword("CBN_SELCHANGE"))      mdp_number = CBN_SELCHANGE;
        else if (mdp_keyword("CBN_DBLCLK"))         mdp_number = CBN_DBLCLK;
        else if (mdp_keyword("CBN_SETFOCUS"))       mdp_number = CBN_SETFOCUS;
        else if (mdp_keyword("CBN_KILLFOCUS"))      mdp_number = CBN_KILLFOCUS;
        else if (mdp_keyword("CBN_EDITCHANGE"))     mdp_number = CBN_EDITCHANGE;
        else if (mdp_keyword("CBN_EDITUPDATE"))     mdp_number = CBN_EDITUPDATE;
        else if (mdp_keyword("CBN_DROPDOWN"))       mdp_number = CBN_DROPDOWN;
        else if (mdp_keyword("CBN_CLOSEUP"))        mdp_number = CBN_CLOSEUP;
        else if (mdp_keyword("CBN_SELENDOK"))       mdp_number = CBN_SELENDOK;
        else if (mdp_keyword("CBN_SELENDCANCEL"))   mdp_number = CBN_SELENDCANCEL;
        else REJECT;
        break;

    case 'CCE':
        //from <cderr.h>
             if (mdp_keyword("CCERR_CHOOSECOLORCODES")) mdp_number = CCERR_CHOOSECOLORCODES;
        else REJECT;
        break;

    case 'CC_':
        //from "AfhFM.h"
             if (mdp_keyword("CC_UNUSED"))          mdp_number = CC_UNUSED;
        else if (mdp_keyword("CC_STANDARD"))        mdp_number = CC_STANDARD;
        else if (mdp_keyword("CC_SCROLLBAR"))       mdp_number = CC_SCROLLBAR;
        else if (mdp_keyword("CC_TRACKBAR"))        mdp_number = CC_TRACKBAR;
        else if (mdp_keyword("CC_SPINNER"))         mdp_number = CC_SPINNER;
        else if (mdp_keyword("CC_UPDOWN"))          mdp_number = CC_UPDOWN;
        else if (mdp_keyword("CC_PUSHBUTTON"))      mdp_number = CC_PUSHBUTTON;
        else if (mdp_keyword("CC_CHECKBOX"))        mdp_number = CC_CHECKBOX;
        else if (mdp_keyword("CC_RADIOBUTTON"))     mdp_number = CC_RADIOBUTTON;
        else if (mdp_keyword("CC_GROUPBOX"))        mdp_number = CC_GROUPBOX;
        else if (mdp_keyword("CC_OWNERDRAW"))       mdp_number = CC_OWNERDRAW;
        else if (mdp_keyword("CC_LISTBOX"))         mdp_number = CC_LISTBOX;
        else if (mdp_keyword("CC_COMBOBOX"))        mdp_number = CC_COMBOBOX;
        else if (mdp_keyword("CC_SLIDER"))          mdp_number = CC_SLIDER;
        else if (mdp_keyword("CC_PROGRESSBAR"))     mdp_number = CC_PROGRESSBAR;
        else if (mdp_keyword("CC_EDIT"))            mdp_number = CC_EDIT;
        else if (mdp_keyword("CC_STATICTEXT"))      mdp_number = CC_STATICTEXT;
        else if (mdp_keyword("CC_FRAME"))           mdp_number = CC_FRAME;
        else if (mdp_keyword("CC_RECT"))            mdp_number = CC_RECT;
        else if (mdp_keyword("CC_IMAGE"))           mdp_number = CC_IMAGE;
        else if (mdp_keyword("CC_BITMAP"))          mdp_number = CC_BITMAP;
        else if (mdp_keyword("CC_ICON"))            mdp_number = CC_ICON;
        else if (mdp_keyword("CC_TOOLTIP"))         mdp_number = CC_TOOLTIP;
        else if (mdp_keyword("CC_ANIMATION"))       mdp_number = CC_ANIMATION;
        else if (mdp_keyword("CC_PREVIEW"))         mdp_number = CC_PREVIEW;
        else if (mdp_keyword("CC_ZOOM"))            mdp_number = CC_ZOOM;
        else if (mdp_keyword("CC_EDIT"))            mdp_number = CC_EDIT;
        else if (mdp_keyword("CC_TAB"))				mdp_number = CC_TAB;
        else REJECT;
        break;
        
    case 'CDE':
        //from <cderr.h>
             if (mdp_keyword("CDERR_DIALOGFAILURE"))    mdp_number = CDERR_DIALOGFAILURE;
        else if (mdp_keyword("CDERR_STRUCTSIZE"))       mdp_number = CDERR_STRUCTSIZE;
        else if (mdp_keyword("CDERR_INITIALIZATION"))   mdp_number = CDERR_INITIALIZATION;
        else if (mdp_keyword("CDERR_NOTEMPLATE"))       mdp_number = CDERR_NOTEMPLATE;
        else if (mdp_keyword("CDERR_NOHINSTANCE"))      mdp_number = CDERR_NOHINSTANCE;
        else if (mdp_keyword("CDERR_LOADSTRFAILURE"))   mdp_number = CDERR_LOADSTRFAILURE;
        else if (mdp_keyword("CDERR_FINDRESFAILURE"))   mdp_number = CDERR_FINDRESFAILURE;
        else if (mdp_keyword("CDERR_LOADRESFAILURE"))   mdp_number = CDERR_LOADRESFAILURE;
        else if (mdp_keyword("CDERR_LOCKRESFAILURE"))   mdp_number = CDERR_LOCKRESFAILURE;
        else if (mdp_keyword("CDERR_MEMALLOCFAILURE"))  mdp_number = CDERR_MEMALLOCFAILURE;
        else if (mdp_keyword("CDERR_MEMLOCKFAILURE"))   mdp_number = CDERR_MEMLOCKFAILURE;
        else if (mdp_keyword("CDERR_NOHOOK"))           mdp_number = CDERR_NOHOOK;
        else if (mdp_keyword("CDERR_REGISTERMSGFAIL"))  mdp_number = CDERR_REGISTERMSGFAIL;
        else REJECT;
        break;
        
    case 'CFE':
        //from <cderr.h>
             if (mdp_keyword("CFERR_NOFONTS"))          mdp_number = CFERR_NOFONTS;
        else if (mdp_keyword("CFERR_MAXLESSTHANMIN"))   mdp_number = CFERR_MAXLESSTHANMIN;
        else REJECT;
        break;
        
    case 'CLO':
        //from <time.h>
             if (mdp_keyword("CLOCKS_PER_SEC"))     mdp_number = CLOCKS_PER_SEC;
        else REJECT;
        break;

    case 'CMY':
        //from "PIGeneral.h"
             if (mdp_keyword("CMYKMode"))           mdp_number = plugInModeCMYKColor;
        else if (mdp_keyword("CMYKMODE"))           mdp_number = plugInModeCMYKColor;
		//Added by Harald Heim, December 6, 2002
        else if (mdp_keyword("CMYK64Mode"))         mdp_number = plugInModeCMYK64;
        else if (mdp_keyword("CMYK64MODE"))         mdp_number = plugInModeCMYK64;
        else REJECT;
        break;

    case 'COL':
        //from <Wingdi.h>
              if (mdp_keyword("COLORONCOLOR"))              mdp_number = COLORONCOLOR;
		//from  Winuser.h
		else if (mdp_keyword("COLOR_SCROLLBAR"))            mdp_number = COLOR_SCROLLBAR;
		else if (mdp_keyword("COLOR_BACKGROUND"))           mdp_number = COLOR_BACKGROUND;
		else if (mdp_keyword("COLOR_ACTIVECAPTION"))        mdp_number = COLOR_ACTIVECAPTION;
		else if (mdp_keyword("COLOR_INACTIVECAPTION"))      mdp_number = COLOR_INACTIVECAPTION;
		else if (mdp_keyword("COLOR_MENU"))                 mdp_number = COLOR_MENU;
		else if (mdp_keyword("COLOR_WINDOW"))               mdp_number = COLOR_WINDOW;
		else if (mdp_keyword("COLOR_WINDOWFRAME"))          mdp_number = COLOR_WINDOWFRAME;
		else if (mdp_keyword("COLOR_MENUTEXT"))             mdp_number = COLOR_MENUTEXT;
		else if (mdp_keyword("COLOR_WINDOWTEXT"))           mdp_number = COLOR_WINDOWTEXT;
		else if (mdp_keyword("COLOR_CAPTIONTEXT"))          mdp_number = COLOR_CAPTIONTEXT;
		else if (mdp_keyword("COLOR_ACTIVEBORDER"))         mdp_number = COLOR_ACTIVEBORDER;
		else if (mdp_keyword("COLOR_INACTIVEBORDER"))       mdp_number = COLOR_INACTIVEBORDER;
		else if (mdp_keyword("COLOR_APPWORKSPACE"))         mdp_number = COLOR_APPWORKSPACE;
		else if (mdp_keyword("COLOR_HIGHLIGHT"))            mdp_number = COLOR_HIGHLIGHT;
		else if (mdp_keyword("COLOR_HIGHLIGHTTEXT"))        mdp_number = COLOR_HIGHLIGHTTEXT;
		else if (mdp_keyword("COLOR_BTNFACE"))              mdp_number = COLOR_BTNFACE;
		else if (mdp_keyword("COLOR_BTNSHADOW"))            mdp_number = COLOR_BTNSHADOW;
		else if (mdp_keyword("COLOR_GRAYTEXT"))             mdp_number = COLOR_GRAYTEXT;
		else if (mdp_keyword("COLOR_BTNTEXT"))              mdp_number = COLOR_BTNTEXT;
		else if (mdp_keyword("COLOR_INACTIVECAPTIONTEXT"))  mdp_number = COLOR_INACTIVECAPTIONTEXT;
		else if (mdp_keyword("COLOR_BTNHIGHLIGHT"))         mdp_number = COLOR_BTNHIGHLIGHT;
		else if (mdp_keyword("COLOR_3DDKSHADOW"))           mdp_number = COLOR_3DDKSHADOW;
		else if (mdp_keyword("COLOR_3DLIGHT"))              mdp_number = COLOR_3DLIGHT;
		else if (mdp_keyword("COLOR_INFOTEXT"))             mdp_number = COLOR_INFOTEXT;
		else if (mdp_keyword("COLOR_INFOBK"))               mdp_number = COLOR_INFOBK;
		else if (mdp_keyword("COLOR_DESKTOP"))              mdp_number = COLOR_DESKTOP;
		else if (mdp_keyword("COLOR_3DFACE "))              mdp_number = COLOR_3DFACE;
		else if (mdp_keyword("COLOR_3DSHADOW"))             mdp_number = COLOR_3DSHADOW;
		else if (mdp_keyword("COLOR_3DHIGHLIGHT"))          mdp_number = COLOR_3DHIGHLIGHT;
		else if (mdp_keyword("COLOR_3DHILIGHT"))            mdp_number = COLOR_3DHILIGHT;
		else if (mdp_keyword("COLOR_BTNHILIGHT"))           mdp_number = COLOR_BTNHILIGHT;
        else REJECT;
        break;		
	
case 'CSI':
             if (mdp_keyword("CSIDL_DESKTOP"))			mdp_number = 0;
		else if (mdp_keyword("CSIDL_INTERNET"))			mdp_number = 1;
		else if (mdp_keyword("CSIDL_PROGRAMS"))			mdp_number = 2;
		else if (mdp_keyword("CSIDL_CONTROLS"))			mdp_number = 3;
		else if (mdp_keyword("CSIDL_PRINTERS"))			mdp_number = 4;
		else if (mdp_keyword("CSIDL_PERSONAL"))			mdp_number = 5;
		else if (mdp_keyword("CSIDL_FAVORITES"))		mdp_number = 6;
		else if (mdp_keyword("CSIDL_STARTUP"))			mdp_number = 7;
		else if (mdp_keyword("CSIDL_RECENT"))			mdp_number = 8;
		else if (mdp_keyword("CSIDL_SENDTO"))			mdp_number = 9;
		else if (mdp_keyword("CSIDL_BITBUCKET"))		mdp_number = 10;

		else if (mdp_keyword("CSIDL_STARTMENU"))		mdp_number = 11;

		else if (mdp_keyword("CSIDL_MYDOCUMENTS"))		mdp_number = 12;
		else if (mdp_keyword("CSIDL_MYMUSIC"))			mdp_number = 13;
		else if (mdp_keyword("CSIDL_MYVIDEO"))			mdp_number = 14;
		
		else if (mdp_keyword("CSIDL_DESKTOPDIRECTORY"))	mdp_number = 16;
		else if (mdp_keyword("CSIDL_DRIVES"))			mdp_number = 17;
		else if (mdp_keyword("CSIDL_NETWORK"))			mdp_number = 18;
		else if (mdp_keyword("CSIDL_NETHOOD"))			mdp_number = 19;
		else if (mdp_keyword("CSIDL_FONTS"))			mdp_number = 20;

		else if (mdp_keyword("CSIDL_TEMPLATES"))		mdp_number = 21;
		else if (mdp_keyword("CSIDL_COMMON_STARTMENU"))	mdp_number = 22;
		else if (mdp_keyword("CSIDL_COMMON_PROGRAMS"))	mdp_number = 23;
		else if (mdp_keyword("CSIDL_COMMON_STARTUP"))	mdp_number = 24;
		else if (mdp_keyword("CSIDL_COMMON_DESKTOPDIRECTORY"))	mdp_number = 25;
		else if (mdp_keyword("CSIDL_APPDATA"))			mdp_number = 26;
		else if (mdp_keyword("CSIDL_PRINTHOOD"))		mdp_number = 27;
		else if (mdp_keyword("CSIDL_LOCAL_APPDATA"))	mdp_number = 28;
		else if (mdp_keyword("CSIDL_ALTSTARTUP"))		mdp_number = 29;
		else if (mdp_keyword("CSIDL_COMMON_ALTSTARTUP")) mdp_number = 30;

		else if (mdp_keyword("CSIDL_COMMON_FAVORITES")) mdp_number = 31;
		else if (mdp_keyword("CSIDL_INTERNET_CACHE"))   mdp_number = 32;
		else if (mdp_keyword("CSIDL_COOKIES"))			mdp_number = 33;
		else if (mdp_keyword("CSIDL_HISTORY"))			mdp_number = 34;
		else if (mdp_keyword("CSIDL_COMMON_APPDATA"))   mdp_number = 35;
		else if (mdp_keyword("CSIDL_WINDOWS"))			mdp_number = 36;
		else if (mdp_keyword("CSIDL_SYSTEM"))			mdp_number = 37;
		else if (mdp_keyword("CSIDL_PROGRAM_FILES"))	mdp_number = 38;
		else if (mdp_keyword("CSIDL_MYPICTURES"))		mdp_number = 39;
		else if (mdp_keyword("CSIDL_PROFILE"))			mdp_number = 40;
		
		else if (mdp_keyword("CSIDL_SYSTEMX86"))		mdp_number = 41;
		else if (mdp_keyword("CSIDL_PROGRAM_FILESX86"))			mdp_number = 42;
		else if (mdp_keyword("CSIDL_PROGRAM_FILES_COMMON"))		mdp_number = 43;
		else if (mdp_keyword("CSIDL_PROGRAM_FILES_COMMONX86"))	mdp_number = 44;
		else if (mdp_keyword("CSIDL_COMMON_TEMPLATES"))	mdp_number = 45;
		else if (mdp_keyword("CSIDL_COMMON_DOCUMENTS"))	mdp_number = 46;
		else if (mdp_keyword("CSIDL_COMMON_ADMINTOOLS"))	mdp_number = 47;
		else if (mdp_keyword("CSIDL_ADMINTOOLS"))		mdp_number = 48;
		else if (mdp_keyword("CSIDL_CONNECTIONS"))		mdp_number = 49;

		else if (mdp_keyword("CSIDL_COMMON_MUSIC"))		mdp_number = 53;
		else if (mdp_keyword("CSIDL_COMMON_PICTURES"))	mdp_number = 54;
		else if (mdp_keyword("CSIDL_COMMON_VIDEO"))		mdp_number = 55;
		else if (mdp_keyword("CSIDL_RESOURCES"))		mdp_number = 56;
		else if (mdp_keyword("CSIDL_RESOURCES_LOCALIZED"))	mdp_number = 57;
		else if (mdp_keyword("CSIDL_COMMON_OEM_LINKS"))	mdp_number = 58;
		else if (mdp_keyword("CSIDL_CDBURN_AREA"))		mdp_number = 59;

		else if (mdp_keyword("CSIDL_COMPUTERSNEARME"))	mdp_number = 61;
		/*else if (mdp_keyword("CSIDL_PROFILES"))			mdp_number = 62;

		else if (mdp_keyword("CSIDL_FOLDER_MASK"))		mdp_number = 0x00ff;*/
		
        else REJECT;
        break;

    case 'CTL':
        //from "AfhFM.h"
             if (mdp_keyword("CTL_OK"))             mdp_number = CTL_OK;
        else if (mdp_keyword("CTL_CANCEL"))         mdp_number = CTL_CANCEL;
        else if (mdp_keyword("CTL_EDIT"))           mdp_number = CTL_EDIT;
        else if (mdp_keyword("CTL_LOGO"))           mdp_number = CTL_LOGO;
        else if (mdp_keyword("CTL_PREVIEW"))        mdp_number = CTL_PREVIEW;
        else if (mdp_keyword("CTL_PROGRESS"))       mdp_number = CTL_PROGRESS;
        else if (mdp_keyword("CTL_HOST"))           mdp_number = CTL_HOST;
        else if (mdp_keyword("CTL_ZOOM"))           mdp_number = CTL_ZOOM;
        else if (mdp_keyword("CTL_LAST_USER"))      mdp_number = CTL_LAST_USER;
        else if (mdp_keyword("CTL_FRAME"))          mdp_number = CTL_FRAME;
        else if (mdp_keyword("CTL_BACKGROUND"))     mdp_number = CTL_BACKGROUND;
        else REJECT;
        break;
        
    case 'CTP':
        //from "AfhFM.h"
             if (mdp_keyword("CTP_TRACK"))          mdp_number = CTP_TRACK;
        else if (mdp_keyword("CTP_READONLY"))       mdp_number = CTP_READONLY;
        else if (mdp_keyword("CTP_HORZ"))           mdp_number = CTP_HORZ;
        else if (mdp_keyword("CTP_VERT"))           mdp_number = CTP_VERT;
        else if (mdp_keyword("CTP_ORIENT_MASK"))    mdp_number = CTP_ORIENT_MASK;
        else if (mdp_keyword("CTP_TOP"))            mdp_number = CTP_TOP;
        else if (mdp_keyword("CTP_BOTTOM"))         mdp_number = CTP_BOTTOM;
        else if (mdp_keyword("CTP_LEFT"))           mdp_number = CTP_LEFT;
        else if (mdp_keyword("CTP_RIGHT"))          mdp_number = CTP_RIGHT;
        else if (mdp_keyword("CTP_SIDE_MASK"))      mdp_number = CTP_SIDE_MASK;
        else if (mdp_keyword("CTP_EXTENDEDUI"))     mdp_number = CTP_EXTENDEDUI;
        else if (mdp_keyword("CTP_MOUSEOVER"))      mdp_number = CTP_MOUSEOVER;
        else if (mdp_keyword("CTP_CONTEXTMENU"))    mdp_number = CTP_CONTEXTMENU;
        else if (mdp_keyword("CTP_DRAWITEM"))       mdp_number = CTP_DRAWITEM;
        else if (mdp_keyword("CTP_MOUSEMOVE"))      mdp_number = CTP_MOUSEMOVE;
        else if (mdp_keyword("CTP_PREVIEWDRAG"))    mdp_number = CTP_PREVIEWDRAG;
        else REJECT;
        break;

    case 'DEE':
        //from "PIGeneral.h"
		//Added by Harald Heim, December 6, 2002		
		     if (mdp_keyword("DeepMultichannelMode"))   mdp_number = plugInModeDeepMultichannel;
        else if (mdp_keyword("DEEPMULTICHANNELMODE"))   mdp_number = plugInModeDeepMultichannel;
        else REJECT;
        break;
        
    case 'DIM':
        //from "AfhFM.h"
             if (mdp_keyword("DIM_EXACT"))          mdp_number = DIM_EXACT;
        else if (mdp_keyword("DIM_TILED"))          mdp_number = DIM_TILED;
        else if (mdp_keyword("DIM_STRETCHED"))      mdp_number = DIM_STRETCHED;
        else REJECT;
        break;

    case 'DUO':
        //from "PIGeneral.h"
             if (mdp_keyword("DuotoneMode"))        mdp_number = plugInModeDuotone;
        else if (mdp_keyword("DUOTONEMODE"))        mdp_number = plugInModeDuotone;
		else if (mdp_keyword("Duotone16Mode"))		mdp_number = plugInModeDuotone16;
        else if (mdp_keyword("DUOTONE16MODE"))      mdp_number = plugInModeDuotone16;
        else REJECT;
        break;	

	 case 'DDL':
		     if (mdp_keyword("DDL_ARCHIVE"))		mdp_number = DDL_ARCHIVE;
		else if (mdp_keyword("DDL_DIRECTORY"))      mdp_number = DDL_DIRECTORY;
		else if (mdp_keyword("DDL_EXCLUSIVE"))      mdp_number = DDL_EXCLUSIVE;
		else if (mdp_keyword("DDL_DRIVES"))			mdp_number = DDL_DRIVES;
		else if (mdp_keyword("DDL_HIDDEN"))			mdp_number = DDL_HIDDEN;
		else if (mdp_keyword("DDL_READONLY"))		mdp_number = DDL_READONLY;
		else if (mdp_keyword("DDL_READWRITE"))      mdp_number = DDL_READWRITE;
		else if (mdp_keyword("DDL_SYSTEM"))			mdp_number = DDL_SYSTEM ;
		
        else REJECT;
        break;

	case 'E2B':
		//from <errno.h>
			 if (mdp_keyword("E2BIG"))			mdp_number = E2BIG;
		else REJECT;
		break;

	case 'EAC':
		//from <errno.h>
			 if (mdp_keyword("EACCES"))			mdp_number = EACCES;
		else REJECT;
		break;

	case 'EAG':
		//from <errno.h>
			 if (mdp_keyword("EAGAIN"))			mdp_number = EAGAIN;
		else REJECT;
		break;

	case 'EBA':
		//from <errno.h>
			 if (mdp_keyword("EBADF"))			mdp_number = EBADF;
		else REJECT;
		break;

	case 'EBU':
		//from <errno.h>
			 if (mdp_keyword("EBUSY"))			mdp_number = EBUSY;
		else REJECT;
		break;

    case 'ECH':
        //from <errno.h>
             if (mdp_keyword("ECHILD"))			mdp_number = ECHILD;
        else REJECT;
        break;
        
    case 'EDE':
        //from <errno.h>
             if (mdp_keyword("EDEADLK"))		mdp_number = EDEADLK;
		else if (mdp_keyword("EDEADLOCK"))		mdp_number = EDEADLOCK;
        else REJECT;
        break;
        
    case 'EDO':
        //from <errno.h>
             if (mdp_keyword("EDOM"))           mdp_number = EDOM;
        else REJECT;
        break;
        
    case 'EEX':
        //from <errno.h>
             if (mdp_keyword("EEXIST"))			mdp_number = EEXIST;
        else REJECT;
        break;
        
    case 'EFA':
        //from <errno.h>
             if (mdp_keyword("EFAULT"))			mdp_number = EFAULT;
        else REJECT;
        break;
        
    case 'EFB':
        //from <errno.h>
             if (mdp_keyword("EFBIG"))			mdp_number = EFBIG;
        else REJECT;
        break;
        
    case 'EIL':
        //from <errno.h>
             if (mdp_keyword("EILSEQ"))			mdp_number = EILSEQ;
        else REJECT;
        break;
        
    case 'EIN':
        //from <errno.h>
             if (mdp_keyword("EINTR"))			mdp_number = EINTR;
		else if (mdp_keyword("EINVAL"))			mdp_number = EINVAL;
        else REJECT;
        break;
        
    case 'EIO':
        //from <errno.h>
             if (mdp_keyword("EIO"))			mdp_number = EIO;
        else REJECT;
        break;
        
    case 'EIS':
        //from <errno.h>
             if (mdp_keyword("EISDIR"))			mdp_number = EISDIR;
        else REJECT;
        break;
        
    case 'EMF':
        //from <errno.h>
             if (mdp_keyword("EMFILE"))			mdp_number = EMFILE;
        else REJECT;
        break;
        
    case 'EML':
        //from <errno.h>
             if (mdp_keyword("EMLINK"))			mdp_number = EMLINK;
        else REJECT;
        break;
        
    case 'EN_':
        //from <Winuser.h>
             if (mdp_keyword("EN_SETFOCUS"))    mdp_number = EN_SETFOCUS;
        else if (mdp_keyword("EN_KILLFOCUS"))   mdp_number = EN_KILLFOCUS;
        else if (mdp_keyword("EN_CHANGE"))      mdp_number = EN_CHANGE;
        else if (mdp_keyword("EN_UPDATE"))      mdp_number = EN_UPDATE;
        else if (mdp_keyword("EN_ERRSPACE"))    mdp_number = EN_ERRSPACE;
        else if (mdp_keyword("EN_MAXTEXT"))     mdp_number = EN_MAXTEXT;
        else if (mdp_keyword("EN_HSCROLL"))     mdp_number = EN_HSCROLL;
        else if (mdp_keyword("EN_VSCROLL"))     mdp_number = EN_VSCROLL;
        else REJECT;
        break;

    case 'ENA':
        //from <errno.h>
             if (mdp_keyword("ENAMETOOLONG"))	mdp_number = ENAMETOOLONG;
        else REJECT;
        break;
        
    case 'ENF':
        //from <errno.h>
             if (mdp_keyword("ENFILE"))			mdp_number = ENFILE;
        else REJECT;
        break;
        
    case 'ENO':
        //from <errno.h>
             if (mdp_keyword("ENODEV"))			mdp_number = ENODEV;
		else if (mdp_keyword("ENOENT"))			mdp_number = ENOENT;
		else if (mdp_keyword("ENOEXEC"))		mdp_number = ENOEXEC;
		else if (mdp_keyword("ENOLCK"))			mdp_number = ENOLCK;
		else if (mdp_keyword("ENOMEM"))			mdp_number = ENOMEM;
		else if (mdp_keyword("ENOSPC"))			mdp_number = ENOSPC;
		else if (mdp_keyword("ENOSYS"))			mdp_number = ENOSYS;
		else if (mdp_keyword("ENOTDIR"))		mdp_number = ENOTDIR;
		else if (mdp_keyword("ENOTEMPTY"))		mdp_number = ENOTEMPTY;
		else if (mdp_keyword("ENOTTY"))			mdp_number = ENOTTY;
        else REJECT;
        break;
        
    case 'ENX':
        //from <errno.h>
             if (mdp_keyword("ENXIO"))			mdp_number = ENXIO;
        else REJECT;
        break;
        
    case 'EOK':
        //from <errno.h>
             if (mdp_keyword("EOK"))			mdp_number = 0;	//not actually in errorno.h??
        else REJECT;
        break;
        
    case 'EPE':
        //from <errno.h>
             if (mdp_keyword("EPERM"))			mdp_number = EPERM;
        else REJECT;
        break;
        
    case 'EPI':
        //from <errno.h>
             if (mdp_keyword("EPIPE"))			mdp_number = EPIPE;
        else REJECT;
        break;
        
    case 'ERA':
        //from <errno.h>
             if (mdp_keyword("ERANGE"))			mdp_number = ERANGE;
        else REJECT;
        break;
        
    case 'ERO':
        //from <errno.h>
             if (mdp_keyword("EROFS"))			mdp_number = EROFS;
        else REJECT;
        break;
        
    case 'ERR':
        //from <Winerror.h>
             if (mdp_keyword("ERROR_SUCCESS"))              mdp_number = ERROR_SUCCESS;
        else if (mdp_keyword("ERROR_FILE_NOT_FOUND"))       mdp_number = ERROR_FILE_NOT_FOUND;
        else if (mdp_keyword("ERROR_MORE_DATA"))            mdp_number = ERROR_MORE_DATA;
#if 0
        //NOT defined in Winerror.h!!!
        else if (mdp_keyword("ERROR_TRANSFER_TOO_LONG"))    mdp_number = ERROR_TRANSFER_TOO_LONG;
#endif
        else if (mdp_keyword("ERROR_NO_MORE_ITEMS"))        mdp_number = ERROR_NO_MORE_ITEMS;
        else if (mdp_keyword("ERROR_INVALID_FUNCTION"))     mdp_number = ERROR_INVALID_FUNCTION;
        else if (mdp_keyword("ERROR_INVALID_DATA"))         mdp_number = ERROR_INVALID_DATA;
        else if (mdp_keyword("ERROR_BADDB"))                mdp_number = ERROR_BADDB;
        else if (mdp_keyword("ERROR_CANTOPEN"))             mdp_number = ERROR_CANTOPEN;
        else if (mdp_keyword("ERROR_CANTREAD"))             mdp_number = ERROR_CANTREAD;
        else if (mdp_keyword("ERROR_CANTWRITE"))            mdp_number = ERROR_CANTWRITE;
        else if (mdp_keyword("ERROR_REGISTRY_CORRUPT"))     mdp_number = ERROR_REGISTRY_CORRUPT;
        else if (mdp_keyword("ERROR_REGISTRY_IO_FAILED"))   mdp_number = ERROR_REGISTRY_IO_FAILED;
        else if (mdp_keyword("ERROR_KEY_DELETED"))          mdp_number = ERROR_KEY_DELETED;
        else if (mdp_keyword("ERROR_KEY_HAS_CHILDREN"))     mdp_number = ERROR_KEY_HAS_CHILDREN;
        else if (mdp_keyword("ERROR_BADKEY"))               mdp_number = ERROR_BADKEY;
        else if (mdp_keyword("ERROR_PATH_NOT_FOUND"))       mdp_number = ERROR_PATH_NOT_FOUND;
        else REJECT;
        break;

    case 'ES_':
        //from <Winuser.h>
             if (mdp_keyword("ES_LEFT"))                    mdp_number = ES_LEFT;
        else if (mdp_keyword("ES_CENTER"))                  mdp_number = ES_CENTER;
        else if (mdp_keyword("ES_RIGHT"))                   mdp_number = ES_RIGHT;
        else if (mdp_keyword("ES_MULTILINE"))               mdp_number = ES_MULTILINE;
        else if (mdp_keyword("ES_UPPERCASE"))               mdp_number = ES_UPPERCASE;
        else if (mdp_keyword("ES_LOWERCASE"))               mdp_number = ES_LOWERCASE;
        else if (mdp_keyword("ES_PASSWORD"))                mdp_number = ES_PASSWORD;
        else if (mdp_keyword("ES_AUTOVSCROLL"))             mdp_number = ES_AUTOVSCROLL;
        else if (mdp_keyword("ES_AUTOHSCROLL"))             mdp_number = ES_AUTOHSCROLL;
        else if (mdp_keyword("ES_NOHIDESEL"))               mdp_number = ES_NOHIDESEL;
        else if (mdp_keyword("ES_OEMCONVERT"))              mdp_number = ES_OEMCONVERT;
        else if (mdp_keyword("ES_READONLY"))                mdp_number = ES_READONLY;
        else if (mdp_keyword("ES_PASSWORD"))                mdp_number = ES_PASSWORD;
        else if (mdp_keyword("ES_WANTRETURN"))              mdp_number = ES_WANTRETURN;
        else if (mdp_keyword("ES_NUMBER"))                  mdp_number = ES_NUMBER;
        else REJECT;
        break;

    case 'ESP':
        //from <errno.h>
             if (mdp_keyword("ESPIPE"))			mdp_number = ESPIPE;
        else REJECT;
        break;
        
    case 'ESR':
        //from <errno.h>
             if (mdp_keyword("ESRCH"))			mdp_number = ESRCH;
        else REJECT;
        break;
        
    case 'EXD':
        //from <errno.h>
             if (mdp_keyword("EXDEV"))			mdp_number = EXDEV;
        else REJECT;
        break;
        
    case 'EXI':
        //from <stdlib.h>
             if (mdp_keyword("EXIT_SUCCESS"))   mdp_number = EXIT_SUCCESS;
        else if (mdp_keyword("EXIT_FAILURE"))   mdp_number = EXIT_FAILURE;
        else REJECT;
        break;
        
    case 'FIL':
        //from <WINNT.H>
             if (mdp_keyword("FILE_ATTRIBUTE_ARCHIVE"))     mdp_number = FILE_ATTRIBUTE_ARCHIVE;
        else if (mdp_keyword("FILE_ATTRIBUTE_COMPRESSED"))  mdp_number = FILE_ATTRIBUTE_COMPRESSED;
        else if (mdp_keyword("FILE_ATTRIBUTE_DIRECTORY"))   mdp_number = FILE_ATTRIBUTE_DIRECTORY;
        else if (mdp_keyword("FILE_ATTRIBUTE_HIDDEN"))      mdp_number = FILE_ATTRIBUTE_HIDDEN;
        else if (mdp_keyword("FILE_ATTRIBUTE_NORMAL"))      mdp_number = FILE_ATTRIBUTE_NORMAL;
        else if (mdp_keyword("FILE_ATTRIBUTE_OFFLINE"))     mdp_number = FILE_ATTRIBUTE_OFFLINE;
        else if (mdp_keyword("FILE_ATTRIBUTE_READONLY"))    mdp_number = FILE_ATTRIBUTE_READONLY;
        else if (mdp_keyword("FILE_ATTRIBUTE_SYSTEM"))      mdp_number = FILE_ATTRIBUTE_SYSTEM;
        else if (mdp_keyword("FILE_ATTRIBUTE_TEMPORARY"))   mdp_number = FILE_ATTRIBUTE_TEMPORARY;
        else REJECT;
        break;

    case 'FME':
        //from "AfhFM.h"
             if (mdp_keyword("FME_UNKNOWN"))        mdp_number = FME_UNKNOWN;
        else if (mdp_keyword("FME_ZERODIVIDE"))     mdp_number = FME_ZERODIVIDE;
        else if (mdp_keyword("FME_DIVIDEOVERFLOW")) mdp_number = FME_DIVIDEOVERFLOW;
        else if (mdp_keyword("FME_CLICKED"))        mdp_number = FME_CLICKED;
        else if (mdp_keyword("FME_DBLCLK"))         mdp_number = FME_DBLCLK;
        else if (mdp_keyword("FME_PAGEUP"))         mdp_number = FME_PAGEUP;
        else if (mdp_keyword("FME_PAGEDOWN"))       mdp_number = FME_PAGEDOWN;
        else if (mdp_keyword("FME_LINEUP"))         mdp_number = FME_LINEUP;
        else if (mdp_keyword("FME_LINEDOWN"))       mdp_number = FME_LINEDOWN;
        else if (mdp_keyword("FME_MOUSEOVER"))      mdp_number = FME_MOUSEOVER;
        else if (mdp_keyword("FME_MOUSEOUT"))       mdp_number = FME_MOUSEOUT;
		else if (mdp_keyword("FME_MOUSEMOVE"))				mdp_number = FME_MOUSEMOVE;
		else if (mdp_keyword("FME_LEFTCLICKED_DOWN"))       mdp_number = FME_LEFTCLICKED_DOWN;
		else if (mdp_keyword("FME_LEFTCLICKED_UP"))			mdp_number = FME_LEFTCLICKED_UP;
		else if (mdp_keyword("FME_RIGHTCLICKED_DOWN"))      mdp_number = FME_RIGHTCLICKED_DOWN;
		else if (mdp_keyword("FME_RIGHTCLICKED_UP"))		mdp_number = FME_RIGHTCLICKED_UP;
		else if (mdp_keyword("FME_TIMER"))					mdp_number = FME_TIMER;
		else if (mdp_keyword("FME_ZOOMCHANGED"))			mdp_number = FME_ZOOMCHANGED;
		else if (mdp_keyword("FME_KEYDOWN"))				mdp_number = FME_KEYDOWN;
		else if (mdp_keyword("FME_KEYUP"))					mdp_number = FME_KEYUP;
        else if (mdp_keyword("FME_CHANGED"))				mdp_number = FME_CHANGED;
		else if (mdp_keyword("FME_VALUECHANGED"))			mdp_number = FME_CHANGED;
        else if (mdp_keyword("FME_SIZE"))			        mdp_number = FME_SIZE;
        else if (mdp_keyword("FME_ENTERSIZE"))			    mdp_number = FME_ENTERSIZE;
        else if (mdp_keyword("FME_EXITSIZE"))			    mdp_number = FME_EXITSIZE;
        else if (mdp_keyword("FME_DRAWITEM"))			    mdp_number = FME_DRAWITEM;
        else if (mdp_keyword("FME_CUSTOMEVENT"))			mdp_number = FME_CUSTOMEVENT;
        else if (mdp_keyword("FME_COMBO_DROPDOWN"))			mdp_number = FME_COMBO_DROPDOWN;
        else if (mdp_keyword("FME_COMBO_CLOSEUP"))			mdp_number = FME_COMBO_CLOSEUP;
        else if (mdp_keyword("FME_CONTEXTMENU"))			mdp_number = FME_CONTEXTMENU;
        else if (mdp_keyword("FME_SETFOCUS"))				mdp_number = FME_SETFOCUS;
		else if (mdp_keyword("FME_SETEDITFOCUS"))			mdp_number = FME_SETFOCUS;
        else if (mdp_keyword("FME_KILLFOCUS"))				mdp_number = FME_KILLFOCUS;
		else if (mdp_keyword("FME_KILLEDITFOCUS"))			mdp_number = FME_KILLFOCUS;
        else if (mdp_keyword("FME_PREVIEWDRAG"))			mdp_number = FME_PREVIEWDRAG;
        else if (mdp_keyword("FME_CANCEL"))					mdp_number = FME_CANCEL;
		else if (mdp_keyword("FME_INIT"))					mdp_number = FME_INIT;
		else if (mdp_keyword("FME_READSCRIPT"))				mdp_number = FME_READSCRIPT;
        else if (mdp_keyword("FME_WRITESCRIPT"))			mdp_number = FME_WRITESCRIPT;
        else REJECT;
        break;
        
    case 'FNE':
        //from <cderr.h>
             if (mdp_keyword("FNERR_SUBCLASSFAILURE"))      mdp_number = FNERR_SUBCLASSFAILURE;
        else if (mdp_keyword("FNERR_INVALIDFILENAME"))      mdp_number = FNERR_INVALIDFILENAME;
        else if (mdp_keyword("FNERR_BUFFERTOOSMALL"))       mdp_number = FNERR_BUFFERTOOSMALL;
        else REJECT;
        break;

    case 'FRE':
        //from <cderr.h>
             if (mdp_keyword("FRERR_BUFFERLENGTHZERO"))     mdp_number = FRERR_BUFFERLENGTHZERO;
        else REJECT;
        break;
        
    case 'GRA':
        //from "PIGeneral.h"
             if (mdp_keyword("GrayScaleMode"))      mdp_number = plugInModeGrayScale;
        else if (mdp_keyword("GRAYSCALEMODE"))      mdp_number = plugInModeGrayScale;
        else if (mdp_keyword("Gray16Mode"))         mdp_number = plugInModeGray16;
        else if (mdp_keyword("GRAY16MODE"))         mdp_number = plugInModeGray16;
        else REJECT;
        break;

    case 'HAL':
        //from <Wingdi.h>
             if (mdp_keyword("HALFTONE"))           mdp_number = HALFTONE;
        else REJECT;
        break;

    case 'HKE':
        //from <Winreg.h>
             if (mdp_keyword("HKEY_CLASSES_ROOT"))      mdp_number = (INT_PTR)HKEY_CLASSES_ROOT;
        else if (mdp_keyword("HKEY_CURRENT_USER"))      mdp_number = (INT_PTR)HKEY_CURRENT_USER;
        else if (mdp_keyword("HKEY_LOCAL_MACHINE"))     mdp_number = (INT_PTR)HKEY_LOCAL_MACHINE;
        else if (mdp_keyword("HKEY_USERS"))             mdp_number = (INT_PTR)HKEY_USERS;
        else if (mdp_keyword("HKEY_PERFORMANCE_DATA"))  mdp_number = (INT_PTR)HKEY_PERFORMANCE_DATA;
        else if (mdp_keyword("HKEY_CURRENT_CONFIG"))    mdp_number = (INT_PTR)HKEY_CURRENT_CONFIG;
        else if (mdp_keyword("HKEY_DYN_DATA"))          mdp_number = (INT_PTR)HKEY_DYN_DATA;
        else REJECT;
        break;

    case 'HSB':
        //from "PIGeneral.h"
             if (mdp_keyword("HSBMode"))            mdp_number = plugInModeHSBColor;
        else if (mdp_keyword("HSBMODE"))            mdp_number = plugInModeHSBColor;
        else REJECT;
        break;

    case 'HSL':
        //from "PIGeneral.h"
             if (mdp_keyword("HSLMode"))            mdp_number = plugInModeHSLColor;
        else if (mdp_keyword("HSLMODE"))            mdp_number = plugInModeHSLColor;
        else REJECT;
        break;

    case 'IDA':
        //from <Winuser.h>
             if (mdp_keyword("IDABORT"))    mdp_number = IDABORT;
        else REJECT;
        break;

    case 'IDC':
        //from <Winuser.h>
             if (mdp_keyword("IDCANCEL"))   mdp_number = IDCANCEL;
        else if (mdp_keyword("IDCLOSE"))    mdp_number = IDCLOSE;
        //
        //Cursor IDs from <Windows.h>
        //
        else if (mdp_keyword("IDC_APPSTARTING"))        mdp_number = (INT_PTR)IDC_APPSTARTING;
        else if (mdp_keyword("IDC_ARROW"))              mdp_number = (INT_PTR)IDC_ARROW;
        else if (mdp_keyword("IDC_CROSS"))              mdp_number = (INT_PTR)IDC_CROSS;
      //else if (mdp_keyword("IDC_HAND"))               mdp_number = (INT_PTR)IDC_HAND;
        else if (mdp_keyword("IDC_HELP"))               mdp_number = (INT_PTR)IDC_HELP;
        else if (mdp_keyword("IDC_IBEAM"))              mdp_number = (INT_PTR)IDC_IBEAM;
        else if (mdp_keyword("IDC_ICON"))               mdp_number = (INT_PTR)IDC_ICON;
        else if (mdp_keyword("IDC_NO"))                 mdp_number = (INT_PTR)IDC_NO;
        else if (mdp_keyword("IDC_SIZE"))               mdp_number = (INT_PTR)IDC_SIZE;
        else if (mdp_keyword("IDC_SIZEALL"))            mdp_number = (INT_PTR)IDC_SIZEALL;
        else if (mdp_keyword("IDC_SIZENESW"))           mdp_number = (INT_PTR)IDC_SIZENESW;
        else if (mdp_keyword("IDC_SIZENS"))             mdp_number = (INT_PTR)IDC_SIZENS;
        else if (mdp_keyword("IDC_SIZENWSE"))           mdp_number = (INT_PTR)IDC_SIZENWSE;
        else if (mdp_keyword("IDC_SIZEWE"))             mdp_number = (INT_PTR)IDC_SIZEWE;
        else if (mdp_keyword("IDC_UPARROW"))            mdp_number = (INT_PTR)IDC_UPARROW;
        else if (mdp_keyword("IDC_WAIT"))               mdp_number = (INT_PTR)IDC_WAIT;
        else REJECT;
        break;

    case 'IDH':
        //from <Winuser.h>
             if (mdp_keyword("IDHELP"))     mdp_number = IDHELP;
        else REJECT;
        break;

    case 'IDI':
        //from <Winuser.h>
             if (mdp_keyword("IDIGNORE"))   mdp_number = IDIGNORE;
        else REJECT;
        break;

    case 'IDN':
        //from <Winuser.h>
             if (mdp_keyword("IDNO"))       mdp_number = IDNO;
        else REJECT;
        break;

    case 'IDO':
        //from <Winuser.h>
             if (mdp_keyword("IDOK"))       mdp_number = IDOK;
        else REJECT;
        break;

    case 'IDR':
        //from <Winuser.h>
             if (mdp_keyword("IDRETRY"))    mdp_number = IDRETRY;
        else REJECT;
        break;

    case 'IDY':
        //from <Winuser.h>
             if (mdp_keyword("IDYES"))      mdp_number = IDYES;
        else REJECT;
        break;

    case 'IND':
        //from "PIGeneral.h"
             if (mdp_keyword("IndexedColorMode"))   mdp_number = plugInModeIndexedColor;
        else if (mdp_keyword("INDEXEDCOLORMODE"))   mdp_number = plugInModeIndexedColor;
        else REJECT;
        break;

    case 'INF':
        //from <Winbase.h>>
             if (mdp_keyword("INFINITE"))           mdp_number = INFINITE;
        else REJECT;
        break;

    case 'INV':
        //from <Winbase.h>
             if (mdp_keyword("INVALID_HANDLE_VALUE"))   mdp_number = (INT_PTR)INVALID_HANDLE_VALUE;
        else if (mdp_keyword("INVALID_FILE_SIZE"))      mdp_number = (int)INVALID_FILE_SIZE;
        else REJECT;
        break;

    case 'LAB':
        //from "PIGeneral.h"
             if (mdp_keyword("LabMode"))        mdp_number = plugInModeLabColor;
        else if (mdp_keyword("LABMODE"))        mdp_number = plugInModeLabColor;
		//Added by Harald Heim, December 6, 2002
		else if (mdp_keyword("Lab48Mode"))      mdp_number = plugInModeLab48;
        else if (mdp_keyword("LAB48MODE"))      mdp_number = plugInModeLab48;
        else REJECT;
        break;

    case 'LBN':
        //from <Winuser.h>
             if (mdp_keyword("LBN_ERRSPACE"))   mdp_number = LBN_ERRSPACE;
        else if (mdp_keyword("LBN_SELCHANGE"))  mdp_number = LBN_SELCHANGE;
        else if (mdp_keyword("LBN_DBLCLK"))     mdp_number = LBN_DBLCLK;
        else if (mdp_keyword("LBN_SELCANCEL"))  mdp_number = LBN_SELCANCEL;
        else if (mdp_keyword("LBN_SETFOCUS"))   mdp_number = LBN_SETFOCUS;
        else if (mdp_keyword("LBN_KILLFOCUS"))  mdp_number = LBN_KILLFOCUS;
        else REJECT;
        break;

    case 'LBS':
        //from <Winuser.h>
             if (mdp_keyword("LBS_NOTIFY"))             mdp_number = LBS_NOTIFY;
        else if (mdp_keyword("LBS_SORT"))               mdp_number = LBS_SORT;
        else if (mdp_keyword("LBS_NOREDRAW"))           mdp_number = LBS_NOREDRAW;
        else if (mdp_keyword("LBS_MULTIPLESEL"))        mdp_number = LBS_MULTIPLESEL;
        else if (mdp_keyword("LBS_OWNERDRAWFIXED"))     mdp_number = LBS_OWNERDRAWFIXED;
        else if (mdp_keyword("LBS_OWNERDRAWVARIABLE"))  mdp_number = LBS_OWNERDRAWVARIABLE;
        else if (mdp_keyword("LBS_HASSTRINGS"))         mdp_number = LBS_HASSTRINGS;
        else if (mdp_keyword("LBS_USETABSTOPS"))        mdp_number = LBS_USETABSTOPS;
        else if (mdp_keyword("LBS_NOINTEGRALHEIGHT"))   mdp_number = LBS_NOINTEGRALHEIGHT;
        else if (mdp_keyword("LBS_MULTICOLUMN"))        mdp_number = LBS_MULTICOLUMN;
        else if (mdp_keyword("LBS_WANTKEYBOARDINPUT"))  mdp_number = LBS_WANTKEYBOARDINPUT;
        else if (mdp_keyword("LBS_EXTENDEDSEL"))        mdp_number = LBS_EXTENDEDSEL;
        else if (mdp_keyword("LBS_DISABLENOSCROLL"))    mdp_number = LBS_DISABLENOSCROLL;
        else if (mdp_keyword("LBS_NODATA"))             mdp_number = LBS_NODATA;
        else if (mdp_keyword("LBS_NOSEL"))              mdp_number = LBS_NOSEL;
        else if (mdp_keyword("LBS_STANDARD"))           mdp_number = LBS_STANDARD;
        else REJECT;
        break;

    case 'LOC':
        //from <Winnt.h>
             if (mdp_keyword("LOCALE_USER_DEFAULT"))    mdp_number = LOCALE_USER_DEFAULT;
        else if (mdp_keyword("LOCALE_SYSTEM_DEFAULT"))  mdp_number = LOCALE_SYSTEM_DEFAULT;
        //Language from <WinNLS.h>
        else if (mdp_keyword("LOCALE_SABBREVLANGNAME")) mdp_number = LOCALE_SABBREVLANGNAME;
        else if (mdp_keyword("LOCALE_SNATIVELANGNAME")) mdp_number = LOCALE_SNATIVELANGNAME;
        else if (mdp_keyword("LOCALE_SLANGUAGE"))		mdp_number = LOCALE_SLANGUAGE;
        else if (mdp_keyword("LOCALE_SENGLANGUAGE"))	mdp_number = LOCALE_SENGLANGUAGE;
        else if (mdp_keyword("LOCALE_ILANGUAGE"))		mdp_number = LOCALE_ILANGUAGE;
        else if (mdp_keyword("LOCALE_IDEFAULTLANGUAGE")) mdp_number = LOCALE_IDEFAULTLANGUAGE;
        //Country from <WinNLS.h>
        else if (mdp_keyword("LOCALE_SENGCOUNTRY"))		mdp_number = LOCALE_SENGCOUNTRY;
        else REJECT;
        break;

    case 'MAX':
        //from <Wingdi.h>
             if (mdp_keyword("MAXSTRETCHBLTMODE"))      mdp_number = MAXSTRETCHBLTMODE;
        //
        //from "AfhFM.h"
        //
        else if (mdp_keyword("MAX_LABEL_SIZE"))         mdp_number = MAX_LABEL_SIZE;
        else if (mdp_keyword("MAX_TOOLTIP_SIZE"))       mdp_number = MAX_TOOLTIP_SIZE;
        else if (mdp_keyword("MAX_SOURCE_CODE_SIZE"))   mdp_number = MAX_SOURCE_CODE_SIZE;
        else if (mdp_keyword("MAX_DLITS"))              mdp_number = MAX_DLITS;
        else if (mdp_keyword("MAX_LOCALS"))             mdp_number = MAX_LOCALS;
        else if (mdp_keyword("MAX_TEMPS"))              mdp_number = MAX_TEMPS;
        else if (mdp_keyword("MAX_CASE_LABELS"))        mdp_number = MAX_CASE_LABELS;
        else REJECT;
        break;

    case 'MB_':
        //from <Winuser.h>
             if (mdp_keyword("MB_ICONERROR"))           mdp_number = MB_ICONERROR;
        else if (mdp_keyword("MB_ICONQUESTION"))        mdp_number = MB_ICONQUESTION;
        else if (mdp_keyword("MB_ICONWARNING"))         mdp_number = MB_ICONWARNING;
        else if (mdp_keyword("MB_ICONINFORMATION"))     mdp_number = MB_ICONINFORMATION;

        else if (mdp_keyword("MB_OK"))                  mdp_number = MB_OK;
        else if (mdp_keyword("MB_OKCANCEL"))            mdp_number = MB_OKCANCEL;
        else if (mdp_keyword("MB_ABORTRETRYIGNORE"))    mdp_number = MB_ABORTRETRYIGNORE;
        else if (mdp_keyword("MB_YESNOCANCEL"))         mdp_number = MB_YESNOCANCEL;
        else if (mdp_keyword("MB_YESNO"))               mdp_number = MB_YESNO;
        else if (mdp_keyword("MB_RETRYCANCEL"))         mdp_number = MB_RETRYCANCEL;

        else if (mdp_keyword("MB_DEFBUTTON1"))			mdp_number = MB_DEFBUTTON1;
        else if (mdp_keyword("MB_DEFBUTTON2"))			mdp_number = MB_DEFBUTTON2;
        else if (mdp_keyword("MB_DEFBUTTON3"))			mdp_number = MB_DEFBUTTON3;
        else if (mdp_keyword("MB_DEFBUTTON4"))			mdp_number = MB_DEFBUTTON4;

        else if (mdp_keyword("MB_APPLMODAL"))			mdp_number = MB_APPLMODAL;
        else if (mdp_keyword("MB_SYSTEMMODAL"))			mdp_number = MB_SYSTEMMODAL;
        else if (mdp_keyword("MB_TASKMODAL"))			mdp_number = MB_TASKMODAL;
        else if (mdp_keyword("MB_HELP"))				mdp_number = MB_HELP;
        else REJECT;
        break;

    case 'MFS':
        //For InsertMenuItem from <Winuser.h>
             if (mdp_keyword("MFS_CHECKED"))            mdp_number = MFS_CHECKED;
        else if (mdp_keyword("MFS_DEFAULT"))            mdp_number = MFS_DEFAULT;
        else if (mdp_keyword("MFS_DISABLED"))           mdp_number = MFS_DISABLED;
        else if (mdp_keyword("MFS_ENABLED"))            mdp_number = MFS_ENABLED;
        else if (mdp_keyword("MFS_GRAYED"))             mdp_number = MFS_GRAYED;
        else if (mdp_keyword("MFS_HILITE"))             mdp_number = MFS_HILITE;
        else if (mdp_keyword("MFS_UNCHECKED"))          mdp_number = MFS_UNCHECKED;
        else if (mdp_keyword("MFS_UNHILITE"))           mdp_number = MFS_UNHILITE;
        else REJECT;
        break;

    case 'MUL':
        //from "PIGeneral.h"
             if (mdp_keyword("MultichannelMode"))       mdp_number = plugInModeMultichannel;
        else if (mdp_keyword("MULTICHANNELMODE"))       mdp_number = plugInModeMultichannel;
        else REJECT;
        break;

    case 'NUL':
        //from <stdlib.h>
             if (mdp_keyword("NULL"))           mdp_number = (int)0;
        else REJECT;
        break;

    case 'N_A':
        //from "AfhFM.h"
             if (mdp_keyword("N_ARRAYS"))       mdp_number = N_ARRAYS;
        else REJECT;
        break;

    case 'N_C':
        //from "AfhFM.h"
             if (mdp_keyword("N_CELLS"))        mdp_number = N_CELLS;
        else if (mdp_keyword("N_CTLS"))         mdp_number = N_CTLS;
        else REJECT;
        break;

    case 'OFN':
        //from <Commdlg.h>
             if (mdp_keyword("OFN_READONLY"))               mdp_number = OFN_READONLY;
        else if (mdp_keyword("OFN_OVERWRITEPROMPT"))        mdp_number = OFN_OVERWRITEPROMPT;
        else if (mdp_keyword("OFN_HIDEREADONLY"))           mdp_number = OFN_HIDEREADONLY;
        else if (mdp_keyword("OFN_NOCHANGEDIR"))            mdp_number = OFN_NOCHANGEDIR;
        else if (mdp_keyword("OFN_SHOWHELP"))               mdp_number = OFN_SHOWHELP;
      //else if (mdp_keyword("OFN_ENABLEHOOK"))             mdp_number = OFN_ENABLEHOOK;
      //else if (mdp_keyword("OFN_ENABLETEMPLATE"))         mdp_number = OFN_ENABLETEMPLATE;
      //else if (mdp_keyword("OFN_ENABLETEMPLATEHANDLE"))   mdp_number = OFN_ENABLETEMPLATEHANDLE;
        else if (mdp_keyword("OFN_NOVALIDATE"))             mdp_number = OFN_NOVALIDATE;
        else if (mdp_keyword("OFN_ALLOWMULTISELECT"))       mdp_number = OFN_ALLOWMULTISELECT;
        else if (mdp_keyword("OFN_EXTENSIONDIFFERENT"))     mdp_number = OFN_EXTENSIONDIFFERENT;
        else if (mdp_keyword("OFN_PATHMUSTEXIST"))          mdp_number = OFN_PATHMUSTEXIST;
        else if (mdp_keyword("OFN_FILEMUSTEXIST"))          mdp_number = OFN_FILEMUSTEXIST;
        else if (mdp_keyword("OFN_CREATEPROMPT"))           mdp_number = OFN_CREATEPROMPT;
        else if (mdp_keyword("OFN_SHAREAWARE"))             mdp_number = OFN_SHAREAWARE;
        else if (mdp_keyword("OFN_NOREADONLYRETURN"))       mdp_number = OFN_NOREADONLYRETURN;
        else if (mdp_keyword("OFN_NOTESTFILECREATE"))       mdp_number = OFN_NOTESTFILECREATE;
      //else if (mdp_keyword("OFN_NONETWORKBUTTON"))        mdp_number = OFN_NONETWORKBUTTON;
      //else if (mdp_keyword("OFN_NOLONGNAMES"))            mdp_number = OFN_NOLONGNAMES;
        else if (mdp_keyword("OFN_EXPLORER"))               mdp_number = OFN_EXPLORER;
        else if (mdp_keyword("OFN_NODEREFERENCELINKS"))     mdp_number = OFN_NODEREFERENCELINKS;
      //else if (mdp_keyword("OFN_LONGNAMES"))              mdp_number = OFN_LONGNAMES;
        else REJECT;
        break;

    case 'PDE':
        //from <cderr.h>
             if (mdp_keyword("PDERR_SETUPFAILURE"))         mdp_number = PDERR_SETUPFAILURE;
        else if (mdp_keyword("PDERR_PARSEFAILURE"))         mdp_number = PDERR_PARSEFAILURE;
        else if (mdp_keyword("PDERR_RETDEFFAILURE"))        mdp_number = PDERR_RETDEFFAILURE;
        else if (mdp_keyword("PDERR_LOADDRVFAILURE"))       mdp_number = PDERR_LOADDRVFAILURE;
        else if (mdp_keyword("PDERR_GETDEVMODEFAIL"))       mdp_number = PDERR_GETDEVMODEFAIL;
        else if (mdp_keyword("PDERR_INITFAILURE"))          mdp_number = PDERR_INITFAILURE;
        else if (mdp_keyword("PDERR_NODEVICES"))            mdp_number = PDERR_NODEVICES;
        else if (mdp_keyword("PDERR_NODEFAULTPRN"))         mdp_number = PDERR_NODEFAULTPRN;
        else if (mdp_keyword("PDERR_DNDMMISMATCH"))         mdp_number = PDERR_DNDMMISMATCH;
        else if (mdp_keyword("PDERR_CREATEICFAILURE"))      mdp_number = PDERR_CREATEICFAILURE;
        else if (mdp_keyword("PDERR_PRINTERNOTFOUND"))      mdp_number = PDERR_PRINTERNOTFOUND;
        else if (mdp_keyword("PDERR_DEFAULTDIFFERENT"))     mdp_number = PDERR_DEFAULTDIFFERENT;
   		else REJECT;
        break;

    case 'PS_':
		     if (mdp_keyword("PS_ENDCAP_ROUND"))		mdp_number = PS_ENDCAP_ROUND;
		else if (mdp_keyword("PS_ENDCAP_SQUARE"))		mdp_number = PS_ENDCAP_SQUARE;
		else if (mdp_keyword("PS_ENDCAP_FLAT"))			mdp_number = PS_ENDCAP_FLAT;
		else if (mdp_keyword("PS_JOIN_ROUND"))			mdp_number = PS_JOIN_ROUND;
		else if (mdp_keyword("PS_JOIN_BEVEL"))			mdp_number = PS_JOIN_BEVEL;
		else if (mdp_keyword("PS_JOIN_MITER"))			mdp_number = PS_JOIN_MITER;
        else REJECT;
        break;

    case 'RAN':
        //from <stdlib.h>
             if (mdp_keyword("RAND_MAX"))       mdp_number = RAND_MAX;
        else REJECT;
        break;

    case 'REG':
        //from <Winnt.h>
             if (mdp_keyword("REG_NONE"))                       mdp_number = REG_NONE;
        else if (mdp_keyword("REG_SZ"))                         mdp_number = REG_SZ;
        else if (mdp_keyword("REG_EXPAND_SZ"))                  mdp_number = REG_EXPAND_SZ;
        else if (mdp_keyword("REG_BINARY"))                     mdp_number = REG_BINARY;
        else if (mdp_keyword("REG_DWORD"))                      mdp_number = REG_DWORD;
        else if (mdp_keyword("REG_DWORD_LITTLE_ENDIAN"))        mdp_number = REG_DWORD_LITTLE_ENDIAN;
        else if (mdp_keyword("REG_DWORD_BIG_ENDIAN"))           mdp_number = REG_DWORD_BIG_ENDIAN;
        else if (mdp_keyword("REG_LINK"))                       mdp_number = REG_LINK;
        else if (mdp_keyword("REG_MULTI_SZ"))                   mdp_number = REG_MULTI_SZ;
        else if (mdp_keyword("REG_RESOURCE_LIST"))              mdp_number = REG_RESOURCE_LIST;
        else if (mdp_keyword("REG_FULL_RESOURCE_DESCRIPTOR"))   mdp_number = REG_FULL_RESOURCE_DESCRIPTOR;
        else if (mdp_keyword("REG_RESOURCE_REQUIREMENTS_LIST")) mdp_number = REG_RESOURCE_REQUIREMENTS_LIST;
        else REJECT;
        break;

    case 'RGB':
        //from "PIGeneral.h"
             if (mdp_keyword("RGBMode"))        mdp_number = plugInModeRGBColor;
        else if (mdp_keyword("RGBMODE"))        mdp_number = plugInModeRGBColor;
        else if (mdp_keyword("RGB48Mode"))      mdp_number = plugInModeRGB48;
        else if (mdp_keyword("RGB48MODE"))      mdp_number = plugInModeRGB48;
        else REJECT;
        break;

    case 'RGN':
        //from <wingdi.h>
             if (mdp_keyword("RGN_AND"))        mdp_number = RGN_AND;
        else if (mdp_keyword("RGN_OR"))         mdp_number = RGN_OR;
        else if (mdp_keyword("RGN_XOR"))        mdp_number = RGN_XOR;
        else if (mdp_keyword("RGN_DIFF"))       mdp_number = RGN_DIFF;
        else if (mdp_keyword("RGN_COPY"))       mdp_number = RGN_COPY;
        else if (mdp_keyword("RGN_MIN"))        mdp_number = RGN_MIN;
        else if (mdp_keyword("RGN_MAX"))        mdp_number = RGN_MAX;
        else REJECT;
        break;

    case 'SBS':
        //from <Winuser.h>
             if (mdp_keyword("SBS_HORZ"))                       mdp_number = SBS_HORZ;
        else if (mdp_keyword("SBS_VERT"))                       mdp_number = SBS_VERT;
        else if (mdp_keyword("SBS_TOPALIGN"))                   mdp_number = SBS_TOPALIGN;
        else if (mdp_keyword("SBS_LEFTALIGN"))                  mdp_number = SBS_LEFTALIGN;
        else if (mdp_keyword("SBS_BOTTOMALIGN"))                mdp_number = SBS_BOTTOMALIGN;
        else if (mdp_keyword("SBS_RIGHTALIGN"))                 mdp_number = SBS_RIGHTALIGN;
        else if (mdp_keyword("SBS_SIZEBOXTOPLEFTALIGN"))        mdp_number = SBS_SIZEBOXTOPLEFTALIGN;
        else if (mdp_keyword("SBS_SIZEBOXBOTTOMRIGHTALIGN"))    mdp_number = SBS_SIZEBOXBOTTOMRIGHTALIGN;
        else if (mdp_keyword("SBS_SIZEBOX"))                    mdp_number = SBS_SIZEBOX;
        else if (mdp_keyword("SBS_SIZEGRIP"))                   mdp_number = SBS_SIZEGRIP;
        else REJECT;
        break;

    case 'SEE':
        //from <Stdio.h>
             if (mdp_keyword("SEEK_CUR"))       mdp_number = SEEK_CUR;
        else if (mdp_keyword("SEEK_END"))       mdp_number = SEEK_END;
        else if (mdp_keyword("SEEK_SET"))       mdp_number = SEEK_SET;
        else REJECT;
        break;

    case 'SIZ':
        //WM_SIZE message values from <Winuser.h>
             if (mdp_keyword("SIZE_MAXHIDE"))           mdp_number = SIZE_MAXHIDE;
        else if (mdp_keyword("SIZE_MAXIMIZED"))         mdp_number = SIZE_MAXIMIZED;
        else if (mdp_keyword("SIZE_MAXSHOW"))           mdp_number = SIZE_MAXSHOW;
        else if (mdp_keyword("SIZE_MINIMIZED"))         mdp_number = SIZE_MINIMIZED;
        else if (mdp_keyword("SIZE_RESTORED"))          mdp_number = SIZE_RESTORED;
        else REJECT;
        break;

    case 'SM_':
        // GetSystemMetrics() code from <winuser.h>
             if (mdp_keyword("SM_CXSCREEN"))            mdp_number = SM_CXSCREEN;
        else if (mdp_keyword("SM_CYSCREEN"))            mdp_number = SM_CYSCREEN;
        else if (mdp_keyword("SM_CXVSCROLL"))           mdp_number = SM_CXVSCROLL;
        else if (mdp_keyword("SM_CYHSCROLL"))           mdp_number = SM_CYHSCROLL;
        else if (mdp_keyword("SM_CYCAPTION"))           mdp_number = SM_CYCAPTION;
        else if (mdp_keyword("SM_CXBORDER"))            mdp_number = SM_CXBORDER;
        else if (mdp_keyword("SM_CYBORDER"))            mdp_number = SM_CYBORDER;
        else if (mdp_keyword("SM_CXDLGFRAME"))          mdp_number = SM_CXDLGFRAME;
        else if (mdp_keyword("SM_CYDLGFRAME"))          mdp_number = SM_CYDLGFRAME;
        else if (mdp_keyword("SM_CYVTHUMB"))            mdp_number = SM_CYVTHUMB;
        else if (mdp_keyword("SM_CXHTHUMB"))            mdp_number = SM_CXHTHUMB;
        else if (mdp_keyword("SM_CXICON"))              mdp_number = SM_CXICON;
        else if (mdp_keyword("SM_CYICON"))              mdp_number = SM_CYICON;
        else if (mdp_keyword("SM_CXCURSOR"))            mdp_number = SM_CXCURSOR;
        else if (mdp_keyword("SM_CYCURSOR"))            mdp_number = SM_CYCURSOR;
        else if (mdp_keyword("SM_CYMENU"))              mdp_number = SM_CYMENU;
        else if (mdp_keyword("SM_CXFULLSCREEN"))        mdp_number = SM_CXFULLSCREEN;
        else if (mdp_keyword("SM_CYFULLSCREEN"))        mdp_number = SM_CYFULLSCREEN;
        else if (mdp_keyword("SM_CYKANJIWINDOW"))       mdp_number = SM_CYKANJIWINDOW;
        else if (mdp_keyword("SM_MOUSEPRESENT"))        mdp_number = SM_MOUSEPRESENT;
        else if (mdp_keyword("SM_CYVSCROLL"))           mdp_number = SM_CYVSCROLL;
        else if (mdp_keyword("SM_CXHSCROLL"))           mdp_number = SM_CXHSCROLL;
        else if (mdp_keyword("SM_DEBUG"))               mdp_number = SM_DEBUG;
        else if (mdp_keyword("SM_SWAPBUTTON"))          mdp_number = SM_SWAPBUTTON;
        else if (mdp_keyword("SM_RESERVED1"))           mdp_number = SM_RESERVED1;
        else if (mdp_keyword("SM_RESERVED2"))           mdp_number = SM_RESERVED2;
        else if (mdp_keyword("SM_RESERVED3"))           mdp_number = SM_RESERVED3;
        else if (mdp_keyword("SM_RESERVED4"))           mdp_number = SM_RESERVED4;
        else if (mdp_keyword("SM_CXMIN"))               mdp_number = SM_CXMIN;
        else if (mdp_keyword("SM_CYMIN"))               mdp_number = SM_CYMIN;
        else if (mdp_keyword("SM_CXSIZE"))              mdp_number = SM_CXSIZE;
        else if (mdp_keyword("SM_CYSIZE"))              mdp_number = SM_CYSIZE;
        else if (mdp_keyword("SM_CXFRAME"))             mdp_number = SM_CXFRAME;
        else if (mdp_keyword("SM_CYFRAME"))             mdp_number = SM_CYFRAME;
        else if (mdp_keyword("SM_CXMINTRACK"))          mdp_number = SM_CXMINTRACK;
        else if (mdp_keyword("SM_CYMINTRACK"))          mdp_number = SM_CYMINTRACK;
        else if (mdp_keyword("SM_CXDOUBLECLK"))         mdp_number = SM_CXDOUBLECLK;
        else if (mdp_keyword("SM_CYDOUBLECLK"))         mdp_number = SM_CYDOUBLECLK;
        else if (mdp_keyword("SM_CXICONSPACING"))       mdp_number = SM_CXICONSPACING;
        else if (mdp_keyword("SM_CYICONSPACING"))       mdp_number = SM_CYICONSPACING;
        else if (mdp_keyword("SM_MENUDROPALIGNMENT"))   mdp_number = SM_MENUDROPALIGNMENT;
        else if (mdp_keyword("SM_PENWINDOWS"))          mdp_number = SM_PENWINDOWS;
        else if (mdp_keyword("SM_DBCSENABLED"))         mdp_number = SM_DBCSENABLED;
        else if (mdp_keyword("SM_CMOUSEBUTTONS"))       mdp_number = SM_CMOUSEBUTTONS;
        else if (mdp_keyword("SM_CXFIXEDFRAME"))        mdp_number = SM_CXFIXEDFRAME;
        else if (mdp_keyword("SM_CYFIXEDFRAME"))        mdp_number = SM_CYFIXEDFRAME;
        else if (mdp_keyword("SM_CXSIZEFRAME"))         mdp_number = SM_CXSIZEFRAME;
        else if (mdp_keyword("SM_CYSIZEFRAME"))         mdp_number = SM_CYSIZEFRAME;
        else if (mdp_keyword("SM_SECURE"))              mdp_number = SM_SECURE;
        else if (mdp_keyword("SM_CXEDGE"))              mdp_number = SM_CXEDGE;
        else if (mdp_keyword("SM_CYEDGE"))              mdp_number = SM_CYEDGE;
        else if (mdp_keyword("SM_CXMINSPACING"))        mdp_number = SM_CXMINSPACING;
        else if (mdp_keyword("SM_CYMINSPACING"))        mdp_number = SM_CYMINSPACING;
        else if (mdp_keyword("SM_CXSMICON"))            mdp_number = SM_CXSMICON;
        else if (mdp_keyword("SM_CYSMICON"))            mdp_number = SM_CYSMICON;
        else if (mdp_keyword("SM_CYSMCAPTION"))         mdp_number = SM_CYSMCAPTION;
        else if (mdp_keyword("SM_CXSMSIZE"))            mdp_number = SM_CXSMSIZE;
        else if (mdp_keyword("SM_CYSMSIZE"))            mdp_number = SM_CYSMSIZE;
        else if (mdp_keyword("SM_CXMENUSIZE"))          mdp_number = SM_CXMENUSIZE;
        else if (mdp_keyword("SM_CYMENUSIZE"))          mdp_number = SM_CYMENUSIZE;
        else if (mdp_keyword("SM_ARRANGE"))             mdp_number = SM_ARRANGE;
        else if (mdp_keyword("SM_CXMINIMIZED"))         mdp_number = SM_CXMINIMIZED;
        else if (mdp_keyword("SM_CYMINIMIZED"))         mdp_number = SM_CYMINIMIZED;
        else if (mdp_keyword("SM_CXMAXTRACK"))          mdp_number = SM_CXMAXTRACK;
        else if (mdp_keyword("SM_CYMAXTRACK"))          mdp_number = SM_CYMAXTRACK;
        else if (mdp_keyword("SM_CXMAXIMIZED"))         mdp_number = SM_CXMAXIMIZED;
        else if (mdp_keyword("SM_CYMAXIMIZED"))         mdp_number = SM_CYMAXIMIZED;
        else if (mdp_keyword("SM_NETWORK"))             mdp_number = SM_NETWORK;
        else if (mdp_keyword("SM_CLEANBOOT"))           mdp_number = SM_CLEANBOOT;
        else if (mdp_keyword("SM_CXDRAG"))              mdp_number = SM_CXDRAG;
        else if (mdp_keyword("SM_CYDRAG"))              mdp_number = SM_CYDRAG;
        else if (mdp_keyword("SM_SHOWSOUNDS"))          mdp_number = SM_SHOWSOUNDS;
        else if (mdp_keyword("SM_CXMENUCHECK"))         mdp_number = SM_CXMENUCHECK;
        else if (mdp_keyword("SM_CYMENUCHECK"))         mdp_number = SM_CYMENUCHECK;
        else if (mdp_keyword("SM_SLOWMACHINE"))         mdp_number = SM_SLOWMACHINE;
        else if (mdp_keyword("SM_MIDEASTENABLED"))      mdp_number = SM_MIDEASTENABLED;
      //else if (mdp_keyword("SM_MOUSEWHEELPRESENT"))   mdp_number = SM_MOUSEWHEELPRESENT;
        else if (mdp_keyword("SM_CMETRICS"))            mdp_number = SM_CMETRICS;
        else REJECT;
        break;

    case 'SS_':
        //from <Winuser.h>
             if (mdp_keyword("SS_LEFT"))                mdp_number = SS_LEFT;
        else if (mdp_keyword("SS_CENTER"))              mdp_number = SS_CENTER;
        else if (mdp_keyword("SS_RIGHT"))               mdp_number = SS_RIGHT;
        else if (mdp_keyword("SS_ICON"))                mdp_number = SS_ICON;
        else if (mdp_keyword("SS_BLACKRECT"))           mdp_number = SS_BLACKRECT;
        else if (mdp_keyword("SS_GRAYRECT"))            mdp_number = SS_GRAYRECT;
        else if (mdp_keyword("SS_WHITERECT"))           mdp_number = SS_WHITERECT;
        else if (mdp_keyword("SS_BLACKFRAME"))          mdp_number = SS_BLACKFRAME;
        else if (mdp_keyword("SS_GRAYFRAME"))           mdp_number = SS_GRAYFRAME;
        else if (mdp_keyword("SS_WHITEFRAME"))          mdp_number = SS_WHITEFRAME;
        else if (mdp_keyword("SS_USERITEM"))            mdp_number = SS_USERITEM;
        else if (mdp_keyword("SS_SIMPLE"))              mdp_number = SS_SIMPLE;
        else if (mdp_keyword("SS_LEFTNOWORDWRAP"))      mdp_number = SS_LEFTNOWORDWRAP;
        else if (mdp_keyword("SS_OWNERDRAW"))           mdp_number = SS_OWNERDRAW;
        else if (mdp_keyword("SS_BITMAP"))              mdp_number = SS_BITMAP;
        else if (mdp_keyword("SS_ENHMETAFILE"))         mdp_number = SS_ENHMETAFILE;
        else if (mdp_keyword("SS_ETCHEDHORZ"))          mdp_number = SS_ETCHEDHORZ;
        else if (mdp_keyword("SS_ETCHEDVERT"))          mdp_number = SS_ETCHEDVERT;
        else if (mdp_keyword("SS_ETCHEDFRAME"))         mdp_number = SS_ETCHEDFRAME;
        else if (mdp_keyword("SS_TYPEMASK"))            mdp_number = SS_TYPEMASK;
        else if (mdp_keyword("SS_NOPREFIX"))            mdp_number = SS_NOPREFIX;
        else if (mdp_keyword("SS_NOTIFY"))              mdp_number = SS_NOTIFY;
        else if (mdp_keyword("SS_CENTERIMAGE"))         mdp_number = SS_CENTERIMAGE;
        else if (mdp_keyword("SS_RIGHTJUST"))           mdp_number = SS_RIGHTJUST;
        else if (mdp_keyword("SS_REALSIZEIMAGE"))       mdp_number = SS_REALSIZEIMAGE;
        else if (mdp_keyword("SS_SUNKEN"))              mdp_number = SS_SUNKEN;
        else if (mdp_keyword("SS_ENDELLIPSIS"))         mdp_number = SS_ENDELLIPSIS;
        else if (mdp_keyword("SS_PATHELLIPSIS"))        mdp_number = SS_PATHELLIPSIS;
        else if (mdp_keyword("SS_WORDELLIPSIS"))        mdp_number = SS_WORDELLIPSIS;
        else if (mdp_keyword("SS_ELLIPSISMASK"))        mdp_number = SS_ELLIPSISMASK;
        else REJECT;
        break;

    case 'STN':
        //from <Winuser.h>
             if (mdp_keyword("STN_CLICKED"))    mdp_number = STN_CLICKED;
        else if (mdp_keyword("STN_DBLCLK"))     mdp_number = STN_DBLCLK;
        else if (mdp_keyword("STN_ENABLE"))     mdp_number = STN_ENABLE;
        else if (mdp_keyword("STN_DISABLE"))    mdp_number = STN_DISABLE;
        else REJECT;
        break;

    case 'STR':
        //from <Wingdi.h>
             if (mdp_keyword("STRETCH_ANDSCANS"))       mdp_number = STRETCH_ANDSCANS;
        else if (mdp_keyword("STRETCH_ORSCANS"))        mdp_number = STRETCH_ORSCANS;
        else if (mdp_keyword("STRETCH_DELETESCANS"))    mdp_number = STRETCH_DELETESCANS;
        else if (mdp_keyword("STRETCH_HALFTONE"))       mdp_number = STRETCH_HALFTONE;
        //from <errno.h>
		else if (mdp_keyword("STRUNCATE"))				mdp_number = STRUNCATE;
        else REJECT;
        break;

    case 'SW_':
        //ShowWindow() commands from <Winuser.h>
             if (mdp_keyword("SW_HIDE"))            mdp_number = SW_HIDE;
        else if (mdp_keyword("SW_MAXIMIZE"))        mdp_number = SW_MAXIMIZE;
        else if (mdp_keyword("SW_MINIMIZE"))        mdp_number = SW_MINIMIZE;
        else if (mdp_keyword("SW_RESTORE"))         mdp_number = SW_RESTORE;
        else if (mdp_keyword("SW_SHOW"))            mdp_number = SW_SHOW;
        else if (mdp_keyword("SW_SHOWDEFAULT"))     mdp_number = SW_SHOWDEFAULT;
        else if (mdp_keyword("SW_SHOWMAXIMIZED"))   mdp_number = SW_SHOWMAXIMIZED;
        else if (mdp_keyword("SW_SHOWMINIMIZED"))   mdp_number = SW_SHOWMINIMIZED;
        else if (mdp_keyword("SW_SHOWMINNOACTIVE")) mdp_number = SW_SHOWMINNOACTIVE;
        else if (mdp_keyword("SW_SHOWNA"))          mdp_number = SW_SHOWNA;
        else if (mdp_keyword("SW_SHOWNOACTIVATE"))  mdp_number = SW_SHOWNOACTIVATE;
        else if (mdp_keyword("SW_SHOWNORMAL"))      mdp_number = SW_SHOWNORMAL;
        else REJECT;
        break;

	case 'TA_':
		     if (mdp_keyword("TA_BASELINE"))		mdp_number = TA_BASELINE;
		else if (mdp_keyword("TA_BOTTOM"))			mdp_number = TA_BOTTOM;
		else if (mdp_keyword("TA_TOP"))				mdp_number = TA_TOP;
		else if (mdp_keyword("TA_CENTER"))			mdp_number = TA_CENTER;
		else if (mdp_keyword("TA_LEFT"))			mdp_number = TA_LEFT;
		else if (mdp_keyword("TA_RIGHT"))			mdp_number = TA_RIGHT;
		else if (mdp_keyword("TA_NOUPDATECP"))		mdp_number = TA_NOUPDATECP;
		else if (mdp_keyword("TA_RTLREADING"))		mdp_number = TA_RTLREADING;
		else if (mdp_keyword("TA_UPDATECP"))		mdp_number = TA_UPDATECP;
        else REJECT;
        break;

    case 'TPM':
        //from <Winuser.h>
			 if (mdp_keyword("TPM_LEFTBUTTON"))     mdp_number = TPM_LEFTBUTTON;
        else if (mdp_keyword("TPM_RIGHTBUTTON"))    mdp_number = TPM_RIGHTBUTTON;
        else if (mdp_keyword("TPM_LEFTALIGN"))      mdp_number = TPM_LEFTALIGN ;
        else if (mdp_keyword("TPM_CENTERALIGN"))    mdp_number = TPM_CENTERALIGN;
        else if (mdp_keyword("TPM_RIGHTALIGN"))     mdp_number = TPM_RIGHTALIGN;
        else if (mdp_keyword("TPM_TOPALIGN"))       mdp_number = TPM_TOPALIGN;
        else if (mdp_keyword("TPM_VCENTERALIGN"))   mdp_number = TPM_VCENTERALIGN;
        else if (mdp_keyword("TPM_BOTTOMALIGN"))    mdp_number = TPM_BOTTOMALIGN;
        else if (mdp_keyword("TPM_HORIZONTAL"))     mdp_number = TPM_HORIZONTAL;
        else if (mdp_keyword("TPM_VERTICAL"))       mdp_number = TPM_VERTICAL;
        else if (mdp_keyword("TPM_NONOTIFY"))       mdp_number = TPM_NONOTIFY;
        else if (mdp_keyword("TPM_RETURNCMD"))      mdp_number = TPM_RETURNCMD;
        else REJECT;
        break;

    case 'TTF':
        //from <Commctrl.h>
        if (mdp_keyword("TTF_CENTERTIP"))          mdp_number = TTF_CENTERTIP;
        else if (mdp_keyword("TTF_RTLREADING"))    mdp_number = TTF_RTLREADING;
        else if (mdp_keyword("TTF_TRACK"))         mdp_number = TTF_TRACK;
        else if (mdp_keyword("TTF_ABSOLUTE"))      mdp_number = TTF_ABSOLUTE;
        else if (mdp_keyword("TTF_TRANSPARENT"))   mdp_number = TTF_TRANSPARENT;
        else REJECT;
        break;

	case 'VK_':
        //Virtual Keys from <Winuser.h>
			 if (mdp_keyword("VK_LBUTTON"))		mdp_number = VK_LBUTTON;
		else if (mdp_keyword("VK_RBUTTON"))     mdp_number = VK_RBUTTON;
		else if (mdp_keyword("VK_CANCEL"))		mdp_number = VK_CANCEL;
		else if (mdp_keyword("VK_MBUTTON"))     mdp_number = VK_MBUTTON;
		else if (mdp_keyword("VK_BACK"))		mdp_number = VK_BACK;
		else if (mdp_keyword("VK_TAB"))			mdp_number = VK_TAB;
		else if (mdp_keyword("VK_CLEAR"))		mdp_number = VK_CLEAR;
		else if (mdp_keyword("VK_RETURN"))		mdp_number = VK_RETURN;
		else if (mdp_keyword("VK_SHIFT"))		mdp_number = VK_SHIFT;
		else if (mdp_keyword("VK_CONTROL"))     mdp_number = VK_CONTROL;
		else if (mdp_keyword("VK_MENU"))		mdp_number = VK_MENU;
		else if (mdp_keyword("VK_PAUSE"))		mdp_number = VK_PAUSE;
		else if (mdp_keyword("VK_CAPITAL"))     mdp_number = VK_CAPITAL;
		else if (mdp_keyword("VK_ESCAPE"))		mdp_number = VK_ESCAPE;
		else if (mdp_keyword("VK_SPACE"))		mdp_number = VK_SPACE;
		else if (mdp_keyword("VK_PRIOR"))		mdp_number = VK_PRIOR;
		else if (mdp_keyword("VK_NEXT"))		mdp_number = VK_NEXT;
		else if (mdp_keyword("VK_END"))			mdp_number = VK_END;
		else if (mdp_keyword("VK_HOME"))		mdp_number = VK_HOME;
		else if (mdp_keyword("VK_LEFT"))		mdp_number = VK_LEFT;
		else if (mdp_keyword("VK_UP"))			mdp_number = VK_UP;
		else if (mdp_keyword("VK_RIGHT"))		mdp_number = VK_RIGHT;
		else if (mdp_keyword("VK_DOWN"))		mdp_number = VK_DOWN;
		else if (mdp_keyword("VK_SELECT"))		mdp_number = VK_SELECT;
		else if (mdp_keyword("VK_PRINT"))		mdp_number = VK_PRINT;
		else if (mdp_keyword("VK_EXECUTE"))     mdp_number = VK_EXECUTE;
		else if (mdp_keyword("VK_SNAPSHOT"))	mdp_number = VK_SNAPSHOT;
		else if (mdp_keyword("VK_INSERT"))		mdp_number = VK_INSERT;
		else if (mdp_keyword("VK_DELETE"))		mdp_number = VK_DELETE;
		else if (mdp_keyword("VK_HELP"))		mdp_number = VK_HELP;
		else if (mdp_keyword("VK_0"))			mdp_number = 0x30;
		else if (mdp_keyword("VK_1"))			mdp_number = 0x31;
		else if (mdp_keyword("VK_2"))			mdp_number = 0x32;
		else if (mdp_keyword("VK_3"))			mdp_number = 0x33;
		else if (mdp_keyword("VK_4"))			mdp_number = 0x34;
		else if (mdp_keyword("VK_5"))			mdp_number = 0x35;
		else if (mdp_keyword("VK_6"))			mdp_number = 0x36;
		else if (mdp_keyword("VK_7"))			mdp_number = 0x37;
		else if (mdp_keyword("VK_8"))			mdp_number = 0x38;
		else if (mdp_keyword("VK_9"))			mdp_number = 0x39;
		else if (mdp_keyword("VK_A"))			mdp_number = 0x41;
		else if (mdp_keyword("VK_B"))			mdp_number = 0x42;
		else if (mdp_keyword("VK_C"))			mdp_number = 0x43;
		else if (mdp_keyword("VK_D"))			mdp_number = 0x44;
		else if (mdp_keyword("VK_E"))			mdp_number = 0x45;
		else if (mdp_keyword("VK_F"))			mdp_number = 0x46;
		else if (mdp_keyword("VK_G"))			mdp_number = 0x47;
		else if (mdp_keyword("VK_H"))			mdp_number = 0x48;
		else if (mdp_keyword("VK_I"))			mdp_number = 0x49;
		else if (mdp_keyword("VK_J"))			mdp_number = 0x4a;
		else if (mdp_keyword("VK_K"))			mdp_number = 0x4b;
		else if (mdp_keyword("VK_L"))			mdp_number = 0x4c;
		else if (mdp_keyword("VK_M"))			mdp_number = 0x4d;
		else if (mdp_keyword("VK_N"))			mdp_number = 0x4e;
		else if (mdp_keyword("VK_O"))			mdp_number = 0x4f;
		else if (mdp_keyword("VK_P"))			mdp_number = 0x50;
		else if (mdp_keyword("VK_Q"))			mdp_number = 0x51;
		else if (mdp_keyword("VK_R"))			mdp_number = 0x52;
		else if (mdp_keyword("VK_S"))			mdp_number = 0x53;
		else if (mdp_keyword("VK_T"))			mdp_number = 0x54;
		else if (mdp_keyword("VK_U"))			mdp_number = 0x55;
		else if (mdp_keyword("VK_V"))			mdp_number = 0x56;
		else if (mdp_keyword("VK_W"))			mdp_number = 0x57;
		else if (mdp_keyword("VK_X"))			mdp_number = 0x58;
		else if (mdp_keyword("VK_Y"))			mdp_number = 0x59;
		else if (mdp_keyword("VK_Z"))			mdp_number = 0x5a;
		else if (mdp_keyword("VK_LWIN"))		mdp_number = VK_LWIN;
		else if (mdp_keyword("VK_RWIN"))		mdp_number = VK_RWIN;
		else if (mdp_keyword("VK_APPS"))		mdp_number = VK_APPS;
		else if (mdp_keyword("VK_NUMPAD0"))     mdp_number = VK_NUMPAD0;
		else if (mdp_keyword("VK_NUMPAD1"))     mdp_number = VK_NUMPAD1;
		else if (mdp_keyword("VK_NUMPAD2"))     mdp_number = VK_NUMPAD2;
		else if (mdp_keyword("VK_NUMPAD3"))     mdp_number = VK_NUMPAD3;
		else if (mdp_keyword("VK_NUMPAD4"))     mdp_number = VK_NUMPAD4;
		else if (mdp_keyword("VK_NUMPAD5"))     mdp_number = VK_NUMPAD5;
		else if (mdp_keyword("VK_NUMPAD6"))     mdp_number = VK_NUMPAD6;
		else if (mdp_keyword("VK_NUMPAD7"))     mdp_number = VK_NUMPAD7;
		else if (mdp_keyword("VK_NUMPAD8"))     mdp_number = VK_NUMPAD8;
		else if (mdp_keyword("VK_NUMPAD9"))     mdp_number = VK_NUMPAD9;
		else if (mdp_keyword("VK_MULTIPLY"))    mdp_number = VK_MULTIPLY;
		else if (mdp_keyword("VK_ADD"))			mdp_number = VK_ADD;
		else if (mdp_keyword("VK_SEPARATOR"))   mdp_number = VK_SEPARATOR;
		else if (mdp_keyword("VK_SUBTRACT"))    mdp_number = VK_SUBTRACT;
		else if (mdp_keyword("VK_DECIMAL"))     mdp_number = VK_DECIMAL;
		else if (mdp_keyword("VK_DIVIDE"))		mdp_number = VK_DIVIDE;
		else if (mdp_keyword("VK_F1"))			mdp_number = VK_F1;
		else if (mdp_keyword("VK_F2"))			mdp_number = VK_F2;
		else if (mdp_keyword("VK_F3"))			mdp_number = VK_F3;
		else if (mdp_keyword("VK_F4"))			mdp_number = VK_F4;
		else if (mdp_keyword("VK_F5"))			mdp_number = VK_F5;
		else if (mdp_keyword("VK_F6"))			mdp_number = VK_F6;
		else if (mdp_keyword("VK_F7"))			mdp_number = VK_F7;
		else if (mdp_keyword("VK_F8"))			mdp_number = VK_F8;
		else if (mdp_keyword("VK_F9"))			mdp_number = VK_F9;
		else if (mdp_keyword("VK_F10"))			mdp_number = VK_F10;
		else if (mdp_keyword("VK_F11"))			mdp_number = VK_F11;
		else if (mdp_keyword("VK_F12"))			mdp_number = VK_F12;
		else if (mdp_keyword("VK_F13"))			mdp_number = VK_F13;
		else if (mdp_keyword("VK_F14"))			mdp_number = VK_F14;
		else if (mdp_keyword("VK_F15"))			mdp_number = VK_F15;
		else if (mdp_keyword("VK_F16"))			mdp_number = VK_F16;
		else if (mdp_keyword("VK_F17"))			mdp_number = VK_F17;
		else if (mdp_keyword("VK_F18"))			mdp_number = VK_F18;
		else if (mdp_keyword("VK_F19"))			mdp_number = VK_F19;
		else if (mdp_keyword("VK_F20"))			mdp_number = VK_F20;
		else if (mdp_keyword("VK_F21"))			mdp_number = VK_F21;
		else if (mdp_keyword("VK_F22"))			mdp_number = VK_F22;
		else if (mdp_keyword("VK_F23"))			mdp_number = VK_F23;
		else if (mdp_keyword("VK_F24"))			mdp_number = VK_F24;
		else if (mdp_keyword("VK_NUMLOCK"))     mdp_number = VK_NUMLOCK;
		else if (mdp_keyword("VK_SCROLL"))		mdp_number = VK_SCROLL;
		else if (mdp_keyword("VK_LSHIFT"))		mdp_number = VK_LSHIFT;
		else if (mdp_keyword("VK_RSHIFT"))		mdp_number = VK_RSHIFT;
		else if (mdp_keyword("VK_LCONTROL"))    mdp_number = VK_LCONTROL;
		else if (mdp_keyword("VK_RCONTROL"))    mdp_number = VK_RCONTROL;
		else if (mdp_keyword("VK_LMENU"))		mdp_number = VK_LMENU;
		else if (mdp_keyword("VK_RMENU"))		mdp_number = VK_RMENU;
		else if (mdp_keyword("VK_PROCESSKEY"))  mdp_number = VK_PROCESSKEY;
		else if (mdp_keyword("VK_ATTN"))		mdp_number = VK_ATTN;
		else if (mdp_keyword("VK_CRSEL"))		mdp_number = VK_CRSEL;
		else if (mdp_keyword("VK_EXSEL"))		mdp_number = VK_EXSEL;
		else if (mdp_keyword("VK_EREOF"))		mdp_number = VK_EREOF;
		else if (mdp_keyword("VK_PLAY"))		mdp_number = VK_PLAY;
		else if (mdp_keyword("VK_ZOOM"))		mdp_number = VK_ZOOM;
		else if (mdp_keyword("VK_NONAME"))		mdp_number = VK_NONAME;
		else if (mdp_keyword("VK_PA1"))			mdp_number = VK_PA1;
		else if (mdp_keyword("VK_OEM_CLEAR"))   mdp_number = VK_OEM_CLEAR;
		else REJECT;
		break;

    case 'VTA':	
        //from <Wingdi.h>
		     if (mdp_keyword("VTA_BASELINE"))	mdp_number = VTA_BASELINE;
		else if (mdp_keyword("VTA_LEFT"))		mdp_number = VTA_LEFT;
		else if (mdp_keyword("VTA_RIGHT"))		mdp_number = VTA_RIGHT;
		else if (mdp_keyword("VTA_CENTER"))		mdp_number = VTA_CENTER;
		else if (mdp_keyword("VTA_BOTTOM"))		mdp_number = VTA_BOTTOM;
		else if (mdp_keyword("VTA_TOP"))		mdp_number = VTA_TOP;
		else REJECT;
		break;

    case 'WAI':
        //from <winbase.h>
             if (mdp_keyword("WAIT_TIMEOUT"))       mdp_number = WAIT_TIMEOUT;
        else if (mdp_keyword("WAIT_FAILED"))        mdp_number = WAIT_FAILED;
        else if (mdp_keyword("WAIT_OBJECT_0"))      mdp_number = WAIT_OBJECT_0;
        else if (mdp_keyword("WAIT_ABANDONED"))     mdp_number = WAIT_ABANDONED;
        else if (mdp_keyword("WAIT_ABANDONED_0"))   mdp_number = WAIT_ABANDONED_0;
        else if (mdp_keyword("WAIT_IO_COMPLETION")) mdp_number = WAIT_IO_COMPLETION;
        else REJECT;
		break;

    case 'WHI':
        //from <Wingdi.h>
             if (mdp_keyword("WHITEONBLACK"))           mdp_number = WHITEONBLACK;
        else REJECT;
        break;

    case 'WIN':
        //from <Wingdi.h>
             if (mdp_keyword("WINDING"))            mdp_number = WINDING;
        else REJECT;
        break;

    case 'WS_':
        //from <Winuser.h>
        /*
         * Window Styles
         */
             if (mdp_keyword("WS_OVERLAPPED"))      mdp_number = WS_OVERLAPPED;
        else if (mdp_keyword("WS_POPUP"))           mdp_number = WS_POPUP;
        else if (mdp_keyword("WS_CHILD"))           mdp_number = WS_CHILD;
        else if (mdp_keyword("WS_MINIMIZE"))        mdp_number = WS_MINIMIZE;
		else if (mdp_keyword("WS_VISIBLE"))         mdp_number = WS_VISIBLE;
		else if (mdp_keyword("WS_DISABLED"))        mdp_number = WS_DISABLED;
		else if (mdp_keyword("WS_CLIPSIBLINGS"))    mdp_number = WS_CLIPSIBLINGS;
        else if (mdp_keyword("WS_CLIPCHILDREN"))    mdp_number = WS_CLIPCHILDREN;
        else if (mdp_keyword("WS_MAXIMIZE"))        mdp_number = WS_MAXIMIZE;
        else if (mdp_keyword("WS_CAPTION"))         mdp_number = WS_CAPTION;
        else if (mdp_keyword("WS_BORDER"))          mdp_number = WS_BORDER;
		else if (mdp_keyword("WS_DLGFRAME"))		mdp_number = WS_DLGFRAME;
        else if (mdp_keyword("WS_VSCROLL"))         mdp_number = WS_VSCROLL;
        else if (mdp_keyword("WS_HSCROLL"))         mdp_number = WS_HSCROLL;
        else if (mdp_keyword("WS_SYSMENU"))         mdp_number = WS_SYSMENU;
        else if (mdp_keyword("WS_THICKFRAME"))      mdp_number = WS_THICKFRAME;
        else if (mdp_keyword("WS_GROUP"))           mdp_number = WS_GROUP;
        else if (mdp_keyword("WS_TABSTOP"))         mdp_number = WS_TABSTOP;
        else if (mdp_keyword("WS_MINIMIZEBOX"))     mdp_number = WS_MINIMIZEBOX;
        else if (mdp_keyword("WS_MAXIMIZEBOX"))     mdp_number = WS_MAXIMIZEBOX;
        else if (mdp_keyword("WS_TILED"))           mdp_number = WS_TILED;
        else if (mdp_keyword("WS_ICONIC"))          mdp_number = WS_ICONIC;
        else if (mdp_keyword("WS_SIZEBOX"))         mdp_number = WS_SIZEBOX;
        else if (mdp_keyword("WS_TILEDWINDOW"))     mdp_number = WS_TILEDWINDOW;
        /*
         * Common Window Styles
         */
        else if (mdp_keyword("WS_OVERLAPPEDWINDOW"))    mdp_number = WS_OVERLAPPEDWINDOW;
        else if (mdp_keyword("WS_POPUPWINDOW"))         mdp_number = WS_POPUPWINDOW;
        else if (mdp_keyword("WS_CHILDWINDOW"))         mdp_number = WS_CHILDWINDOW;
        /*
         * Extended Window Styles
         */
		else if (mdp_keyword("WS_EX_DLGMODALFRAME"))    mdp_number = WS_EX_DLGMODALFRAME;
        else if (mdp_keyword("WS_EX_NOPARENTNOTIFY"))   mdp_number = WS_EX_NOPARENTNOTIFY;
        else if (mdp_keyword("WS_EX_TOPMOST"))          mdp_number = WS_EX_TOPMOST;
		else if (mdp_keyword("WS_EX_ACCEPTFILES"))      mdp_number = WS_EX_ACCEPTFILES;
		else if (mdp_keyword("WS_EX_TRANSPARENT"))      mdp_number = WS_EX_TRANSPARENT;
        else if (mdp_keyword("WS_EX_MDICHILD"))         mdp_number = WS_EX_MDICHILD;
        else if (mdp_keyword("WS_EX_TOOLWINDOW"))       mdp_number = WS_EX_TOOLWINDOW;
        else if (mdp_keyword("WS_EX_WINDOWEDGE"))       mdp_number = WS_EX_WINDOWEDGE;
        else if (mdp_keyword("WS_EX_CLIENTEDGE"))       mdp_number = WS_EX_CLIENTEDGE;
        else if (mdp_keyword("WS_EX_CONTEXTHELP"))      mdp_number = WS_EX_CONTEXTHELP;
		else if (mdp_keyword("WS_EX_RIGHT"))            mdp_number = WS_EX_RIGHT;
        else if (mdp_keyword("WS_EX_LEFT"))             mdp_number = WS_EX_LEFT;
		else if (mdp_keyword("WS_EX_RTLREADING"))       mdp_number = WS_EX_RTLREADING;
		else if (mdp_keyword("WS_EX_LTRREADING"))       mdp_number = WS_EX_LTRREADING;
        else if (mdp_keyword("WS_EX_LEFTSCROLLBAR"))    mdp_number = WS_EX_LEFTSCROLLBAR;
        else if (mdp_keyword("WS_EX_RIGHTSCROLLBAR"))   mdp_number = WS_EX_RIGHTSCROLLBAR;
        else if (mdp_keyword("WS_EX_CONTROLPARENT"))    mdp_number = WS_EX_CONTROLPARENT;
        else if (mdp_keyword("WS_EX_STATICEDGE"))       mdp_number = WS_EX_STATICEDGE;
        else if (mdp_keyword("WS_EX_APPWINDOW"))        mdp_number = WS_EX_APPWINDOW;
		else if (mdp_keyword("WS_EX_OVERLAPPEDWINDOW")) mdp_number = WS_EX_OVERLAPPEDWINDOW;
        else if (mdp_keyword("WS_EX_PALETTEWINDOW"))    mdp_number = WS_EX_PALETTEWINDOW;
        else REJECT;
        break;

    case '_MA':
        //from <stdlib.h>
             if (mdp_keyword("_MAX_PATH"))      mdp_number = _MAX_PATH;
        else if (mdp_keyword("_MAX_DRIVE"))     mdp_number = _MAX_DRIVE;
        else if (mdp_keyword("_MAX_DIR"))       mdp_number = _MAX_DIR;
        else if (mdp_keyword("_MAX_FNAME"))     mdp_number = _MAX_FNAME;
        else if (mdp_keyword("_MAX_EXT"))       mdp_number = _MAX_EXT;
        else REJECT;
        break;

    default:
        REJECT;
    } //switch
}//end scope

    push_value(LIT_INTEGER);
    ACCEPT;
} /*named_constant*/

/************************** <named_floating_constant> ************************
;
; Syntax:
;
;       <named_floating_constant> ::= "M_E" | "M_LOG2E" | ... 
;
;
; Semantic Descriptor:
;
;       +-------+-------+-------+-------+
;       |             4 |     1 |     2 |
;       +---------------+-------+-------+
;       |           32-bit value        |
;       +-------------------------------+
;
*/
STATE(named_floating_constant)
    MARK;
    //need a fast out to speed up the parser???  hash it, or switch on 1st char(s)?
    
    switch (*inp) {		//// ACKK! Worthless switch!!

    case 'M':
        //from <math.h> (http://www.opengroup.org/onlinepubs/009695399/basedefs/math.h.html)
			 if (mdp_keyword("M_E"))        	mdp_real_num = M_E;
        else if (mdp_keyword("M_LOG2E"))        mdp_real_num = M_LOG2E ;
        else if (mdp_keyword("M_LOG10E"))       mdp_real_num = M_LOG10E;
        else if (mdp_keyword("M_LN2"))        	mdp_real_num = M_LN2;
        else if (mdp_keyword("M_LN10"))        	mdp_real_num = M_LN10;
        else if (mdp_keyword("M_PI"))        	mdp_real_num = M_PI;
        else if (mdp_keyword("M_PI_2"))        	mdp_real_num = M_PI_2;
        else if (mdp_keyword("M_PI_4"))        	mdp_real_num = M_PI_4;
        else if (mdp_keyword("M_1_PI"))        	mdp_real_num = M_1_PI;
        else if (mdp_keyword("M_2_PI"))        	mdp_real_num = M_2_PI;
        else if (mdp_keyword("M_2_SQRTPI"))     mdp_real_num = M_2_SQRTPI;
        else if (mdp_keyword("M_SQRT2"))        mdp_real_num = M_SQRT2;
        else if (mdp_keyword("M_SQRT1_2"))      mdp_real_num = M_SQRT1_2;
        else if (mdp_keyword("MAXFLOAT"))       mdp_real_num = MAXFLOAT;
        else REJECT;
        break;

    default:
        REJECT;
    } //switch

	push_real_sd();
    ACCEPT;
} /*named_floating_constant*/	

/*************************** <character_constant> ************************
;
; Syntax:
;
;	<character_constant> ::= ["L"]"'"<c_char_sequence>"'"
;
;	<c_char_sequence> ::= [<c_char>]+
;
;	<c_char> ::=
;		any member of the source character set except the
;		   single quote ('), backslash (\), or new-line character
;		| <escape_sequence>
;
;	<escape_sequence> ::=
;		<simple_escape_sequence> |
;		<octal_escape_sequence> |
;		<hexadecimal_escape_sequence> |
#ifdef NYI
;		<Unicode_escape_sequence>
#endif
;
;	<simple_escape_sequence> ::=
;		"\'" | '\"' | "\?" | "\\" |
;       "\a" | "\b" | "\f" | "\n" | "\r" | "\t" | "\v"
;
;	<octal_escape_sequence> ::=	"\" <octal_digit>(1-3)
;
;	<hexadecimal_escape_sequence> ::= "\x" {<hexadecimal_digit>}+
;
;
#ifdef NYI
;	<Unicode_escape_sequence> ::=
;		<UTF-8_escape_sequence> |
;		<UTF-16_escape_sequence>
;
;	<UTF-8_escape_sequence> ::= "\u" <hexadecimal_digit>(4)
;
;	<UTF-16_escape_sequence> ::= "\U" <hexadecimal_digit>(8)
#endif
;
*/

// QUESTION:  Is \<new-line> allowed to continue a <character_constant>
// (as opposed to a <string_literal>) across a line boundary?? -- Sure,
// it's part of the lexical rules???

void scan_string(char quotechar) {
    char ch;	//signed or unsigned???
    int n;
    while (*inp != quotechar && *inp != 0) {
        if (*inp == '\r' || *inp == '\n') {
            // error -- cr or lf inside string...
            // For now, just let it quietly terminate the string.
            break; //exit from while
        }
        else if (*inp == '\\') {
            // escape sequence...
            inp++;
            switch (*inp) {
            case 0: //eos ERROR!!!
                // For now, just add '\\' to the string and quietly terminate the string.
                add_char_to_acc('\\');
				mdp_warnv("Escape sequence terminated prematurely at end of line %d.\r\n"
						  "Taken to be a literal '\\' character code.",
						  mdp_line_from_inp(inp));
                break;
            case '\r':  //ignore \cr[lf] (line-continuation chars)
                if (inp[1] == '\n') inp++;
                break;
            case '\n':  //ignore \lf (line-continuation chars)
                break;
            case 'a':
                add_char_to_acc('\a');      //alert
                break;
            case 'b':
                add_char_to_acc('\b');      //backspace
                break;
            case 'f':
                add_char_to_acc('\f');      //form feed
                break;
            case 'n':
                add_char_to_acc('\n');      //new line
                break;
            case 'r':
                add_char_to_acc('\r');      //carriage return
                break;
            case 't':
                add_char_to_acc('\t');      //tab
                break;
            case 'v':
                add_char_to_acc('\v');      //vertical tab
                break;
			case 'u':						//UTF-8 escape sequence
			case 'U':						//UTF-16 escape sequence
				mdp_warnv("Unicode escape sequence %.2s in line %d is not yet supported.\r\n"
						  "Taken to be a hex escape sequence '\\x'.",
						  inp - 1, mdp_line_from_inp(inp));
				//fall through to hex escape sequence for now...
            case 'x':	                    //hex escape sequence
                if (mdp_ishexdigit(inp[1])) {
                    inp++;
                    ch = 0;
                    while (mdp_ishexdigit(*inp)) {
						//should we check for overflow (e.g. \x0100)??
                        ch = (ch << 4) | ( mdp_isdigit(*inp) ? *inp - '0' : (*inp & 7) + 9 );
                        inp++;
                    }
                    add_char_to_acc(ch);
                    inp--;
                }
                else {
				   //(MSVC: error C2153 hex constants must have at least one hex digit.)
                   // error, no hex digits after \x -- so just take x literally
#if 0
					mdp_warnv("Hex escape sequence %.2s in line %d\r\n"
							  "must have at least one hex digit.\r\n"
							  "'%.1s' is taken literally as a character code.",
							  inp - 1, mdp_line_from_inp(inp), inp);
#else
					if (FML_VERSION >= VERSION(2,0)) {
						//silent warning if version < 2.0
						mdp_warnv("Hex escape sequence %.2s in line %d\r\n"
								  "must have at least one hex digit.\r\n"
								  "The characters '%.2s' are taken literally.",
								  inp - 1, mdp_line_from_inp(inp), inp - 1);
					}
                    add_char_to_acc('\\');
#endif
                    add_char_to_acc(*inp);
                }
                break;
            case '0':
            case '1':
            case '2':
            case '3':
            case '4':
            case '5':
            case '6':
            case '7':
                // octal escape sequence
                ch = 0;
                n = 0;
                while (n < 3 && mdp_isoctaldigit(*inp)) {
					//should we check for overflow (e.g. \777)??
                    ch = (ch << 3) | (*inp & 7);
                    inp++;
                    n++;
                }
                add_char_to_acc(ch);
                inp--;
                break;
            default:
				//unrecognized escape sequence
#if 0
				mdp_warnv("Unrecognized escape sequence %.2s in line %d.\r\n"
						  "'%.1s' is taken literally as a character code.",
						  inp - 1, mdp_line_from_inp(inp), inp);
#else
				if (FML_VERSION >= VERSION(2,0)) {
					//silent warning if version < 2.0
					mdp_warnv("Unrecognized escape sequence %.2s in line %d.\r\n"
							  "The characters '%.2s' are taken literally.",
							  inp - 1, mdp_line_from_inp(inp), inp - 1);
				}
                add_char_to_acc('\\');
#endif
				//fall through to literal char case...
			case '"':
			case '\'':
			case '\\':
			case '?':
				// just take char literally
                add_char_to_acc(*inp);
                break;
           } /*switch*/
        }
        else {
            // ordinary char
            add_char_to_acc(*inp);
        }
        if (*inp != 0) inp++;
    } /*while*/;

    if (*inp != quotechar) {
        // Error if terminated by cr or lf or eos, but for now
        // we just give a warning.
		mdp_warnv("Quoted string prematurely terminated at end of line %d.",
				  mdp_line_from_inp(inp));
    }
    else {
        inp++;  // Skip over final quote char.
    }
} /*scan_string*/

STATE(character_constant)
    MARK;
    // Error if mdp_string_len > 8 for SBC or > 1 for WC (per MSVC)
    // Convert to an integer literal (big-endian!!!)
    init_string_acc();
    if (*inp == '\'') {
		// narrow-character constant
        inp++;
        scan_string('\'');
		convert_char_const_to_int();
		if (mdp_string_len > 8) {
			mdp_warnv("In line %d, a character constant with more than 8 characters\r\n"
					  "will be truncated to value 0x%I64x",
					  mdp_line_from_inp(inp_save),
					  mdp_number);
		}
    }
    else if (*inp == 'L' && inp[1] == '\'') {
        // wide character constant -- NYI, same as SBC constant for now
        inp += 2;
        scan_string('\'');
	    convert_char_const_to_int();
		if (mdp_string_len > 1) {
			mdp_warnv("In line %d, a wide-character constant with more than 1 character\r\n"
					  "has an implementation-dependent value 0x%I64x",
					  mdp_line_from_inp(inp_save),
					  mdp_number);
		}
		else {
			mdp_warnv("Wide character constant in line %d, not yet supported.\r\n"
					  "Treated as a narrow character constant with value 0x%I64x",
					  mdp_line_from_inp(inp_save),
					  mdp_number);
		}
    } else REJECT;
    if (mdp_string_len == 0) {
        //an empty character constant is not allowed by ANSI C  ///FM EXTENSION!!
        //REJECT;
		mdp_warnv("Empty character constant in line %d.\r\n"
				  "Treated as the value 0.",
				  mdp_line_from_inp(inp_save));
    }
    push_value(LIT_INTEGER);
    ACCEPT;
} /*character_constant*/

/*************************** <string_literal> ************************
;
; Syntax:
;
;	<string_literal> ::= {  '"'[<s_char_sequence>]'"' }+     |
;                        { 'L"'[<s_char_sequence>]'"' }+
;
;	<s_char_sequence> ::= {<s_char>}+
;
;	<s_char>:
;		any member of the source character set except the
;		   double quote ("), backslash (\), or new-line character
;		| <escape_sequence>
;
; Semantic Descriptor:
;
;       +-------+-------+-------+-------+
;       |    # of chars |     4 |     2 |
;       +---------------+-------+-------+
;       |                               |
;       |       chars (padded with      |
;       |       blanks if needed)       |
;       +-------------------------------+
;
*/
STATE(string_literal)
    MARK;
    init_string_acc();
    if (single_string_literal()) {
        // concatenate adjacent strings
        while (single_string_literal())		//warn if wide string encountered??
            ;
    }
    else if (wide_string_literal()) {
        // concatenate adjacent wide strings
        while (wide_string_literal())		//warn if narrow string encountered??
            ;
		mdp_warnv("Wide string literal in line %d, not yet supported.\r\n"
				  "Treated as a narrow string literal.",
				  mdp_line_from_inp(inp_save));
    }
    else REJECT;
	/* Add a terminating NUL */
	if (!add_char_to_acc(0)) {
		// No room for terminator!
		// Force a terminator at end of string accumulator.
		mdp_string_acc[sizeof(mdp_string_acc) - 1] = 0;
		mdp_warnv("String literal too long in line %d:\r\n"
				  "%.20s...\r\n"
				  "Truncated to %d characters.",
				  mdp_line_from_inp(inp_save),
				  inp_save,
				  sizeof(mdp_string_acc) - 1);
	}
    push_char_sd();
    ACCEPT;
} /*string_literal*/

STATE(single_string_literal)
    MARK;
    if (*inp == '\"') {
        inp++;
        scan_string('\"');
        ACCEPT;
    } else REJECT;
} /*single_string_literal*/

STATE(wide_string_literal)
    MARK;
    if (*inp == 'L' && inp[1] == '\"') {			//should WARN???
        inp += 2;
        scan_string('\"');
        ACCEPT;
    } else REJECT;
} /*wide_string_literal*/


// 3.1.5 Operators

/*************************** <any_operator> ************************
;
; Syntax:
;
;	<any_operator> ::=
;		"[" | "]" | "(" | ")" | "." | "->" |
;		"++" | "--" | "&" | "*" | "+" | "-" | "~" | "!" | "sizeof" |
;		"/" | "%" | <power_operator> |
;		"<<" | ">>" | "<" | ">" | "<=" | ">=" | "==" | "!=" |
;		"^" | "|" | "&&" | "||" |
;		"?" | ":" |
;		"=" | "*=" | "/=" | "%=" | "+=" | "-=" |
;		"<<=" | ">>=" | "&=" | "^=" | "|=" |
;		"," | "#" | "##"
;
*/
STATE(indirect_component_selection_op)
    MARK;
    skip_whitespace = FALSE;
    if (mdp_char('-') && mdp_char('>')) ACCEPT; else REJECT;
} /*indirect_component_selection_op*/

STATE(increment_operator)
    MARK;
    skip_whitespace = FALSE;
    if (mdp_char('+') && mdp_char('+')) ACCEPT; else REJECT;
} /*increment_operator*/

STATE(decrement_operator)
    MARK;
    skip_whitespace = FALSE;
    if (mdp_char('-') && mdp_char('-')) ACCEPT; else REJECT;
} /*decrement_operator*/

STATE(left_shift_operator)
    MARK;
    skip_whitespace = FALSE;
    if (mdp_char('<') && mdp_char('<')) op = OP_LSH;
    else REJECT;
    ACCEPT;
} /*left_shift_operator*/

STATE(right_shift_operator)
    MARK;
    skip_whitespace = FALSE;
    if (mdp_char('>') && mdp_char('>')) op = OP_RSH;
    else REJECT;
    ACCEPT;
} /*right_shift_operator*/

STATE(lt_operator)
    MARK;
    if (mdp_char('<')) op = OP_LT;
    else REJECT;
    ACCEPT;
} /*lt_operator*/

STATE(gt_operator)
    MARK;
    if (mdp_char('>')) op = OP_GT;
    else REJECT;
    ACCEPT;
} /*gt_operator*/

STATE(le_operator)
    MARK;
    skip_whitespace = FALSE;
    if (mdp_char('<') && mdp_char('=')) op = OP_LE;
    else REJECT;
    ACCEPT;
} /*le_operator*/

STATE(ge_operator)
    MARK;
    skip_whitespace = FALSE;
    if (mdp_char('>') && mdp_char('=')) op = OP_GE;
    else REJECT;
    ACCEPT;
} /*ge_operator*/

STATE(eq_operator)
    MARK;
    skip_whitespace = FALSE;
    if (mdp_char('=') && mdp_char('=')) op = OP_EQ;
    else REJECT;
    ACCEPT;
} /*eq_operator*/

STATE(ne_operator)
    MARK;
    skip_whitespace = FALSE;
    if (mdp_char('!') && mdp_char('=')) op = OP_NE;
    else REJECT;
    ACCEPT;
} /*ne_operator*/

STATE(logical_and_operator)
    MARK;
    skip_whitespace = FALSE;
    if (mdp_char('&') && mdp_char('&')) op = OP_LOG_AND;
    else REJECT;
    ACCEPT;
} /*logical_and_operator*/

STATE(logical_or_operator)
    MARK;
    skip_whitespace = FALSE;
    if (mdp_char('|') && mdp_char('|')) op = OP_LOG_OR;
    else REJECT;
    ACCEPT;
} /*logical_or_operator*/

STATE(and_operator)
    MARK;
    if (logical_and_operator()) REJECT; /* avoid spurious match */
    if (mdp_char('&')) op = OP_AND;
    else REJECT;
    ACCEPT;
} /*and_operator*/

STATE(or_operator)
    MARK;
    if (logical_or_operator()) REJECT;  /* avoid spurious match */
    if (mdp_char('|')) op = OP_IOR;
    else REJECT;
    ACCEPT;
} /*or_operator*/

STATE(compound_assignment_op)
    MARK;
    skip_whitespace = FALSE;
#ifdef DONT_SYNTHESIZE_COMPOUND_ASSIGNMENTS
    if      (mdp_char('+')) op = OP_ASS_ADD;
    else if (mdp_char('-')) op = OP_ASS_SUB;
    else if (mdp_char('*')) op = OP_ASS_MUL;
    else if (mdp_char('/')) op = OP_ASS_DIV;
    else if (mdp_char('%')) op = OP_ASS_REM;
    else if (mdp_char('&')) op = OP_ASS_AND;
    else if (mdp_char('^')) op = OP_ASS_XOR;
    else if (mdp_char('|')) op = OP_ASS_IOR;
    else if (left_shift_operator()) op = OP_ASS_LSH;
    else if (right_shift_operator()) op = OP_ASS_RSH;
#ifndef NO_POWER_OPERATOR
    else if (power_operator()) op = OP_ASS_POW;
#endif
#else
    /* Synthesize compound assignment operators from */
    /* basic operators...                            */
    if      (mdp_char('+')) op = OP_ADD;
    else if (mdp_char('-')) op = OP_SUB;
    else if (mdp_char('*')) op = OP_MUL;
    else if (mdp_char('/')) op = OP_DIV;
    else if (mdp_char('%')) op = OP_REM;
    else if (mdp_char('&')) op = OP_AND;
    else if (mdp_char('^')) op = OP_XOR;
    else if (mdp_char('|')) op = OP_IOR;
    else if (left_shift_operator()) op = OP_LSH;
    else if (right_shift_operator()) op = OP_RSH;
#ifndef NO_POWER_OPERATOR
    else if (power_operator()) op = OP_POW;
#endif
#endif
    else REJECT;
    if (mdp_char('=')) ACCEPT;
    REJECT;
} /*compound_assignment_op*/

STATE(assignment_operator)
    MARK;
    if (eq_operator()) REJECT;  /* avoid spurious match!!! */
    if (mdp_char('=')) op = OP_STORE;
    else if (!compound_assignment_op()) REJECT;
    ACCEPT;
} /*assignment_operator*/

STATE(concat_operator)
    MARK;
    skip_whitespace = FALSE;
    if (mdp_char('#') && mdp_char('#')) op = OP_CONCAT;
    else REJECT;
    ACCEPT;
} /*concat_operator*/

STATE(any_operator)
    MARK;
    if (indirect_component_selection_op()) ACCEPT;
    if (increment_operator()) ACCEPT;
    if (decrement_operator()) ACCEPT;
    if (left_shift_operator()) ACCEPT;
    if (right_shift_operator()) ACCEPT;
    if (le_operator()) ACCEPT;
    if (ge_operator()) ACCEPT;
    if (eq_operator()) ACCEPT;
    if (ne_operator()) ACCEPT;
    if (logical_and_operator()) ACCEPT;
    if (logical_or_operator()) ACCEPT;
    if (compound_assignment_op()) ACCEPT;
    if (concat_operator()) ACCEPT;
    if (lt_operator()) ACCEPT;
    if (gt_operator()) ACCEPT;
    if (and_operator()) ACCEPT;
    if (or_operator()) ACCEPT;
    if (assignment_operator()) ACCEPT;
#ifndef NO_POWER_OPERATOR
    if (power_operator()) ACCEPT;
#endif
    if (mdp_char('[')) ACCEPT;
    if (mdp_char(']')) ACCEPT;
    if (mdp_char('(')) ACCEPT;
    if (mdp_char(')')) ACCEPT;
    if (mdp_char('.')) ACCEPT;
    if (mdp_char('*')) ACCEPT;
    if (mdp_char('+')) ACCEPT;
    if (mdp_char('-')) ACCEPT;
    if (mdp_char('~')) ACCEPT;
    if (mdp_char('!')) ACCEPT;
    if (mdp_char('/')) ACCEPT;
    if (mdp_char('%')) ACCEPT;
    if (mdp_char('^')) ACCEPT;
    if (mdp_char('?')) ACCEPT;
    if (mdp_char(':')) ACCEPT;
    if (mdp_char(',')) ACCEPT;
    if (mdp_char('#')) ACCEPT;
    REJECT;
} /*any_operator*/


// 3.1.6 Punctuators

/*************************** <punctuator> ************************
;
; Syntax:
;
;	<punctuator> ::=
;		"[" | "]" | "(" | ")" | "{" | "}" | "*" |
;		"," | ":" | "=" | ";" | "..." | "#"
;
*/
STATE(ellipsis)
    MARK;
    skip_whitespace = FALSE;
    if (mdp_char('.') && mdp_char('.') && mdp_char('.')) ACCEPT;
    REJECT;
} /*ellipsis*/

STATE(punctuator)
    MARK;
    if (ellipsis()) ACCEPT;
    if (mdp_char('[')) ACCEPT;
    if (mdp_char(']')) ACCEPT;
    if (mdp_char('(')) ACCEPT;
    if (mdp_char(')')) ACCEPT;
    if (mdp_char('{')) ACCEPT;
    if (mdp_char('}')) ACCEPT;
    if (mdp_char('*')) ACCEPT;
    if (mdp_char(',')) ACCEPT;
    if (mdp_char(':')) ACCEPT;
    if (mdp_char('=')) ACCEPT;
    if (mdp_char(';')) ACCEPT;
    if (mdp_char('#')) ACCEPT;
    REJECT;
} /*punctuator*/


/****************************** <C_language_expression> ***********************/
/*                                                                            */
/*  Reference: ANSI Programming Language - C                                  */
/*             ANSI X3.159-1989                                               */
/*             Section A.2.1 Expressions                                      */
/*                                                                            */
/*----------------------------------------------------------------------------*/

STATE(C_language_expression)
    int kind;
    MARK;
    if (!(kind = expression())) REJECT;
    if (kind == LVALUE) {
        push_op(OP_LOAD);
        kind = RVALUE;
    }
    ACCEPT;
} /*C_language_expression*/

/*************************** <primary_expression> ************************
;
; Syntax:
;
;	<primary_expression> ::=
;		<identifier> |
;		<constant> |
;		<string_literal> |
;		"(" <expression> ")"
;
*/
STATE(primary_expression)
    int kind;
    MARK;
    if (mdp_char('(')) {
        if (!(kind = expression())) REJECT;
        if (!mdp_char(')')) REJECT;
        ACCEPTX(kind);
    }
    else if (constant()) {
        ACCEPTX(RVALUE);
    }
    else if (string_literal()) {
        ACCEPTX(RVALUE);
    }
    else if (identifier()) {
        push_op(OP_LOOKUP);
        ACCEPTX(LVALUE);
    }
    REJECT;
} /*primary_expression*/

/*************************** <postfix_expression> ************************
;
; Syntax:
;
;	<postfix_expression> ::=
;		<primary_expression> |
;		<postfix_expression> "[" <subscript_expression_list> "]" |
;		<postfix_expression> "(" [<argument_expression_list>] ")" |
;		<postfix_expression> "." <identifier> |
;		<postfix_expression> "->" <identifier> |
;		<postfix_expression> "++" |
;		<postfix_expression> "--"
;
;	<subscript_expression_list> ::=
;		<assignment_expression> {"," <assignment_expression>}*
;
;	<argument_expression_list> ::=
;		<assignment_expression> {"," <assignment_expression>}*
;
*/
STATE(postfix_expression)
    int kind;
    int keep_looking;
    MARK;

    if (!(kind = primary_expression())) REJECT;

    keep_looking = TRUE;
    do {
        if (mdp_char('[')) {
            if (kind == LVALUE)
                push_op(OP_LOAD);           /* need RVALUE */
            if (!(kind = expression())) REJECT;
            if (kind == LVALUE)
                push_op(OP_LOAD);           /* need RVALUE */
            if (!mdp_char(']')) REJECT;
            push_op(OP_ADD);
            kind = LVALUE;
        }
        else if (mdp_char('(')) {
            if (kind == LVALUE)
                push_op(OP_LOAD);           /* need RVALUE */
			push_op(OP_ARGLIST);			/* start of argument list */
            if (!argument_expression_list()) {
                // Empty arg list: Nargs = 0;
                push_constant(LIT_INTEGER);
                push_constant(0);
            }
            if (!mdp_char(')')) REJECT;
            push_op(OP_CALL);
            kind = RVALUE;
        }
        else if (mdp_char('.')) {
#if 0
            // for e.name, e must be an lvalue UNLESS e is
            // a function call returning a struct or union,
            // in which case the result of e.name is also
            // an rvalue, not an lvalue
            if (kind != LVALUE) REJECT;     /* must be LVALUE */
#endif
            if (!identifier()) REJECT;
            push_op(OP_MEMBER);
#if 0
            // See note above.  Result is an lvalue if e is an lvalue,
            // but an rvalue if e is a function call returning a
            // struct or union.  Note that f().a CANNOT be used
            // as an LVALUE; e.g., "f().a = 1" and "&f().a" are
            // illegal, so we must FORCE f().a to be an RVALUE
            // by always issuing an OP_LOAD.  See H&S 7.4.2.
            // Of course, we could accept the above syntactically,
            // and weed it out later semantically...
            // Other cases, such as the illegal "2.name", must be
            // sorted out semantically, not syntactically.
            kind = LVALUE;
#else
            if (kind == RVALUE)
                push_op(OP_LOAD);           /* load the resulting RVALUE */
#endif
        }
        else if (indirect_component_selection_op()) {
            if (kind == LVALUE)
                push_op(OP_LOAD);           /* need RVALUE */
            if (!identifier()) REJECT;
            push_op(OP_MEMBER);
            kind = LVALUE;
        }
        else if (increment_operator()) {
            if (kind != LVALUE) REJECT;     /* must be LVALUE */
#ifdef EMIT_POSTINC
            push_op(OP_POSTINC);
#else
            /* Synthesize the post-increment operator */
            push_op(OP_DUPL);               /* i i */
            push_op(OP_LOAD);               /* i [i] */
            push_op(OP_XCHG);               /* [i] i */
            push_op(OP_DUPL);               /* [i] i i */
            push_op(OP_LOAD);               /* [i] i [i] */
            push_param(LIT_INTEGER);        /* [i] i [i] 1 */
            push_constant(1);
            push_op(OP_ADD);                /* [i] i [i]+1 */
            push_op(OP_STORE);              /* [i] [i]+1 */
            push_op(OP_POP);                /* [i] */
#endif
            kind = RVALUE;
        }
        else if (decrement_operator()) {
            if (kind != LVALUE) REJECT;     /* must be LVALUE */
#ifdef EMIT_POSTDEC
            push_op(OP_POSTDEC);
#else
            /* Synthesize the post-decrement operator */
            push_op(OP_DUPL);               /* i i */
            push_op(OP_LOAD);               /* i [i] */
            push_op(OP_XCHG);               /* [i] i */
            push_op(OP_DUPL);               /* [i] i i */
            push_op(OP_LOAD);               /* [i] i [i] */
            push_param(LIT_INTEGER);        /* [i] i [i] 1 */
            push_constant(1);
            push_op(OP_SUB);                /* [i] i [i]-1 */
            push_op(OP_STORE);              /* [i] [i]-1 */
            push_op(OP_POP);                /* [i] */
#endif
            kind = RVALUE;
        }
        else keep_looking = FALSE;
    } while (keep_looking);
    ACCEPTX(kind);
} /*postfix_expression*/

STATE(subscript_expression_list)
    int kind;
    MARK;

    REJECT; //needs work

    do {
        if (!(kind = assignment_expression())) REJECT;
        if (kind == LVALUE)
            push_op(OP_LOAD);               /* need RVALUE */
        push_op(OP_ADD);
    } while (mdp_char(','));
    ACCEPT;
} /*subscript_expression_list*/

STATE(argument_expression_list)
    int kind;
    int nargs;
    MARK;
    nargs = 0;
    do {
        if (!(kind = assignment_expression())) REJECT;
        if (kind == LVALUE)
            push_op(OP_LOAD);               /* need RVALUE */
        push_op(OP_ARG);
        nargs++;
    } while (mdp_char(','));
    /* Put number of args on stack for VPL-style call */
    push_constant(LIT_INTEGER);
    push_param(nargs);
    ACCEPT;
} /*argument_expression_list*/

/*************************** <unary_expression> ************************
;
; Syntax:
;
;	<unary_expression> ::=
;		<postfix_expression> |
;		"++" <unary_expression> |
;		"--" <unary_expression> |
;		<unary_operator> <cast_expression> |
;		"sizeof" <unary_expression> |
;		"sizeof" "(" <type_name> ")" |
;       <FFP_inline_function> |
;       <FFP_intrinsic_function>
;
;	<unary_operator> ::=
;		"&" | "*" | "+" | "-" | "~" | "!"
;
*/
STATE(unary_expression)
    int kind;
    MARK;
    if (increment_operator()) {
        if (unary_expression() != LVALUE) REJECT;
#ifdef EMIT_PREINC
        push_op(OP_PREINC);
#else
        /* Synthesize the pre-increment operator */
        push_op(OP_DUPL);           /* i i */
        push_op(OP_LOAD);           /* i [i] */
        push_param(LIT_INTEGER);    /* i [i] 1 */
        push_constant(1);
        push_op(OP_ADD);            /* i [i]+1 */
        push_op(OP_STORE);          /* [i]+1 */
#endif
        kind = RVALUE;
    }
    else if (decrement_operator()) {
        if (unary_expression() != LVALUE) REJECT;
#ifdef EMIT_PREDEC
        push_op(OP_PREDEC);
#else
        /* Synthesize the pre-decrement operator */
        push_op(OP_DUPL);           /* i i */
        push_op(OP_LOAD);           /* i [i] */
        push_param(LIT_INTEGER);    /* i [i] 1 */
        push_constant(1);
        push_op(OP_SUB);            /* i [i]-1 */
        push_op(OP_STORE);          /* [i]-1 */
#endif
        kind = RVALUE;
    }
    else if (unary_operator()) {
        UINT4 unaryop = op;
        if (!(kind = cast_expression())) REJECT;
        if (kind == LVALUE)
            push_op(OP_LOAD);               /* need RVALUE */
        kind = RVALUE;
#if 0
        /* "short-circuit" version of ! */
        /* Can probably do better with explicit OP_LOG_NOT... */
        if (unaryop == OP_LOG_NOT) {
            push_op(OP_THEN);
            push_param(LIT_INTEGER);
            push_constant(0);
            push_op(OP_POP);
            push_op(OP_ELSE);
            push_param(LIT_INTEGER);
            push_constant(1);
            push_op(OP_IF);
        }
        else
#endif
        if (unaryop == OP_LOAD) {
            /* dereference operator (*rvalue) */
            kind = LVALUE;  /* OP_LOAD is deferred... */
        }
        else if (unaryop != OP_NOP) push_op(unaryop);
    }
    else if (and_operator()) {
        /* address-of operator (&lvalue) */
        if (cast_expression() != LVALUE) REJECT;
        kind = RVALUE;      /* suppresses OP_LOAD */
    }
    else if (mdp_keyword("sizeof")) {
        /* Could we somehow suppress codegen for the arg to sizeof??? */
#if 1
		/* Kludge to parse some simple forms for now... */
		int op;
		mdp_number_overflow = false;
		mdp_number_unsigned = true;	//correct??
		mdp_number_bitlen = 32;	//default
		if (op = restricted_type_name_in_parens()) {
			switch (op) {
			case OP_BOOL:	//changed to 1 byte for FM64	//later 4??
			case OP_BYTE:	//changed to 1 byte for FM64
			case OP_CHAR:	//changed to 1 byte for FM64
				mdp_number = 1;
				break;
			case OP_SHORT:	//changed to 2 bytes for FM64
			case OP_WORD:	//changed to 2 bytes for FM64
			case OP_HALF:
				mdp_number = 2;
				break;
			case OP_INT:	//4 bytes
			case OP_DWORD:	//4 bytes
			case OP_SIGNED: //4 bytes
			case OP_UNSIGNED: //4 bytes
			case OP_LONG:	//4 bytes
			case OP_FLOAT:	//4 bytes
				mdp_number = 4;
				break;
#ifdef X64
			case OP_INT_PTR:	//chameleon type
			case OP_UINT_PTR:	//chameleon type
				mdp_number = X64GEN ? 8 : 4;	//pointer size, 4 or 8 bytes
				break;
			case OP_INT64:	//8 bytes
			case OP_QWORD:	//8 bytes
#endif //X64
			case OP_DOUBLE:	//8 bytes
				mdp_number = 8;
				break;
			case OP_TEMPREAL: //10 bytes
			case OP_BCD:	//10 bytes
				mdp_number = 10;
				break;
			case OP_VOID:	//0 bytes
				mdp_number = 0;
				break;
			default:
				REJECT;		//compiler bug???
				break;
			}//switch op
			push_value(LIT_INTEGER);
		}
        else if (sizeof_operand()) {
        }
		else
#endif //simple forms
        if (type_name_in_parens()) {
            push_op(OP_SIZEOF);
        }
        else if (kind = unary_expression()) {
#if 0
            if (kind == LVALUE)
                push_op(OP_LOAD);           /* need RVALUE?? */ //WRONG????
#endif
            push_op(OP_SIZEOF);
        }
        else REJECT;
        kind = RVALUE;
    }//sizeof

#ifndef NO_FFP_INLINES
    else if (FFP_inline_function()) {
        kind = RVALUE;
    }
#endif
    else if (FFP_intrinsic_function()) {
        kind = RVALUE;
    }
    else if (!(kind = postfix_expression())) REJECT;
    ACCEPTX(kind);
} /*unary_expression*/


STATE(sizeof_operand)   // another 'sizeof' kludge
    MARK;
    if (mdp_char('(')) {
        if (!sizeof_operand()) REJECT;
        if (mdp_char(')')) ACCEPT;
        REJECT;
    }
    else if (string_literal()) {
        //pop string lit from SDL...
        int i;
        for (i = (mdp_string_len + 3)/4; i > 0; i--) {
            pop_param();    //pop 4 chars per longword (last may have 3 padding blanks)
        }
        pop_param();    //pop the LIT_CHAR descriptor header dword
        //now push an SD for the # of chars, including final NUL
        mdp_number = mdp_string_len;    //already includes final NUL
        push_value(LIT_INTEGER);        //i.e., strlen(lit) + 1
    }
    else if (floating_constant()) {
        pop_param();        //discard LIT_REAL
        pop_param();
        pop_param();
        mdp_number = 8;     //for now (should be 4 if 'F' suffix present)
		push_value(LIT_INTEGER);
    }
    else if (integer_constant()) {
        pop_param();        //discard LIT_INTEGER
        pop_param();
        mdp_number = 4;     //for now (should be 8 if int64, etc.)
		push_value(LIT_INTEGER);
    }
#if 0
    // Causes problems for now ... disambiguate later!
    else if (enumeration_constant()) {
        pop_param();        //discard LIT_INTEGER
        pop_param();
        mdp_number = 4;     //for now (should be 8 if int64, etc.)
		push_value(LIT_INTEGER);
    }
#endif
    else if (character_constant()) {
        pop_param();        //discard LIT_INTEGER
        pop_param();
        mdp_number = 4;     //for now (C++ returns # of chars in lit); or 2 for L'c'.
		push_value(LIT_INTEGER);
    }
    else if (boolean_constant()) {
        pop_param();        //discard LIT_INTEGER
        pop_param();
        mdp_number = 4;     //for now			//X64???
		push_value(LIT_INTEGER);
    }
    else if (named_constant()) {
        pop_param();        //discard LIT_INTEGER (Check: is it always LIT_INTEGER?? YES for now!)
        pop_param();
        mdp_number = 4;     //for now
		push_value(LIT_INTEGER);
    }
    else if (identifier()) {    // simple subset of last case in 'sizeof' code above???
        push_op(OP_LOOKUP); //for now, lookup scalar var
        push_op(OP_SIZEOF);
    }
    else REJECT;
    ACCEPT;
} /* sizeof_operand */


STATE(unary_operator)
    MARK;
    if      (mdp_char('*')) op = OP_LOAD;   /* dereference operator */
    else if (mdp_char('+')) op = OP_NOP;    /* it's a no-op */
    else if (mdp_char('-')) op = OP_NEG;
    else if (mdp_char('~')) op = OP_NOT;
    else if (mdp_char('!')) op = OP_LOG_NOT;
    //// See clever use of "and_operator" in unary_expression above,
    //// as a workaround(?) for the ambiguity below...
    /* else if (logical_and_operator()) REJECT; /* avoid spurious match */
    /* else if (mdp_char('&')) op = OP_ADDR; */     // lvalue!!!
    else REJECT;
    ACCEPT;
} /*unary_operator*/

STATE(type_name_in_parens)
    MARK;
    if (mdp_char('(')) {
        if (!type_name()) REJECT;
        if (mdp_char(')')) ACCEPT;
    }
    REJECT;
} /*type_name_in_parens*/

/*
;   <FFP_intrinsic_function>:
;       "COLOR" "(" <color> ")"
;
*/
STATE(FFP_intrinsic_function)
    MARK;
    if (mdp_keyword("COLOR")) {
        if (!mdp_char('(')) REJECT;
        if (!color()) REJECT;
        if (!mdp_char(')')) REJECT;
    }
    else REJECT;
    ACCEPT;
} /*FFP_intrinsic_function*/

/*
;   <FFP_inline_function>:
;       <FFP_inline_function_name> "(" [<argument_expression_list>] ")"
;
;   <FFP_inline_function_name>:
;       "get" | "put" | "min" | "max" |
;       "ctl" | "chk" | "getCtlVal" | 
#if 0
;       "val" |
#endif
;       "sin" | "cos" | "tan" |
;       "sqr" | "abs" | "dif" |
;if (!X64GEN)
;       "cnvX" | "cnvY" |
;endif
;       "gamma" |
#if 0 //nyi
;       "ftol" |                                //N.B. ftol == ichop == (int)
#endif
;       "floor" | "ceil" | "round" | "chop" | "pow" |
#if 0 //nyi
;       "ifloor" | "iceil" | "iround" | "ichop" |
;       "acos" | "asin" | "atan" | "atan2" |
;       "exp" | "fabs" | "fmod" | "ldexp" | "log" | "log10" |
#endif
#if 0 //following are probably not feasible to in-line (at present...)
;       "cosh" | "sinh" | "tanh"(?) | "frexp" (crtl??) | "modf"(?) |
#endif
#if 0 //other possible in-line candidates...
;       "src8_fast"    | "src8z0_fast"    | "src8z1_fast"    | "src8z2_fast"    | "src8z3_fast"    | "src8z4_fast"    |
;       "src16_fast"   | "src16z0_fast"   | "src16z1_fast"   | "src16z2_fast"   | "src16z3_fast"   | "src16z4_fast"   |
;       "pget8_fast"   | "pget8z0_fast"   | "pget8z1_fast"   | "pget8z2_fast"   | "pget8z3_fast"   | "pget8z4_fast"   |
;       "pget16_fast"  | "pget16z0_fast"  | "pget16z1_fast"  | "pget16z2_fast"  | "pget16z3_fast"  | "pget16z4_fast"  |
;       "pset8_fast"   | "pset8z0_fast"   | "pset8z1_fast"   | "pset8z2_fast"   | "pset8z3_fast"   | "pset8z4_fast"   |
;       "pset16_fast"  | "pset16z0_fast"  | "pset16z1_fast"  | "pset16z2_fast"  | "pset16z3_fast"  | "pset16z4_fast"  |
;       "tget8_fast"   | "tget8z0_fast"   | "tget8z1_fast"   | "tget8z2_fast"   | "tget8z3_fast"   | "tget8z4_fast"   |
;       "tget16_fast"  | "tget16z0_fast"  | "tget16z1_fast"  | "tget16z2_fast"  | "tget16z3_fast"  | "tget16z4_fast"  |
;       "tset8_fast"   | "tset8z0_fast"   | "tset8z1_fast"   | "tset8z2_fast"   | "tset8z3_fast"   | "tset8z4_fast"   |
;       "tset16_fast"  | "tset16z0_fast"  | "tset16z1_fast"  | "tset16z2_fast"  | "tset16z3_fast"  | "tset16z4_fast"  |
;       "t2get8_fast"  | "t2get8z0_fast"  | "t2get8z1_fast"  | "t2get8z2_fast"  | "t2get8z3_fast"  | "t2get8z4_fast"  |
;       "t2get16_fast" | "t2get16z0_fast" | "t2get16z1_fast" | "t2get16z2_fast" | "t2get16z3_fast" | "t2get16z4_fast" |
;       "t2set8_fast"  | "t2set8z0_fast"  | "t2set8z1_fast"  | "t2set8z2_fast"  | "t2set8z3_fast"  | "t2set8z4_fast"  |
;       "t2set16_fast" | "t2set16z0_fast" | "t2set16z1_fast" | "t2set16z2_fast" | "t2set16z3_fast" | "t2set16z4_fast" |
;       "srcRGB"   | "srcRGBA" | "srcGrey" | "srcDuo" | "srcCMYK" | etc.
;       "pgetRGB"  | ...
;       "psetRGB"  | ...
;       "tgetRGB"  | ...
;       "tsetRGB"  | ...
;       "t2getRGB" | ...
;       "t2setRGB" | ...
;       "c2d" | "c2m"(?) | "cnv"(?) | "rad"(?) | "r2x" | "r2y" | "scl" | "map"(?) | "add" | "sub" | "rnd"(?) | "mix" |
;       "RGB" | RGBA | Rval | Gval | Bval | Aval
#endif
;
*/
#ifndef NO_FFP_INLINES
STATE(FFP_inline_function)
    MARK;
    if (!(mdp_flags & MDP_USE_INLINES)) REJECT;
    if (mdp_keyword("get")) {
        if (!one_inline_argument()) REJECT;
        push_op(OP_GET);
    }
    else if (mdp_keyword("put")) {
        if (!two_inline_arguments()) REJECT;
        push_op(OP_PUT);
    }
    else if (mdp_keyword("min")) {
        if (!mdp_char('(')) REJECT;
        if (!argument_to_inline_function()) REJECT;
        if (!mdp_char(',')) REJECT;
        if (!argument_to_inline_function()) REJECT;
        push_op(OP_MIN);
        while (mdp_char(',')) {
            if (!argument_to_inline_function()) REJECT;
            push_op(OP_MIN);
        }
        if (!mdp_char(')')) REJECT;
    }
    else if (mdp_keyword("max")) {
        if (!mdp_char('(')) REJECT;
        if (!argument_to_inline_function()) REJECT;
        if (!mdp_char(',')) REJECT;
        if (!argument_to_inline_function()) REJECT;
        push_op(OP_MAX);
        while (mdp_char(',')) {
            if (!argument_to_inline_function()) REJECT;
            push_op(OP_MAX);
        }
        if (!mdp_char(')')) REJECT;
    }
    else if (mdp_keyword("ctl")) {
        if (!one_inline_argument()) REJECT;
        push_op(OP_CTL);
    }
    else if (mdp_keyword("getCtlVal")) {
        // "getCtlVal" is the long name for "ctl"
        if (!one_inline_argument()) REJECT;
        push_op(OP_CTL);
    }
    else if (mdp_keyword("chk")) {
        // "chk" is a deprecated synonym for "ctl"
        if (!one_inline_argument()) REJECT;
        push_op(OP_CTL);
    }
#if 0
    else if (mdp_keyword("val")) {
        if (!three_inline_arguments()) REJECT;
        push_op(OP_VAL);
    }
#endif
    else if (mdp_keyword("sin")) {
        if (!one_inline_argument()) REJECT;
        push_op(OP_SIN);
    }
    else if (mdp_keyword("cos")) {
        if (!one_inline_argument()) REJECT;
        push_op(OP_COS);
    }
    else if (mdp_keyword("tan")) {
        if (!one_inline_argument()) REJECT;
        push_op(OP_TAN);
    }
    else if (mdp_keyword("sqr")) {
        if (!one_inline_argument()) REJECT;
        push_op(OP_SQR);
    }
    else if (mdp_keyword("abs")) {
        if (!one_inline_argument()) REJECT;
        push_op(OP_ABS);
    }
    else if (mdp_keyword("dif")) {
        if (!two_inline_arguments()) REJECT;
        push_op(OP_DIF);
    }
    else if (mdp_keyword("cnvX")) {
		if (X64GEN) REJECT;	//for now, until fixed for X64
        if (!seven_inline_arguments()) REJECT;
        push_op(OP_CNVX);
    }
    else if (mdp_keyword("cnvY")) {
		if (X64GEN) REJECT;	//for now, until fixed for X64
        if (!seven_inline_arguments()) REJECT;
        push_op(OP_CNVY);
    }
    else if (mdp_keyword("gamma")) {
        if (!one_inline_argument()) REJECT;
        push_op(OP_GAMMA);
    }
#if 0
    else if (mdp_keyword("ftol")) {
        if (!one_inline_argument()) REJECT;
        push_op(OP_FTOL);
    }
#endif
    else if (mdp_keyword("floor")) {
        if (!one_inline_argument()) REJECT;
        push_op(OP_FLOOR);
    }
    else if (mdp_keyword("ceil")) {
        if (!one_inline_argument()) REJECT;
        push_op(OP_CEIL);
    }
    else if (mdp_keyword("round")) {
        if (!one_inline_argument()) REJECT;
        push_op(OP_ROUND);
    }
    else if (mdp_keyword("chop")) {
        if (!one_inline_argument()) REJECT;
        push_op(OP_CHOP);
    }
#if 0   // not yet implemented in AfhFMcompiler.c
    else if (mdp_keyword("ifloor")) {
        if (!one_inline_argument()) REJECT;
        push_op(OP_IFLOOR);
    }
    else if (mdp_keyword("iceil")) {
        if (!one_inline_argument()) REJECT;
        push_op(OP_ICEIL);
    }
    else if (mdp_keyword("iround")) {
        if (!one_inline_argument()) REJECT;
        push_op(OP_IROUND);
    }
    else if (mdp_keyword("ichop")) {
        if (!one_inline_argument()) REJECT;
        push_op(OP_ICHOP);
    }
#endif
    else if (mdp_keyword("pow")) {
        if (!two_inline_arguments()) REJECT;
        push_op(OP_POW);
    }
    else REJECT;
    ACCEPT;
} /*FFP_inline_function*/

STATE(argument_to_inline_function)
    int kind;
    MARK;
    if (!(kind = assignment_expression())) REJECT;
    if (kind == LVALUE) {
        push_op(OP_LOAD);               /* need RVALUE */
    }
    ACCEPT;
} /*argument_to_inline_function*/

STATE(one_inline_argument)
    MARK;
    if (!mdp_char('(')) REJECT;
    if (!argument_to_inline_function()) REJECT;
    if (!mdp_char(')')) REJECT;
    ACCEPT;
} /*one_inline_argument*/

STATE(two_inline_arguments)
    MARK;
    if (!mdp_char('(')) REJECT;
    if (!argument_to_inline_function()) REJECT;
    if (!mdp_char(',')) REJECT;
    if (!argument_to_inline_function()) REJECT;
    if (!mdp_char(')')) REJECT;
    ACCEPT;
} /*two_inline_arguments*/

STATE(three_inline_arguments)
    MARK;
    if (!mdp_char('(')) REJECT;
    if (!argument_to_inline_function()) REJECT;
    if (!mdp_char(',')) REJECT;
    if (!argument_to_inline_function()) REJECT;
    if (!mdp_char(',')) REJECT;
    if (!argument_to_inline_function()) REJECT;
    if (!mdp_char(')')) REJECT;
    ACCEPT;
} /*three_inline_arguments*/

STATE(seven_inline_arguments)
    MARK;
    if (!mdp_char('(')) REJECT;
    if (!argument_to_inline_function()) REJECT;
    if (!mdp_char(',')) REJECT;
    if (!argument_to_inline_function()) REJECT;
    if (!mdp_char(',')) REJECT;
    if (!argument_to_inline_function()) REJECT;
    if (!mdp_char(',')) REJECT;
    if (!argument_to_inline_function()) REJECT;
    if (!mdp_char(',')) REJECT;
    if (!argument_to_inline_function()) REJECT;
    if (!mdp_char(',')) REJECT;
    if (!argument_to_inline_function()) REJECT;
    if (!mdp_char(',')) REJECT;
    if (!argument_to_inline_function()) REJECT;
    if (!mdp_char(')')) REJECT;
    ACCEPT;
} /*seven_inline_arguments*/
#endif //!NO_FFP_INLINES

/*************************** <cast_expression> ************************
;
; Syntax:
;
;	<cast_expression> ::=
;		<unary_expression> |
;		"(" <type_name> ")" <cast_expression>
;
*/
#if 0
STATE(cast_expression)
    int kind;
    MARK;
    if (type_name_in_parens()) {
        if (!(kind = cast_expression())) REJECT;
        if (kind == LVALUE)
            push_op(OP_LOAD);               /* need RVALUE */
        push_op(OP_CAST);
        kind = RVALUE;
    }
    else if (!(kind = unary_expression())) REJECT;
    ACCEPTX(kind);
} /*cast_expression*/
#else
// For now, we just handle a restricted set of type names...
STATE(restricted_type_name_in_parens)
    int op;
    MARK;
    if (mdp_char('(')) {					//should also parse "long long" as "int64"???
											//also "signed" or "unsigned" + restricted type???
											//also "*" + restricted type???
		if (prog_rev_major < 2) {
			//Before rev 2.0, there were only two implementation types:
			//'int' and 'double', and all types were signed
			//(arranged roughly in descending frequency of occurrence)
			if      (mdp_keyword("int"))		op = OP_INT;
			else if (mdp_keyword("char"))		op = OP_INT;
			else if (mdp_keyword("double"))		op = OP_DOUBLE;
			else if (mdp_keyword("float"))		op = OP_DOUBLE;
			else if (mdp_keyword("bool"))		op = OP_INT;
			else if (mdp_keyword("long"))		op = OP_INT;
			else if (mdp_keyword("short"))		op = OP_INT;
			else if (mdp_keyword("byte"))		op = OP_INT;
			else if (mdp_keyword("word"))		op = OP_INT;
			else if (mdp_keyword("dword"))		op = OP_INT;
			else if (mdp_keyword("signed"))		op = OP_SIGNED;
			else if (mdp_keyword("unsigned"))	op = OP_SIGNED;	//sic!!
#ifdef X64
			//Following keywords did not exist before 2.0,
			//so might as well retain their 2.0 definitions
			else if (mdp_keyword("int64"))		op = OP_INT64;
			else if (mdp_keyword("_int64"))		op = OP_INT64;
			else if (mdp_keyword("qword"))		op = OP_QWORD;
			else if (mdp_keyword("INT_PTR"))	op = OP_INT_PTR;
			else if (mdp_keyword("UINT_PTR"))	op = OP_UINT_PTR;
			else if (mdp_keyword("half"))		op = OP_HALF;
			else if (mdp_keyword("tempreal"))	op = OP_TEMPREAL;
			else if (mdp_keyword("bcd"))		op = OP_BCD;
#endif //X64
			else if (mdp_keyword("void"))		op = OP_VOID;
			else REJECT;
		}
		else {
			//Following types are implemented in rev 2.0 and later
			//(arranged roughly in descending frequency of occurrence)
			if		(mdp_keyword("int"))		op = OP_INT;
			else if (mdp_keyword("char"))		op = OP_CHAR;
			else if (mdp_keyword("double"))		op = OP_DOUBLE;
			else if (mdp_keyword("float"))		op = OP_FLOAT;
			else if (mdp_keyword("bool"))		op = OP_BOOL;
			else if (mdp_keyword("long"))		op = OP_LONG;
			else if (mdp_keyword("short"))		op = OP_SHORT;
			else if (mdp_keyword("byte"))		op = OP_BYTE;
			else if (mdp_keyword("word"))		op = OP_WORD;
			else if (mdp_keyword("dword"))		op = OP_DWORD;
			else if (mdp_keyword("signed"))		op = OP_SIGNED;
			else if (mdp_keyword("unsigned"))	op = OP_UNSIGNED;
#ifdef X64
			else if (mdp_keyword("int64"))		op = OP_INT64;
			else if (mdp_keyword("_int64"))		op = OP_INT64;
			else if (mdp_keyword("qword"))		op = OP_QWORD;
			else if (mdp_keyword("INT_PTR"))	op = OP_INT_PTR;
			else if (mdp_keyword("UINT_PTR"))	op = OP_UINT_PTR;
			else if (mdp_keyword("half"))		op = OP_HALF;
			else if (mdp_keyword("tempreal"))	op = OP_TEMPREAL;
			else if (mdp_keyword("bcd"))		op = OP_BCD;
#endif //X64
			else if (mdp_keyword("void"))		op = OP_VOID;
			else REJECT;
		}
        if (mdp_char(')')) ACCEPTX(op);
    }
    REJECT;
} /*restricted_type_name_in_parens*/

STATE(cast_expression)
    int kind;
    int op;
    MARK;
    if (op = restricted_type_name_in_parens()) {
        if (!(kind = cast_expression())) REJECT;
        if (kind == LVALUE)
            push_op(OP_LOAD);               /* need RVALUE */
        push_op(op);
        kind = RVALUE;
    }
    else if (!(kind = unary_expression())) REJECT;
    ACCEPTX(kind);
} /*cast_expression*/
#endif


/*************************** <power_expression> ************************
;
; Syntax:
;
;   <power_expression> ::=
;   	<cast_expression> |
;   	<cast_expression> <power_operator> <power_expression>
;
;   <power_operator> ::= "@" | ... | "*^"   //designer's choice
*/
STATE(power_expression)
	int kind;
	MARK;
    if (!(kind = cast_expression())) REJECT;
#ifndef NO_POWER_OPERATOR
    if (power_operator()) {
        if (kind == LVALUE)
            push_op(OP_LOAD);               /* need RVALUE */
        if (kind = power_expression()) {
            if (kind == LVALUE)
                push_op(OP_LOAD);           /* need RVALUE */
            push_op(OP_POW);
            kind = RVALUE;
        }
        else REJECT;
    }
#endif
    ACCEPTX(kind);
} /*power_expression*/

#ifndef NO_POWER_OPERATOR
STATE(power_operator)
    MARK;
    skip_whitespace = FALSE;
#if 1
    if (mdp_char('@')) op = OP_POW;                     // a @ b    <==== CURRENT CHOICE =====
#elif 1
    if (mdp_char('$')) op = OP_POW;                     // a $ b  ($ allowed in symbol names?)
#elif 1
    if (mdp_char('~')) op = OP_POW;                     // a ~ b
#elif 1
    if (mdp_char('`')) op = OP_POW;                     // a ` b
#elif 1
    if (mdp_char('!')) op = OP_POW;                     // a ! b  (but conflict with != comp assn op)
#elif 1
    if (mdp_char('@') && mdp_char('@')) op = OP_POW;    // a @@ b
#elif 1
    if (mdp_char('$') && mdp_char('$')) op = OP_POW;    // a $$ b  ($ allowed in symbol names?)
#elif 1
    if (mdp_char('^') && mdp_char('^')) op = OP_POW;    // a ^^ b
#elif 1
    if (mdp_char('~') && mdp_char('~')) op = OP_POW;    // a ~~ b
#elif 1
    if (mdp_char('`') && mdp_char('`')) op = OP_POW;    // a `` b
#elif 1
    if (mdp_char('?') && mdp_char('?')) op = OP_POW;    // a ?? b
#elif 1
    if (mdp_char('*') && mdp_char('^')) op = OP_POW;    // a *^ b (Matt Austern)
#else
    if (mdp_char('*') && mdp_char('*')) op = OP_POW;    // a ** b (ambiguous: a*(*b) !!)
#endif
    else REJECT;
    ACCEPT;
} /*power_operator*/
#endif

/*************************** <multiplicative_expression> ************************
;
; Syntax:
;
;	<multiplicative_expression> ::=
;		<power_expression> |
;		<multiplicative_expression> "*" <power_expression> |
;		<multiplicative_expression> "/" <power_expression> |
;		<multiplicative_expression> "%" <power_expression>
;
*/
STATE(multiplicative_expression)
    int kind;
    char op;
    MARK;
    if (!(kind = power_expression())) REJECT;
    while (mdp_char(op='*')||mdp_char(op='/')||mdp_char(op='%')) {
        if (kind == LVALUE)
            push_op(OP_LOAD);               /* need RVALUE for first operand */
        if (!(kind = power_expression())) REJECT;
        if (kind == LVALUE)
            push_op(OP_LOAD);               /* need RVALUE for second operand */
        if      (op == '*') push_op(OP_MUL);
        else if (op == '/') push_op(OP_DIV);
        else                push_op(OP_REM);
        kind = RVALUE;
    }
    ACCEPTX(kind);
} /*multiplicative_expression*/

/*************************** <additive_expression> ************************
;
; Syntax:
;
;	<additive_expression> ::=
;		<multiplicative_expression> |
;		<additive_expression> {"+"|"-"} <multiplicative_expression>
;
*/
STATE(additive_expression)
    int kind;
    char op;
    MARK;
    if (!(kind = multiplicative_expression())) REJECT;
    while (mdp_char(op='+')||mdp_char(op='-')) {
        if (kind == LVALUE)
            push_op(OP_LOAD);               /* need RVALUE for first operand */
        if (!(kind = multiplicative_expression())) REJECT;
        if (kind == LVALUE)
            push_op(OP_LOAD);               /* need RVALUE for second operand */
        if (op == '+') push_op(OP_ADD);
        else           push_op(OP_SUB);
        kind = RVALUE;
    }
    ACCEPTX(kind);
} /*additive_expression*/

/*************************** <shift_expression> ************************
;
; Syntax:
;
;	<shift_expression> ::=
;		<additive_expression> |
;		<shift_expression> "<<" <additive_expression> |
;		<shift_expression> ">>" <additive_expression>
;
*/
STATE(shift_expression)
    int kind;
    UINT4 shiftop;
    MARK;
    if (!(kind = additive_expression())) REJECT;
    while (left_shift_operator()||right_shift_operator()) {
        shiftop = op;
        if (kind == LVALUE)
            push_op(OP_LOAD);               /* need RVALUE for first operand */
        if (!(kind = additive_expression())) REJECT;
        if (kind == LVALUE)
            push_op(OP_LOAD);               /* need RVALUE for second operand */
        push_op(shiftop);
        kind = RVALUE;
    }
    ACCEPTX(kind);
} /*shift_expression*/

/*************************** <relational_expression> ************************
;
; Syntax:
;
;	<relational_expression> ::=
;		<shift_expression> |
;		<relational_expression> <relational_operator> <shift_expression>
;
;	<relational_operator> ::=
;		 "<" | ">" | "<=" | ">="
;
*/
STATE(relational_expression)
    int kind;
    UINT4 relop;
    MARK;
    if (!(kind = shift_expression())) REJECT;
    while (le_operator()||ge_operator()||lt_operator()||gt_operator()) {
        relop = op;
        if (kind == LVALUE)
            push_op(OP_LOAD);               /* need RVALUE for first operand */
        if (!(kind = shift_expression())) REJECT;
        if (kind == LVALUE)
            push_op(OP_LOAD);               /* need RVALUE for second operand */
        push_op(relop);
        kind = RVALUE;
    }
    ACCEPTX(kind);
} /*relational_expression*/

STATE(relational_operator)    // Dryrot, or TBD???
    MARK;
    if (le_operator()) ACCEPT;
    if (ge_operator()) ACCEPT;
    if (lt_operator()) ACCEPT;
    if (gt_operator()) ACCEPT;
    REJECT;
} /*relational_operator*/

/*************************** <equality_expression> ************************
;
; Syntax:
;
;	<equality_expression> ::=
;		<relational_expression> |
;		<equality_expression> <equality_operator> <relational_expression>
;
;	<equality_operator> ::= "==" | "!="
;
*/
STATE(equality_expression)
    int kind;
    UINT4 eqop;
    MARK;
    if (!(kind = relational_expression())) REJECT;
    while (eq_operator()||ne_operator()) {
        eqop = op;
        if (kind == LVALUE)
            push_op(OP_LOAD);               /* need RVALUE for first operand */
        if (!(kind = relational_expression())) REJECT;
        if (kind == LVALUE)
            push_op(OP_LOAD);               /* need RVALUE for second operand */
        push_op(eqop);
        kind = RVALUE;
    }
    ACCEPTX(kind);
} /*equality_expression*/

STATE(equality_operator)    // Dryrot, or TBD???
    MARK;
    if (eq_operator()) ACCEPT;
    if (ne_operator()) ACCEPT;
    REJECT;
} /*equality_operator*/

/*************************** <AND_expression> ************************
;
; Syntax:
;
;	<AND_expression>:
;		<equality_expression> |
;		<AND_expression> "&" <equality_expression>
;
*/
STATE(AND_expression)
    int kind;
    MARK;
    if (!(kind = equality_expression())) REJECT;
    while (and_operator()) {
        if (kind == LVALUE)
            push_op(OP_LOAD);               /* need RVALUE for first operand */
        if (!(kind = equality_expression())) REJECT;
        if (kind == LVALUE)
            push_op(OP_LOAD);               /* need RVALUE for second operand */
        push_op(OP_AND);
        kind = RVALUE;
    }
    ACCEPTX(kind);
} /*AND_expression*/

/*************************** <exclusive_OR_expression> ************************
;
; Syntax:
;
;	<exclusive_OR_expression> ::=
;		<AND_expression> |
;		<exclusive_OR_expression> "^" <AND_expression>
;
*/
STATE(exclusive_OR_expression)
    int kind;
    MARK;
    if (!(kind = AND_expression())) REJECT;
    while (mdp_char('^')) {
        if (kind == LVALUE)
            push_op(OP_LOAD);               /* need RVALUE for first operand */
        if (!(kind = AND_expression())) REJECT;
        if (kind == LVALUE)
            push_op(OP_LOAD);               /* need RVALUE for second operand */
        push_op(OP_XOR);
        kind = RVALUE;
    }
    ACCEPTX(kind);
} /*exclusive_OR_expression*/

/*************************** <inclusive_OR_expression> ************************
;
; Syntax:
;
;	<inclusive_OR_expression>:
;		<exclusive_OR_expression> |
;		<inclusive_OR_expression> "|" <exclusive_OR_expression>
;
*/
STATE(inclusive_OR_expression)
    int kind;
    MARK;
    if (!(kind = exclusive_OR_expression())) REJECT;
    while (or_operator()) {
        if (kind == LVALUE)
            push_op(OP_LOAD);               /* need RVALUE for first operand */
        if (!(kind = exclusive_OR_expression())) REJECT;
        if (kind == LVALUE)
            push_op(OP_LOAD);               /* need RVALUE for second operand */
        push_op(OP_IOR);
        kind = RVALUE;
    }
    ACCEPTX(kind);
} /*inclusive_OR_expression*/

/*************************** <logical_AND_expression> ************************
;
; Syntax:
;
;	<logical_AND_expression> ::=
;		<inclusive_OR_expression> |
;		<logical_AND_expression> "&&" <inclusive_OR_expression>
;
*/
STATE(logical_AND_expression)
    int kind;
    MARK;
    if (!(kind = inclusive_OR_expression())) REJECT;
#if 1
    /* optimized version for a && b && c ... */
    /* a ? b ? c ? 1 : 0 : 0 : 0             */
    {   int count = 0;
        while (logical_and_operator()) {
            count++;
            if (kind == LVALUE)
                push_op(OP_LOAD);           /* need RVALUE for first operand */
            push_op(OP_THEN);
            if (!(kind = inclusive_OR_expression())) REJECT;
            if (kind == LVALUE)
                push_op(OP_LOAD);           /* need RVALUE for second operand */
            kind = RVALUE;
        }
        if (count) {
            push_op(OP_THEN);
            push_param(LIT_INTEGER);
            push_constant(1);
            push_op(OP_POP);
            push_op(OP_ELSE);
            push_param(LIT_INTEGER);
            push_constant(0);
            push_op(OP_IF);
            while (count--) {
                push_op(OP_POP);
                push_op(OP_ELSE);
                push_param(LIT_INTEGER);
                push_constant(0);
                push_op(OP_IF);
            }
        }
    }
#elif 0
    /* unoptimized version for a && b && c ... */
    /* (a ? b ? 1 : 0 : 0) ? c ? 1 : 0 : 0     */
    while (logical_and_operator()) {
        if (kind == LVALUE)
            push_op(OP_LOAD)                /* need RVALUE for first operand */
        push_op(OP_THEN);
        if (!(kind = inclusive_OR_expression())) REJECT;
        if (kind == LVALUE)
            push_op(OP_LOAD);               /* need RVALUE for second operand */
        kind = RVALUE;
        push_op(OP_THEN);
        push_param(LIT_INTEGER);
        push_constant(1);
        push_op(OP_POP);
        push_op(OP_ELSE);
        push_param(LIT_INTEGER);
        push_constant(0);
        push_op(OP_IF);
        push_op(OP_POP);
        push_op(OP_ELSE);
        push_param(LIT_INTEGER);
        push_constant(0);
        push_op(OP_IF);
    }
#else
    /* non-short-circuited version */
    while (logical_and_operator()) {
        if (kind == LVALUE)
            push_op(OP_LOAD)                /* need RVALUE for first operand */
        if (!(kind = inclusive_OR_expression())) REJECT;
        if (kind == LVALUE)
            push_op(OP_LOAD);               /* need RVALUE for second operand */
        push_op(OP_LOG_AND);
        kind = RVALUE;
    }
#endif
    ACCEPTX(kind);
} /*logical_AND_expression*/

/*************************** <logical_OR_expression> ************************
;
; Syntax:
;
;	<logical_OR_expression> ::=
;		<logical_AND_expression> |
;		<logical_OR_expression> "||" <logical_AND_expression>
;
*/
STATE(logical_OR_expression)
    int kind;
    MARK;
    if (!(kind = logical_AND_expression())) REJECT;
#if 1
    /* optimized version for a || b || c ... */
    /* a ? 1 : b ? 1 ? c : 1 : 0             */
    {   int count = 0;
        while (logical_or_operator()) {
            count++;
            if (kind == LVALUE)
                push_op(OP_LOAD);           /* need RVALUE for first operand */
            push_op(OP_THEN);
            push_param(LIT_INTEGER);
            push_constant(1);
            push_op(OP_POP);
            push_op(OP_ELSE);
            if (!(kind = logical_AND_expression())) REJECT;
            if (kind == LVALUE)
                push_op(OP_LOAD);           /* need RVALUE for second operand */
            kind = RVALUE;
        }
        if (count) {
            push_op(OP_THEN);
            push_param(LIT_INTEGER);
            push_constant(1);
            push_op(OP_POP);
            push_op(OP_ELSE);
            push_param(LIT_INTEGER);
            push_constant(0);
            push_op(OP_IF);
            while (count--) push_op(OP_IF);
        }
    }
#elif 0
    /* unoptimized version */
    /* (a ? 1 : b ? 1 : 0) ? 1 : c ? 1 : 0 */
    while (logical_or_operator()) {
        if (kind == LVALUE)
            push_op(OP_LOAD)                /* need RVALUE for first operand */
        push_op(OP_THEN);
        push_param(LIT_INTEGER);
        push_constant(1);
        push_op(OP_POP);
        push_op(OP_ELSE);
        if (!(kind = logical_AND_expression())) REJECT;
        if (kind == LVALUE)
            push_op(OP_LOAD);               /* need RVALUE for second operand */
        kind = RVALUE;
        push_op(OP_THEN);
        push_param(LIT_INTEGER);
        push_constant(1);
        push_op(OP_POP);
        push_op(OP_ELSE);
        push_param(LIT_INTEGER);
        push_constant(0);
        push_op(OP_IF);
        push_op(OP_IF);
    }
#else
    /* non-short-circuited version */
    while (logical_or_operator()) {
        if (kind == LVALUE)
            push_op(OP_LOAD)                /* need RVALUE for first operand */
        if (!(kind = logical_AND_expression())) REJECT;
        if (kind == LVALUE)
            push_op(OP_LOAD);               /* need RVALUE for second operand */
        push_op(OP_LOG_OR);
        kind = RVALUE;
    }
#endif
    ACCEPTX(kind);
} /*logical_OR_expression*/

/*************************** <conditional_expression> ************************
;
; Syntax:
;
;	<conditional_expression> ::=
;		<logical_OR_expression> |
;		<logical_OR_expression> "?" <expression> ":" <conditional_expression>
;
*/
STATE(conditional_expression)
    int kind;
    MARK;
    if (!(kind = logical_OR_expression())) REJECT;
    if (mdp_char('?')) {
        if (kind == LVALUE)
            push_op(OP_LOAD);               /* need RVALUE */
        push_op(OP_THEN);
        if ((kind = expression()) && mdp_char(':')) {
            if (kind == LVALUE)
                push_op(OP_LOAD);           /* need RVALUE */
            push_op(OP_DEC_STACK);
            push_op(OP_ELSE);
            if (kind = conditional_expression()) {
                if (kind == LVALUE)
                    push_op(OP_LOAD);       /* need RVALUE */
                push_op(OP_IF);
                kind = RVALUE;
            }
            else REJECT;
        }
        else REJECT;
    }
    ACCEPTX(kind);
} /*conditional_expression*/

/*************************** <assignment_expression> ************************
;
; Syntax:
;
;	<assignment_expression> ::=
;		<conditional_expression> |
;		<unary_expression> <assignment_operator> <assignment_expression>
;
;	<assignment_operator> ::=
;		"=" | "*=" | "/=" | "%=" | "+=" | "-=" |
;		"<<=" | ">>=" | "&=" | "^=" | "|="
#ifndef NO_POWER_OPERATOR
;       | <power_operator>"="
#endif
;
*/
STATE(assignment_stmt)
    int kind;
    UINT4 assop;
    MARK;
    if (unary_expression() != LVALUE) REJECT;
    if (!assignment_operator()) REJECT;
    assop = op;
#ifdef DONT_SYNTHESIZE_COMPOUND_ASSIGNMENTS
    if (!(kind = assignment_expression())) REJECT;
    if (kind == LVALUE)
        push_op(OP_LOAD);                   /* need RVALUE */
    push_op(assop);
#else
    if (assop == OP_STORE) {
        if (!(kind = assignment_expression())) REJECT;
        if (kind == LVALUE)
            push_op(OP_LOAD);               /* need RVALUE */
        push_op(assop);
    }
    else {
        /* synthesize a compound assignment */
        push_op(OP_DUPL);       /* lhs lhs */
        push_op(OP_LOAD);       /* lhs [lhs] */
        if (!(kind = assignment_expression())) REJECT;
        if (kind == LVALUE)     /* lhs [lhs] rhs */
            push_op(OP_LOAD);               /* need RVALUE */
        push_op(assop);         /* lhs ([lhs] op rhs) */
        push_op(OP_STORE);      /* ([lhs] op rhs) */
        /* may need to cast rhs to type of lhs???? */
    }
#endif
    ACCEPTX(RVALUE);
} /*assignment_stmt*/

STATE(assignment_expression)
    int kind;
    MARK;
    if (kind = assignment_stmt()) {
        ACCEPTX(kind);
    }
    else if (kind = conditional_expression()) {
        ACCEPTX(kind);
    }
    REJECT;
} /*assignment_expression*/

/*************************** <expression> ************************
;
; Syntax:
;
;	<expression> ::=
;		<assignment_expression> |
;		<expression> "," <assignment_expression>
;
*/
STATE(expression)
    int kind;
    MARK;
    if (!(kind = assignment_expression())) REJECT;
    while (mdp_char(',')) {
        if (kind == LVALUE)
            push_op(OP_LOAD);
        push_op(OP_POP);
        if (!(kind = assignment_expression())) REJECT;
        if (kind == LVALUE) {
            push_op(OP_LOAD);
        }
        kind = RVALUE;
    }
    /* No need to convert to rvalue's except for the very */
    /* last expression?  This avoids a few unneeded       */
    /* OP_LOAD operations. I don't think there are any    */
    /* cases of converting an lvalue to an rvalue that    */
    /* could have side-effects.                           */
    /* N.B.: If there is ONLY one value, do NOT convert   */
    /*       it to an rvalue!                             */
    ACCEPTX(kind);
} /*expression*/


// 3.4 Constant Expressions

/*************************** <constant_expression> ************************
;
; Syntax:
;
;	<constant_expression> ::=
;		<conditional_expression>
;
*/
STATE(constant_expression)
#if 0
    MARK;
    if (conditional_expression()) ACCEPT;
    REJECT;
#else
    int kind;
    MARK;
    if (!(kind = conditional_expression())) REJECT;
    if (kind == LVALUE) {
        push_op(OP_LOAD);
        kind = RVALUE;
    }
    ACCEPT;
#endif
}

STATE(constant_integer_expression)
    MARK;
    if (!constant_expression()) REJECT;
    //check that type is integral???
    ACCEPT;
} /*constant_integer_expression*/


// 3.5 Declarations

/*************************** <declaration> ************************
;
; Syntax:
;
;	<declaration> ::=
;		<declaration_specifiers> [<init_declarator_list>] ";"  // is idl really optional??? YES
;
;	<declaration_specifiers> ::=
;		<storage_class_specifier> [<declaration_specifiers>] |
;		<type_specifier> [<declaration_specifiers>] |
;		<type_qualifier> [<declaration_specifiers>]
;
;	<init_declarator_list> ::=
;		<init_declarator> |
;		<init_declarator_list> "," <init_declarator>
;
;	<init_declarator> ::=
;		<declarator> |
;		<declarator> "=" <initializer>
;
*/
STATE(declaration)
    MARK;
    push_op(OP_DECLARE);
    if (!declaration_specifiers()) REJECT;
    init_declarator_list(); //optional
    if (!mdp_char(';')) REJECT;
    push_op(OP_END_DECL);
    ACCEPT;
}

STATE(declaration_specifiers)
    MARK;
    if (storage_class_specifier() || type_specifier() || type_qualifier()) {
        while (storage_class_specifier() || type_specifier() || type_qualifier())
            ;
        push_op(OP_COMPOSE_TYPE);
        ACCEPT;
    }
    REJECT;
}

STATE(init_declarator_list)
    MARK;
    do {
        if (!init_declarator()) REJECT;
    } while (mdp_char(','));
    ACCEPT;
}

STATE(init_declarator)
    MARK;
    if (!declarator()) REJECT;
    push_op(OP_ENTER);
    if (mdp_char('=')) {
        push_op(OP_BEGIN_INIT);
        if (!initializer()) REJECT;
        push_op(OP_END_INIT);
    }
    push_op(OP_POP);    //pop the VAR or the stored value
    ACCEPT;
}

/*************************** <storage_class_specifier> ************************
;
; Syntax:
;
;	<storage_class_specifier> ::=
;		"typedef" | "extern" | "static" | "auto" | "register"
;
*/
STATE(storage_class_specifier)
    MARK;
    if (mdp_keyword("typedef")) {
        push_op(DCL_TYPEDEF);
    }
    else if (mdp_keyword("extern")) {
        push_op(DCL_EXTERN);
    }
    else if (mdp_keyword("static")) {
        push_op(DCL_STATIC);
    }
    else if (mdp_keyword("auto")) {
        push_op(DCL_AUTO);
    }
    else if (mdp_keyword("register")) {
        push_op(DCL_REGISTER);
    }
    else REJECT;
    ACCEPT;
}

/*************************** <type_specifier> ************************
;
; Syntax:
;
;	<type_specifier> ::=
;		"void" | "char" | "short" | "int" | "long" |
;		"bool" | "byte" | "word" | "dword" |
#ifdef X64
;		"int64" | "_int64" | "qword" |
;		"INT_PTR" | "UINT_PTR" |
#endif
;       "half" | "float" | "double" | "tempreal" | "bcd" |
;		"signed" | "unsigned" | "saturated" |
;		<struct_or_union_specifier> |
;		<enum_specifier> |
;		<typedef_name>
;
*/
STATE(type_specifier)
    MARK;
	//Following keywords are defined the same in all prog revs.
	//(arranged roughly in descending frequency of occurrence)
    if (mdp_keyword("int")) {
        push_op(DCL_INT);
    }
    else if (mdp_keyword("double")) {
        push_op(DCL_DOUBLE);
    }
    else if (mdp_keyword("void")) {
        push_op(DCL_VOID);
    }
    else if (mdp_keyword("signed")) {
        push_op(DCL_SIGNED);
    }
	else if (prog_rev_major < 2) {
		//Before rev 2.0, the only types implemented were 'int' and 'double',
		//and all types were signed
		if (mdp_keyword("char")) {
			push_op(DCL_INT);
		}
		else if (mdp_keyword("float")) {
			push_op(DCL_DOUBLE);
		}
		else if (mdp_keyword("bool")) {
			push_op(DCL_INT);
		}
		else if (mdp_keyword("long")) {
			push_op(DCL_INT);
		}
		else if (mdp_keyword("short")) {
			push_op(DCL_INT);
		}
		else if (mdp_keyword("byte")) {
			push_op(DCL_INT);
		}
		else if (mdp_keyword("word")) {
			push_op(DCL_INT);
		}
		else if (mdp_keyword("dword")) {
			push_op(DCL_INT);
		}
		else if (mdp_keyword("unsigned")) {
			push_op(DCL_SIGNED);	//sic!
		}
		else goto next;
	}
	else {
		//The following types are defined in 2.0 and later
		if (mdp_keyword("char")) {
			push_op(DCL_CHAR);
		}
		else if (mdp_keyword("float")) {
			push_op(DCL_FLOAT);
		}
		else if (mdp_keyword("bool")) {
			push_op(DCL_BOOL);
		}
		else if (mdp_keyword("long")) {
			push_op(DCL_LONG);
		}
		else if (mdp_keyword("short")) {
			push_op(DCL_SHORT);
		}
		else if (mdp_keyword("byte")) {
			push_op(DCL_BYTE);
		}
		else if (mdp_keyword("word")) {
			push_op(DCL_WORD);
		}
		else if (mdp_keyword("dword")) {
			push_op(DCL_DWORD);
		}
		else if (mdp_keyword("unsigned")) {
			push_op(DCL_UNSIGNED);
		}
		else goto next;
	}
	ACCEPT;

next:	//resume here if none of the above...
#ifdef X64
	//The following keywords did not exist before 2.0,
	//so we might as well retain their 2.0 definitions
    if (mdp_keyword("int64")) {
        push_op(DCL_INT64);
    }
    else if (mdp_keyword("_int64")) {	// _int64 for Microsoft compatibility
        push_op(DCL_INT64);
    }
    else if (mdp_keyword("qword")) {
        push_op(DCL_QWORD);
    }
    else if (mdp_keyword("INT_PTR")) {
        push_op(DCL_INT_PTR);
    }
    else if (mdp_keyword("UINT_PTR")) {
        push_op(DCL_UINT_PTR);
    }
    else if (mdp_keyword("half")) {
        push_op(DCL_HALF);
    }
	else if (mdp_keyword("tempreal")) {
		push_op(DCL_TEMPREAL);
	}
	else if (mdp_keyword("bcd")) {
		push_op(DCL_BCD);
	}
    else if (mdp_keyword("saturated")) {
        push_op(DCL_SATURATED);
    }
#endif //X64
    else if (struct_or_union_specifier()) {
    }
    else if (enum_specifier()) {
    }
    else if (typedef_name()) {
    }
    else REJECT;
    ACCEPT;
}

/*************************** <struct_or_union_specifier> ************************
;
; Syntax:
;
;	<struct_or_union_specifier> ::=
;		<struct_or_union> [<identifier>] "{" <struct_declaration_list> "}" |
;		<struct_or_union> identifier
;
;	<struct_or_union> ::= "struct" | "union"
;
;	<struct_declaration_list> ::= {<struct_declaration>}+
;
;	<struct_declaration> ::=
;		<specifier_qualifier_list> <struct_declarator_list> ";"
;
;	<specifier_qualifier_list> ::=
;		{<type_specifier>|<type_qualifier>}+
;
;	<struct_declarator_list> ::=
;		<struct_declarator> {"," <struct_declarator>}*
;
;	<struct_declarator> ::=
;		<declarator> |
;		[<declarator>] ":" <constant_expression>   // NOT IMPLEMENTED
;
*/
STATE(struct_or_union_specifier)
    MARK;
    if (!struct_or_union()) REJECT;
    if (identifier()) {
        // Tag is present, so the <struct_declaration_list> is optional.
        if (mdp_char('{')) {
            if (!struct_declaration_list()) REJECT;
            if (!mdp_char('}')) REJECT;
        }
    } else {
        // No tag, so MUST have a <struct_declaration_list>.
        if (!mdp_char('{')) REJECT;
        if (!struct_declaration_list()) REJECT;
        if (!mdp_char('}')) REJECT;
    }
    ACCEPT;
}

STATE(struct_or_union)
    MARK;
    if (mdp_keyword("struct")) {
        push_op(DCL_STRUCT);
    }
    else if (mdp_keyword("union")) {
        push_op(DCL_UNION);
    }
    else REJECT;
    ACCEPT;
}

STATE(struct_declaration_list)
    MARK;
    if (!struct_declaration()) REJECT;
    while (struct_declaration())
            ;
    ACCEPT;
}

STATE(struct_declaration)
    MARK;
    if (!specifier_qualifier_list()) REJECT;
    if (!struct_declarator_list()) REJECT;
    if (!mdp_char(';')) REJECT;
    ACCEPT;
}

STATE(specifier_qualifier_list)
    MARK;
    if (!type_specifier() && !type_qualifier()) REJECT;
    while (type_specifier() || type_qualifier())
            ;
    push_op(OP_COMPOSE_TYPE);
    ACCEPT;
}

STATE(struct_declarator_list)
    MARK;
    do {
        if (!struct_declarator()) REJECT;
    } while (mdp_char(','));
    ACCEPT;
}

STATE(struct_declarator)
    MARK;
    if (declarator()) {
        push_op(DCL_MEMBER);
        ACCEPT;
    }
    // Bit fields are NOT implemented...
    REJECT;
}

/*************************** <enum_specifier> ************************
;
; Syntax:
;
;	<enum_specifier> ::=
;		"enum" [<identifier>] "{" <enumerator_list> "}" |
;		"enum" <identifier>
;
;	<enumerator_list> ::= <enumerator> {"," <enumerator>}*
;
;	<enumerator> ::=
;		<enumeration_constant> ["=" <constant_expression>]
;
*/
STATE(enum_specifier)
    MARK;
    if (!mdp_keyword("enum")) REJECT;
    push_op(DCL_ENUM);
    if (identifier()) {
        // Tag is present, so the <enumerator_list> is optional.
        if (mdp_char('{')) {
            if (!enumerator_list()) REJECT;
            if (!mdp_char('}')) REJECT;
        }
    } else {
        // No tag, so MUST have an <enumerator_list>.
        if (!mdp_char('{')) REJECT;
        if (!enumerator_list()) REJECT;
        if (!mdp_char('}')) REJECT;
    }
    ACCEPT;
}

STATE(enumerator_list)
    MARK;
    do {
        if (!enumerator()) REJECT;
        push_op(DCL_MEMBER);
    } while (mdp_char(','));
    ACCEPT;
}

STATE(enumerator)
    MARK;
    if (!enumeration_constant()) REJECT;
    if (mdp_char('=')) {
        if (!constant_expression()) REJECT;
    }
    ACCEPT;
}

/*************************** <type_qualifier> ************************
;
; Syntax:
;
;	<type_qualifier> ::=
;		"const" | "volatile"
;
*/
STATE(type_qualifier)
    MARK;
    if (mdp_keyword("const")) {
        push_op(DCL_CONST);
    }
    else if (mdp_keyword("volatile")) {
        push_op(DCL_VOLATILE);
    }
    else REJECT;
    ACCEPT;
}

/*************************** <declarator> ************************
;
; Syntax:
;
#if 0
;	<declarator> ::= [<pointer>] <direct_declarator>
#else
;   <declarator> ::=
;       "*" <qualified_declarator> |
;       <direct_declarator>
;
;   <qualified_declarator> ::=
;       <type_qualifier> <qualified_declarator> |
;       <declarator>
#endif
;
;	<direct_declarator> ::=
;		<identifier> |
;		"(" <declarator> ")" |
;		<direct_declarator> "[" [<array_dimension_list>] "]" |  // FM extension
;		<direct_declarator> "(" [<parameter_type_list>] ")"     // FM change
;
;	<pointer> ::= {"*" [<type_qualifier_list>]}+
;
;	<type_qualifier_list> ::= {<type_qualifier>}+
;
;   <array_dimension_list> ::= <conditional_expression> {"," <conditional_expression>}*
;
;	<parameter_type_list> ::= <parameter_list> ["," "..." ]
;
;	<parameter_list> ::=
;		<parameter_declaration> {"," <parameter_declaration>}*
;
;	<parameter_declaration> ::=
;		<declaration_specifiers> <declarator> |
;		<declaration_specifiers> [<abstract_declarator>]
;
*/
#if 0
STATE(declarator)
    MARK;
    pointer();  //optional
    if (direct_declarator()) ACCEPT;
    REJECT;
}
#else
STATE(qualified_declarator)
    MARK;
    if (mdp_keyword("const")) {
        if (!qualified_declarator()) REJECT;
        push_op(DCL_CONST);
    }
    else if (mdp_keyword("volatile")) {
        if (!qualified_declarator()) REJECT;
        push_op(DCL_VOLATILE);
    }
    else if (!declarator1()) REJECT;
    ACCEPT;
}

STATE(declarator1)
    MARK;
    if (mdp_char('*')) {
        if (!qualified_declarator()) REJECT;
        push_op(DCL_POINTER);
    }
    else if (!direct_declarator()) REJECT;
    ACCEPT;
}

STATE(declarator)
    MARK;
    push_op(OP_DUPL);   //duplicate the base type
    if (!declarator1()) REJECT;
    push_op(OP_COMPOSE_DECL);
    ACCEPT;
}
#endif

STATE(direct_declarator)
    char ch;
    MARK;
    if (mdp_char('(')) {
        if (!declarator1()) REJECT;
        if (!mdp_char(')')) REJECT;
    }
    else if (identifier()) {
        push_op(OP_XCHG);   //exchange ID and base type
    }
    else REJECT;
    while (mdp_char(ch='[') || mdp_char(ch='(')) {
        if (ch=='[') {
            if (!array_dimension_list()) {
                push_op(DCL_ARRAYDIMNUL);
            }
            if (!mdp_char(']')) REJECT;
        }
        else {
            assert(ch == '(');
            if (parameter_type_list()) {
            }
            else {
                // FM treats empty parameter list "()"
                // as equivalent to "(void)".
            }
            if (!mdp_char(')')) REJECT;
        }
    }
    ACCEPT;
}

STATE(pointer)
    MARK;
    if (!mdp_char('*')) REJECT;
    push_op(DCL_POINTER);
    type_qualifier_list();  //optional
    while (mdp_char('*')) {
        push_op(DCL_POINTER);
        type_qualifier_list();  //optional
    }
    ACCEPT;
}

STATE(type_qualifier_list)
    MARK;
    if (!type_qualifier()) REJECT;
    while (type_qualifier())
            ;
    ACCEPT;
}

STATE(array_dimension_list)
    MARK;
    do {
        if (!conditional_expression()) REJECT;
        push_op(DCL_ARRAYDIM);
    } while (mdp_char(','));
    ACCEPT;
} /*array_dimension_list*/

STATE(parameter_type_list)
    MARK;
    if (!parameter_declaration()) REJECT;
    push_op(DCL_PARAM);
    while (mdp_char(',')) {
        if (ellipsis()) {
            //push_op(DCL_ELLIPSIS);
            ACCEPT;
        }
        if (!parameter_declaration()) REJECT;
        push_op(DCL_PARAM);
    }
    ACCEPT;
}

#if 0
STATE(parameter_list)
    MARK;
    do {
        if (!parameter_declaration()) REJECT;
        push_op(DCL_PARAM);
    } while (mdp_char(','));
    ACCEPT;
}
#endif

STATE(parameter_declaration)
    MARK;
    if (!declaration_specifiers()) REJECT;
    if (declarator()) ACCEPT;
    abstract_declarator(); //optional
    ACCEPT;
}

/*************************** <type_name> ************************
;
; Syntax:
;
;	<type_name> ::=
;		<specifier_qualifier_list> [<abstract_declarator>]
;
#if 0
;	<abstract_declarator> ::=
;		<pointer> |
;		[<pointer>] <direct_abstract_declarator>
#else
;   <abstract_declarator> ::=
;       "*" <qualified_abstract_declarator> |
;       <direct_abstract_declarator>
;
;   <qualified_abstract_declarator> ::=
;       <type_qualifier> <qualified_abstract_declarator> |
;       [<abstract_declarator>]
#endif
;
;	<direct_abstract_declarator> ::=
;		"(" <abstract_declarator> ")" |
;		[<direct_abstract_declarator>] "[" [<array_dimension_list>] "]" |
#if 0
;		[<direct_abstract_declarator>] "(" <parameter_type_list> ")"
#else
;		[<direct_abstract_declarator>] "(" [<parameter_type_list>] ")"  //FM change
#endif
;
*/
STATE(type_name)
    MARK;
    if (!specifier_qualifier_list()) REJECT;
    abstract_declarator(); //optional
    ACCEPT;
}

#if 0
STATE(abstract_declarator)
    MARK;
    if (pointer()) {
        direct_abstract_declarator();   //optional
        ACCEPT;
    }
    if (!direct_abstract_declarator()) REJECT;
    ACCEPT;
}
#else
STATE(qualified_abstract_declarator)
    MARK;
    if (mdp_keyword("const")) {
        qualified_abstract_declarator(); //can't fail!
        push_op(DCL_CONST);
    }
    else if (mdp_keyword("volatile")) {
        qualified_abstract_declarator(); //can't fail!
        push_op(DCL_VOLATILE);
    }
    else {
        abstract_declarator1();  //optional
    }
    ACCEPT;
}

STATE(abstract_declarator1)
    MARK;
    if (mdp_char('*')) {
        qualified_abstract_declarator(); //can't fail!
        push_op(DCL_POINTER);
    }
    else if (!direct_abstract_declarator()) REJECT;
    ACCEPT;
}

STATE(abstract_declarator)
    MARK;
    if (!abstract_declarator1()) REJECT;
    push_op(OP_COMPOSE_DECL);
    ACCEPT;
}
#endif

STATE(direct_abstract_declarator)
    char ch;
    MARK;
    if (mdp_char('(')) {
        // any ambiguity???
        if (abstract_declarator1()) {
        }
        else if (parameter_type_list()) {
        }
        //else REJECT;  //FM change: parameter_type_list is optional
        if (!mdp_char(')')) REJECT;
    }
    else if (mdp_char('[')) {
        if (!array_dimension_list()) {
            push_op(DCL_ARRAYDIMNUL);
        }
        if (!mdp_char(']')) REJECT;
    }
    else REJECT;
    while (mdp_char(ch='[') || mdp_char(ch='(')) {	//['=' sic]
        if (ch=='[') {
            if (!array_dimension_list()) {
                push_op(DCL_ARRAYDIMNUL);
            }
            if (!mdp_char(']')) REJECT;
        }
        else {
            assert(ch=='(');
            //if (!parameter_type_list()) REJECT;
            parameter_type_list();  //FM change: parameter_type_list is optional
            if (!mdp_char(')')) REJECT;
        }
    }/*while*/
    ACCEPT;
}

/*************************** <typedef_name> ************************
;
; Syntax:
;
;	<typedef_name> ::= <identifier>
;
*/
STATE(typedef_name)
    MARK;
#if DO_TYPEDEF
    //can cause ambiguities!!!
    if (identifier()) ACCEPT;
#endif
    REJECT;
}

/*************************** <initializer> ************************
;
; Syntax:
;
;	<initializer> ::=
;		<assignment_expression> |
;		"{" <initializer_list> [","] "}"
;
;	<initializer_list> ::= <initializer> {"," <initializer>}*
;
*/
STATE(initializer)
    int kind;
    MARK;
    if (mdp_char('{')) {
        if (!initializer_list()) REJECT;
        mdp_char(',');  //optional
        if (!mdp_char('}')) REJECT;
    }
    else if (kind = assignment_expression()) {
        if (kind == LVALUE)
            push_op(OP_LOAD);               /* need RVALUE */
        push_op(OP_STORE);
    }
    else REJECT;
    ACCEPT;
}

STATE(initializer_list)
    MARK;
    do {
        if (!initializer()) REJECT;
    } while (mdp_char(','));
    ACCEPT;
}


// 3.6 Statements

/******************************** <statement> *********************************/
/*                                                                            */
/*  Reference: ANSI Programming Language - C                                  */
/*             ANSI X3.159-1989                                               */
/*             Section A.2.3 Statements                                       */
/*                                                                            */
/*----------------------------------------------------------------------------*
;
; Syntax:
;
;	<statement> ::=
;   $if PROG_FML
;       <FML_specification> [";"] |
;   $endif
;		<labeled_statement> |
;		<compound_statement> |
;		<expression_statement> |
;		<selection_statement> |
;		<iteration_statement> |
;		<jump_statement> |
;       <asm_statement> |
;       <FFP_statement>
;
*/
STATE(statement)
    MARK;
    ////if (prog_type == PROG_FML && FML_specification()) {
    if (prog_type == PROG_FML && FFP_eval_specification()) {    //QUICK KLUDGE!!
        mdp_char(';');  //optional
        ACCEPT;
    }
    else if (compound_statement()) ACCEPT;
    else if (selection_statement()) ACCEPT;
    else if (iteration_statement()) ACCEPT;
    else if (jump_statement()) ACCEPT;
    else if (labeled_statement()) ACCEPT;
    else if (expression_statement()) ACCEPT;
    else if (asm_statement()) ACCEPT;
    else if (FFP_statement()) ACCEPT;
    REJECT;
} /*statement*/

/*************************** <labeled_statement> ************************
;
; Syntax:
;
;	<labeled_statement> ::=
;		<identifier> ":" <statement> |
;		"case" <constant_expression> ":" <statement> |
;		"default" ":" <statement>
;
*/
STATE(labeled_statement)
    MARK;
    if (mdp_keyword("case")) {
        push_op(OP_CASE);
#if 1
        if (!constant_expression()) REJECT;
#else
        // for now...
        if (!C_language_expression()) REJECT; //!! wrong allows case 1,2,3: !!!!!
#endif
        if (!mdp_char(':')) REJECT;
        push_op(OP_CASETEST);
        if (!statement()) REJECT;
        ACCEPT;
    }
    else if (mdp_keyword("default")) {
        push_op(OP_DEFAULT);
        if (mdp_char(':') && statement()) ACCEPT;
        REJECT;
    }
#if 1 //defined(ALLOW_GOTO)
    else if (identifier()) {
        push_op(OP_LABEL);
        if (mdp_char(':') && statement()) ACCEPT;
        REJECT;
    }
#endif
    REJECT;
} /*labeled_statement*/

#ifdef INTERMIX
/*************************** <compound_statement> ************************
;
; Syntax:
;
;	<compound_statement> ::=
;		"{" [<declaration_statement_list>] "}"
;
;	<declaration_statement_list> ::= { <declaration> | <statement> }+
;
*/
STATE(compound_statement)
    MARK;
    if (mdp_char('{')) {
        push_op(OP_BEGIN_SCOPE);
        while (declaration() || statement())
            ;
        push_op(OP_END_SCOPE);
        if (mdp_char('}')) ACCEPT;
    }
    REJECT;
} /*compound_statement*/

#else
/*************************** <compound_statement> ************************
;
; Syntax:
;
;	<compound_statement> ::=
;   $if PROG_FML
;		"{" {<declaration>|<statement>}* "}"
;   $else
;		"{" [<declaration_list>] [<statement_list>] "}"
;   $endif
;
*/
STATE(compound_statement)
    MARK;
    if (mdp_char('{')) {
        push_op(OP_BEGIN_SCOPE);
        if (prog_type == PROG_FML) {
            while (declaration() || statement())
                ;
        }
        else {
            declaration_list();     //optional
            statement_list();       //optional
        }
        push_op(OP_END_SCOPE);
        if (mdp_char('}')) ACCEPT;
    }
    REJECT;
} /*compound_statement*/

/********************* <non_scoping_compound_statement> ******************
;
; Syntax:
;
;	<non_scoping_compound_statement> ::=
;               // Same as <compound_statement>, but without
;               // generating a scope around it; for use by the
;               // C++ style <for_statement>, in which the scope
;               // must also include declarations within the
;               // for-expressions, not just the subsidiary 
;               // <statement> body.
;
*/
STATE(non_scoping_compound_statement)
    MARK;
    if (mdp_char('{')) {
        if (prog_type == PROG_FML) {
            while (declaration() || statement())
                ;
        }
        else {
            declaration_list();     //optional
            statement_list();       //optional
        }
        if (mdp_char('}')) ACCEPT;
    }
    REJECT;
} /*non_scoping_compound_statement*/

/*
;	<declaration_list> ::= {<declaration>}+
;
;	<statement_list> ::= {<statement>}+
;
*/
STATE(declaration_list)
    MARK;
    if (declaration()) {
        while (declaration())
            ;
        ACCEPT;
    }
    REJECT;
} /*declaration_list*/

STATE(statement_list)
    MARK;
    if (statement()) {
        while (statement())
            ;
        ACCEPT;
    }
    REJECT;
} /*statement_list*/

#endif

/*************************** <expression_statement> ************************
;
; Syntax:
;
;	<expression_statement> ::= [<expression>] ";"
;
*/
STATE(expression_statement)
    MARK;
    if (C_language_expression()) {
        //don't need the op_load??? In that case, call expression() instead.
        //--but for now, FFP needs the load to return the final value
        //in a channel expression...
        // But will that sometimes omit invoking side effects???
        push_op(OP_POP);
    }
    if (mdp_char(';')) ACCEPT;
    REJECT;
} /*expression_statement*/

/*************************** <selection_statement> ************************
;
; Syntax:
;
;	<selection_statement> ::=
;		"if" "(" <expression> ")" <statement> ["else" <statement>] |
;		"switch" "(" <expression> ")" <statement>
#if 0   ///// Does C++ allow the following selection statements?
;       "if "(" <declaration-no-;> ")" <statement1> ["else" <statement2>]
;   where the scope of <declaration-no-;> spans <statement1> and <statement2>
;   and also
;       "switch" "(" <declaration-no-;> ")" <statement>
#endif
;
*/
STATE(selection_statement)
    MARK;
    if (mdp_keyword("if")) {
        if (mdp_char('(') && C_language_expression() && mdp_char(')')) {
            push_op(OP_THEN);
            if (!statement()) REJECT;
            if (mdp_keyword("else")) {
                push_op(OP_ELSE);
                if (!statement()) REJECT;
            }
            push_op(OP_IF);
            ACCEPT;
        }
    }
    else if (mdp_keyword("switch")) {
        if (mdp_char('(') && C_language_expression() && mdp_char(')')) {
            push_op(OP_SWITCH);
            if (!statement()) REJECT;
            push_op(OP_ENDSW);
            ACCEPT;
        }
    }
    REJECT;
} /*selection_statement*/

/*************************** <iteration_statement> ************************
;
; Syntax:
;
;	<iteration_statement> ::=
;		"while" "(" <expression> ")" <statement> |
#if 0   //////// DOES C++ or C99 allow the following <while>? /////////////
;       "while" "(" <declaration-no-;> ")" <statement> |
#endif
;		"do" <statement> "while" "(" <expression> ")" ";" |
;		"for" "(" [<expression>] ";" [<expression>] ";" [<expression>] ")" <statement>
;		"for" "(" {<declaration> | [<expression>] ";"} [<expression>] ";" [<expression>] ")" <statement>
;
*/
STATE(iteration_statement)
    MARK;
    if (mdp_keyword("for")) {
        if (!mdp_char('(')) REJECT;
        push_op(OP_BEGIN_SCOPE);
        if (declaration()) {
            //optional declaration (includes trailing ";")
        }
        else if (C_language_expression()) {
            //optional init expression
            push_op(OP_POP);
			if (!mdp_char(';')) REJECT;
        }
        else if (!mdp_char(';')) REJECT;
        push_op(OP_FOR1);
        if (C_language_expression()) {
            //optional test
            push_op(OP_BREAKF);
        }
        if (!mdp_char(';')) REJECT;
        push_op(OP_FOR2);
        if (C_language_expression()) {
            //optional iterator
            push_op(OP_POP);
        }
        if (!mdp_char(')')) REJECT;
        push_op(OP_FOR3);
        if (non_scoping_compound_statement()) {
            // okay
        }
        else if (statement()) {
            // okay - a bit redundant, since <statement> also
            // tries to parse a <compound_statement>, but harmless
            // since the compound statement will already have been
            // picked up by <non_scoping_compound_statement>.
        }
        else REJECT;
        push_op(OP_FOR4);
        push_op(OP_END_SCOPE);
        ACCEPT;
    }
    else if (mdp_keyword("while")) {
        push_op(OP_FOR1);               // WHILE is a degenerate FOR
        if (mdp_char('(') && C_language_expression() && mdp_char(')')) {
            push_op(OP_BREAKF);
            push_op(OP_FOR2);
            push_op(OP_FOR3);
            if (!statement()) REJECT;
            push_op(OP_FOR4);
            ACCEPT;
        }
    }
    else if (mdp_keyword("do")) {
        push_op(OP_DO1);
        if (!statement()) REJECT;
        if (!mdp_keyword("while")) REJECT;
        if (!mdp_char('(')) REJECT;
        push_op(OP_DO2);
        if (!C_language_expression()) REJECT;
        if (!mdp_char(')')) REJECT;
        if (!mdp_char(';')) REJECT;
        push_op(OP_DO3);
        ACCEPT;
    }
    REJECT;
} /*iteration_statement*/

/*************************** <jump_statement> ************************
;
; Syntax:
;
;	<jump_statement> ::=
;		"goto" <identifier> ";" |
;		"continue" ";" |
;		"break" ";" |
;		"return" [<expression>] ";"
;
*/
STATE(jump_statement)
    MARK;
    if (mdp_keyword("return")) {
        C_language_expression();    //optional return value
        if (!mdp_char(';')) REJECT;
        push_op(OP_RETURN);
        ACCEPT;
    }
    else if (mdp_keyword("break")) {
        if (!mdp_char(';')) REJECT;
        push_op(OP_BREAK);
        ACCEPT;
    }
    else if (mdp_keyword("continue")) {
        if (!mdp_char(';')) REJECT;
        push_op(OP_CONTINUE);
        ACCEPT;
    }
#if 1 //defined(ALLOW_GOTO)
    else if (mdp_keyword("goto")) {
        if (!identifier()) REJECT;
        if (!mdp_char(';')) REJECT;
        push_op(OP_GOTO);
        ACCEPT;
    }
#endif
    REJECT;
} /*jump_statement*/

/*************************** <asm_statement> ************************
;
; Syntax:
;
;	<asm_statement> ::=
;		"__asm" "{" <asm_instruction> {<eol> <asm_instruction>}* "}"
;
;   <eol> ::= <cr> | <lf>
;
*/
STATE(asm_statement)
    MARK;
    if (mdp_keyword("__asm")) {
        if (!mdp_char('{')) REJECT;
        do (asm_instruction()) ; while (eol());
        if (!mdp_char('}')) REJECT;
        ACCEPT;
    }
    REJECT;
} /*asm_statement*/

STATE(eol)
    MARK;
    if (mdp_char('\r')) ACCEPT;
    if (mdp_char('\n')) ACCEPT;
    REJECT;
} /*eol*/

/*************************** <asm_instruction> ************************
;
; Syntax:
;
;	<asm_instruction> ::= [<x86_instruction>] [';' <asm_comment>]
;
;   <asm_comment> ::= {any char except <cr>, <lf>, or <EOS>}*
;
*/
STATE(asm_instruction)
    MARK;
    x86_instruction();
    if (mdp_char(';'))
    {   //scan over asm-style comment
        while (inp[0] != '\r' && inp[0] != '\n' && inp[0] != 0)
            inp++;
    }
    ACCEPT;
} /*asm_instruction*/

/*************************** <x86_instruction> ************************
;
; Syntax:
;
;	<x86_instruction> ::= 
;       <x86_opcode> [<x86_operand>
;                    [',' <x86_operand>
;                    [',' <x86_operand>]]]    //for now
;
;
*/
STATE(x86_instruction)
    MARK;
    if (!x86_opcode()) REJECT;
    if (x86_operand()) {
        if (mdp_char(',')) {
            if (x86_operand()) {
                if (mdp_char(',')) {
                    if (x86_operand()) ACCEPT;
                    else REJECT;
                }
            }
            else REJECT;
        }
    }
    ACCEPT;
} /*x86_instruction*/

/*************************** <x86_opcode> ************************
;
; Syntax:
;
;	<x86_opcode> ::= 'add' | 'sub' | 'mov' | 'xor' | 'ret' | 'hlt'  //for now
;
;
*/
STATE(x86_opcode)
    MARK;
    if (mdp_keywordi("add")) {
    }
    else if (mdp_keywordi("sub")) {
    }
    else if (mdp_keywordi("mov")) {
    }
    else if (mdp_keywordi("xor")) {
    }
    else if (mdp_keywordi("ret")) {
    }
    else if (mdp_keywordi("hlt")) {
    }
    else REJECT;
    ACCEPT;
} /*x86_opcode*/

/*************************** <x86_operand> ************************
;
; Syntax:
;
;	<x86_operand> ::=   <x86_accumulator> | <x86_segreg> |
;                       <x86_reg8> | <x86_reg16> | <x86_reg32 |
;                       <x86_imm8> | <x86_imm16> | <x86_imm32> |
;                       <x86_rm8> | <x86_rm16> | <x86_rm32> |
;                       <x86_rel8> |
;                       <x86_ptr16> | <x86_ptr32> | <x86_offs16> | <x86_offs32>
;                       
;
;
*/
STATE(x86_operand)
    MARK;
    if (x86_accumulator()) {
    }
    else if (x86_reg32()) {
    }
    else if (x86_reg16()) {
    }
    else if (x86_reg8()) {
    }
    else if (x86_segreg()) {
    }
    else REJECT;
    ACCEPT;
} /*x86_operand*/

/*************************** <x86_accumulator> ************************
;
; Syntax:
;
;	<x86_accumulator> ::=  ['%']'eax' | ['%']'ax' | ['%']'al'
;
*/
STATE(x86_accumulator)
    MARK;
    skip_whitespace = FALSE;
    mdp_char('%');
    if (mdp_keywordi("eax")) {
    }
    else if (mdp_keywordi("ax")) {
    }
    else if (mdp_keywordi("al")) {
    }
    else REJECT;
    ACCEPT;
} /*x86_accumulator*/

/*************************** <x86_reg32> ************************
;
; Syntax:
;
;	<x86_reg32> ::=  ['%']'eax' | ['%']'ebx' | ['%']'ecx' ['%']'edx' |
;	                 ['%']'esi' | ['%']'edi' | ['%']'ebp' ['%']'esp'
;
;
*/
STATE(x86_reg32)
    MARK;
    skip_whitespace = FALSE;
    mdp_char('%');
    if (mdp_keywordi("eax")) {
    }
    else if (mdp_keywordi("ebx")) {
    }
    else if (mdp_keywordi("ecx")) {
    }
    else if (mdp_keywordi("edx")) {
    }
    else if (mdp_keywordi("esi")) {
    }
    else if (mdp_keywordi("edi")) {
    }
    else if (mdp_keywordi("ebp")) {
    }
    else if (mdp_keywordi("esp")) {
    }
    else REJECT;
    ACCEPT;
} /*x86_reg32*/

/*************************** <x86_reg16> ************************
;
; Syntax:
;
;	<x86_reg16> ::=  ['%']'ax' | ['%']'bx' | ['%']'cx' ['%']'dx' |
;	                 ['%']'si' | ['%']'di' | ['%']'bp' ['%']'sp'
;
;
*/
STATE(x86_reg16)
    MARK;
    skip_whitespace = FALSE;
    mdp_char('%');
    if (mdp_keywordi("ax")) {
    }
    else if (mdp_keywordi("bx")) {
    }
    else if (mdp_keywordi("cx")) {
    }
    else if (mdp_keywordi("dx")) {
    }
    else if (mdp_keywordi("si")) {
    }
    else if (mdp_keywordi("di")) {
    }
    else if (mdp_keywordi("bp")) {
    }
    else if (mdp_keywordi("sp")) {
    }
    else REJECT;
    ACCEPT;
} /*x86_reg16*/

/*************************** <x86_reg8> ************************
;
; Syntax:
;
;	<x86_reg8> ::=  ['%']'ah' | ['%']'al' | ['%']'bh' ['%']'bl' |
;	                ['%']'ch' | ['%']'cl' | ['%']'dh' ['%']'dl'
;
;
*/
STATE(x86_reg8)
    MARK;
    skip_whitespace = FALSE;
    mdp_char('%');
    if (mdp_keywordi("ah")) {
    }
    else if (mdp_keywordi("al")) {
    }
    else if (mdp_keywordi("bh")) {
    }
    else if (mdp_keywordi("bl")) {
    }
    else if (mdp_keywordi("ch")) {
    }
    else if (mdp_keywordi("cl")) {
    }
    else if (mdp_keywordi("dh")) {
    }
    else if (mdp_keywordi("dl")) {
    }
    else REJECT;
    ACCEPT;
} /*x86_reg8*/

/*************************** <x86_segreg> ************************
;
; Syntax:
;
;	<x86_segreg> ::=  ['%']'cs' | ['%']'ss' | ['%']'ds' ['%']'es' |
;	                  ['%']'fs' | ['%']'gs'
;
;
*/
STATE(x86_segreg)
    MARK;
    skip_whitespace = FALSE;
    mdp_char('%');
    if (mdp_keywordi("ss")) {
    }
    else if (mdp_keywordi("cs")) {
    }
    else if (mdp_keywordi("ds")) {
    }
    else if (mdp_keywordi("es")) {
    }
    else if (mdp_keywordi("fs")) {
    }
    else if (mdp_keywordi("gs")) {
    }
    else REJECT;
    ACCEPT;
} /*x86_segreg*/


/*************************** <FFP_statement> ************************
;
; Syntax:
;
;	<FFP_statement> ::=
;		"_eval_FFP" "{" <FFP_eval_specification>* "}"
;
*/
STATE(FFP_statement)
    MARK;
    if (mdp_keyword("_eval_FFP")) {
        if (!mdp_char('{')) REJECT;
        while (FFP_eval_specification())
            ;
        if (!mdp_char('}')) REJECT;
        ACCEPT;
    }
    REJECT;
} /*FFP_statement*/

/********************* <FFP_eval_specification> **********************
;
; Syntax:
;
;	<FFP_eval_specification> ::=
;           <FFW_key_value_pair_but_not_FFP_key>                |
;           <FFP_channel_list> ":" <FFP_formula>                |   //WRONG ???!!
;		    <organization_specification>			            |
;		    <URL_specification>				                    |
;		    <version_specification>				                |
;           <dialog_specification>                              |
;           <control_specification>
;
*/
STATE(FFP_eval_specification)
    MARK;
    if (FFW_key_value_pair_but_not_FFP_key()) ACCEPT;   //Should exclude 'val' key????
    else if (FFP_channel_list(NULL)) ACCEPT;
    else if (organization_specification()) ACCEPT;
    else if (URL_specification()) ACCEPT;
    else if (version_specification()) ACCEPT;
    else if (control_specification()) ACCEPT;
    else if (dialog_specification()) ACCEPT;
    REJECT;
} /*FFP_eval_specification*/


// 3.7 Global Definitions

/********************** <program_revision_number> *******************
;
; Syntax:
;
;	<program_revision_number> ::= '-' <mdp_decimal> '.' <mdp_decimal>
;
*/
STATE(program_revision_number)
	MARK;
	skip_whitespace = FALSE;
	if (!mdp_char('-')) REJECT;
	if (!mdp_decimal()) REJECT;
	prog_rev_major = (int)mdp_number;
	if (!mdp_char('.')) REJECT;
	if (!mdp_decimal()) REJECT;
	prog_rev_minor = (int)mdp_number;
	ACCEPT;
} /*program_revision_number*/

/*************************** <AFS_program> ************************
;
; Syntax:
;
;	<AFS_program> ::= '%RGB-1.0'
;                     <preset>          //slider 0
;                     <preset>          //slider 1
;                     <preset>          //slider 2
;                     <preset>          //slider 3
;                     <preset>          //slider 4
;                     <preset>          //slider 5
;                     <preset>          //slider 6
;                     <preset>          //slider 7
;                     <C_language_expression> <p>
;                     <C_language_expression> <p>
;                     <C_language_expression> <p>
;                     <C_language_expression> [<p>]
;                     <end_of_program>
;
;   <preset> ::= <decimal_constant>
;
;   <p> ::= <lf><lf> | <lf><cr>
;
*/
STATE(AFS_program)
    int i;
    MARK;
    prog_type = PROG_AFS;
	prog_rev_major = INT_MAX;	//max available
	prog_rev_minor = INT_MAX;	//max available
	//optional program header
    if (!mdp_keywordi("%RGB")) REJECT;
	//optional revision number
	program_revision_number();
	//push an SD_PROG item onto the SDL
	push_constant(SD_PROG);
	push_param(prog_type);
	push_param(prog_rev_major);
	push_param(prog_rev_minor);

    // 8 slider presets
    for (i=0; i<8; i++) {
        push_proc_addr(setCtlVal);      //<setCtlVal>
        push_constant(LIT_INTEGER);
        push_param(i);                  //<index>
        push_op(OP_DUPL);               //duplicate <index>
        push_proc_addr(initCtl);        //<initCtl>
        push_op(OP_XCHG);               //exchange <index>, <initCtl>
        push_op(OP_ARG);                //push arg <index>
        push_constant(LIT_INTEGER);
        push_constant(1);               //nargs=1
        push_op(OP_CALL);               //call initCtl(<index>)
        push_op(OP_POP);                //discard return value
        push_op(OP_ARG);                //push arg <index>
        if (!decimal_constant()) REJECT;//<val>
        push_op(OP_ARG);                //push arg <val>
        push_constant(LIT_INTEGER);
        push_constant(2);               //nargs=2
        push_op(OP_CALL);               //call setCtlVal(<index>, <val>)
        push_op(OP_POP);                //discard return value
    } //for

    //RGBA formulas
    check_for_paragraph = TRUE; //turn on checking for paragraph marks
    while (mdp_paragraph())
        ;   //skip over any extra paragraph marks

    push_op(OP_SETCHAN_R);
    if (!C_language_expression()) REJECT;
    push_op(OP_SETVAL_R);

    if (!mdp_paragraph()) REJECT;
    while (mdp_paragraph())
        ;   //skip over any extra paragraph marks

    push_op(OP_SETCHAN_G);
    if (!C_language_expression()) REJECT;
    push_op(OP_SETVAL_G);

    if (!mdp_paragraph()) REJECT;
    while (mdp_paragraph())
        ;   //skip over any extra paragraph marks

    push_op(OP_SETCHAN_B);
    if (!C_language_expression()) REJECT;
    push_op(OP_SETVAL_B);

    if (!mdp_paragraph()) REJECT;
    while (mdp_paragraph())
        ;   //skip over any extra paragraph marks

    push_op(OP_SETCHAN_A);
    if (!C_language_expression()) REJECT;
    push_op(OP_SETVAL_A);

    while (mdp_paragraph())
        ;   //skip over any extra paragraph marks
    check_for_paragraph = FALSE; //turn off checking for paragraph marks

    if (end_of_program()) ACCEPT;
    REJECT;
} /*AFS_program*/

/*************************** <FFW_program> ************************
;
; Syntax:
;
;	<FFW_program> ::= [ {'%FFW'|'=FF='} [<program_revision_number>] ]	//optional header
;                     { <FFW_key_value_pair> }+
;                     <end_of_program>
;
;   <FFW_key_value_pair> ::=
;           'Category'       ":" <text_field>                   |
;           {'Title'|'Name'} ":" <text_field>                   |
;           'Copyright'      ":" <text_paragraph>               |
;           'Author'         ":" <text_field>                   |
;           'Filename'       ":" <text_field>                   |
;           'Description'    ":" <text_paragraph>               |
;           'R'              ":" <C_language_expression>        |
;           'G'              ":" <C_language_expression>        |
;           'B'              ":" <C_language_expression>        |
;           'A'              ":" <C_language_expression>        |
;           'ctl' <index>    ":" <text_field>                   |
;           'val' <index>    ":" <preset>
;
;   <text_field> ::= <string_literal> |
;                    <bare_text_field>          //obsolescent
;
;   <text_paragraph> ::= <string_literal> |
;                        <bare_text_paragraph>  //obsolescent
;
;   <bare_text_field> ::= {<char>}* <eol>
;
;   <bare_text_paragraph> ::= {<char>}* <p>
;
;   <index> ::= "[" <integer_constant> "]"  |
;               "(" <integer_constant> ")"      //obsolescent
;
*/
STATE(FFW_program)
    MARK;
    prog_type = PROG_FFW;
	prog_rev_major = INT_MAX;	//max available
	prog_rev_minor = INT_MAX;	//max available
	//optional program header
	if (mdp_keywordi("%FFW") || mdp_keywordi("=FF=")) {
		//optional revision number
		program_revision_number();
	}
	//push an SD_PROG item onto the SDL
	push_constant(SD_PROG);
	push_param(prog_type);
	push_param(prog_rev_major);
	push_param(prog_rev_minor);

    if (!FFW_key_value_pair()) REJECT;  //at least one!
    while (FFW_key_value_pair())
        ;
    if (end_of_program()) ACCEPT;
    REJECT;
} /*FFW_program*/

STATE(separator)
    // For historical reasons, we allow '=' in addition to ':'
    // But FML allows only ':' -- else too many ambiguities!
    MARK;
    if (mdp_char(':')) ACCEPT;
    if (prog_type != PROG_FML && mdp_char('=')) ACCEPT;
    REJECT;
} /*separator*/

STATE(FFW_key_value_pair_but_not_FFP_key)
    MARK;
    if (mdp_keywordi("Category")) {
        if (!separator()) REJECT;
        push_proc_addr(strncpy);
        push_var_addr(filterCategoryText);
        push_op(OP_ARG);
        if (!text_field()) REJECT;
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(255);             //max chars to copy
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(3);               //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard result
		// Kludge to set filterCategoryText immediately, since it
		// may be needed *before* the filter initialization code
		// is executed.
		if (gParams->filterCategoryText[0] == '\0') {
			//Copy only first time parsed in case later set dynamically
			strncpy(gParams->filterCategoryText, mdp_string_acc, 255);
			gParams->filterCategoryText[255] = 0;	//ensure NUL-terminated
		}
        ACCEPT;
    }
    else if (mdp_keywordi("Title") | mdp_keywordi("Name")) {
        // setDialogText(strncpy(filterTitleText,<text_field>,255))
        if (!separator()) REJECT;
#if 0
        push_proc_addr(setDialogText);
#endif
        push_proc_addr(strncpy);
        push_var_addr(filterTitleText);
        push_op(OP_ARG);
        if (!text_field()) REJECT;
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(255);             //max chars to copy
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(3);               //nargs
        push_op(OP_CALL);               //strncpy()
#if 0
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);               //nargs
        push_op(OP_CALL);               //setDialogText()
#endif
        push_op(OP_POP);                //discard result
		// Kludge to set filterTitleText immediately, since it
		// may be needed *before* the filter initialization code
		// is executed.
		if (gParams->filterTitleText[0] == '\0') {
			//Copy only first time parsed in case later set dynamically
			strncpy(gParams->filterTitleText, mdp_string_acc, 255);
			gParams->filterTitleText[255] = 0;	//ensure NUL-terminated
		}
        ACCEPT;
    }
    else if (mdp_keywordi("Copyright")) {
        //strncpy(filterCopyrightText,<text_paragraph>,255)
        if (!separator()) REJECT;
        push_proc_addr(strncpy);
        push_var_addr(filterCopyrightText);
        push_op(OP_ARG);
        if (!text_paragraph()) REJECT;
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(255);             //max chars to copy
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(3);               //nargs
        push_op(OP_CALL);
        push_op(OP_POP);
        ACCEPT;
    }
    else if (mdp_keywordi("Author")) {
        if (!separator()) REJECT;
        push_proc_addr(strncpy);
        push_var_addr(filterAuthorText);
        push_op(OP_ARG);
        if (!text_field()) REJECT;
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(255);             //max chars to copy
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(3);               //nargs
        push_op(OP_CALL);
        push_op(OP_POP);
        ACCEPT;
    }
    else if (mdp_keywordi("Filename")) {
        if (!separator()) REJECT;
        push_proc_addr(strncpy);
        push_var_addr(filterFilenameText);
        push_op(OP_ARG);
        if (!text_field()) REJECT;
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(255);             //max chars to copy
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(3);               //nargs
        push_op(OP_CALL);
        push_op(OP_POP);
		// Kludge to set filterFilenameText immediately, since it
		// may be needed *before* the filter initialization code
		// is executed.
		if (gParams->filterFilenameText[0] == '\0') {
			//Copy only first time parsed in case later set dynamically
			strncpy(gParams->filterFilenameText, mdp_string_acc, 255);
			gParams->filterFilenameText[255] = 0;	//ensure NUL-terminated
		}
        ACCEPT;
    }
    else if (mdp_keywordi("Description")) {
        if (!separator()) REJECT;
        push_proc_addr(strncpy);
        push_var_addr(filterDescriptionText);
        push_op(OP_ARG);
        if (!text_paragraph()) REJECT;
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(255);             //max chars to copy
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(3);               //nargs
        push_op(OP_CALL);
        push_op(OP_POP);
        ACCEPT;
    }
    else if (mdp_keywordi("val")) {
        push_proc_addr(setCtlVal);
        if (!index()) REJECT;
        push_op(OP_ARG);
        if (!separator()) REJECT;
        if (!decimal_constant()) REJECT;
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(2);               //nargs
        push_op(OP_CALL);
        push_op(OP_POP);
        ACCEPT;
    }
    else if (mdp_keywordi("UniqueID")) {
        if (!separator()) REJECT;
        push_proc_addr(strncpy);
        push_var_addr(filterUniqueID);
        push_op(OP_ARG);
        if (!text_paragraph()) REJECT;
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(36);             //max chars to copy
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(3);               //nargs
        push_op(OP_CALL);
        push_op(OP_POP);
        ACCEPT;
    }
    REJECT;
} /*FFW_key_value_pair_but_not_FFP_key*/

STATE(FFW_key_value_pair)
    MARK;
    if (FFW_key_value_pair_but_not_FFP_key()) {
        ACCEPT;
    }
    else if (mdp_keywordi("ctl")) {
        push_proc_addr(setCtlText);
        if (!index()) REJECT;
        if (!separator()) REJECT;
        push_op(OP_DUPL);               //duplicate <index>
        push_proc_addr(initCtl);
        push_op(OP_XCHG);               //exchange <index>, <proc_addr>
        push_op(OP_ARG);                //push arg <index>
        push_constant(LIT_INTEGER);
        push_constant(1);               //nargs
        push_op(OP_CALL);               //call initCtl(<index>)
        push_op(OP_POP);                //discard return value
        push_op(OP_ARG);                //push arg <index>
        if (!text_field()) REJECT;
        push_op(OP_ARG);                //push arg <text_field>
        push_constant(LIT_INTEGER);
        push_constant(2);               //nargs
        push_op(OP_CALL);               //call setCtltext(<index>, <text_field>)
        push_op(OP_POP);                //discard return value
        ACCEPT;
    }
    else if (mdp_keywordi("R")) {
        if (!separator()) REJECT;
        push_op(OP_SETCHAN_R);
        if (!C_language_expression()) REJECT;
        push_op(OP_SETVAL_R);
        ACCEPT;
    }
    else if (mdp_keywordi("G")) {
        if (!separator()) REJECT;
        push_op(OP_SETCHAN_G);
        if (!C_language_expression()) REJECT;
        push_op(OP_SETVAL_G);
        ACCEPT;
    }
    else if (mdp_keywordi("B")) {
        if (!separator()) REJECT;
        push_op(OP_SETCHAN_B);
        if (!C_language_expression()) REJECT;
        push_op(OP_SETVAL_B);
        ACCEPT;
    }
    else if (mdp_keywordi("A")) {
        if (!separator()) REJECT;
        push_op(OP_SETCHAN_A);
        if (!C_language_expression()) REJECT;
        push_op(OP_SETVAL_A);
        ACCEPT;
    }
    REJECT;
} /*FFW_key_value_pair*/

STATE(text_field)
    MARK;
    if (string_literal()) ACCEPT;
    if (bare_text_field()) ACCEPT;
    REJECT;
} /*text_field*/

STATE(text_paragraph)
    MARK;
    if (string_literal()) ACCEPT;
#if 0
    if (bare_text_paragraph()) ACCEPT;
#else
    if (bare_text_field()) ACCEPT;
#endif
    REJECT;
} /*text_paragraph*/

STATE(bare_text_field)
    MARK;
    init_string_acc();
    while (*inp != '\r' && *inp != '\n' && *inp != 0) {
        add_char_to_acc(*inp++);
    }
    add_char_to_acc(0);  /* Add a terminating NUL? */		//ERROR IF THIS CALL FAILS!!!
    push_char_sd();
    ACCEPT;
} /*bare_text_field*/

STATE(bare_text_paragraph)
    MARK;
    init_string_acc();
    while (*inp != 0 &&
           !(inp[0] == '\n' && (inp[1] == '\r' || inp[1] == '\n')))
    {
        if (*inp != '\r') {
            // embedded <lf>'s are copied to string, but not <cr>'s.
            add_char_to_acc(*inp);
        }
        if (*inp == '\n') {
            //Strip off whitespace after <lf>...
            while ((*inp & 0x7f) == ' ' || *inp == '\t' || *inp == '\f' || *inp == '\v')
                inp++;
        }
        inp++;
    }
    add_char_to_acc(0);  /* Add a terminating NUL? */
    push_char_sd();
    ACCEPT;
} /*bare_text_paragraph*/

STATE(index)
    MARK;
    if (mdp_char('[')) {
        if (!constant_integer_expression()) REJECT;
        if (!mdp_char(']')) REJECT;
        ACCEPT;
    }
    else if (mdp_char('(')) {
        if (!constant_integer_expression()) REJECT;
        if (!mdp_char(')')) REJECT;
        ACCEPT;
    }
    REJECT;
} /*index*/


/*************************** <FFP_program> ************************
;
; Syntax:
;
;	<FFP_program> ::= [ '%FFP' [<program_revision_number>] ]	//optional header
;                     { <FFP_key_value_pair> }+
;                     <end_of_program>
;
*/
STATE(FFP_program)
    MARK;
    prog_type = PROG_FFP;
	prog_rev_major = INT_MAX;	//max available
	prog_rev_minor = INT_MAX;	//max available
	//optional program header
	if (mdp_keywordi("%FFP")) {
		//optional revision number
		program_revision_number();
	}
	//push an SD_PROG item onto the SDL
	push_constant(SD_PROG);
	push_param(prog_type);
	push_param(prog_rev_major);
	push_param(prog_rev_minor);

    if (!FFP_key_value_pair()) REJECT;  //at least one!
    while (FFP_key_value_pair())
        ;
    if (end_of_program()) ACCEPT;
    REJECT;
} /*FFP_program*/

/*
;   <FFP_key_value_pair> ::=
;           <FFW_key_value_pair>                                |
;           <FFP_channel_list> ":" <FFP_formula>                |
;		    <organization_specification>				        |
;		    <URL_specification>				                    |
;		    <version_specification>				                |
;		    <PIKind_specification>				                |
;		    <PIVersion_specification>			                |
;		    <PIPriority_specification>			                |
;		    <supported_modes_specification>		                |
;		    <enable_info_specification>			                |
;		    <required_host_specification>		                |
;		    <filter_case_info_specification>		            |
;		    <has_terminology_specification>		                |
;		    <PiMI_specification>					            |
;		    <PiPL_specification>					            |
;           <preset_specification>                              |
;           <event_handler_specification>                       |
;           <dialog_specification>                              |
;           <about_specification>                               |
;           <embed_specification>                               |
;           <control_specification>                             |
;           <debug_specification>
;
*/
STATE(FFP_key_value_pair)
    MARK;
         if (FFW_key_value_pair_but_not_FFP_key()) ACCEPT;
    else if (FFP_channel_list(NULL)) ACCEPT;
    else if (organization_specification()) ACCEPT;
    else if (URL_specification()) ACCEPT;
    else if (version_specification()) ACCEPT;
#ifndef BETA
    else if (PIKIND_specification()) ACCEPT;
    else if (PIVersion_specification()) ACCEPT;
    else if (PIPriority_specification()) ACCEPT;
    else if (supported_modes_specification()) ACCEPT;
    else if (enable_info_specification()) ACCEPT;
    else if (required_host_specification()) ACCEPT;
    else if (filter_case_info_specification()) ACCEPT;
    else if (has_terminology_specification()) ACCEPT;
    else if (PiMI_specification()) ACCEPT;
    else if (PiPL_specification()) ACCEPT;
#endif //ifndef BETA
    else if (event_handler_specification()) ACCEPT;
    else if (control_specification()) ACCEPT;
    else if (dialog_specification()) ACCEPT;
    else if (about_specification()) ACCEPT;
    else if (embed_specification()) ACCEPT;
    else REJECT;
    ACCEPT;
} /*FFP_key_value_pair*/

/*
;	<FFP_channel_list>:
;		<FFP_channel> {"," <FFP_channel>}*
;
;	<FFP_channel>:
;			'R' |		//Red channel
;			'G' |		//Green channel
;		    'B' |		//Blue channel
;			'A' |		//Alpha channel
;			'S'			//Selection mask channel
;
;   <FFP_formula>:
;       <C_language_expression>    |
;       <compound_statement>
;
*/
static void do_channel_list(channel_node *list)
{
    if (list) {
        do_channel_list(list->prev); //depth first
        push_op(list->c);                                //OP_SETCHAN_x
        push_op(OP_DUMP_CHANNEL_CODE);
        //WARNING: Following assumes contiguous operator values!!!
        push_op(list->c + (OP_SETVAL_R - OP_SETCHAN_R)); //OP_SETVAL_x
    }
} /*do_channel_list*/

static int FFP_channel(void)
{
    char *state_name = "FFP_channel";
    MARK;
    if (mdp_keywordi("R")) ACCEPTX(OP_SETCHAN_R);
    if (mdp_keywordi("G")) ACCEPTX(OP_SETCHAN_G);
    if (mdp_keywordi("B")) ACCEPTX(OP_SETCHAN_B);
    if (mdp_keywordi("A")) ACCEPTX(OP_SETCHAN_A);
    if (mdp_keywordi("S")) ACCEPTX(OP_SETCHAN_S);
    REJECT;
} /*FFP_channel*/

static int FFP_channel_list(channel_node *list)
{
    char *state_name = "FFP_channel_list";
    channel_node cn;
    MARK;
    if (!(cn.c = FFP_channel())) REJECT;
    cn.prev = list;
    if (mdp_char(',')) {
        if (!FFP_channel_list(&cn)) REJECT;
        ACCEPT;
    }
    if (!separator()) REJECT;
    push_op(OP_SAVE_CHANNEL_CODE);
    if (compound_statement() || C_language_expression()) {
        push_op(OP_END_CHANNEL_CODE);
        do_channel_list(&cn);
        ACCEPT;
    }
    REJECT;
} /*FFP_channel_list*/

/*
;	<organization_specification>:
;		'Organization' ":" <text_field>
;
*/
STATE(organization_specification)
    MARK;
        if (!mdp_keywordi("Organization")) REJECT;
        if (!separator()) REJECT;
        push_proc_addr(strncpy);
        push_var_addr(filterOrganizationText);
        push_op(OP_ARG);
        if (!text_field()) REJECT;
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(255);             //max chars to copy
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(3);               //nargs
        push_op(OP_CALL);
        push_op(OP_POP);
    ACCEPT;
} /*organization_specification*/

/*
;	<URL_specification>:
;		'URL' ":" <text_field>
;
*/
STATE(URL_specification)
    MARK;
        if (!mdp_keywordi("URL")) REJECT;
        if (!separator()) REJECT;
        push_proc_addr(strncpy);
        push_var_addr(filterURLText);
        push_op(OP_ARG);
        if (!text_field()) REJECT;
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(255);             //max chars to copy
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(3);               //nargs
        push_op(OP_CALL);
        push_op(OP_POP);
    ACCEPT;
} /*URL_specification*/

/*
;	<version_specification>:
;		'Version' ":" <text_field>
;
*/
STATE(version_specification)
    MARK;
        if (!mdp_keywordi("Version")) REJECT;
        if (!separator()) REJECT;
        push_proc_addr(strncpy);
        push_var_addr(filterVersionText);
        push_op(OP_ARG);
        if (!text_field()) REJECT;
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(255);             //max chars to copy
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(3);               //nargs
        push_op(OP_CALL);
        push_op(OP_POP);
    ACCEPT;
} /*version_specification*/

#ifndef BETA

/*
;	<PIKind_specification>:
;		'PIKind' ":" <plug-in_kind>
;
;	<plug-in_kind>:
;		'Filter'		|	// Filter module (default)
;		'Parser'		|	// Parser module
;		'ImageFormat'	|	// Format module
;		'Extension'		|	// Accelerator extension
;		'Acquire'		|	// Import (acquisition) module
;		'Export'		|	// Export module
;		'Selection'		|	// Selection module
;		'Picker'		|	// Color picker module
;		<OSType>		|	// to specify the actual OS Type code
;		<integer_constant>	// to specify an arbitrary 32-bit value
;
;	<OSType>:
;		'8BFM'	|			// Filter module (default)
;		'8BYM'	|			// Parser module
;		'8BIF'	|			// Format module
;		'8BXM'	|			// Accelerator extension
;		'8BAM'	|			// Import (acquisition) module
;		'8BEM'	|			// Export module
;		'8BSM'	|			// Selection module
;       '8BCM'              // Color picker module
;
*/
STATE(PIKIND_specification)
    MARK;
    if (!mdp_keywordi("PIKind")) REJECT;
    if (!separator()) REJECT;
    if (!plugin_kind()) REJECT;
    ACCEPT;
} /*PIKIND_specification*/

STATE(plugin_kind)
    MARK;
    if (mdp_keywordi("Filter") | mdp_keywordi("8BFM")) {
    }
    else if (mdp_keywordi("Parser") | mdp_keywordi("8BYM")) {
    }
    else if (mdp_keywordi("ImageFormat") | mdp_keywordi("8BIF")) {
    }
    else if (mdp_keywordi("Extension") | mdp_keywordi("8BXM")) {
    }
    else if (mdp_keywordi("Acquire") | mdp_keywordi("8BAM")) {
    }
    else if (mdp_keywordi("Export") | mdp_keywordi("8BEM")) {
    }
    else if (mdp_keywordi("Selection") | mdp_keywordi("8BSM")) {
    }
    else if (mdp_keywordi("Picker") | mdp_keywordi("8BCM")) {
    }
    else if (constant_integer_expression()) {
    }
    else REJECT;
    ACCEPT;
} /*plugin_kind*/

/*
;	<PIVersion_specification>:
;		'PIVersion' ":" <plug-in_interface_version>
;
;	<plug-in_interface_version>:
;		<constant_integer_expression>	|		//to specify it numerically
;		<wired_version_ID>				|		//predefined versions
;		"(" <wired_version_ID_high> "<<" "16" ")" "|"
;		"(" <wired_version_ID_low> ")"			//predefined major and minor
;
;	<wired_version_ID>:
;		'FilterVersion'			|		//current filter version
;		'ParserVersion'			|		//current parser version
;		'ImageFormatVersion'	|		//current format version
;		'ExtensionVersion'		|		//current extension version
;		'AcquireVersion'		|		//current import version
;		'ExportVersion'			|		//current export version
;		'SelectionVersion'		|		//current selection version
;		'PickerVersion'					//current color picker version
;
;	<wired_version_ID_high>:
;		'latestFilterVersion'		|	//current filter major version
;		'latestParserVersion'		|	//current parser major version
;		'latestImageFormatVersion'	|	//current format major version
;		'latestExtensionVersion'	|	//current extension major version
;		'latestAcquireVersion'		|	//current import major version
;		'latestExportVersion'		|	//current export major version
;		'latestSelectionVersion'	|	//current selection major version
;		'latestPickerVersion'			//current color picker major version
;
;	<wired_version_ID_low>:
;		'latestFilterSubVersion'		|	//current filter minor version
;		'latestParserSubVersion'		|	//current parser minor version
;		'latestImageFormatSubVersion'	|	//current format minor version
;		'latestExtensionSubVersion'		|	//current extension minor version
;		'latestAcquireSubVersion'		|	//current import minor version
;		'latestExportSubVersion'		|	//current export minor version
;		'latestSelectionSubVersion'		|	//current selection minor version
;		'latestPickerSubVersion'			//current color picker minor version
;
*/
STATE(PIVersion_specification)
    MARK;
    if (!mdp_keywordi("PIVersion")) REJECT;
    if (!separator()) REJECT;
    if (mdp_char('(')) {
        if (mdp_keywordi("latestFilterVersion")) {
        }
        else if (mdp_keywordi("latestParserVersion")) {
        }
        else if (mdp_keywordi("latestImageFormatVersion")) {
        }
        else if (mdp_keywordi("latestExtensionVersion")) {
        }
        else if (mdp_keywordi("latestAcquireVersion")) {
        }
        else if (mdp_keywordi("latestExportVersion")) {
        }
        else if (mdp_keywordi("latestSelectionVersion")) {
        }
        else if (mdp_keywordi("latestPickerVersion")) {
        }
        else REJECT;
        if (!left_shift_operator()) REJECT;
        if (!mdp_keywordi("16")) REJECT;
        if (!mdp_char(')')) REJECT;
        if (!or_operator()) REJECT;
        if (!mdp_char('(')) REJECT;
        if (mdp_keywordi("latestFilterSubVersion")) {
        }
        else if (mdp_keywordi("latestParserSubVersion")) {
        }
        else if (mdp_keywordi("latestImageFormatSubVersion")) {
        }
        else if (mdp_keywordi("latestExtensionSubVersion")) {
        }
        else if (mdp_keywordi("latestAcquireSubVersion")) {
        }
        else if (mdp_keywordi("latestExportSubVersion")) {
        }
        else if (mdp_keywordi("latestSelectionSubVersion")) {
        }
        else if (mdp_keywordi("latestPickerSubVersion")) {
        }
        else REJECT;
        if (!mdp_char(')')) REJECT;
    }
    else if (mdp_keywordi("FilterVersion")) {
    }
    else if (mdp_keywordi("ParserVersion")) {
    }
    else if (mdp_keywordi("ImageFormatVersion")) {
    }
    else if (mdp_keywordi("ExtensionVersion")) {
    }
    else if (mdp_keywordi("AcquireVersion")) {
    }
    else if (mdp_keywordi("ExportVersion")) {
    }
    else if (mdp_keywordi("SelectionVersion")) {
    }
    else if (mdp_keywordi("PickerVersion")) {
    }
    else if (constant_integer_expression()) {
    }
    else REJECT;
    ACCEPT;
} /*PIVersion_specification*/

/*
;	<PIPriority_specification>:
;		'PIPriority' ":" <constant_integer_expression>
;
*/
STATE(PIPriority_specification)
    MARK;
    if (!mdp_keywordi("PIPriority")) REJECT;
    if (!separator()) REJECT;
    if (!constant_integer_expression()) REJECT;
    ACCEPT;
} /*PIPriority_specification*/

/*
;	<supported_modes_specification>:
;		'SupportedModes' ":" <image_mode_list>
;
;	<image_mode_list>:
;		<image_mode> {"," <image_mode>}*
;
;	<image_mode>:
;		'Bitmap*Mode'		|	//Bitmap mode
;		'Gray*ScaleMode'	|	//Grayscale mode
;		'Indexed*ColorMode'	|	//Indexed color mode
;		'RGB*Mode'			|	//RGB color mode
;		'CMYK*Mode'			|	//CMYK color mode
;		'HSL*Mode'			|	//HSL color mode
;		'HSB*Mode'			|	//HSB color mode
;		'Multichannel*Mode'	|	//Multichannel mode
;		'Duotone*Mode'		|	//Duotone mode
;		'Lab*Mode'			|	//Lab color mode
;		'Gray16*Mode'		|	//Grayscale mode, 16 bits per channel
;		'RGB48*Mode'		|	//RGB color mode, 16 bits per channel
;       'Lab48*Mode'        |   //Lab color mode, 16 bits per channel
;       'CMYK64*Mode'       |   //CMYK color mode, 16 bits per channel
;       'DeepMultichannel*Mode' |   //Multichannel mode, 16 bits per channel
;       'Duotone16*Mode'    |   //Duotone mode, 16 bits per channel
;		'All*Modes'				//Any mode (don't gray out my filter menu!)
;
*/
STATE(supported_modes_specification)
    MARK;
    if (!mdp_keywordi("SupportedModes")) REJECT;
    if (!separator()) REJECT;
    gParams->SupportedModes = 0;
    do {
        if (!image_mode()) REJECT;
    } while (mdp_char(','));
    ACCEPT;
} /*supported_modes_specification*/

STATE(image_mode)
    MARK;
    if (mdp_keywordi("Bitmap*Mode")) {
        gParams->SupportedModes |= FLAG(plugInModeBitmap);
    }
    else if (mdp_keywordi("Gray*ScaleMode")) {
        gParams->SupportedModes |= FLAG(plugInModeGrayScale);
    }
    else if (mdp_keywordi("Indexed*ColorMode")) {
        gParams->SupportedModes |= FLAG(plugInModeIndexedColor);
    }
    else if (mdp_keywordi("RGB*Mode")) {
        gParams->SupportedModes |= FLAG(plugInModeRGBColor);
    }
    else if (mdp_keywordi("CMYK*Mode")) {
        gParams->SupportedModes |= FLAG(plugInModeCMYKColor);
    }
    else if (mdp_keywordi("HSL*Mode")) {
        gParams->SupportedModes |= FLAG(plugInModeHSLColor);
    }
    else if (mdp_keywordi("HSB*Mode")) {
        gParams->SupportedModes |= FLAG(plugInModeHSBColor);
    }
    else if (mdp_keywordi("Multichannel*Mode")) {
        gParams->SupportedModes |= FLAG(plugInModeMultichannel);
    }
    else if (mdp_keywordi("Duotone*Mode")) {
        gParams->SupportedModes |= FLAG(plugInModeDuotone);
    }
    else if (mdp_keywordi("Lab*Mode")) {
        gParams->SupportedModes |= FLAG(plugInModeLabColor);
    }
    else if (mdp_keywordi("Gray16*Mode")) {
        gParams->SupportedModes |= FLAG(plugInModeGray16);
    }
    else if (mdp_keywordi("RGB48*Mode")) {
        gParams->SupportedModes |= FLAG(plugInModeRGB48);
    }


	//Added by Harald Heim, December 6, 2002
	else if (mdp_keywordi("Lab48*Mode")) {
        gParams->SupportedModes |= FLAG(plugInModeLab48);
    }
    else if (mdp_keywordi("CMYK64*Mode")) {
        gParams->SupportedModes |= FLAG(plugInModeCMYK64);
    }
    else if (mdp_keywordi("DeepMultichannel*Mode")) {
        gParams->SupportedModes |= FLAG(plugInModeDeepMultichannel);
    }
    else if (mdp_keywordi("Duotone16*Mode")) {
        gParams->SupportedModes |= FLAG(plugInModeDuotone16);


	}
    else if (mdp_keywordi("All*Modes")) {
        gParams->SupportedModes |= FLAG(plugInModeBitmap)       |
                                   FLAG(plugInModeGrayScale)    |
                                   FLAG(plugInModeIndexedColor) |
                                   FLAG(plugInModeRGBColor)     |
                                   FLAG(plugInModeCMYKColor)    |
                                   FLAG(plugInModeHSLColor)     |
                                   FLAG(plugInModeHSBColor)     |
                                   FLAG(plugInModeMultichannel) |
                                   FLAG(plugInModeDuotone)      |
                                   FLAG(plugInModeLabColor)     |
                                   FLAG(plugInModeGray16)       |
                                   FLAG(plugInModeRGB48)        |
								   //Added by Harald Heim, December 6, 2002
								  FLAG(plugInModeLab48)        |
								  FLAG(plugInModeCMYK64)       |
								  FLAG(plugInModeDeepMultichannel)|
								  FLAG(plugInModeDuotone16);
    }
    else REJECT;
    ACCEPT;
} /*image_mode*/

/*
;	<enable_info_specification>:
;		'EnableInfo' ":" <booleanExpression>		//See Appendix K
;
*/
STATE(enable_info_specification)
    MARK;
    if (!mdp_keywordi("EnableInfo")) REJECT;
    if (!separator()) REJECT;
    gParams->EI_start_index = (int)(inp - inp_start);
    if (!EI_booleanExpression()) REJECT;
    gParams->EI_count = (int)(inp - inp_start) - gParams->EI_start_index;
    gParams->EI_specified = TRUE;
    ACCEPT;
} /*enable_info_specification*/

/*
;	<required_host_specification>:
;		'RequiredHost' ":" <host_specification>
;
;	<host_specification>:
;		"'    '"			|			//no required host
;		<host_signature>	|			//see Appendix L
;		<constant_integer_expression>	//to specify it numerically
;
*/
STATE(required_host_specification)
    MARK;
    if (!mdp_keywordi("RequiredHost")) REJECT;
    if (!separator()) REJECT;
    if (!constant_integer_expression()) REJECT;
    ACCEPT;
} /*required_host_specification*/

/*
;	<filter_case_info_specification>:
;		'FilterCaseInfo' ":" <filter_case_list>
;
;	<filter_case_list>:
;		<filter_case_spec> {";" <filter_case_spec>}*
;
;	<filter_case_spec>:
;		<filter_case_name_list> "=" <filter_handling_list>
;
;	<filter_case_name_list>:
;       'DEFAULT'                           |
;		<filter_case_name> {"," <filter_case_name>}*
;
;	<filter_case_name>:
;		'FlatImageNoSelection'				|
;		'FlatImageWithSelection'			|
;		'FloatingSelection'					|
;		'EditableTransparencyNoSelection'	|
;		'EditableTransparencyWithSelection'	|
;		'ProtectedTransparencyNoSelection'	|
;		'ProtectedTransparencyWithSelection'
;
;	<filter_handling_list>:
;		<filter_handling_item> {"," <filter_handling_item>}*
;
;	<filter_handling_item>:
;		<filter_input_handling_mode>	|
;		<filter_output_handling_mode>	|
;		<filter_handling_property>
;
;	<filter_input_handling_mode>:
;		'inCantFilter'		|
;		'inStraightData'	|
;		'inBlackMat'		|
;		'inGrayMat'			|
;		'inWhiteMat'		|
;		'inDefringe'		|
;		'inBlackZap'		|
;		'inGrayZap'			|
;		'inWhiteZap'		|
;		'inBackgroundZap'	|
;		'inForegroundZap'	
;
;	<filter_output_handling_mode>:
;		'outCantFilter'		|
;		'outStraightData'	|
;		'outBlackMat'		|
;		'outGrayMat'		|
;		'outWhiteMat'		|
;		'outFillMask'
;
;	<filter_handling_property>:
;		'copySourceToDestination'		| 'doNotCopySourceToDestination'	|
;		'doesNotWorkWithBlankData'		| 'worksWithBlankData'				|
;		'doesNotFilterLayerMasks'		| 'filtersLayerMasks'				|
;		'doesNotWriteOutsideSelection'	| 'writesOutsideSelection'
;
*/
STATE(filter_case_info_specification)
    MARK;
    if (!mdp_keywordi("FilterCaseInfo")) REJECT;
    if (!separator()) REJECT;
    do {
        if (!filter_case_spec()) REJECT;
    } while (mdp_char(';'));
    ACCEPT;
} /*filter_case_info_specification*/

STATE(filter_case_spec)
    MARK;
    bitsetFiciCases = 0;
    // set default fici properties...
    fici.inputHandling = filterDataHandlingNone;
    fici.outputHandling = filterDataHandlingNone;
    fici.flags1 = BIT(PIFilterDontCopyToDestinationBit);
    fici.flags2 = 0;    //reserved
    if (mdp_keywordi("DEFAULT")) {
        bitsetFiciCases = -1;
    }
    else {
        do {
            if (!filter_case_name()) REJECT;
        } while (mdp_char(','));
    }
    if (!mdp_char('=')) REJECT;
    do {
        if (!filter_handling_item()) REJECT;
    } while (mdp_char(','));
    {   int i;
        for (i = filterCaseFlatImageNoSelection;
             i <= filterCaseProtectedTransparencyWithSelection;
             ++i)
        {
            if (bitsetFiciCases & BIT(i)) {
                gParams->fici[i-1] = fici;
            }
        }
    }
    ACCEPT;
} /*filter_case_spec*/

STATE(filter_case_name)
    MARK;
    if (mdp_keywordi("FlatImageNoSelection")) {
        bitsetFiciCases |= BIT(filterCaseFlatImageNoSelection);
    }
    else if (mdp_keywordi("FlatImageWithSelection")) {
        bitsetFiciCases |= BIT(filterCaseFlatImageWithSelection);
    }
    else if (mdp_keywordi("FloatingSelection")) {
        bitsetFiciCases |= BIT(filterCaseFloatingSelection);
    }
    else if (mdp_keywordi("EditableTransparencyNoSelection")) {
        bitsetFiciCases |= BIT(filterCaseEditableTransparencyNoSelection);
    }
    else if (mdp_keywordi("EditableTransparencyWithSelection")) {
        bitsetFiciCases |= BIT(filterCaseEditableTransparencyWithSelection);
    }
    else if (mdp_keywordi("ProtectedTransparencyNoSelection")) {
        bitsetFiciCases |= BIT(filterCaseProtectedTransparencyNoSelection);
    }
    else if (mdp_keywordi("ProtectedTransparencyWithSelection")) {
        bitsetFiciCases |= BIT(filterCaseProtectedTransparencyWithSelection);
    }
    else REJECT;
    ACCEPT;
} /*filter_case_name*/

STATE(filter_handling_item)
    MARK;
    if (mdp_keywordi("inCantFilter")) {
        fici.inputHandling = filterDataHandlingCantFilter;
    }
    else if (mdp_keywordi("inStraightData")) {
        fici.inputHandling = filterDataHandlingNone;
    }
    else if (mdp_keywordi("inBlackMat")) {
        fici.inputHandling = filterDataHandlingBlackMat;
    }
    else if (mdp_keywordi("inGrayMat")) {
        fici.inputHandling = filterDataHandlingGrayMat;
    }
    else if (mdp_keywordi("inWhiteMat")) {
        fici.inputHandling = filterDataHandlingWhiteMat;
    }
    else if (mdp_keywordi("inDefringe")) {
        fici.inputHandling = filterDataHandlingDefringe;
    }
    else if (mdp_keywordi("inBlackZap")) {
        fici.inputHandling = filterDataHandlingBlackZap;
    }
    else if (mdp_keywordi("inGrayZap")) {
        fici.inputHandling = filterDataHandlingGrayZap;
    }
    else if (mdp_keywordi("inWhiteZap")) {
        fici.inputHandling = filterDataHandlingWhiteZap;
    }
    else if (mdp_keywordi("inBackgroundZap")) {
        fici.inputHandling = filterDataHandlingBackgroundZap;
    }
    else if (mdp_keywordi("inForegroundZap")) {
        fici.inputHandling = filterDataHandlingForegroundZap;
    }
    else if (mdp_keywordi("outCantFilter")) {
        fici.outputHandling = filterDataHandlingCantFilter;
    }
    else if (mdp_keywordi("outStraightData")) {
        fici.outputHandling = filterDataHandlingNone;
    }
    else if (mdp_keywordi("outBlackMat")) {
        fici.outputHandling = filterDataHandlingBlackMat;
    }
    else if (mdp_keywordi("outGrayMat")) {
        fici.outputHandling = filterDataHandlingGrayMat;
    }
    else if (mdp_keywordi("outWhiteMat")) {
        fici.outputHandling = filterDataHandlingWhiteMat;
    }
    else if (mdp_keywordi("outFillMask")) {
        fici.outputHandling = filterDataHandlingFillMask;
    }
    else if (mdp_keywordi("copySourceToDestination")) {
        fici.flags1 &= ~BIT(PIFilterDontCopyToDestinationBit);
    }
    else if (mdp_keywordi("doNotCopySourceToDestination")) {
        fici.flags1 |= BIT(PIFilterDontCopyToDestinationBit);
    }
    else if (mdp_keywordi("doesNotWorkWithBlankData")) {
        fici.flags1 &= ~BIT(PIFilterWorksWithBlankDataBit);
    }
    else if (mdp_keywordi("worksWithBlankData")) {
        fici.flags1 |= BIT(PIFilterWorksWithBlankDataBit);
    }
    else if (mdp_keywordi("doesNotFilterLayerMasks")) {
        fici.flags1 &= ~BIT(PIFilterFiltersLayerMaskBit);
    }
    else if (mdp_keywordi("filtersLayerMasks")) {
        fici.flags1 |= BIT(PIFilterFiltersLayerMaskBit);
    }
    else if (mdp_keywordi("doesNotWriteOutsideSelection")) {
        fici.flags1 &= ~BIT(PIFilterWritesOutsideSelectionBit);
    }
    else if (mdp_keywordi("writesOutsideSelection")) {
        fici.flags1 |= BIT(PIFilterWritesOutsideSelectionBit);
    }
    else if (mdp_keywordi("doNotWriteOutsideSelection")) { //per Adobe (deprecate!)
        fici.flags1 &= ~BIT(PIFilterWritesOutsideSelectionBit);
    }
    else if (mdp_keywordi("writeOutsideSelection")) {  //per Adobe (deprecate!)
        fici.flags1 |= BIT(PIFilterWritesOutsideSelectionBit);
    }
    else REJECT;
    ACCEPT;
} /*filter_handling_item*/

/*
;	<has_terminology_specification>:
;		'HasTerminology' ":" <terminolgy_list>
;
;	<terminology_list>:
;		<terminology_item> {"," <terminology_item>}*
;
;	<terminology_item>:
;		'classID' "=" <constant_integer_expression>		|
;		'eventID' "=" <constant_integer_expression>		|
;		'aeteResNum' "=" <constant_integer_expression>	|
;		'uniqueID' "=" <quoted_string>
;
*/
STATE(has_terminology_specification)
    MARK;
    if (!mdp_keywordi("HasTerminology")) REJECT;
    if (!separator()) REJECT;
    do {
        if (!terminology_item()) REJECT;
    } while (mdp_char(','));
    ACCEPT;
} /*has_terminology_specification*/

STATE(terminology_item)
    MARK;
    if (mdp_keywordi("classID")) {
        if (!mdp_char('=')) REJECT;
        if (!constant_integer_expression()) REJECT;
    }
    else if (mdp_keywordi("eventID")) {
        if (!mdp_char('=')) REJECT;
        if (!constant_integer_expression()) REJECT;
    }
    else if (mdp_keywordi("aeteResNum")) {
        if (!mdp_char('=')) REJECT;
        if (!constant_integer_expression()) REJECT;
    }
    else if (mdp_keywordi("uniqueID")) {
    }
        if (!mdp_char('=')) REJECT;
        if (!string_literal()) REJECT;
    else REJECT;
    ACCEPT;
} /*terminology_item*/

/*
;	<PiMI_specification>:
;		'PiMI' ":" <PiMI_resource>		//See Appendix M
;
;	<PiPL_specification>:
;		'PiPL' ":" <PiPL_resource>		//See Appendix J
;
*/
STATE(PiMI_specification)
    MARK;
    if (!mdp_keywordi("PiMI")) REJECT;
    if (!separator()) REJECT;
    if (!PiMI_resource()) REJECT;
    ACCEPT;
} /*PiMI_specification*/

STATE(PiPL_specification)
    MARK;
    if (!mdp_keywordi("PiPL")) REJECT;
    if (!separator()) REJECT;
    if (!PiPL_resource()) REJECT;
    ACCEPT;
} /*PiPL_specification*/

#endif //ifndef BETA

/*
;	<preset_specification>:
;		'Preset' <index> ":" ['Name' "="] <preset_name> {"," <preset_item>}*
;
;	<preset_name>: <string_literal>
;
;	<preset_item>: <n> "=" <decimal_constant>  |   //<n> is the control index
;                  <quoted_string> "=" <decimal_constant>   //<quoted_string> is control label
;
;	<n>: <decimal_constant>
;*/


/*
;   <event_handler_specification>:
;           <event_handler> ":" <FFP_formula>
;
;	<event_handler>:
;		'OnInitDialog'
;		'OnCtl' <index_or_n>
;		'OnZeroDivide'
;		'OnDivideOverflow'
;		'OnIntegerOverflow'
;		'OnBoundsError'
;		'OnBreakpoint' <index_or_n>
;		'OnWatchpoint' <index_or_n>
;		'OnFloatError'
;		'OnDomainError'
;		'OnRangeError'
;		'OnAbort'
;		'OnOK'		//??
;		'OnCancel'	//??
;		'OnResize'
;		'OnInterruptCheck'
;		'OnFilterStart' | 'OFS'			//so can check image mode, do init's, etc.
;		'OnFilterEnd'	| 'OFE'		//for cleanup, reset progress bar, etc.
;       'ForEveryPixel' | 'FEP'
;       'ForEveryTile'  | 'FET'
;*/

static int op_on;

STATE(event_handler_specification)
    MARK;
    if (event_handler()) {
        if (!separator()) REJECT;
        if (compound_statement() || C_language_expression()) {
            switch (op_on) {
            case OP_FOR_EVERY_PIXEL:
                push_op(OP_FOR_EVERY_PIXEL_END);
                break;
            case OP_FOR_EVERY_TILE:
                push_op(OP_FOR_EVERY_END);
                break;
            case OP_ON_ZERO_DIVIDE:
            case OP_ON_DIVIDE_OVERFLOW:
                //these assume ebx already set; no need to restore ebx // IS THIS CORRECT ???
                push_op(OP_ON_HANDLER_END);
                break;
            default:
                //these set and restore ebx
                push_op(OP_ON_END);
                break;
            }//switch op_on
            ACCEPT;
        }
    }
    REJECT;
} /*event_handler_specification*/

STATE(event_handler)
    MARK;
    if (mdp_keywordi("OnInitDialog")) {
        push_op(op_on = OP_ON_INIT_DIALOG);
    }
    else if (mdp_keywordi("OnCtl")) {
        if (!index_or_n()) REJECT;
        push_op(op_on = OP_ON_CTL);
    }
    else if (mdp_keywordi("OnZeroDivide")) {
        push_op(op_on = OP_ON_ZERO_DIVIDE);
    }
    else if (mdp_keywordi("OnDivideOverflow")) {
        push_op(op_on = OP_ON_DIVIDE_OVERFLOW);
    }
#if 0
    else if (mdp_keywordi("OnIntegerOverflow")) {
        push_op(op_on = OP_ON_INTEGER_OVERFLOW);
    }
    else if (mdp_keywordi("OnBoundsError")) {
        push_op(op_on = OP_ON_BOUNDS_ERROR);
    }
    else if (mdp_keywordi("OnFloatError")) {
        push_op(op_on = OP_ON_FLOAT_ERROR);
    }
    else if (mdp_keywordi("OnDomainError")) {
        push_op(op_on = OP_ON_DOMAIN_ERROR);
    }
    else if (mdp_keywordi("OnRangeError")) {
        push_op(op_on = OP_ON_RANGE_ERROR);
    }
#endif
    else if (mdp_keywordi("OnFilterStart") || mdp_keywordi("OFS")) {
        push_op(op_on = OP_ON_FILTER_START);
    }
    else if (mdp_keywordi("ForEveryTile") || mdp_keywordi("FET")) {
        push_op(op_on = OP_FOR_EVERY_TILE);
    }
    else if (mdp_keywordi("ForEveryPixel") || mdp_keywordi("FEP")) {
        push_op(op_on = OP_FOR_EVERY_PIXEL);
    }
    else if (mdp_keywordi("OnFilterEnd") || mdp_keywordi("OFE")) {
        push_op(op_on = OP_ON_FILTER_END);
    }
    else REJECT;
    ACCEPT;
} /*event_handler*/

/*
;	<index_or_n>:
;		"[" {<decimal_constant> | "n"} [ "," "e" ] "]"	|	//literal "n" is control index param, "e" is event param
;		"(" {<decimal_constant> | "n"} [ "," "e" ] ")"		// onCtl[n, e] is obsolescent; onCtl(n, e) is preferred.
;                                                           // onCtl(<decimal_constant>) is not supported (never was!)
;*/
STATE(index_or_n)
    MARK;
    if (mdp_char('[')) {
#if 0
        if (decimal_constant()) {
        }
        else
#endif
        if (mdp_keywordi("n")) {
        }
        else REJECT;
        if (mdp_char(',')) {
            if (!mdp_keywordi("e")) REJECT;
        }
        if (!mdp_char(']')) REJECT;
        ACCEPT;
    }
    else if (mdp_char('(')) {
#if 0
        if (decimal_constant()) {
        }
        else
#endif
        if (mdp_keywordi("n")) {
        }
        else REJECT;
        if (mdp_char(',')) {
            if (!mdp_keywordi("e")) REJECT;
        }
        if (!mdp_char(')')) REJECT;
        ACCEPT;
    }
    REJECT;
} /*index_or_n*/

/*
;	<dialog_specification>:
;		'Dialog' ":" <dialog_subkey_list>
;
;	<dialog_subkey_list>:
;		<dialog_subkey_item> {"," <dialog_subkey_item>}*
;
*/
STATE(dialog_specification)
    MARK;
    if (mdp_keywordi("Dialog")) {
        if (!separator()) REJECT;
        do {
            if (!dialog_subkey_item()) REJECT;
        } while (mdp_char(','));
        ACCEPT;
    }
    REJECT;
} /*dialog_specification*/

/*
;	<dialog_subkey_item>:
;		<dialog_scaling_specification>				|
;       <dialog_style_specification>                |       ???
;		<dialog_position_specification>				|
;		<dialog_size_specification>					|
;		<dialog_border_specification>				|
;		<dialog_hscroll_specification>				|		??
;		<dialog_vscroll_specification>				|		??
;		<dialog_scroll_align_specification>			|		??
;		<dialog_3Dlook_specification>				|
;		<dialog_background_specification>			|
;		<dialog_region_specification>				|		??
;		<dialog_drag_specification>					|
;		<dialog_accept_files_specification>			|		??
;		<dialog_titlebar_specification>				|
;		<dialog_system_menu_specification>			|
;		<dlalog_minbox_specification>				|
;		<dialog_maxbox_specification>				|
;		<dialog_context_help_specification>			|
;		<dialog_tool_window_specification>			|
;   	<dialog_fixedsys_specification>             |
;		<dialog_menubar_specification>				|
;		<dialog_statusbar_specification>			|
;		<dialog_caption_specification>				|		??
;		<dialog_fontname_specification>				|
;		<dialog_fontsize_specification>				|
;		<dialog_fontcolor_specification>			|
;		<dialog_text_align_specification>			|
;		<dialog_reading_order_specification>
;
*/
STATE(dialog_subkey_item)
    MARK;

/*
;	<dialog_fontcolor_specification>:
;		'FontColor' "=" <color>
;
*/
    if (mdp_keywordi("FontColor")) {
        if (!mdp_char('=')) REJECT;
        if (!color()) REJECT;
    }

/*
;	<dialog_caption_specification>:
;		['Text' "="] <quoted_string>
;
*/
    else if (mdp_keywordi("Text")) {
        if (!mdp_char('=')) REJECT;
        push_proc_addr(setDialogText);
        if (!string_literal()) REJECT;
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);               //nargs
        push_op(OP_CALL);               //setDialogText(<string>)
        push_op(OP_POP);                //discard return value
    }
    else if (string_literal()) {
        //obsolescent: string by itself presumes Text=
        push_proc_addr(setDialogText);
        push_op(OP_XCHG);               //exchange <string>, <proc_addr>
        push_op(OP_ARG);                //push <string> arg
        push_constant(LIT_INTEGER);
        push_constant(1);               //nargs
        push_op(OP_CALL);               //setDialogText(<string>)
        push_op(OP_POP);                //discard return value
    }

#if 0
/*
;   <dialog_style_specification>:
;       'Popup' | 'Overlapped' | 'Child'
;
*/
    else if (mdp_keywordi("Style")) {
        if (!mdp_char('=')) REJECT;
        push_proc_addr(clearDialogStyle);   //clear all window styles
        push_constant(LIT_INTEGER);
        push_constant(WS_POPUPWINDOW|WS_OVERLAPPEDWINDOW|WS_CHILDWINDOW);
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                    //discard return value
        push_proc_addr(setDialogStyle);     //then set just the styles we need
        push_constant(LIT_INTEGER);
        if (mdp_keywordi("Popup")) {
            push_constant(WS_POPUPWINDOW);      //flags=WS_POPUPWINDOW
        }
        else if (mdp_keywordi("Overlapped")) {
            push_constant(WS_OVERLAPPEDWINDOW); //flags=WS_OVERLAPPEDWINDOW
        }
        else if (mdp_keywordi("Child")) {
            push_constant(WS_CHILDWINDOW);      //flags=WS_CHILDWINDOW
        }
        else REJECT;
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }
#endif

#if 0
    /** NFG **/
/*
;	<dialog_clip_children_specification>:
;		'ClipChildren' | 'NoClipChildren'
;
*/
    else if (mdp_keywordi("ClipChildren")) {
        push_proc_addr(setDialogStyle);
        push_constant(LIT_INTEGER);
        push_constant(WS_CLIPCHILDREN);          //flags=WS_CLIPCHILDREN
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("NoClipChildren")) {
        push_proc_addr(clearDialogStyle);
        push_constant(LIT_INTEGER);
        push_constant(WS_CLIPCHILDREN);          //flags=WS_CLIPCHILDREN
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }
#endif

#if 1
/*
;	<dialog_clip_siblings_specification>:
;		'ClipSiblings' | 'NoClipSiblings'
;
*/
    else if (mdp_keywordi("ClipSiblings")) {
        push_proc_addr(setDialogStyle);
        push_constant(LIT_INTEGER);
        push_constant(WS_CLIPSIBLINGS);          //flags=WS_CLIPSIBLINGS
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("NoClipSiblings")) {
        push_proc_addr(clearDialogStyle);
        push_constant(LIT_INTEGER);
        push_constant(WS_CLIPSIBLINGS);          //flags=WS_CLIPSIBLINGS
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }
#endif

/*
;	<dialog_system_menu_specification>:
;		'SystemMenu' | 'NoSystemMenu'
;
*/
    else if (mdp_keywordi("SystemMenu")) {
        push_proc_addr(setDialogStyle);
        push_constant(LIT_INTEGER);
        push_constant(WS_SYSMENU);          //flags=WS_SYSMENU
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("NoSystemMenu")) {
        push_proc_addr(clearDialogStyle);
        push_constant(LIT_INTEGER);
        push_constant(WS_SYSMENU);          //flags=WS_SYSMENU
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }

/*
;	<dialog_titlebar_specification>:
;		'TitleBar' | 'NoTitleBar'
;
*/
    else if (mdp_keywordi("TitleBar")) {
        push_proc_addr(setDialogStyle);
        push_constant(LIT_INTEGER);
        push_constant(WS_CAPTION);          //flags=WS_CAPTION
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("NoTitleBar")) {
        push_proc_addr(clearDialogStyle);
        push_constant(LIT_INTEGER);
        push_constant(WS_CAPTION);          //flags=WS_CAPTION
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }

/*
;	<dialog_exitevent_specification>:
;		'InitEvent' | 'NoInitEvent'
;
*/
    else if (mdp_keywordi("InitEvent")) {
        push_proc_addr(setDialogEvent);
        push_constant(LIT_INTEGER);
        push_constant(1);				//flags=WS_CAPTION
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("NoInitEvent")) {
        push_proc_addr(clearDialogEvent);
        push_constant(LIT_INTEGER);
        push_constant(1);				//flags=WS_CAPTION
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }

/*
;	<dialog_event_specification>:
;		'CancelEvent' | 'NoCancelEvent'
;
*/
    else if (mdp_keywordi("CancelEvent")) {
        push_proc_addr(setDialogEvent);
        push_constant(LIT_INTEGER);
        push_constant(2);          //flags=WS_CAPTION
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("NoCancelEvent")) {
        push_proc_addr(clearDialogEvent);
        push_constant(LIT_INTEGER);
        push_constant(2);          //flags=WS_CAPTION
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }

/*
;	<dialog_event_specification>:
;		'KeyDownEvents' | 'NoKeyDownEvents'
;
*/
    else if (mdp_keywordi("KeyEvents")) {
        push_proc_addr(setDialogEvent);
        push_constant(LIT_INTEGER);
        push_constant(4);          //flags=WS_CAPTION
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("NoKeyEvents")) {
        push_proc_addr(clearDialogEvent);
        push_constant(LIT_INTEGER);
        push_constant(4);          //flags=WS_CAPTION
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }

/*
;	<dialog_minbox_specification>:
;		'MinBox' | 'NoMinBox'
;
*/
    else if (mdp_keywordi("MinBox")) {
        push_proc_addr(setDialogStyle);
        push_constant(LIT_INTEGER);
        push_constant(WS_MINIMIZEBOX);          //flags=WS_MINIMIZEBOX
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("NoMinBox")) {
        push_proc_addr(clearDialogStyle);
        push_constant(LIT_INTEGER);
        push_constant(WS_MINIMIZEBOX);          //flags=WS_MINIMIZEBOX
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }

/*
;	<dialog_maxbox_specification>:
;		'MaxBox' | 'NoMaxBox'
;
*/
    else if (mdp_keywordi("MaxBox")) {
        push_proc_addr(setDialogStyle);
        push_constant(LIT_INTEGER);
        push_constant(WS_MAXIMIZEBOX);          //flags=WS_MAXIMIZEBOX
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("NoMaxBox")) {
        push_proc_addr(clearDialogStyle);
        push_constant(LIT_INTEGER);
        push_constant(WS_MAXIMIZEBOX);          //flags=WS_MAXIMIZEBOX
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }

/*
;	<dialog_hscroll_specification>:
;		'HScroll' | 'NoHScroll'
;
*/
    else if (mdp_keywordi("HScroll")) {
        push_proc_addr(setDialogStyle);
        push_constant(LIT_INTEGER);
        push_constant(WS_HSCROLL);          //flags=WS_HSCROLL
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("NoHScroll")) {
        push_proc_addr(clearDialogStyle);
        push_constant(LIT_INTEGER);
        push_constant(WS_HSCROLL);          //flags=WS_HSCROLL
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }

/*
;	<dialog_vscroll_specification>:
;		'VScroll' | 'NoVScroll'
;
*/
    else if (mdp_keywordi("VScroll")) {
        push_proc_addr(setDialogStyle);
        push_constant(LIT_INTEGER);
        push_constant(WS_VSCROLL);          //flags=WS_VSCROLL
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("NoVScroll")) {
        push_proc_addr(clearDialogStyle);
        push_constant(LIT_INTEGER);
        push_constant(WS_VSCROLL);          //flags=WS_VSCROLL
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }

/*
;	<dialog_fixedsys_specification>:
;		'FixedSys' | 'NoFixedSys'
;
*/
    else if (mdp_keywordi("FixedSys")) {
        push_proc_addr(setDialogStyle);
        push_constant(LIT_INTEGER);
        push_constant(DS_FIXEDSYS);          //flags=DS_FIXEDSYS
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("NoFixedSys")) {
        push_proc_addr(clearDialogStyle);
        push_constant(LIT_INTEGER);
        push_constant(DS_FIXEDSYS);          //flags=DS_FIXEDSYS
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }

/*
;	<dialog_3Dlook_specification>:		//Win95/WinNT only
;		'3Dlook' | 'No3Dlook'
;
*/
    else if (mdp_keywordi("3Dlook")) {
        push_proc_addr(setDialogStyle);
        push_constant(LIT_INTEGER);
        push_constant(DS_3DLOOK);          //flags=DS_3DLOOK
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("No3Dlook")) {
        push_proc_addr(clearDialogStyle);
        push_constant(LIT_INTEGER);
        push_constant(DS_3DLOOK);          //flags=DS_3DLOOK
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }

/*
;	<dialog_border_specification>:
;		'Border' "=" 'DialogFrame' | 'Resizing' | 'Thin' | 'None'
;
*/
    else if (mdp_keywordi("Border")) {
        if (!mdp_char('=')) REJECT;
#if 0       // DON'T WE NEED THIS, E.G. FOR 'NONE' ???
        push_proc_addr(clearDialogStyle);   //clear all border styles
        push_constant(LIT_INTEGER);
        push_constant(WS_BORDER|WS_THICKFRAME|WS_DLGFRAME);
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                    //discard return value
#endif
        push_proc_addr(setDialogStyle);
        push_constant(LIT_INTEGER);
        if (mdp_keywordi("None")) {
            push_constant(0);               //no border styles
        }
        else if (mdp_keywordi("Thin")) {
            push_constant(WS_BORDER);       //set thin border
        }
        else if (mdp_keywordi("Resizing")) {
            push_constant(WS_THICKFRAME);   //set thick border
        }
        else if (mdp_keywordi("DialogFrame")) {
            push_constant(WS_DLGFRAME);     //set dialog frame border
        }
        else REJECT;
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }

/*
;	<dialog_tool_window_specification>:
;		'ToolWindow' | 'NoToolWindow'
;
*/
    else if (mdp_keywordi("ToolWindow")) {
        push_proc_addr(setDialogStyleEx);
        push_constant(LIT_INTEGER);
        push_constant(WS_EX_TOOLWINDOW);    //flags=WS_EX_TOOLWINDOW
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("NoToolWindow")) {
        push_proc_addr(clearDialogStyleEx);
        push_constant(LIT_INTEGER);
        push_constant(WS_EX_TOOLWINDOW);    //flags=WS_EX_TOOLWINDOW
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }

/*
;	<dialog_context_help_specification>:
;		'ContextHelp' | 'NoContextHelp'
;
*/
    else if (mdp_keywordi("ContextHelp")) {
        push_proc_addr(setDialogStyleEx);
        push_constant(LIT_INTEGER);
        push_constant(WS_EX_CONTEXTHELP);    //flags=WS_EX_CONTEXTHELP
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("NoContextHelp")) {
        push_proc_addr(clearDialogStyleEx);
        push_constant(LIT_INTEGER);
        push_constant(WS_EX_CONTEXTHELP);    //flags=WS_EX_CONTEXTHELP
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }

/*
;	<dialog_accept_files_specification>:
;		'AcceptFiles' | 'NoAcceptFiles'
;
*/
    else if (mdp_keywordi("AcceptFiles")) {
        push_proc_addr(setDialogStyleEx);
        push_constant(LIT_INTEGER);
        push_constant(WS_EX_ACCEPTFILES);    //flags=WS_EX_ACCEPTFILES
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("NoAcceptFiles")) {
        push_proc_addr(clearDialogStyleEx);
        push_constant(LIT_INTEGER);
        push_constant(WS_EX_ACCEPTFILES);    //flags=WS_EX_ACCEPTFILES
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }

/*
;	<dialog_scroll_align_specification>:
;		'ScrollbarAlign' "=" 'Left' | 'Right'
;
*/
    else if (mdp_keywordi("ScrollbarAlign")) {
        if (!mdp_char('=')) REJECT;
        if (mdp_keywordi("Left")) {
            push_proc_addr(setDialogStyleEx);   //set WS_EX_LEFTSCROLLBAR
        }
        else if (mdp_keywordi("Right")) {
            push_proc_addr(clearDialogStyleEx); //clear WS_EX_LEFTSCROLLBAR
        }
        else REJECT;
        push_constant(LIT_INTEGER);
        push_constant(WS_EX_LEFTSCROLLBAR);    //flags=WS_EX_LEFTSCROLLBAR
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }

/*
;	<dialog_text_align_specification>:
;		'TextAlign' "=" 'Left' | 'Right'
;
*/
    else if (mdp_keywordi("TextAlign")) {
        if (!mdp_char('=')) REJECT;
        if (mdp_keywordi("Left")) {
            push_proc_addr(clearDialogStyleEx); //clear WS_EX_RIGHT
        }
        else if (mdp_keywordi("Right")) {
            push_proc_addr(setDialogStyleEx);   //set WS_EX_RIGHT
        }
        else REJECT;
        push_constant(LIT_INTEGER);
        push_constant(WS_EX_RIGHT);    //flags=WS_EX_RIGHT
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }

/*
;	<dialog_reading_order_specification>:
;		'ReadingOrder' "=" 'LeftToRight' | 'RightToLeft'
;
*/
    else if (mdp_keywordi("ReadingOrder")) {
        if (!mdp_char('=')) REJECT;
        if (mdp_keywordi("Left*ToRight")) {
            push_proc_addr(clearDialogStyleEx); //clear WS_EX_RTLREADING
        }
        else if (mdp_keywordi("Right*ToLeft")) {
            push_proc_addr(setDialogStyleEx);   //set WS_EX_RTLREADING
        }
        else REJECT;
        push_constant(LIT_INTEGER);
        push_constant(WS_EX_RTLREADING);    //flags=WS_EX_RTLREADING
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }

/*
;	<dialog_region_specification>:
;		'Region' "=" <region_expression>
;
*/
    else if (mdp_keywordi("Region")) {
        if (!mdp_char('=')) REJECT;
        push_proc_addr(setDialogRegion);
        if (!region_expression()) REJECT;
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);               //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }

/*
;	<dialog_position_specification>:
;		'Pos' "=" {'Screen'|'Client'} "(" {<xPos>  "," <yPos> | 'Center'} ")" 
;
*/
    else if (mdp_keywordi("Pos*ition")) {
        if (!mdp_char('=')) REJECT;
        push_proc_addr(setDialogPos);
        push_constant(LIT_INTEGER);
        if (mdp_keywordi("Client")) {
            push_constant(0);           //fAbs=false
        }
        else if (mdp_keywordi("Screen")) {
            push_constant(1);            //fAbs=true
        }
        else REJECT;
        push_op(OP_ARG);                //push arg <fAbs>
        if (couple()) {                 //push arg <x>, <y>
        }
        else {
            //must be '(Center)'
            if (!mdp_char('(')) REJECT;
            if (!mdp_keywordi("Center")) REJECT;
            if (!mdp_char(')')) REJECT;     //(x,y)=(-1,-1) to mean 'Center'
            push_constant(LIT_INTEGER);
            push_constant(-1);              //x = -1
            push_op(OP_ARG);                //push arg <x>
            push_constant(LIT_INTEGER);
            push_constant(-1);              //y = -1
            push_op(OP_ARG);                //push arg <y>
        }
        push_constant(LIT_INTEGER);
        push_constant(-1);              //w = -1
        push_op(OP_ARG);                //push arg <w>
        push_constant(LIT_INTEGER);
        push_constant(-1);              //h = -1
        push_op(OP_ARG);                //push arg <h>
        push_constant(LIT_INTEGER);
        push_constant(5);               //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }

/*
;	<dialog_size_specification>:
;		'Size' "=" "(" <width>  "," <height> ")" 
;
*/
    else if (mdp_keywordi("Size")) {
        if (!mdp_char('=')) REJECT;
        push_proc_addr(setDialogPos);
        push_constant(LIT_INTEGER);
        push_constant(0);               //fAbs=false
        push_op(OP_ARG);                //push arg <fAbs>
        push_constant(LIT_INTEGER);
        push_constant(-1);              //x = -1 means don't move
        push_op(OP_ARG);                //push arg <x>
        push_constant(LIT_INTEGER);
        push_constant(0);               //y = 0
        push_op(OP_ARG);                //push arg <y>
        if (!couple()) REJECT;          //push arg <w>, <h>
        push_constant(LIT_INTEGER);
        push_constant(5);               //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }

/*
;	<dialog_background_specification>:
;		'Color' "=" <color>											|
;		'Grad*ient' "=" "(" <color> "," <color> ["," {'H'|'V'}] ")"	|
;		'Image' "=" <filename>					                    |
;       'Bitmap' "=" <filename>                                     |
;       'Metafile' "=" <filename>                                   |
;		'Tile*d' | 'Stretch*ed' ["(" <stretch_mode> ")"] | 'Exact'
;
*/
    else if (mdp_keywordi("Theme")) {
        if (!mdp_char('=')) REJECT;
        push_proc_addr(setDialogTheme);
        push_constant(LIT_INTEGER);
        if (mdp_keywordi("on")) {
            push_constant(1);   
        }
        else if (mdp_keywordi("off")) {
            push_constant(0);
        }
        else REJECT;
		push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);               //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }
	
	else if (mdp_keywordi("Color")) {
        if (!mdp_char('=')) REJECT;
        push_proc_addr(setDialogColor);
        if (!color()) REJECT;
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);               //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }
	else if (mdp_keywordi("SysColor")) {
        if (!mdp_char('=')) REJECT;
        push_proc_addr(setDialogSysColor);
        if (!constant_integer_expression()) REJECT;
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);               //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("Grad*ient")) {
        if (!mdp_char('=')) REJECT;
        push_proc_addr(setDialogGradient);
        if (!gradient()) REJECT;
        push_constant(LIT_INTEGER);
        push_constant(3);               //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("Image") ||
             mdp_keywordi("Bitmap")) {
        if (!mdp_char('=')) REJECT;
        push_proc_addr(setDialogImage);
        if (!string_literal()) REJECT;
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);               //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("Exact")) {
        push_proc_addr(setDialogImageMode);
        push_constant(LIT_INTEGER);
        push_constant(DIM_EXACT);       // Exact==0
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(0);               // stretchMode==ignore
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(2);               //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("Tile*d")) {
        push_proc_addr(setDialogImageMode);
        push_constant(LIT_INTEGER);
        push_constant(DIM_TILED);       // Tiled==1
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(0);               // stretchMode==ignore
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(2);               //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("Stretch*ed")) {
        push_proc_addr(setDialogImageMode);
        push_constant(LIT_INTEGER);
        push_constant(DIM_STRETCHED);   // Stretched==2
        push_op(OP_ARG);
        if (mdp_char('(')) {
            if (!stretch_mode()) REJECT;
            if (!mdp_char(')')) REJECT;
        }
        else {
            push_constant(LIT_INTEGER);
            push_constant(0);               // stretchMode==ignore
        }
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(2);               //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }

/*
;	<dialog_fontsize_specification>:
;		'FontSize' "=" <n>
;
*/
    else if (mdp_keywordi("FontSize")) {
        if (!mdp_char('=')) REJECT;
        if (!constant_integer_expression()) REJECT;
    }

/*
;	<dialog_fontname_specification>:
;		'Font' "=" <quoted_string>
;
*/
    else if (mdp_keywordi("Font")) {
        if (!mdp_char('=')) REJECT;
        if (!string_literal()) REJECT;
    }

/*
;	<dialog_drag_specification>:
;		'Drag' = 'TitleBar' | 'Background' | 'None'
;
*/
    else if (mdp_keywordi("Drag")) {
        if (!mdp_char('=')) REJECT;
        push_proc_addr(setDialogDragMode);
        push_constant(LIT_INTEGER);
        
        if (mdp_keywordi("TitleBar")) {
            push_constant(0);       // TitleBar==0
        }
        else if (mdp_keywordi("Background")) {
            push_constant(1);       // Background==1
        }
        else if (mdp_keywordi("None")) {
            push_constant(2);       // None==2
        }
#if THEPLUGINSITE
        else if (mdp_keywordi("Control")) {
            push_constant(25);       //Control 25
        }
#endif
        else REJECT;
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);               //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }

/*
;	<dialog_scaling_specification>:
;		'Scaling' | 'NoScaling'
;
*/
    else if (mdp_keywordi("Scaling")) {
    }
    else if (mdp_keywordi("NoScaling")) {
    }


/*
;	<ctls_scaling_specification>:
;		'NoCtlScaling' | 'CtlScaling'
;
*/
    else if (mdp_keywordi("NoCtlScaling")) {
        push_proc_addr(lockCtlScaling);
        push_constant(LIT_INTEGER);
        push_constant(-1);              //flag=true
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("CtlScaling")) {
        push_proc_addr(lockCtlScaling);
        push_constant(LIT_INTEGER);
        push_constant(0);               //flag=false
        push_op(OP_ARG);
        push_constant(LIT_INTEGER);
        push_constant(1);                   //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }


/*
;	<dialog_menubar_specification>:
;		'MenuBar' "=" (TBD)
;
*/
    else if (mdp_keywordi("MenuBar")) {
        if (!mdp_char('=')) REJECT;
    }

/*
;	<dialog_statusbar_specification>:
;		'StatusBar' "=" (TBD)
;
*/
    else if (mdp_keywordi("StatusBar")) {
        if (!mdp_char('=')) REJECT;
    }
    else REJECT;
    ACCEPT;
} /*dialog_subkey_item*/

/*
;	<stretch_mode>:
;		'ANDSCANS' | 'BLACKONWHITE'     |
;		'ORSCANS'  | 'WHITEONBLACK'     |
;		'DELETESCANS' |  'COLORONCOLOR' |
;		'HALFTONE'
;
*/
STATE(stretch_mode)
    MARK;
    push_constant(LIT_INTEGER);
    if (mdp_keywordi("ANDSCANS") || mdp_keywordi("BLACKONWHITE")) {
        push_constant(BLACKONWHITE);
    }
    else if (mdp_keywordi("ORSCANS") || mdp_keywordi("WHITEONBLACK")) {
        push_constant(WHITEONBLACK);
    }
    else if (mdp_keywordi("DELETESCANS") || mdp_keywordi("COLORONCOLOR")) {
        push_constant(COLORONCOLOR);
    }
    else if (mdp_keywordi("HALFTONE")) {
        push_constant(HALFTONE);
    }
    else REJECT;
    ACCEPT;
} /*stretch_mode*/

STATE(gradient)
    MARK;
    if (!mdp_char('(')) REJECT;
    if (!color()) REJECT;
    push_op(OP_ARG);
    if (!mdp_char(',')) REJECT;
    if (!color()) REJECT;
    push_op(OP_ARG);
    push_constant(LIT_INTEGER);
    if (mdp_char(',')) {
        //optional direction
        if (mdp_keywordi("H*orizontal")) {
            push_constant(1);
        }
        else if (mdp_keywordi("V*ertical")) {
            push_constant(0);
        }
        else REJECT;
    }
    else {
        //default direction=vertical
        push_constant(0);
    }
    push_op(OP_ARG);
    if (!mdp_char(')')) REJECT;
    ACCEPT;
} /*gradient*/


/*
;	<about_specification>:
;		'About' ":" <about_subkey_list>
;
;	<about_subkey_list>:
;		<about_subkey_item> {"," <about_subkey_item>}*
;
*/
STATE(about_specification)
    MARK;
    if (mdp_keywordi("About")) {
        if (!separator()) REJECT;
        do {
            if (!about_subkey_item()) REJECT;
        } while (mdp_char(','));
        ACCEPT;
    }
    REJECT;
} /*about_specification*/

/*
;	<about_subkey_item>:
;		<about_scaling_specification>				|
;		<about_position_specification>				|
;		<about_size_specification>					|
;		<about_border_specification>				|
;		<about_3Dlook_specification>				|
;		<about_background_specification>			|
;		<about_drag_specification>					|
;		<about_titlebar_specification>				|
;		<about_system_menu_specification>			|
;		<about_minbox_specification>				|
;		<about_maxbox_specification>				|
;		<about_context_help_specification>			|
;		<about_tool_window_specification>			|
;   	<about_fixedsys_specification>              |
;		<about_menubar_specification>				|
;		<about_statusbar_specification>			    |
;		<about_title_specification>				    |		??
;		<about_text_specification>				    |		??
;		<about_fontname_specification>				|
;		<about_fontsize_specification>				|
;		<about_fontcolor_specification>			    |
;		<about_text_align_specification>			|
;		<about_reading_order_specification>
;
*/
STATE(about_subkey_item)
    MARK;

/*
;	<about_fontcolor_specification>:
;		'FontColor' "=" <color>
;
*/
    if (mdp_keywordi("FontColor")) {
        if (!mdp_char('=')) REJECT;
        if (!color()) REJECT;
    }

/*
;	<about_text_specification>:
;		['Text' "="] <quoted_string>
;
*/
    else if (mdp_keywordi("Text")) {
        if (!mdp_char('=')) REJECT;
        if (!string_literal()) REJECT;
        push_op(OP_ABOUT);                  //temporary!!!!
    }
    else if (string_literal()) {
        //obsolescent: string by itself presumes Text=
        push_op(OP_ABOUT);                  //temporary!!!!
    }

/*
;	<about_title_specification>:
;		'Title' "=" <quoted_string>
;
*/
    else if (mdp_keywordi("Title")) {
        if (!mdp_char('=')) REJECT;
        if (!string_literal()) REJECT;
    }

/*
;	<about_system_menu_specification>:
;		'SystemMenu' | 'NoSystemMenu'
;
*/
    else if (mdp_keywordi("SystemMenu")) {
    }
    else if (mdp_keywordi("NoSystemMenu")) {
    }

/*
;	<about_titlebar_specification>:
;		'TitleBar' | 'NoTitleBar'
;
*/
    else if (mdp_keywordi("TitleBar")) {
    }
    else if (mdp_keywordi("NoTitleBar")) {
    }

/*
;	<about_minbox_specification>:
;		'MinBox' | 'NoMinBox'
;
*/
    else if (mdp_keywordi("MinBox")) {
    }
    else if (mdp_keywordi("NoMinBox")) {
    }

/*
;	<about_maxbox_specification>:
;		'MaxBox' | 'NoMaxBox'
;
*/
    else if (mdp_keywordi("MaxBox")) {
    }
    else if (mdp_keywordi("NoMaxBox")) {
    }

/*
;	<about_fixedsys_specification>:
;		'FixedSys' | 'NoFixedSys'
;
*/
    else if (mdp_keywordi("FixedSys")) {
    }
    else if (mdp_keywordi("NoFixedSys")) {
    }

/*
;	<about_3Dlook_specification>:		//Win95/WinNT only
;		'3Dlook' | 'No3Dlook'
;
*/
    else if (mdp_keywordi("3Dlook")) {
        //push_constant(DS_3DLOOK);          //flags=DS_3DLOOK
    }
    else if (mdp_keywordi("No3Dlook")) {
        //push_constant(DS_3DLOOK);          //flags=DS_3DLOOK
    }

/*
;	<about_border_specification>:
;		'Border' "=" 'DialogFrame' | 'Resizing' | 'Thin' | 'None'
;
*/
    else if (mdp_keywordi("Border")) {
        if (!mdp_char('=')) REJECT;
        //push_constant(WS_BORDER|WS_THICKFRAME|WS_DLGFRAME);
        if (mdp_keywordi("None")) {
            //push_constant(0);               //no border styles
        }
        else if (mdp_keywordi("Thin")) {
            //push_constant(WS_BORDER);       //set thin border
        }
        else if (mdp_keywordi("Resizing")) {
            //push_constant(WS_THICKFRAME);   //set thick border
        }
        else if (mdp_keywordi("DialogFrame")) {
            //push_constant(WS_DLGFRAME);     //set dialog frame border
        }
        else REJECT;
    }

/*
;	<about_tool_window_specification>:
;		'ToolWindow' | 'NoToolWindow'
;
*/
    else if (mdp_keywordi("ToolWindow")) {
        //push_constant(WS_EX_TOOLWINDOW);    //flags=WS_EX_TOOLWINDOW
    }
    else if (mdp_keywordi("NoToolWindow")) {
        //push_constant(WS_EX_TOOLWINDOW);    //flags=WS_EX_TOOLWINDOW
    }

/*
;	<about_context_help_specification>:
;		'ContextHelp' | 'NoContextHelp'
;
*/
    else if (mdp_keywordi("ContextHelp")) {
        //push_constant(WS_EX_CONTEXTHELP);    //flags=WS_EX_CONTEXTHELP
    }
    else if (mdp_keywordi("NoContextHelp")) {
        //push_constant(WS_EX_CONTEXTHELP);    //flags=WS_EX_CONTEXTHELP
    }

/*
;	<about_text_align_specification>:
;		'TextAlign' "=" 'Left' | 'Right'
;
*/
    else if (mdp_keywordi("TextAlign")) {
        if (!mdp_char('=')) REJECT;
        if (mdp_keywordi("Left")) {
            //push_proc_addr(clearAboutStyleEx); //clear WS_EX_RIGHT
        }
        else if (mdp_keywordi("Right")) {
            //push_proc_addr(setAboutStyleEx);   //set WS_EX_RIGHT
        }
        else REJECT;
        //push_constant(WS_EX_RIGHT);    //flags=WS_EX_RIGHT
    }

/*
;	<about_reading_order_specification>:
;		'ReadingOrder' "=" 'LeftToRight' | 'RightToLeft'
;
*/
    else if (mdp_keywordi("ReadingOrder")) {
        if (!mdp_char('=')) REJECT;
        if (mdp_keywordi("Left*ToRight")) {
            //push_proc_addr(clearAboutStyleEx); //clear WS_EX_RTLREADING
        }
        else if (mdp_keywordi("Right*ToLeft")) {
            //push_proc_addr(setAboutStyleEx);   //set WS_EX_RTLREADING
        }
        else REJECT;
        //push_constant(WS_EX_RTLREADING);    //flags=WS_EX_RTLREADING
    }

/*
;	<about_position_specification>:
;		'Pos*ition' "=" {'Screen'|'Client'} "(" {<xPos>  "," <yPos> | 'Center'} ")" 
;
*/
    else if (mdp_keywordi("Pos*ition")) {
        if (!mdp_char('=')) REJECT;
        //push_proc_addr(setAboutPos);
        //push_constant(LIT_INTEGER);
        if (mdp_keywordi("Client")) {
            //push_constant(0);           //fAbs=false
        }
        else if (mdp_keywordi("Screen")) {
            //push_constant(1);            //fAbs=true
        }
        else REJECT;
        //push_op(OP_ARG);                //push arg <fAbs>
        if (couple()) {                 //push arg <x>, <y>
        }
        else {
            //must be '(Center)'
            if (!mdp_char('(')) REJECT;
            if (!mdp_keywordi("Center")) REJECT;
            if (!mdp_char(')')) REJECT;     //(x,y)=(-1,-1) to mean 'Center'
            //push_constant(LIT_INTEGER);
            //push_constant(-1);              //x = -1
            //push_op(OP_ARG);                //push arg <x>
            //push_constant(LIT_INTEGER);
            //push_constant(-1);              //y = -1
            //push_op(OP_ARG);                //push arg <y>
        }
        //push_constant(LIT_INTEGER);
        //push_constant(-1);              //w = -1
        //push_op(OP_ARG);                //push arg <w>
        //push_constant(LIT_INTEGER);
        //push_constant(-1);              //h = -1
        //push_op(OP_ARG);                //push arg <h>
        //push_constant(LIT_INTEGER);
        //push_constant(5);               //nargs
        //push_op(OP_CALL);
        //push_op(OP_POP);                //discard return value
    }

/*
;	<about_size_specification>:
;		'Size' "=" "(" <width>  "," <height> ")" 
;
*/
    else if (mdp_keywordi("Size")) {
        if (!mdp_char('=')) REJECT;
        //push_proc_addr(setAboutPos);
        //push_constant(LIT_INTEGER);
        //push_constant(0);               //fAbs=false
        //push_op(OP_ARG);                //push arg <fAbs>
        //push_constant(LIT_INTEGER);
        //push_constant(-1);              //x = -1 means don't move
        //push_op(OP_ARG);                //push arg <x>
        //push_constant(LIT_INTEGER);
        //push_constant(0);               //y = 0
        //push_op(OP_ARG);                //push arg <y>
        if (!couple()) REJECT;          //push arg <w>, <h>
        //push_constant(LIT_INTEGER);
        //push_constant(5);               //nargs
        //push_op(OP_CALL);
        //push_op(OP_POP);                //discard return value
    }

/*
;	<about_background_specification>:
;		'Color' "=" <color>											|
;		'Grad*ient' "=" "(" <color> "," <color> ["," {'H'|'V'}] ")"	|
;		'Image' "=" <filename>					                    |
;       'Bitmap' "=" <filename>                                     |
;       'Metafile' "=" <filename>                                   |
;		'Tile*d' | 'Stretch*ed' ["(" <stretch_mode> ")"] | 'Exact'
;
*/
    else if (mdp_keywordi("Color")) {
        if (!mdp_char('=')) REJECT;
        //push_proc_addr(setAboutColor);
        if (!color()) REJECT;
        //push_op(OP_ARG);
        //push_constant(LIT_INTEGER);
        //push_constant(1);               //nargs
        //push_op(OP_CALL);
        //push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("Grad*ient")) {
        if (!mdp_char('=')) REJECT;
        //push_proc_addr(setAboutGradient);
        if (!gradient()) REJECT;
        //push_constant(LIT_INTEGER);
        //push_constant(3);               //nargs
        //push_op(OP_CALL);
        //push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("Image") ||
             mdp_keywordi("Bitmap")) {
        if (!mdp_char('=')) REJECT;
        //push_proc_addr(setAboutImage);
        if (!string_literal()) REJECT;
        //push_op(OP_ARG);
        //push_constant(LIT_INTEGER);
        //push_constant(1);               //nargs
        //push_op(OP_CALL);
        //push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("Exact")) {
        //push_proc_addr(setAboutImageMode);
        //push_constant(LIT_INTEGER);
        //push_constant(DIM_EXACT);       // Exact==0
        //push_op(OP_ARG);
        //push_constant(LIT_INTEGER);
        //push_constant(0);               // stretchMode==ignore
        //push_op(OP_ARG);
        //push_constant(LIT_INTEGER);
        //push_constant(2);               //nargs
        //push_op(OP_CALL);
        //push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("Tile*d")) {
        //push_proc_addr(setAboutImageMode);
        //push_constant(LIT_INTEGER);
        //push_constant(DIM_TILED);       // Tiled==1
        //push_op(OP_ARG);
        //push_constant(LIT_INTEGER);
        //push_constant(0);               // stretchMode==ignore
        //push_op(OP_ARG);
        //push_constant(LIT_INTEGER);
        //push_constant(2);               //nargs
        //push_op(OP_CALL);
        //push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("Stretch*ed")) {
        //push_proc_addr(setAboutImageMode);
        //push_constant(LIT_INTEGER);
        //push_constant(DIM_STRETCHED);   // Stretched==2
        //push_op(OP_ARG);
        if (mdp_char('(')) {
            if (!stretch_mode()) REJECT;
            if (!mdp_char(')')) REJECT;
        }
        else {
            //push_constant(LIT_INTEGER);
            //push_constant(0);               // stretchMode==ignore
        }
        //push_op(OP_ARG);
        //push_constant(LIT_INTEGER);
        //push_constant(2);               //nargs
        //push_op(OP_CALL);
        //push_op(OP_POP);                //discard return value
    }

/*
;	<about_fontsize_specification>:
;		'FontSize' "=" <n>
;
*/
    else if (mdp_keywordi("FontSize")) {
        if (!mdp_char('=')) REJECT;
        if (!constant_integer_expression()) REJECT;
    }

/*
;	<about_fontname_specification>:
;		'Font' "=" <quoted_string>
;
*/
    else if (mdp_keywordi("Font")) {
        if (!mdp_char('=')) REJECT;
        if (!string_literal()) REJECT;
    }

/*
;	<about_drag_specification>:
;		'Drag' = 'TitleBar' | 'Background' | 'None'
;
*/
    else if (mdp_keywordi("Drag")) {
        if (!mdp_char('=')) REJECT;
        //push_proc_addr(setAboutDragMode);
        //push_constant(LIT_INTEGER);
        if (mdp_keywordi("TitleBar")) {
            //push_constant(0);       // TitleBar==0
        }
        else if (mdp_keywordi("Background")) {
            //push_constant(1);       // Background==1
        }
        else if (mdp_keywordi("None")) {
            //push_constant(2);       // None==2
        }
        else REJECT;
        //push_op(OP_ARG);
        //push_constant(LIT_INTEGER);
        //push_constant(1);               //nargs
        //push_op(OP_CALL);
        //push_op(OP_POP);                //discard return value
    }

/*
;	<about_scaling_specification>:
;		'Scaling' | 'NoScaling'
;
*/
    else if (mdp_keywordi("Scaling")) {
    }
    else if (mdp_keywordi("NoScaling")) {
    }

/*
;	<about_menubar_specification>:
;		'MenuBar' "=" (TBD)
;
*/
    else if (mdp_keywordi("MenuBar")) {
        if (!mdp_char('=')) REJECT;
    }

/*
;	<about_statusbar_specification>:
;		'StatusBar' "=" (TBD)
;
*/
    else if (mdp_keywordi("StatusBar")) {
        if (!mdp_char('=')) REJECT;
    }
    else REJECT;
    ACCEPT;
} /*about_subkey_item*/


/*
;	<embed_specification>:
;		'Embed' ":" <embed_subkey_list>
;
;	<embed_subkey_list>:
;		<embed_subkey_item> {"," <embed_subkey_item>}*
;
*/
STATE(embed_specification)
    MARK;
    if (mdp_keywordi("Embed")) {
        if (!separator()) REJECT;
        do {
            if (!embed_subkey_item()) REJECT;
        } while (mdp_char(','));
        ACCEPT;
    }
    REJECT;
} /*embed_specification*/

/*
;	<embed_subkey_item>:
;		<embed_bitmap_specification>				|
;		<embed_icon_specification>				    |
;		<embed_metafile_specification>				|
;       <embed_wave_specification>
;
*/
STATE(embed_subkey_item)
    MARK;

/*
;	<embed_bitmap_specification>:
;		'Bitmap' "=" <quoted_string>
;
*/
    if (mdp_keywordi("Bitmap")) {
        if (!mdp_char('=')) REJECT;
        if (!string_literal()) REJECT;
        push_op(OP_EMBED_BITMAP);
    }

/*
;	<embed_icon_specification>:
;		'Icon' "=" <quoted_string>
;
*/
    else if (mdp_keywordi("Icon")) {
        if (!mdp_char('=')) REJECT;
        if (!string_literal()) REJECT;
        push_op(OP_EMBED_ICON);
    }

/*
;	<embed_metafile_specification>:
;		'Metafile' "=" <quoted_string>
;
*/
    else if (mdp_keywordi("Metafile")) {
        if (!mdp_char('=')) REJECT;
        if (!string_literal()) REJECT;
        push_op(OP_EMBED_METAFILE);
    }

/*
;	<embed_other_specification>:
;		'Other' "=" <quoted_string>
;
*/
    else if (mdp_keywordi("Other")) {
        if (!mdp_char('=')) REJECT;
        if (!string_literal()) REJECT;
        push_op(OP_EMBED_OTHER);
    }

/*
;	<embed_other_specification>:
;		'Cursor' "=" <quoted_string>
;
*/
    else if (mdp_keywordi("Cursor")) {
        if (!mdp_char('=')) REJECT;
        if (!string_literal()) REJECT;
        push_op(OP_EMBED_CURSOR);
    }

/*
;	<embed_wave_specification>:
;		'Wave' "=" <quoted_string>
;
*/
    else if (mdp_keywordi("Wave")) {
        if (!mdp_char('=')) REJECT;
        if (!string_literal()) REJECT;
        push_op(OP_EMBED_WAVE);
    }
    else REJECT;
    ACCEPT;
} /*embed_subkey_item*/

  
/*
;	<control_specification>:
;		'Ctl' <index> ":"
;           { <control_class_specification> {"," <control_subkey_item>}*   |
;             <control_subkey_item>         {"," <control_subkey_item>}*   }
;
*/
STATE(control_specification)
    MARK;
    if (mdp_keywordi("Ctl")) {
        if (!index()) REJECT;
        if (!separator()) REJECT;
        if (control_class_specification()) {
            while (mdp_char(',')) {
                if (!control_subkey_item()) REJECT;
            }
        }
        else {
            //No class specification:
            //Initialize control to default (STANDARD) values...
            push_op(OP_DUPL);               //duplicate <index>
            push_proc_addr(initCtl);
            push_op(OP_XCHG);               //exchange <index>, <proc_addr>
            push_op(OP_ARG);                //push arg <index>
            push_constant(LIT_INTEGER);
            push_constant(1);               //nargs
            push_op(OP_CALL);               //call initCtl(<index>)
            push_op(OP_POP);                //discard return value
            do {
                if (!control_subkey_item()) REJECT;
            } while (mdp_char(','));
        }
        push_op(OP_POP);        //pop the <index>
        ACCEPT;
    }
    REJECT;
} /*control_specification*/

/*
;   <control_class_specification>:
;       ['Class' "="] {
;           'NONE'                                                  |
;           'MODIFY'                                                |
;           'PREVIEW'       ["(" <control_standard_styles> ")"]     |
;           'ZOOM'		    ["(" <control_standard_styles> ")"]     |
;           'PROGRESSBAR'   ["(" <control_standard_styles> ")"]		|
;           'STANDARD'      ["(" <control_standard_styles> ")"]     |
;           'SCROLLBAR'     ["(" <control_scrollbar_styles> ")"]    |
;           'TRACKBAR'      ["(" <control_trackbar_styles> ")"]     |
;     //    'SPINNER'       ["(" <control_spinner_styles> ")"]      |
;     //    'UPDOWN'        ["(" <control_updown_styles> ")"]       |
;           'PUSHBUTTON'    ["(" <control_pushbutton_styles> ")"]   |
;           'CHECKBOX'      ["(" <control_checkbox_styles> ")"]     |
;           'RADIO*BUTTON'  ["(" <control_radiobutton_styles> ")"]  |
;           'GROUPBOX'      ["(" <control_groupbox_styles> ")"]     |
;           'OWNERDRAW'     ["(" <control_ownerdraw_styles> ")"]    |
;           'LISTBOX'       ["(" <control_listbox_styles> ")"]      |
;           'COMBOBOX'      ["(" <control_combobox_styles> ")"]     |
;           'STATICTEXT'    ["(" <control_statictext_styles> ")"]   |
;           'EDIT'          ["(" <control_edit_styles> ")"]         |
;           'TAB'           ["(" <control_tab_styles> ")"]			|
;           'SLIDER'        ["(" <control_trackbar_styles> ")"]     |
;           'FRAME'         ["(" <control_frame_styles> ")"]        |
;           'RECT*ANGLE'    ["(" <control_rect_styles> ")"]         |
;           'IMAGE'         ["(" <control_image_styles> ")"]        |
;           'BITMAP'        ["(" <control_bitmap_styles> ")"]       |
;           'ICON'          ["(" <control_icon_styles> ")"]         |
;           'METAFILE'      ["(" <control_metafile_styles> ")"]     |
;     //    'TOOLTIP'       ["(" <control_tooltip_styles> ")"]      |
;     //    'ANIMATION'     ["(" <control_animation_styles> ")"]    |
;       }
;
*/
static int control_class = 0;
static int control_style = 0;
static int control_styleEx = 0;
static int control_properties = 0;
static int control_style_mask = 0;
static int control_styleEx_mask = 0;
static int control_properties_mask = 0;

STATE(control_class_specification)
    MARK;
    control_style = control_styleEx = control_properties = 0;
    control_style_mask = control_styleEx_mask = control_properties_mask = 0;
    if (mdp_keywordi("Class")) {
        if (!mdp_char('=')) REJECT;
    }
    if (mdp_keywordi("NONE")) {
        push_op(OP_DUPL);               //duplicate <index>
        push_proc_addr(deleteCtl);
        push_op(OP_XCHG);               //exchange <index>, <proc_addr>
        push_op(OP_ARG);                //push arg <index>
        push_constant(LIT_INTEGER);
        push_constant(1);                //nargs=1
        push_op(OP_CALL);               //call deleteCtl(<index>)
        push_op(OP_POP);                //discard return value
        ACCEPT;
    }
    else if (mdp_keywordi("MODIFY")) {
        ACCEPT;
    }
    else if (mdp_keywordi("PREVIEW")) {
        control_class = CC_PREVIEW;
        if (mdp_char('(')) {
            do {
                if (!control_standard_style()) REJECT;
            } while (mdp_char(','));
            if (!mdp_char(')')) REJECT;
        }
    }
    else if (mdp_keywordi("ZOOM")) {
        control_class = CC_ZOOM;
        if (mdp_char('(')) {
            do {
                if (!control_standard_style()) REJECT;
            } while (mdp_char(','));
            if (!mdp_char(')')) REJECT;
        }
    }
    else if (mdp_keywordi("PROGRESSBAR")) {
        control_class = CC_PROGRESSBAR;
        if (mdp_char('(')) {
            do {
                if (!control_standard_style()) REJECT;
            } while (mdp_char(','));
            if (!mdp_char(')')) REJECT;
        }
    }
    else if (mdp_keywordi("STANDARD")) {
        control_class = CC_STANDARD;
        if (mdp_char('(')) {
            do {
                if (!control_standard_style()) REJECT;
            } while (mdp_char(','));
            if (!mdp_char(')')) REJECT;
        }
    }
    else if (mdp_keywordi("SCROLLBAR")) {
        control_class = CC_SCROLLBAR;
        if (mdp_char('(')) {
            do {
                if (!control_scrollbar_style()) REJECT;
            } while (mdp_char(','));
            if (!mdp_char(')')) REJECT;
        }
    }
    else if (mdp_keywordi("TRACKBAR")) {
        control_class = CC_TRACKBAR;
        if (mdp_char('(')) {
            do {
                if (!control_trackbar_style()) REJECT;
            } while (mdp_char(','));
            if (!mdp_char(')')) REJECT;
        }
    }
    else if (mdp_keywordi("PUSHBUTTON")) {
        control_class = CC_PUSHBUTTON;
        control_style |= BS_PUSHBUTTON;
        control_style_mask |= BS_PUSHBUTTON;
        if (mdp_char('(')) {
            do {
                if (!control_pushbutton_style()) REJECT;
            } while (mdp_char(','));
            if (!mdp_char(')')) REJECT;
        }
    }
    else if (mdp_keywordi("CHECKBOX")) {
        control_class = CC_CHECKBOX;
        control_style |= BS_AUTOCHECKBOX;
        control_style_mask |= BS_AUTOCHECKBOX;
        if (mdp_char('(')) {
            do {
                if (!control_checkbox_style()) REJECT;
            } while (mdp_char(','));
            if (!mdp_char(')')) REJECT;
        }
    }
    else if (mdp_keywordi("RADIO*BUTTON")) {
        control_class = CC_RADIOBUTTON;
        control_style |= BS_AUTORADIOBUTTON;
        if (mdp_char('(')) {
            do {
                if (!control_radiobutton_style()) REJECT;
            } while (mdp_char(','));
            if (!mdp_char(')')) REJECT;
        }
    }
    else if (mdp_keywordi("GROUPBOX")) {
        control_class = CC_GROUPBOX;
        control_style |= BS_GROUPBOX;
        if (mdp_char('(')) {
            do {
                if (!control_groupbox_style()) REJECT;
            } while (mdp_char(','));
            if (!mdp_char(')')) REJECT;
        }
    }
    else if (mdp_keywordi("OWNERDRAW")) {
        control_class = CC_OWNERDRAW;
        control_style |= BS_OWNERDRAW;
        if (mdp_char('(')) {
            do {
                if (!control_ownerdraw_style()) REJECT;
            } while (mdp_char(','));
            if (!mdp_char(')')) REJECT;
        }
    }
    else if (mdp_keywordi("LISTBOX")) {
        control_class = CC_LISTBOX;
        control_style |= LBS_NOTIFY;
        if (mdp_char('(')) {
            do {
                if (!control_listbox_style()) REJECT;
            } while (mdp_char(','));
            if (!mdp_char(')')) REJECT;
        }
    }
    else if (mdp_keywordi("COMBOBOX")) {
        control_class = CC_COMBOBOX;
        control_style |= CBS_DROPDOWNLIST;
        if (mdp_char('(')) {
            do {
                if (!control_combobox_style()) REJECT;
            } while (mdp_char(','));
            if (!mdp_char(')')) REJECT;
        }
    }
    else if (mdp_keywordi("STATICTEXT")) {
        control_class = CC_STATICTEXT;
        control_style |= SS_LEFT;
        if (mdp_char('(')) {
            do {
                if (!control_statictext_style()) REJECT;
            } while (mdp_char(','));
            if (!mdp_char(')')) REJECT;
        }
    }
    else if (mdp_keywordi("EDIT")) {
        control_class = CC_EDIT;
        control_style |= SS_LEFT;
        if (mdp_char('(')) {
            do {
                if (!control_edit_style()) REJECT;
            } while (mdp_char(','));
            if (!mdp_char(')')) REJECT;
        }
    }
	else if (mdp_keywordi("TAB")) {
        control_class = CC_TAB;
        if (mdp_char('(')) {
            do {
                if (!control_tab_style()) REJECT;
            } while (mdp_char(','));
            if (!mdp_char(')')) REJECT;
        }
    }
	else if (mdp_keywordi("SLIDER")) {
        control_class = CC_SLIDER;
        if (mdp_char('(')) {
            do {
                if (!control_trackbar_style()) REJECT;
            } while (mdp_char(','));
            if (!mdp_char(')')) REJECT;
        }
    }
	else if (mdp_keywordi("FRAME")) {
        control_class = CC_FRAME;
        control_style |= SS_BLACKFRAME;
        if (mdp_char('(')) {
            do {
                if (!control_frame_style()) REJECT;
            } while (mdp_char(','));
            if (!mdp_char(')')) REJECT;
        }
    }
    else if (mdp_keywordi("RECT*ANGLE")) {
        control_class = CC_RECT;
        control_style |= SS_BLACKRECT;
        if (mdp_char('(')) {
            do {
                if (!control_rect_style()) REJECT;
            } while (mdp_char(','));
            if (!mdp_char(')')) REJECT;
        }
    }
    else if (mdp_keywordi("IMAGE")) {
        control_class = CC_IMAGE;
        control_style |= SS_OWNERDRAW;
        if (mdp_char('(')) {
            do {
                if (!control_image_style()) REJECT;
            } while (mdp_char(','));
            if (!mdp_char(')')) REJECT;
        }
    }
    else if (mdp_keywordi("BITMAP")) {
        control_class = CC_BITMAP;
        control_style |= SS_BITMAP;
        if (mdp_char('(')) {
            do {
                if (!control_bitmap_style()) REJECT;
            } while (mdp_char(','));
            if (!mdp_char(')')) REJECT;
        }
    }
    else if (mdp_keywordi("ICON")) {
        control_class = CC_ICON;
        control_style |= SS_ICON;
        if (mdp_char('(')) {
            do {
                if (!control_icon_style()) REJECT;
            } while (mdp_char(','));
            if (!mdp_char(')')) REJECT;
        }
    }
    else if (mdp_keywordi("METAFILE")) {
        control_class = CC_METAFILE;
        control_style |= SS_ENHMETAFILE;
        if (mdp_char('(')) {
            do {
                if (!control_metafile_style()) REJECT;
            } while (mdp_char(','));
            if (!mdp_char(')')) REJECT;
        }
    }
    else REJECT;
    push_op(OP_DUPL);               //duplicate <index>
    push_proc_addr(createCtl);
    push_op(OP_XCHG);               //exchange <index>, <proc_addr>
    push_op(OP_ARG);                //push arg <index>
    push_constant(LIT_INTEGER);
    push_param(control_class);
    push_op(OP_ARG);                //push arg <class>
    push_constant(LIT_INTEGER);
    push_constant(0);
    push_op(OP_ARG);                //push arg <text>=NULL;
    push_constant(LIT_INTEGER);
    push_constant(-1);
    push_op(OP_ARG);                //push arg <x>=-1;
    push_constant(LIT_INTEGER);
    push_constant(-1);
    push_op(OP_ARG);                //push arg <y>=-1;
    push_constant(LIT_INTEGER);
    push_constant(-1);
    push_op(OP_ARG);                //push arg <w>=-1;
    push_constant(LIT_INTEGER);
    push_constant(-1);
    push_op(OP_ARG);                //push arg <h>=-1;
    push_constant(LIT_INTEGER);
    push_param(control_style);
    push_op(OP_ARG);                //push arg <style>
    push_constant(LIT_INTEGER);
    push_param(control_styleEx);
    push_op(OP_ARG);                //push arg <styleEx>
    push_constant(LIT_INTEGER);
    push_param(control_properties);
    push_op(OP_ARG);                //push arg <properties>
    push_constant(LIT_INTEGER);
    push_constant(3);
    push_op(OP_ARG);                //push arg <e>=visible/enabled;
    push_constant(LIT_INTEGER);
    push_constant(11);               //nargs=11
    push_op(OP_CALL);               //call createCtl(<index>,...)
    push_op(OP_POP);                //discard return value
    ACCEPT;
} /*control_class_specification*/

/*
;   <control_common_style>:
;       '[NO]BORDER' |
;       'TABSTOP' |
;       'GROUP' |
;       'HSCROLL' | 'VSCROLL' |
;       'VISIBLE' | 'DISABLED' |
;       'CLIPCHILDREN' | 'CLIPSIBLINGS' |
;       'CLIENTEDGE' | 'STATICEDGE' | 'MODALFRAME' |
;       'ACCEPTFILES' |
;       'TRANSPARENT' |
;       'RIGHTALIGNEDTEXT' |
;       'RTLREADING' | 'LTRREADING' |
;       'MOUSEOVER' |
;       'MOUSEMOVE' |
;       'CONTEXTMENU' |
;       'DRAWITEM' |
;       'PREVIEWDRAG'
;
*/
STATE(control_common_style)
    MARK;
    if (mdp_keywordi("BORDER")) {
        control_style_mask |= WS_BORDER;
        control_style |= WS_BORDER;
    }
    else if (mdp_keywordi("NOBORDER")) {
        control_style_mask |= WS_BORDER;
        control_style &= ~WS_BORDER;
    }
    else if (mdp_keywordi("TABSTOP")) {
        control_style |= WS_TABSTOP;
    }
    else if (mdp_keywordi("GROUP")) {
        control_style |= WS_GROUP;
    }
    else if (mdp_keywordi("HSCROLL")) {
        control_style |= WS_HSCROLL;
    }
    else if (mdp_keywordi("VSCROLL")) {
        control_style |= WS_VSCROLL;
    }
    else if (mdp_keywordi("VISIBLE")) {
        control_style |= WS_VISIBLE;
    }
    else if (mdp_keywordi("DISABLED")) {
        control_style |= WS_DISABLED;
    }
#if 1
    else if (mdp_keywordi("CLIPCHILDREN")) {
        control_style |= WS_CLIPCHILDREN;
    }
    else if (mdp_keywordi("CLIPSIBLINGS")) {
        control_style |= WS_CLIPSIBLINGS;
    }
#endif
    else if (mdp_keywordi("CLIENTEDGE")) {
        control_styleEx |= WS_EX_CLIENTEDGE;
    }
    else if (mdp_keywordi("STATICEDGE")) {
        control_styleEx |= WS_EX_STATICEDGE;
    }
    else if (mdp_keywordi("MODALFRAME")) {
        control_styleEx |= WS_EX_DLGMODALFRAME;
    }
    else if (mdp_keywordi("ACCEPTFILES")) {
        control_styleEx |= WS_EX_ACCEPTFILES;
    }
    else if (mdp_keywordi("TRANSPARENT")) {
        control_styleEx |= WS_EX_TRANSPARENT;
    }
    else if (mdp_keywordi("RIGHTALIGNEDTEXT")) {
        control_styleEx |= WS_EX_RIGHT;
    }
    else if (mdp_keywordi("RTLREADING")) {
        control_styleEx_mask |= WS_EX_RTLREADING;
        control_styleEx |= WS_EX_RTLREADING;
    }
    else if (mdp_keywordi("LTRREADING")) {
        //control_styleEx |= WS_EX_LTRREADING;    //NFG = 0
        control_styleEx_mask |= WS_EX_RTLREADING;
        control_styleEx     &= ~WS_EX_RTLREADING;
    }
    else if (mdp_keywordi("MOUSEOVER")) {
        control_properties |= CTP_MOUSEOVER;
    }
    else if (mdp_keywordi("MOUSEMOVE")) {
        control_properties |= CTP_MOUSEMOVE;
    }
    else if (mdp_keywordi("CONTEXTMENU")) {
        control_properties |= CTP_CONTEXTMENU;
    }
    else if (mdp_keywordi("DRAWITEM")) {
        control_properties |= CTP_DRAWITEM;
    }
    else if (mdp_keywordi("PREVIEWDRAG")) {
        control_properties |= CTP_PREVIEWDRAG;
    }
    else REJECT;
    ACCEPT;
} /*control_common_style*/

/*
;   <control_standard_styles>:
;       <control_standard_style> {"," <control_standard_style>}*
;
;   <control_standard_style>:
;       <control_common_style> |
;       'HORZ' |						//legacy abbreviation
;		'HORIZ*ONTAL' | 'VERT*ICAL' |
;       'READONLY'
;
*/
STATE(control_standard_style)
    MARK;
    if (control_common_style()) {
    }
    else if (mdp_keywordi("HORZ") || mdp_keywordi("HORIZ*ONTAL")) {
        //control_style |= SBS_HORZ;
        control_style_mask |= SBS_VERT;
        control_style &= ~SBS_VERT;
    }
    else if (mdp_keywordi("VERT*ICAL")) {
        control_style |= SBS_VERT;
    }
    else if (mdp_keywordi("READONLY")) {
        control_properties |= CTP_READONLY;
    }
    else REJECT;
    ACCEPT;
} /*control_standard_style*/

/*
;   <control_scrollbar_styles>:
;       <control_scrollbar_style> {"," <control_scrollbar_style>}*
;
;   <control_scrollbar_style>:
;       <control_common_style> |
;       'HORZ' |						//legacy abbreviation
;		'HORIZ*ONTAL' | 'VERT*ICAL' |
*/
STATE(control_scrollbar_style)
    MARK;
    if (control_common_style()) {
    }
    else if (mdp_keywordi("HORZ") || mdp_keywordi("HORIZ*ONTAL")) {
        //control_style |= SBS_HORZ;
        control_style &= ~SBS_VERT;
    }
    else if (mdp_keywordi("VERT*ICAL")) {
        control_style |= SBS_VERT;
    }
    else REJECT;
    ACCEPT;
} /*control_scrollbar_style*/

/*
;
;   <control_trackbar_styles>:
;       <control_trackbar_style> {"," <control_trackbar_style>}*
;
;   <control_trackbar_style>:
;       <control_common_style> |
;       'HORZ' |						//legacy abbreviation
;		'HORIZ*ONTAL' | 'VERT*ICAL' |
;       'NOTICKS' | 'TICKS' | 'AUTOTICKS' |
;       'TOP' | 'BOTTOM' | 'LEFT' | 'RIGHT' | 'BOTH' |
;       'ENABLESELRANGE' |
;       'NOTHUMB' | 'THUMB' | 'FIXEDLENGTH' |
;	//	'THUMBLENGTH' "=" <int> |
;       'TOPTIP' | 'BOTTOMTIP' | 'LEFTTIP' | 'RIGHTTIP'
;
*/
STATE(control_trackbar_style)
    MARK;
    if (control_common_style()) {
    }
    else if (mdp_keywordi("HORZ") || mdp_keywordi("HORIZ*ONTAL")) {
        //control_style |= TBS_HORZ;
        control_style &= ~TBS_VERT;
    }
    else if (mdp_keywordi("VERT*ICAL")) {
        control_style |= TBS_VERT;
    }
    else if (mdp_keywordi("NOTICKS")) {
        control_style |= TBS_NOTICKS;
    }
    else if (mdp_keywordi("TICKS")) {
        control_style &= ~TBS_NOTICKS;
    }
    else if (mdp_keywordi("AUTOTICKS")) {
        control_style |= TBS_AUTOTICKS;
    }
    else if (mdp_keywordi("TOP")) {
        control_style |= TBS_TOP;
    }
    else if (mdp_keywordi("BOTTOM")) {
        //control_style |= TBS_BOTTOM;
        control_style &= ~TBS_TOP;
    }
    else if (mdp_keywordi("LEFT")) {
        control_style |= TBS_LEFT;
    }
    else if (mdp_keywordi("RIGHT")) {
        //control_style |= TBS_RIGHT;
        control_style &= ~TBS_LEFT;
    }
    else if (mdp_keywordi("BOTH")) {
        control_style |= TBS_BOTH;
    }
    else if (mdp_keywordi("ENABLESELRANGE")) {
        control_style |= TBS_ENABLESELRANGE;
    }
    else if (mdp_keywordi("NOTHUMB")) {
        control_style |= TBS_NOTHUMB;
    }
    else if (mdp_keywordi("THUMB")) {
        control_style &= ~TBS_NOTHUMB;
    }
    else if (mdp_keywordi("FIXEDLENGTH")) {
        control_style |= TBS_FIXEDLENGTH;
    }
    else if (mdp_keywordi("TOPTIP")) {
        control_style |= TBS_TOOLTIPS;
        control_properties &= ~CTP_SIDE_MASK;
        control_properties |= CTP_TOP;
    }
    else if (mdp_keywordi("BOTTOMTIP")) {
        control_style |= TBS_TOOLTIPS;
        control_properties &= ~CTP_SIDE_MASK;
        control_properties |= CTP_BOTTOM;
    }
    else if (mdp_keywordi("LEFTTIP")) {
        control_style |= TBS_TOOLTIPS;
        control_properties &= ~CTP_SIDE_MASK;
        control_properties |= CTP_LEFT;
    }
    else if (mdp_keywordi("RIGHTTIP")) {
        control_style |= TBS_TOOLTIPS;
        control_properties &= ~CTP_SIDE_MASK;
        control_properties |= CTP_RIGHT;
    }
    else REJECT;
    ACCEPT;
} /*control_trackbar_style*/

/*
;   <control_pushbutton_styles>:
;       <control_pushbutton_style> {"," <control_pushbutton_style>}*
;
;   <control_pushbutton_style>:
;       <control_common_style> |
;       'NOTIFY' |
;       'FLAT' |
;       'DEFAULT' |
;       'LEFT' | 'RIGHT' | 'CENTER' |
;       'TOP' | 'BOTTOM' | 'VCENTER' |
;       'MULTILINE' |
;       'TEXT' | 'BITMAP' | 'ICON'
;
*/
STATE(control_pushbutton_style)
    MARK;
    if (control_common_style()) {
    }
    else if (mdp_keywordi("NOTIFY")) {
        control_style |= BS_NOTIFY;
    }
    else if (mdp_keywordi("FLAT")) {
        control_style |= BS_FLAT;
    }
    else if (mdp_keywordi("DEFAULT")) {
        switch (control_style & BS_TYPEMASK) {
        case BS_PUSHBUTTON:
            control_style &= ~BS_TYPEMASK;
            control_style |= BS_DEFPUSHBUTTON;
            break;
        }//switch
    }
    else if (mdp_keywordi("LEFT")) {
        control_style &= ~(BS_LEFT | BS_RIGHT | BS_CENTER);
        control_style |= BS_LEFT;
    }
    else if (mdp_keywordi("RIGHT")) {
        control_style &= ~(BS_LEFT | BS_RIGHT | BS_CENTER);
        control_style |= BS_RIGHT;
    }
    else if (mdp_keywordi("CENTER")) {
        control_style &= ~(BS_LEFT | BS_RIGHT | BS_CENTER);
        control_style |= BS_CENTER;
    }
    else if (mdp_keywordi("TOP")) {
        control_style &= ~(BS_TOP | BS_BOTTOM | BS_VCENTER);
        control_style |= BS_TOP;
    }
    else if (mdp_keywordi("BOTTOM")) {
        control_style &= ~(BS_TOP | BS_BOTTOM | BS_VCENTER);
        control_style |= BS_BOTTOM;
    }
    else if (mdp_keywordi("VCENTER")) {
        control_style &= ~(BS_TOP | BS_BOTTOM | BS_VCENTER);
        control_style |= BS_VCENTER;
    }
    else if (mdp_keywordi("MULTILINE")) {
        control_style |= BS_MULTILINE;
    }
    else if (mdp_keywordi("TEXT")) {
        control_style &= ~(BS_TEXT | BS_BITMAP | BS_ICON);
        control_style |= BS_TEXT;
    }
    else if (mdp_keywordi("BITMAP")) {
        control_style &= ~(BS_TEXT | BS_BITMAP | BS_ICON);
        control_style |= BS_BITMAP;
    }
    else if (mdp_keywordi("ICON")) {
        control_style &= ~(BS_TEXT | BS_BITMAP | BS_ICON);
        control_style |= BS_ICON;
    }
    else REJECT;
    ACCEPT;
} /*control_pushbutton_style*/

/*
;   <control_checkbox_styles>:
;       <control_checkbox_style> {"," <control_checkbox_style>}*
;
;   <control_checkbox_style>:
;       <control_common_style> |
;       'NOTIFY' |
;       'FLAT' | 'PUSHLIKE' | 'AUTO' | 'NOAUTO' | '3STATE' |
;       'LEFTTEXT' | 'RIGHTBUTTON' |
;       'LEFT' | 'RIGHT' | 'CENTER' |
;       'TOP' | 'BOTTOM' | 'VCENTER' |
;       'MULTILINE' |
;       'TEXT' | 'BITMAP' | 'ICON'
;
*/
STATE(control_checkbox_style)
    MARK;
    if (control_common_style()) {
    }
    else if (mdp_keywordi("NOTIFY")) {
        control_style |= BS_NOTIFY;
    }
    else if (mdp_keywordi("FLAT")) {
        control_style |= BS_FLAT;
    }
    else if (mdp_keywordi("PUSHLIKE")) {
        control_style |= BS_PUSHLIKE;
    }
    else if (mdp_keywordi("AUTO")) {
        switch (control_style & BS_TYPEMASK) {
        case BS_CHECKBOX:
            control_style &= ~BS_TYPEMASK;
            control_style |= BS_AUTOCHECKBOX;
            break;
        case BS_3STATE:
            control_style &= ~BS_TYPEMASK;
            control_style |= BS_AUTO3STATE;
            break;
        }//switch
    }
    else if (mdp_keywordi("NOAUTO")) {
        switch (control_style & BS_TYPEMASK) {
        case BS_AUTOCHECKBOX:
            control_style &= ~BS_TYPEMASK;
            control_style |= BS_CHECKBOX;
            break;
        case BS_AUTO3STATE:
            control_style &= ~BS_TYPEMASK;
            control_style |= BS_3STATE;
            break;
        }//switch
    }
    else if (mdp_keywordi("3STATE")) {
        switch (control_style & BS_TYPEMASK) {
        case BS_CHECKBOX:
            control_style &= ~BS_TYPEMASK;
            control_style |= BS_3STATE;
            break;
        case BS_AUTOCHECKBOX:
            control_style &= ~BS_TYPEMASK;
            control_style |= BS_AUTO3STATE;
            break;
        }//switch
    }
    else if (mdp_keywordi("LEFTTEXT") || mdp_keywordi("RIGHTBUTTON")) {
        control_style |= BS_LEFTTEXT;
    }
    else if (mdp_keywordi("LEFT")) {
        control_style &= ~(BS_LEFT | BS_RIGHT | BS_CENTER);
        control_style |= BS_LEFT;
    }
    else if (mdp_keywordi("RIGHT")) {
        control_style &= ~(BS_LEFT | BS_RIGHT | BS_CENTER);
        control_style |= BS_RIGHT;
    }
    else if (mdp_keywordi("CENTER")) {
        control_style &= ~(BS_LEFT | BS_RIGHT | BS_CENTER);
        control_style |= BS_CENTER;
    }
    else if (mdp_keywordi("TOP")) {
        control_style &= ~(BS_TOP | BS_BOTTOM | BS_VCENTER);
        control_style |= BS_TOP;
    }
    else if (mdp_keywordi("BOTTOM")) {
        control_style &= ~(BS_TOP | BS_BOTTOM | BS_VCENTER);
        control_style |= BS_BOTTOM;
    }
    else if (mdp_keywordi("VCENTER")) {
        control_style &= ~(BS_TOP | BS_BOTTOM | BS_VCENTER);
        control_style |= BS_VCENTER;
    }
    else if (mdp_keywordi("MULTILINE")) {
        control_style |= BS_MULTILINE;
    }
    else if (mdp_keywordi("TEXT")) {
        control_style &= ~(BS_TEXT | BS_BITMAP | BS_ICON);
        control_style |= BS_TEXT;
    }
    else if (mdp_keywordi("BITMAP")) {
        control_style &= ~(BS_TEXT | BS_BITMAP | BS_ICON);
        control_style |= BS_BITMAP;
    }
    else if (mdp_keywordi("ICON")) {
        control_style &= ~(BS_TEXT | BS_BITMAP | BS_ICON);
        control_style |= BS_ICON;
    }
    else REJECT;
    ACCEPT;
} /*control_checkbox_style*/

/*
;   <control_radiobutton_styles>:
;       <control_radiobutton_style> {"," <control_radiobutton_style>}*
;
;   <control_radiobutton_style>:
;       <control_common_style> |
;       'NOTIFY' |
;       'FLAT' | 'PUSHLIKE' | 'AUTO' | 'NOAUTO'
;       'LEFTTEXT' | 'RIGHTBUTTON' |
;       'LEFT' | 'RIGHT' | 'CENTER' |
;       'TOP' | 'BOTTOM' | 'VCENTER' |
;       'MULTILINE' |
;       'TEXT' | 'BITMAP' | 'ICON'
;
*/
STATE(control_radiobutton_style)
    MARK;
    if (control_common_style()) {
    }
    else if (mdp_keywordi("NOTIFY")) {
        control_style |= BS_NOTIFY;
    }
    else if (mdp_keywordi("FLAT")) {
        control_style |= BS_FLAT;
    }
    else if (mdp_keywordi("PUSHLIKE")) {
        control_style |= BS_PUSHLIKE;
    }
    else if (mdp_keywordi("AUTO")) {
        switch (control_style & BS_TYPEMASK) {
        case BS_RADIOBUTTON:
            control_style &= ~BS_TYPEMASK;
            control_style |= BS_AUTORADIOBUTTON;
            break;
        }//switch
    }
    else if (mdp_keywordi("NOAUTO")) {
        switch (control_style & BS_TYPEMASK) {
        case BS_AUTORADIOBUTTON:
            control_style &= ~BS_TYPEMASK;
            control_style |= BS_RADIOBUTTON;
            break;
        }//switch
    }
    else if (mdp_keywordi("LEFTTEXT") || mdp_keywordi("RIGHTBUTTON")) {
        control_style |= BS_LEFTTEXT;
    }
    else if (mdp_keywordi("LEFT")) {
        control_style &= ~(BS_LEFT | BS_RIGHT | BS_CENTER);
        control_style |= BS_LEFT;
    }
    else if (mdp_keywordi("RIGHT")) {
        control_style &= ~(BS_LEFT | BS_RIGHT | BS_CENTER);
        control_style |= BS_RIGHT;
    }
    else if (mdp_keywordi("CENTER")) {
        control_style &= ~(BS_LEFT | BS_RIGHT | BS_CENTER);
        control_style |= BS_CENTER;
    }
    else if (mdp_keywordi("TOP")) {
        control_style &= ~(BS_TOP | BS_BOTTOM | BS_VCENTER);
        control_style |= BS_TOP;
    }
    else if (mdp_keywordi("BOTTOM")) {
        control_style &= ~(BS_TOP | BS_BOTTOM | BS_VCENTER);
        control_style |= BS_BOTTOM;
    }
    else if (mdp_keywordi("VCENTER")) {
        control_style &= ~(BS_TOP | BS_BOTTOM | BS_VCENTER);
        control_style |= BS_VCENTER;
    }
    else if (mdp_keywordi("MULTILINE")) {
        control_style |= BS_MULTILINE;
    }
    else if (mdp_keywordi("TEXT")) {
        control_style &= ~(BS_TEXT | BS_BITMAP | BS_ICON);
        control_style |= BS_TEXT;
    }
    else if (mdp_keywordi("BITMAP")) {
        control_style &= ~(BS_TEXT | BS_BITMAP | BS_ICON);
        control_style |= BS_BITMAP;
    }
    else if (mdp_keywordi("ICON")) {
        control_style &= ~(BS_TEXT | BS_BITMAP | BS_ICON);
        control_style |= BS_ICON;
    }
    else REJECT;
    ACCEPT;
} /*control_radiobutton_style*/

/*
;   <control_groupbox_styles>:
;       <control_groupbox_style> {"," <control_groupbox_style>}*
;
;   <control_groupbox_style>:
;       <control_common_style> |
;       'NOTIFY' |
;       'FLAT' |
;       'LEFT' | 'RIGHT' | 'CENTER'
;
*/
STATE(control_groupbox_style)
    MARK;
    if (control_common_style()) {
    }
    else if (mdp_keywordi("NOTIFY")) {
        control_style |= BS_NOTIFY;
    }
    else if (mdp_keywordi("FLAT")) {
        control_style |= BS_FLAT;
    }
    else if (mdp_keywordi("LEFT")) {
        control_style &= ~(BS_LEFT | BS_RIGHT | BS_CENTER);
        control_style |= BS_LEFT;
    }
    else if (mdp_keywordi("RIGHT")) {
        control_style &= ~(BS_LEFT | BS_RIGHT | BS_CENTER);
        control_style |= BS_RIGHT;
    }
    else if (mdp_keywordi("CENTER")) {
        control_style &= ~(BS_LEFT | BS_RIGHT | BS_CENTER);
        control_style |= BS_CENTER;
    }
    else REJECT;
    ACCEPT;
} /*control_groupbox_style*/

/*
;   <control_ownerdraw_styles>:
;       <control_ownerdraw_style> {"," <control_ownerdraw_style>}*
;
;   <control_ownerdraw_style>:
;       <control_common_style> |
;       'NOTIFY'
;
*/
STATE(control_ownerdraw_style)
    MARK;
    if (control_common_style()) {
    }
    else if (mdp_keywordi("NOTIFY")) {
        control_style |= BS_NOTIFY;
    }
    else REJECT;
    ACCEPT;
} /*control_ownerdraw_style*/

/*
;   <control_listbox_styles>:
;      <control_listbox_style> {"," <control_listbox_style>}*
;
;   <control_listbox_style>:
;      <control_common_style> |
;      'SORT' |
;      'MULTIPLESEL' | 'EXTENDEDSEL' | 'NOSEL' |
;      'MULTICOLUMN' |
;      'INTEGRALHEIGHT' | 'NOINTEGRALHEIGHT' |
;      'DISABLENOSCROLL' |
;      'USETABSTOPS' |
;      'OWNERDRAWFIXED' | 'OWNERDRAWVARIABLE |
;      'HASSTRINGS'
;
*/
STATE(control_listbox_style)
    MARK;
    if (control_common_style()) {
    }
    else if (mdp_keywordi("SORT")) {
        control_style |= LBS_SORT;
    }
    else if (mdp_keywordi("MULTIPLESEL")) {
        control_style &= ~(LBS_MULTIPLESEL | LBS_EXTENDEDSEL | LBS_NOSEL);
        control_style |= LBS_MULTIPLESEL;
    }
    else if (mdp_keywordi("EXTENDEDSEL")) {
        control_style &= ~(LBS_MULTIPLESEL | LBS_EXTENDEDSEL | LBS_NOSEL);
        control_style |= LBS_EXTENDEDSEL;
    }
    else if (mdp_keywordi("NOSEL")) {
        control_style &= ~(LBS_MULTIPLESEL | LBS_EXTENDEDSEL | LBS_NOSEL);
        control_style |= LBS_NOSEL;
    }
    else if (mdp_keywordi("MULTICOLUMN")) {
        control_style |= LBS_MULTICOLUMN;
    }
    else if (mdp_keywordi("INTEGRALHEIGHT")) {
        control_style &= ~LBS_NOINTEGRALHEIGHT;
    }
    else if (mdp_keywordi("NOINTEGRALHEIGHT")) {
        control_style |= LBS_NOINTEGRALHEIGHT;
    }
    else if (mdp_keywordi("DISABLENOSCROLL")) {
        control_style |= LBS_DISABLENOSCROLL;
    }
    else if (mdp_keywordi("USETABSTOPS")) {
        control_style |= LBS_USETABSTOPS;
    }
    else if (mdp_keywordi("OWNERDRAWFIXED")) {
        control_style &= ~LBS_OWNERDRAWVARIABLE;
        control_style |= LBS_OWNERDRAWFIXED;
    }
    else if (mdp_keywordi("OWNERDRAWVARIABLE")) {
        control_style &= ~LBS_OWNERDRAWFIXED;
        control_style |= LBS_OWNERDRAWVARIABLE;
    }
    else if (mdp_keywordi("HASSTRINGS")) {
        control_style |= LBS_HASSTRINGS;
    }
    else REJECT;
    ACCEPT;
} /*control_listbox_style*/

/*
;   <control_combobox_styles>:
;      <control_combobox_style> {"," <control_combobox_style>}*
;
;   <control_combobox_style>:
;      <control_common_style> |
;      'SIMPLE' | 'DROPDOWN' | 'DROPDOWNLIST' |
;      'SORT' |
;      'AUTOHSCROLL' |
;      'INTEGRALHEIGHT' | 'NOINTEGRALHEIGHT' |
;      'DISABLENOSCROLL' |
;      'UPPERCASE' | 'LOWERCASE' | 'OEMCONVERT' |
;      'OWNERDRAWFIXED' | 'OWNERDRAWVARIABLE |
;      'HASSTRINGS' |
;      'EXTENDEDUI'
;
*/
STATE(control_combobox_style)
    MARK;
    if (control_common_style()) {
    }
    else if (mdp_keywordi("SIMPLE")) {
        control_style &= ~CBS_TYPEMASK;
        control_style |= CBS_SIMPLE;
    }
    else if (mdp_keywordi("DROPDOWN")) {
        control_style &= ~CBS_TYPEMASK;
        control_style |= CBS_DROPDOWN;
    }
    else if (mdp_keywordi("DROPDOWNLIST")) {
        control_style &= ~CBS_TYPEMASK;
        control_style |= CBS_DROPDOWNLIST;
    }
    else if (mdp_keywordi("SORT")) {
        control_style |= CBS_SORT;
    }
    else if (mdp_keywordi("AUTOHSCROLL")) {
        control_style |= CBS_AUTOHSCROLL;
    }
    else if (mdp_keywordi("INTEGRALHEIGHT")) {
        control_style &= ~CBS_NOINTEGRALHEIGHT;
    }
    else if (mdp_keywordi("NOINTEGRALHEIGHT")) {
        control_style |= CBS_NOINTEGRALHEIGHT;
    }
    else if (mdp_keywordi("DISABLENOSCROLL")) {
        control_style |= CBS_DISABLENOSCROLL;
    }
    else if (mdp_keywordi("UPPERCASE")) {
        control_style &= ~CBS_LOWERCASE;
        control_style |= CBS_UPPERCASE;
    }
    else if (mdp_keywordi("LOWERCASE")) {
        control_style &= ~CBS_UPPERCASE;
        control_style |= CBS_LOWERCASE;
    }
    else if (mdp_keywordi("OEMCONVERT")) {
        control_style |= CBS_OEMCONVERT;
    }
    else if (mdp_keywordi("OWNERDRAWFIXED")) {
        control_style &= ~CBS_OWNERDRAWVARIABLE;
        control_style |= CBS_OWNERDRAWFIXED;
    }
    else if (mdp_keywordi("OWNERDRAWVARIABLE")) {
        control_style &= ~CBS_OWNERDRAWFIXED;
        control_style |= CBS_OWNERDRAWVARIABLE;
    }
    else if (mdp_keywordi("HASSTRINGS")) {
        control_style |= CBS_HASSTRINGS;
    }
    else if (mdp_keywordi("EXTENDEDUI")) {
        control_properties |= CTP_EXTENDEDUI;
    }
    else REJECT;
    ACCEPT;
} /*control_combobox_style*/

/*
;   <control_statictext_styles>:
;       <control_statictext_style> {"," <control_statictext_style>}*
;
;   <control_statictext_style>:
;       <control_common_style> |
;       'LEFTNOWORDWRAP' | 'LEFT' | 'CENTER' | 'RIGHT' | 'SIMPLE' |
;       'PREFIX' | 'NOPREFIX' |
;       'ENDELLIPSIS' | 'PATHELLIPSIS' | 'WORDELLIPSIS' |
;       'SUNKEN' |
;       'NOTIFY'
;
*/
STATE(control_statictext_style)
    MARK;
    if (control_common_style()) {
    }
    else if (mdp_keywordi("LEFTNOWORDWRAP")) {
        control_style &= ~SS_TYPEMASK;
        control_style |= SS_LEFTNOWORDWRAP;
    }
    else if (mdp_keywordi("LEFT")) {
        control_style &= ~SS_TYPEMASK;
        control_style |= SS_LEFT;
    }
    else if (mdp_keywordi("CENTER")) {
        control_style &= ~SS_TYPEMASK;
        control_style |= SS_CENTER;
    }
    else if (mdp_keywordi("RIGHT")) {
        control_style &= ~SS_TYPEMASK;
        control_style |= SS_RIGHT;
    }
    else if (mdp_keywordi("SIMPLE")) {
        control_style &= ~SS_TYPEMASK;
        control_style |= SS_SIMPLE;
    }
    else if (mdp_keywordi("PREFIX")) {
        control_style &= ~SS_NOPREFIX;
    }
    else if (mdp_keywordi("NOPREFIX")) {
        control_style |= SS_NOPREFIX;
    }
    else if (mdp_keywordi("ENDELLIPSIS")) {
        control_style &= ~SS_ELLIPSISMASK;
        control_style |= SS_ENDELLIPSIS;
    }
    else if (mdp_keywordi("PATHELLIPSIS")) {
        control_style &= ~SS_ELLIPSISMASK;
        control_style |= SS_PATHELLIPSIS;
    }
    else if (mdp_keywordi("WORDELLIPSIS")) {
        control_style &= ~SS_ELLIPSISMASK;
        control_style |= SS_WORDELLIPSIS;
    }
    else if (mdp_keywordi("SUNKEN")) {
        control_style |= SS_SUNKEN;
    }
    else if (mdp_keywordi("NOTIFY")) {
        control_style |= SS_NOTIFY;
    }
    else REJECT;
    ACCEPT;
} /*control_statictext_style*/

/*
;   <control_edit_styles>:
;       <control_edit_style> {"," <control_edit_style>}*
;
;   <control_edit_style>:
;       <control_common_style> |
;       'LEFT' | 'CENTER' | 'RIGHT' | 'MULTILINE' |
;		'UPPERCASE' | 'LOWERCASE' | 'PASSWORD' |
;		'AUTOVSCROLL' | 'AUTOHSCROLL' | 'NOHIDESEL' |
;		'OEMCONVERT' | 'READONLY' | 'WANTRETURN' |
;		'NUMBER'
;
*/

STATE(control_edit_style)
    MARK;
    if (control_common_style()) {
    }
    else if (mdp_keywordi("LEFT")) {
        control_style |= ES_LEFT;
    }
    else if (mdp_keywordi("CENTER")) {
        control_style |= ES_CENTER;
    }
    else if (mdp_keywordi("RIGHT")) {
        control_style |= ES_RIGHT;
    }
	else if (mdp_keywordi("MULTILINE")) {
        control_style |= ES_MULTILINE;
    }
	else if (mdp_keywordi("UPPERCASE")) {
        control_style |= ES_UPPERCASE;
    }
	else if (mdp_keywordi("LOWERCASE")) {
        control_style |= ES_LOWERCASE;
    }
    else if (mdp_keywordi("PASSWORD")) {
        control_style |= ES_PASSWORD;
    }
	else if (mdp_keywordi("AUTOVSCROLL")) {
        control_style |= ES_AUTOVSCROLL ;
    }
	else if (mdp_keywordi("AUTOHSCROLL")) {
        control_style |= ES_AUTOHSCROLL;
    }
	else if (mdp_keywordi("NOHIDESEL")) {
        control_style |= ES_NOHIDESEL;
    }
	else if (mdp_keywordi("OEMCONVERT")) {
        control_style |= ES_OEMCONVERT;
    }
	else if (mdp_keywordi("READONLY")) {
        control_style |= ES_READONLY;
    }
	else if (mdp_keywordi("WANTRETURN")) {
        control_style |= ES_WANTRETURN ;
    }
	else if (mdp_keywordi("NUMBER")) {
        control_style |= ES_NUMBER;
    }
    else REJECT;
    ACCEPT;
} /*control_edit_style*/

/*
;   <control_tab_styles>:
;       <control_tab_style> {"," <control_tab_style>}*
;
;   <control_tab_style>:
;       <control_common_style> |
;       'SCROLLOPPOSITE' | 'BOTTOM' | 'RIGHT' |
;		'MULTISELECT' | 'FLATBUTTONS' |
;		'FORCEICONLEFT' | 'FORCELABELLEFT' |
;		'HOTTRACK' | 'VERTICAL' | 'TABS' |
;		'BUTTONS' | 'SINGLELINE' | 'MULTILINE' |
;		'RIGHTJUSTIFY' | 'FIXEDWIDTH' | 'RAGGEDRIGHT' |
;		'FOCUSONBUTTONDOWN' | 'OWNERDRAWFIXED' |
;		'TOOLTIPS' | 'FOCUSNEVER'
;
*/

STATE(control_tab_style)
    MARK;
    if (control_common_style()) {
    }
    else if (mdp_keywordi("SCROLLOPPOSITE")) {
        control_style |= TCS_SCROLLOPPOSITE;
    }
	else if (mdp_keywordi("BOTTOM")) {
        control_style |= TCS_BOTTOM;
    }
	else if (mdp_keywordi("RIGHT")) {
        control_style |= TCS_RIGHT;
    }
	else if (mdp_keywordi("MULTISELECT")) {
        control_style |= TCS_MULTISELECT;
    }
	else if (mdp_keywordi("FLATBUTTONS")) {
        control_style |= TCS_FLATBUTTONS;
    }
	else if (mdp_keywordi("FORCEICONLEFT")) {
        control_style |= TCS_FORCEICONLEFT;
    }
	else if (mdp_keywordi("FORCELABELLEFT")) {
        control_style |= TCS_FORCELABELLEFT;
    }
	else if (mdp_keywordi("HOTTRACK")) {
        control_style |= TCS_HOTTRACK;
    }
	else if (mdp_keywordi("VERTICAL")) {
        control_style |= TCS_VERTICAL ;
    }
	else if (mdp_keywordi("TABS")) {
        control_style |= TCS_TABS;
    }
	else if (mdp_keywordi("BUTTONS")) {
        control_style |= TCS_BUTTONS;
    }
	else if (mdp_keywordi("SINGLELINE")) {
        control_style |= TCS_SINGLELINE;
    }
	else if (mdp_keywordi("MULTILINE")) {
        control_style |= TCS_MULTILINE;
    }
	else if (mdp_keywordi("RIGHTJUSTIFY")) {
        control_style |= TCS_RIGHTJUSTIFY;
    }
	else if (mdp_keywordi("FIXEDWIDTH")) {
        control_style |= TCS_FIXEDWIDTH;
    }
	else if (mdp_keywordi("RAGGEDRIGHT")) {
        control_style |= TCS_RAGGEDRIGHT;
    }
	else if (mdp_keywordi("FOCUSONBUTTONDOWN")) {
        control_style |= TCS_FOCUSONBUTTONDOWN;
    }
	else if (mdp_keywordi("OWNERDRAWFIXED")) {
        control_style |= TCS_OWNERDRAWFIXED;
    }
	else if (mdp_keywordi("TOOLTIPS")) {
        control_style |= TCS_TOOLTIPS;
    }
	else if (mdp_keywordi("FOCUSNEVER")) {
        control_style |= TCS_FOCUSNEVER;
    }
    
    else REJECT;
    ACCEPT;
} /*control_tab_style*/

/*
;   <control_frame_styles>:
;       <control_frame_style> {"," <control_frame_style>}*
;
;   <control_frame_style>:
;       <control_common_style> |
;       'BLACK' | 'GRAY' | 'WHITE' | 'ETCHED' | 'ETCHEDHORZ' | 'ETCHEDVERT' |
;       'SUNKEN' |
;       'NOTIFY'
;
*/
STATE(control_frame_style)
    MARK;
    if (control_common_style()) {
    }
    else if (mdp_keywordi("BLACK")) {
        control_style &= ~SS_TYPEMASK;
        control_style |= SS_BLACKFRAME;
    }
    else if (mdp_keywordi("GRAY")) {
        control_style &= ~SS_TYPEMASK;
        control_style |= SS_GRAYFRAME;
    }
    else if (mdp_keywordi("WHITE")) {
        control_style &= ~SS_TYPEMASK;
        control_style |= SS_WHITEFRAME;
    }
    else if (mdp_keywordi("ETCHED")) {
        control_style &= ~SS_TYPEMASK;
        control_style |= SS_ETCHEDFRAME;
    }
    else if (mdp_keywordi("ETCHEDHORZ")) {
        control_style &= ~SS_TYPEMASK;
        control_style |= SS_ETCHEDHORZ;
    }
    else if (mdp_keywordi("ETCHEDVERT")) {
        control_style &= ~SS_TYPEMASK;
        control_style |= SS_ETCHEDVERT;
    }
    else if (mdp_keywordi("SUNKEN")) {
        control_style |= SS_SUNKEN;
    }
    else if (mdp_keywordi("NOTIFY")) {
        control_style |= SS_NOTIFY;
    }
    else REJECT;
    ACCEPT;
} /*control_frame_style*/

/*
;   <control_rect_styles>:
;       <control_rect_style> {"," <control_rect_style>}*
;
;   <control_rect_style>:
;       <control_common_style> |
;       'BLACK' | 'GRAY' | 'WHITE' |
;       'SUNKEN' |
;       'NOTIFY'
;
*/
STATE(control_rect_style)
    MARK;
    if (control_common_style()) {
    }
    else if (mdp_keywordi("BLACK")) {
        control_style &= ~SS_TYPEMASK;
        control_style |= SS_BLACKRECT;
    }
    else if (mdp_keywordi("GRAY")) {
        control_style &= ~SS_TYPEMASK;
        control_style |= SS_GRAYRECT;
    }
    else if (mdp_keywordi("WHITE")) {
        control_style &= ~SS_TYPEMASK;
        control_style |= SS_WHITERECT;
    }
    else if (mdp_keywordi("SUNKEN")) {
        control_style |= SS_SUNKEN;
    }
    else if (mdp_keywordi("NOTIFY")) {
        control_style |= SS_NOTIFY;
    }
    else REJECT;
    ACCEPT;
} /*control_rect_style*/

/*
;   <control_image_styles>:
;       <control_image_style> {"," <control_image_style>}*
;
;   <control_image_style>:
;       <control_common_style> |
;       'SUNKEN' |
;       'NOTIFY'
;
*/
STATE(control_image_style)
    MARK;
    if (control_common_style()) {
    }
    else if (mdp_keywordi("SUNKEN")) {
        control_style |= SS_SUNKEN;
    }
    else if (mdp_keywordi("NOTIFY")) {
        control_style |= SS_NOTIFY;
    }
    else REJECT;
    ACCEPT;
} /*control_image_style*/

/*
;   <control_static_image_style>:
;               'CENTERIMAGE' | 'RIGHTJUST' |
;               'REALSIZEIMAGE'
;               'SUNKEN' |
;               'NOTIFY'
;
*/
STATE(control_static_image_style)
    MARK;
    if (mdp_keywordi("CENTERIMAGE")) {
        control_style |= SS_CENTERIMAGE;
    }
    else if (mdp_keywordi("RIGHTJUST")) {
        control_style |= SS_RIGHTJUST;
    }
    else if (mdp_keywordi("REALSIZEIMAGE")) {
        control_style |= SS_REALSIZEIMAGE;
    }
    else if (mdp_keywordi("SUNKEN")) {
        control_style |= SS_SUNKEN;
    }
    else if (mdp_keywordi("NOTIFY")) {
        control_style |= SS_NOTIFY;
    }
    else REJECT;
    ACCEPT;
} /*control_static_image_style*/

/*
;   <control_bitmap_styles>:
;               <control_bitmap_style> {"," <control_bitmap_style>}*
;
;   <control_bitmap_style>:
;               <control_common_style> |
;               <control_static_image_style>
;
*/
STATE(control_bitmap_style)
    MARK;
    if (control_common_style()) {
    }
    else if (control_static_image_style()) {
    }
    else REJECT;
    ACCEPT;
} /*control_bitmap_style*/

/*
;   <control_icon_styles>:
;               <control_icon_style> {"," <control_icon_style>}*
;
;   <control_icon_style>:
;               <control_common_style> |
;               <control_static_image_style>
;
*/
STATE(control_icon_style)
    MARK;
    if (control_common_style()) {
    }
    else if (control_static_image_style()) {
    }
    else REJECT;
    ACCEPT;
} /*control_icon_style*/

/*
;   <control_metafile_styles>:
;               <control_metafile_style> {"," <control_metafile_style>}*
;
;   <control_metafile_style>:
;               <control_common_style> |
;               <control_static_image_style>
;
*/
STATE(control_metafile_style)
    MARK;
    if (control_common_style()) {
    }
    else if (control_static_image_style()) {
    }
    else REJECT;
    ACCEPT;
} /*control_metafile_style*/

/*
;	<control_subkey_item>:
;               <control_text_specification>                            |
;               <control_value_specification>                           |
;               <control_range_specification>                           |
;               <control_linesize_specification>                        |
;               <control_pagesize_specification>                        |
;				<control_thumbsize_specification>                       |
;               <control_divisor_specification>                         |
;               <control_gamma_specification>                           |
;               <control_position_specification>                        |
;               <control_size_specification>                            |
;               <control_editsize_specification>                        |
;               <control_editstyle_specification>                       |
:               <control_editstyleex_specification>                     |
;               <control_noeditborder_specification>                    |
;               <control_color_specification>                           |
;               <control_syscolor_specification>                        |
;               <control_tooltip_specification>                         |
;               <control_font_specification>                            |
;               <control_fontname_specification>                        |   ?? same as font?
;               <control_fontsize_specification>                        |
;               <control_fontcolor_specification>                       |
;               <control_fontsyscolor_specification>                    |
;               <control_tic_specification>                             |
;               <control_ticfreq_specification>                         |
;               <control_ticlist_specification>                         |   ?? same as tic?
;               <control_enabled_specification>                         |
;               <control_disabled_specification>                        |
;               <control_visible_specification>                         |
;               <control_invisible_specification>                       |
;               <control_tracking_specification>                        |   ?? same as track?
;               <control_track_specification>                           |
;               <control_notrack_specification>                         |
;               <control_action_specification>                          |
;               <control_image_specification>                           |
;               <control_theme_specification>                           |
;               <control_anchor_specification>                          |
;               <control_tab_specification>                             
;
*/
STATE(couple)
    MARK;
    if (!mdp_char('(')) REJECT;
    if (constant_integer_expression()) {
    }
    else if (mdp_char('*')) {
        // '*' means unspecified (-1)
        push_constant(LIT_INTEGER);
        push_constant(-1);
    }
    else REJECT;
    push_op(OP_ARG);
    if (!mdp_char(',')) REJECT;
    if (constant_integer_expression()) {
    }
    else if (mdp_char('*')) {
        // '*' means unspecified (-1)
        push_constant(LIT_INTEGER);
        push_constant(-1);
    }
    else REJECT;
    push_op(OP_ARG);
    if (!mdp_char(')')) REJECT;
    ACCEPT;
} /*couple*/

STATE(control_text_item)
    MARK;
    push_op(OP_DUPL);               //duplicate <index>
    push_proc_addr(setCtlText);
    push_op(OP_XCHG);               //exchange <index>, <proc_addr>
    push_op(OP_ARG);                //push arg <index>
    if (mdp_keywordi("Text")) {
        if (!mdp_char('=')) REJECT;
        if (!string_literal()) REJECT;
    }
    else if (string_literal()) {
        //obsolescent: string by itself presumes Text=
        //Should we check that it's the first subkey??? NO!
    }
    else REJECT;
    push_op(OP_ARG);                //push arg <string>
    push_constant(LIT_INTEGER);
    push_constant(2);               //nargs
    push_op(OP_CALL);               //call setCtlText(<index>, <string>)
    push_op(OP_POP);                //discard return value
    ACCEPT;
} /*control_text_item*/

STATE(control_subkey_item)
    int iType;
    MARK;
    if (control_text_item()) {
    }
    else if (mdp_keywordi("ToolTip")) {
        if (!mdp_char('=')) REJECT;
        push_op(OP_DUPL);               //duplicate <index>
        push_proc_addr(setCtlToolTip);
        push_op(OP_XCHG);               //exchange <index>, <proc_addr>
        push_op(OP_ARG);                //push arg <index>
        if (!string_literal()) REJECT;
        push_op(OP_ARG);                //push arg <text>
        push_constant(LIT_INTEGER);
        push_constant(0);               //style = 0
        push_op(OP_ARG);                //push arg <style>
        push_constant(LIT_INTEGER);
        push_constant(3);               //nargs
        push_op(OP_CALL);               //call setCtlToolTip(<index>, <text>, <style>)
        push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("FontColor") || mdp_keywordi("TextColor")) {
        if (!mdp_char('=')) REJECT;
        push_op(OP_DUPL);               //duplicate <index>
        push_proc_addr(setCtlFontColor);
        push_op(OP_XCHG);               //exchange <index>, <proc_addr>
        push_op(OP_ARG);                //push arg <index>
        if (!color()) REJECT;
        push_op(OP_ARG);                //push arg <color>
        push_constant(LIT_INTEGER);
        push_constant(2);               //nargs
        push_op(OP_CALL);               //call setCtlFontColor(<index>, <color>)
        push_op(OP_POP);                //discard return value
    }

    else if (mdp_keywordi("FontSysColor") || mdp_keywordi("TextSysColor")) {
        if (!mdp_char('=')) REJECT;
        push_op(OP_DUPL);               //duplicate <index>
        push_proc_addr(setCtlFontSysColor);
        push_op(OP_XCHG);               //exchange <index>, <proc_addr>
        push_op(OP_ARG);                //push arg <index>
        if (!constant_integer_expression()) REJECT;
        push_op(OP_ARG);                //push arg <color>
        push_constant(LIT_INTEGER);
        push_constant(2);               //nargs
        push_op(OP_CALL);               //call setCtlFontSysColor(<index>, <color>)
        push_op(OP_POP);                //discard return value
    }

    else if (mdp_keywordi("Theme")) {
        if (!mdp_char('=')) REJECT;
        push_op(OP_DUPL);               //duplicate <index>
        push_proc_addr(setCtlTheme);
        push_op(OP_XCHG);               //exchange <index>, <proc_addr>
        push_op(OP_ARG);                //push arg <index>
        push_constant(LIT_INTEGER);
		if (mdp_keywordi("on"))
            push_constant(1);          
        else if (mdp_keywordi("off"))
			push_constant(0);
		else if (mdp_keywordi("default"))
            push_constant(-1);
		else REJECT;
        push_op(OP_ARG);                //push arg <state>
        push_constant(LIT_INTEGER);
        push_constant(2);               //nargs
        push_op(OP_CALL);               //call setCtlTheme(<index>, <state>)
        push_op(OP_POP);                //discard return value
    }
	
    else if (mdp_keywordi("Scripting")) {
        if (!mdp_char('=')) REJECT;
        push_op(OP_DUPL);               //duplicate <index>
        push_proc_addr(setCtlScripting);
        push_op(OP_XCHG);               //exchange <index>, <proc_addr>
        push_op(OP_ARG);                //push arg <index>
        push_constant(LIT_INTEGER);
		if (mdp_keywordi("on"))
            push_constant(1);          
        else if (mdp_keywordi("off"))
			push_constant(0);
		else if (mdp_keywordi("default"))
            push_constant(-1);
		else REJECT;
        push_op(OP_ARG);                //push arg <state>
        push_constant(LIT_INTEGER);
        push_constant(2);               //nargs
        push_op(OP_CALL);               //call setCtlScripting(<index>, <state>)
        push_op(OP_POP);                //discard return value
    }

	else if (mdp_keywordi("editstyleex")) {
        if (!mdp_char('=')) REJECT;
        push_op(OP_DUPL);               //duplicate <index>
        push_proc_addr(setCtlBuddyStyleEx);
		push_op(OP_XCHG);               //exchange <index>, <proc_addr>
        push_op(OP_ARG);                //push arg <index>
        push_constant(LIT_INTEGER);
        push_constant(1);              //buddy = 1
        push_op(OP_ARG);                //push arg <buddy>
		if (mdp_keywordi("staticedge")){
            push_constant(LIT_INTEGER);
			push_constant(WS_EX_STATICEDGE);          
        } else if (mdp_keywordi("clientedge")){
			push_constant(LIT_INTEGER);
			push_constant(WS_EX_CLIENTEDGE);
		} else if (mdp_keywordi("dialogframe")){
			push_constant(LIT_INTEGER);
			push_constant(WS_EX_DLGMODALFRAME);
		} else if (!constant_integer_expression()) REJECT;
        push_op(OP_ARG);                //push arg <flags>
        push_constant(LIT_INTEGER);
        push_constant(3);               //nargs
        push_op(OP_CALL);               //call setCtlBuddyStyleEx(<index>, <buddy=1>, <flags>)
        push_op(OP_POP);                //discard return value
    }

	else if (mdp_keywordi("editstyle")) {
        if (!mdp_char('=')) REJECT;
        push_op(OP_DUPL);               //duplicate <index>
        push_proc_addr(setCtlBuddyStyle);
		push_op(OP_XCHG);               //exchange <index>, <proc_addr>
        push_op(OP_ARG);                //push arg <index>
        push_constant(LIT_INTEGER);
        push_constant(1);              //buddy = 1
        push_op(OP_ARG);                //push arg <buddy>
		if (mdp_keywordi("border")){
            push_constant(LIT_INTEGER);
			push_constant(WS_BORDER);   
		} else if (!constant_integer_expression()) REJECT;
		push_op(OP_ARG);                //push arg <flags>
        push_constant(LIT_INTEGER);
        push_constant(3);               //nargs
        push_op(OP_CALL);               //call setCtlBuddyStyle(<index>, <buddy=1>, <flags>)
        push_op(OP_POP);                //discard return value
    }

	else if (mdp_keywordi("noeditborder")) {
        push_op(OP_DUPL);               //duplicate <index>
        push_proc_addr(clearCtlBuddyStyle);
        push_op(OP_XCHG);               //exchange <index>, <proc_addr>
        push_op(OP_ARG);                //push arg <index>
        push_constant(LIT_INTEGER);
        push_constant(1);               //buddy = 1
        push_op(OP_ARG);                //push arg <buddy>
		push_constant(LIT_INTEGER);
        push_constant(WS_BORDER);       //WS_BORDER
        push_op(OP_ARG);                //push arg <flags>
        push_constant(LIT_INTEGER);
        push_constant(3);               //nargs
        push_op(OP_CALL);               //call clearCtlBuddyStyle(<index>, <buddy=1>, <flags>)
        push_op(OP_POP);                //discard return value
    }

    else if (mdp_keywordi("Order")) {
        if (!mdp_char('=')) REJECT;
        push_op(OP_DUPL);               //duplicate <index>
        push_proc_addr(setCtlOrder);
        push_op(OP_XCHG);               //exchange <index>, <proc_addr>
        push_op(OP_ARG);                //push arg <index>
        if (!constant_integer_expression()) REJECT;
        push_op(OP_ARG);                //push arg <color>
        push_constant(LIT_INTEGER);
        push_constant(2);               //nargs
        push_op(OP_CALL);               //call setCtlColor(<index>, <color>)
        push_op(OP_POP);                //discard return value
    }

	else if (mdp_keywordi("Color")) {
        if (!mdp_char('=')) REJECT;
        push_op(OP_DUPL);               //duplicate <index>
        push_proc_addr(setCtlColor);
        push_op(OP_XCHG);               //exchange <index>, <proc_addr>
        push_op(OP_ARG);                //push arg <index>
        if (mdp_keywordi("Transparent")) {
            push_constant(LIT_INTEGER);
            push_constant(-1);          // -1 means transparent.
        }
        else if (!color()) REJECT;
        push_op(OP_ARG);                //push arg <color>
        push_constant(LIT_INTEGER);
        push_constant(2);               //nargs
        push_op(OP_CALL);               //call setCtlColor(<index>, <color>)
        push_op(OP_POP);                //discard return value
    }

	else if (mdp_keywordi("SysColor")) {
        if (!mdp_char('=')) REJECT;
        push_op(OP_DUPL);               //duplicate <index>
        push_proc_addr(setCtlSysColor);
        push_op(OP_XCHG);               //exchange <index>, <proc_addr>
        push_op(OP_ARG);                //push arg <index>
        if (!constant_integer_expression()) REJECT;
        push_op(OP_ARG);                //push arg <color>
        push_constant(LIT_INTEGER);
        push_constant(2);               //nargs
        push_op(OP_CALL);               //call setCtlSysColor(<index>, <color>)
        push_op(OP_POP);                //discard return value
    }
	else if (mdp_keywordi("Anchor")) {
        if (!mdp_char('=')) REJECT;
        push_op(OP_DUPL);               //duplicate <index>
        push_proc_addr(setCtlAnchor);
        push_op(OP_XCHG);               //exchange <index>, <proc_addr>
        push_op(OP_ARG);                //push arg <index>
        if (!constant_integer_expression()) REJECT;
        push_op(OP_ARG);                //push arg <flags>
        push_constant(LIT_INTEGER);
        push_constant(2);               //nargs
        push_op(OP_CALL);               //call setCtlAnchor(<index>, <flags>)
        push_op(OP_POP);                //discard return value
    }

    
	else if (mdp_keywordi("DefVal")) {
        if (!mdp_char('=')) REJECT;
        push_op(OP_DUPL);               //duplicate <index>
        push_proc_addr(setCtlDefVal);
        push_op(OP_XCHG);               //exchange <index>, <proc_addr>
        push_op(OP_ARG);                //push arg <index>
        if (!constant_integer_expression()) REJECT;
        push_op(OP_ARG);                //push arg <value>
        push_constant(LIT_INTEGER);
        push_constant(2);               //nargs
        push_op(OP_CALL);               //call setCtlVal(<index>, <value>)
        push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("Val*ue")) {
        if (!mdp_char('=')) REJECT;
        push_op(OP_DUPL);               //duplicate <index>
        push_proc_addr(setCtlVal);
        push_op(OP_XCHG);               //exchange <index>, <proc_addr>
        push_op(OP_ARG);                //push arg <index>
        if (!constant_integer_expression()) REJECT;
        push_op(OP_ARG);                //push arg <value>
        push_constant(LIT_INTEGER);
        push_constant(2);               //nargs
        push_op(OP_CALL);               //call setCtlVal(<index>, <value>)
        push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("Range")) {
        if (!mdp_char('=')) REJECT;
        push_op(OP_DUPL);               //duplicate <index>
        push_proc_addr(setCtlRange);
        push_op(OP_XCHG);               //exchange <index>, <proc_addr>
        push_op(OP_ARG);                //push arg <index>
        if (!couple()) REJECT;          //push arg <lo>, <hi>
        push_constant(LIT_INTEGER);
        push_constant(3);               //nargs
        push_op(OP_CALL);               //call setCtlRange(<index>, <lo>, <hi>)
        push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("Pos*ition")) {
        if (!mdp_char('=')) REJECT;
        push_op(OP_DUPL);               //duplicate <index>
        push_proc_addr(setCtlPos);
        push_op(OP_XCHG);               //exchange <index>, <proc_addr>
        push_op(OP_ARG);                //push arg <index>
        if (!couple()) REJECT;          //push arg <x>, <y>
        push_constant(LIT_INTEGER);
        push_constant(-1);              //w = -1
        push_op(OP_ARG);                //push arg <w>
        push_constant(LIT_INTEGER);
        push_constant(-1);              //h = -1
        push_op(OP_ARG);                //push arg <h>
        push_constant(LIT_INTEGER);
        push_constant(5);               //nargs
        push_op(OP_CALL);               //call setCtlPos(<index>, <x>, <y>, -1, -1)
        push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("Size")) {
        if (!mdp_char('=')) REJECT;
        push_op(OP_DUPL);               //duplicate <index>
        push_proc_addr(setCtlPos);
        push_op(OP_XCHG);               //exchange <index>, <proc_addr>
        push_op(OP_ARG);                //push arg <index>
        push_constant(LIT_INTEGER);
        push_constant(-1);              //x = -1
        push_op(OP_ARG);                //push arg <x>
        push_constant(LIT_INTEGER);
        push_constant(-1);              //y = -1
        push_op(OP_ARG);                //push arg <y>
        if (!couple()) REJECT;          //push arg <w>, <h>
        push_constant(LIT_INTEGER);
        push_constant(5);               //nargs
        push_op(OP_CALL);               //call setCtlPos(<index>, -1, -1, <w>, <h>)
        push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("EditSize")) {
        if (!mdp_char('=')) REJECT;
        push_op(OP_DUPL);               //duplicate <index>
        push_proc_addr(setCtlEditSize);
        push_op(OP_XCHG);               //exchange <index>, <proc_addr>
        push_op(OP_ARG);                //push arg <index>
        if (!couple()) REJECT;          //push arg <w>, <h>
        push_constant(LIT_INTEGER);
        push_constant(3);               //nargs
        push_op(OP_CALL);               //call setCtlEditSize(<index>, <w>, <h>)
        push_op(OP_POP);                //discard return value
    }
	else if (mdp_keywordi("Tab")) {
        if (!mdp_char('=')) REJECT;
        push_op(OP_DUPL);               //duplicate <index>
        push_proc_addr(setCtlTab);
        push_op(OP_XCHG);               //exchange <index>, <proc_addr>
        push_op(OP_ARG);                //push arg <index>
        if (!couple()) REJECT;          //push arg <t>, <s>
        push_constant(LIT_INTEGER);
        push_constant(3);               //nargs
        push_op(OP_CALL);               //call setCtlTab(<index>, <t>, <s>)
        push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("FontSize")) {    //not yet implemented
        if (!mdp_char('=')) REJECT;
        if (!constant_integer_expression()) REJECT;
    }
    else if (mdp_keywordi("Font")) {        //not yet implemented
        if (!mdp_char('=')) REJECT;
        if (!string_literal()) REJECT;
    }
    else if (mdp_keywordi("Line*Size")) {
        if (!mdp_char('=')) REJECT;
        push_op(OP_DUPL);               //duplicate <index>
        push_proc_addr(setCtlLineSize);
        push_op(OP_XCHG);               //exchange <index>, <proc_addr>
        push_op(OP_ARG);                //push arg <index>
        if (!constant_integer_expression()) REJECT;
        push_op(OP_ARG);                //push arg <value>
        push_constant(LIT_INTEGER);
        push_constant(2);               //nargs
        push_op(OP_CALL);               //call setCtlLineSize(<index>, <value>)
        push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("Page*Size")) {
        if (!mdp_char('=')) REJECT;
        push_op(OP_DUPL);               //duplicate <index>
        push_proc_addr(setCtlPageSize);
        push_op(OP_XCHG);               //exchange <index>, <proc_addr>
        push_op(OP_ARG);                //push arg <index>
        if (!constant_integer_expression()) REJECT;
        push_op(OP_ARG);                //push arg <value>
        push_constant(LIT_INTEGER);
        push_constant(2);               //nargs
        push_op(OP_CALL);               //call setCtlPageSize(<index>, <value>)
        push_op(OP_POP);                //discard return value
    }
	else if (mdp_keywordi("Thumb*Size")) {
        if (!mdp_char('=')) REJECT;
        push_op(OP_DUPL);               //duplicate <index>
        push_proc_addr(setCtlThumbSize);
        push_op(OP_XCHG);               //exchange <index>, <proc_addr>
        push_op(OP_ARG);                //push arg <index>
        if (!constant_integer_expression()) REJECT;
        push_op(OP_ARG);                //push arg <value>
        push_constant(LIT_INTEGER);
        push_constant(2);               //nargs
        push_op(OP_CALL);               //call setCtlThumbSize(<index>, <value>)
        push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("Divisor")) {
        if (!mdp_char('=')) REJECT;
        push_op(OP_DUPL);               //duplicate <index>
        push_proc_addr(setCtlDivisor);
        push_op(OP_XCHG);               //exchange <index>, <proc_addr>
        push_op(OP_ARG);                //push arg <index>
        if (!constant_integer_expression()) REJECT;
        push_op(OP_ARG);                //push arg <value>
        push_constant(LIT_INTEGER);
        push_constant(2);               //nargs
        push_op(OP_CALL);               //call setCtlDivisor(<index>, <value>)
        push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("Gamma")) {
        if (!mdp_char('=')) REJECT;
        push_op(OP_DUPL);               //duplicate <index>
        push_proc_addr(setCtlGamma);
        push_op(OP_XCHG);               //exchange <index>, <proc_addr>
        push_op(OP_ARG);                //push arg <index>
        if (!constant_integer_expression()) REJECT;
        push_op(OP_ARG);                //push arg <value>
        push_constant(LIT_INTEGER);
        push_constant(2);               //nargs
        push_op(OP_CALL);               //call setCtlGamma(<index>, <value>)
        push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("TicFreq")) {
        if (!mdp_char('=')) REJECT;
        push_op(OP_DUPL);               //duplicate <index>
        push_proc_addr(setCtlTicFreq);
        push_op(OP_XCHG);               //exchange <index>, <proc_addr>
        push_op(OP_ARG);                //push arg <index>
        if (!constant_integer_expression()) REJECT;
        push_op(OP_ARG);                //push arg <value>
        push_constant(LIT_INTEGER);
        push_constant(2);               //nargs
        push_op(OP_CALL);               //call setCtlTicFreq(<index>, <value>)
        push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("Tic")) {     //not yet implemented
        if (!mdp_char('=')) REJECT;
        if (!mdp_char('(')) REJECT;
        do {
            if (!constant_integer_expression()) REJECT;
        } while (mdp_char(','));
        if (!mdp_char(')')) REJECT;
    }
    else if (mdp_keywordi("Enable*d")) {
        push_op(OP_DUPL);               //duplicate <index>
        push_proc_addr(enableCtl);
        push_op(OP_XCHG);               //exchange <index>, <proc_addr>
        push_op(OP_ARG);                //push arg <index>
        push_constant(LIT_INTEGER);
        push_constant(3);               //3=enabled/visible
        push_op(OP_ARG);                //push arg <level>
        push_constant(LIT_INTEGER);
        push_constant(2);               //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("Disable*d")) {
        push_op(OP_DUPL);               //duplicate <index>
        push_proc_addr(enableCtl);
        push_op(OP_XCHG);               //exchange <index>, <proc_addr>
        push_op(OP_ARG);                //push arg <index>
        push_constant(LIT_INTEGER);
        push_constant(1);               //1=disabled/visible
        push_op(OP_ARG);                //push arg <level>
        push_constant(LIT_INTEGER);
        push_constant(2);               //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }
#if 0
    else if (mdp_keywordi("Visible")) { //not yet implemented
    }
#endif
    else if (mdp_keywordi("Invisible")) {
        push_op(OP_DUPL);               //duplicate <index>
        push_proc_addr(enableCtl);
        push_op(OP_XCHG);               //exchange <index>, <proc_addr>
        push_op(OP_ARG);                //push arg <index>
        push_constant(LIT_INTEGER);
        push_constant(0);               //0=disabled/invisible
        push_op(OP_ARG);                //push arg <level>
        push_constant(LIT_INTEGER);
        push_constant(2);               //nargs
        push_op(OP_CALL);
        push_op(OP_POP);                //discard return value
    }
#if 0   // following belong in control_trackbar_style!!!
    else if (mdp_keywordi("Horz")) {
    }
    else if (mdp_keywordi("Horiz*ontal")) {
    }
    else if (mdp_keywordi("Vert*ical")) {
    }
    else if (mdp_keywordi("AutoTicks")) {
    }
    else if (mdp_keywordi("NoTicks")) {
    }
    else if (mdp_keywordi("Bottom")) {
    }
    else if (mdp_keywordi("Top")) {
    }
    else if (mdp_keywordi("Right")) {
    }
    else if (mdp_keywordi("Left")) {
    }
    else if (mdp_keywordi("Both")) {
    }
    else if (mdp_keywordi("EnableSelRange")) {
    }
    else if (mdp_keywordi("FixedLength")) {
    }
    else if (mdp_keywordi("NoThumb")) {
    }
#endif
    else if (mdp_keywordi("NoTrack")) {
        push_op(OP_DUPL);               //duplicate <index>
        push_proc_addr(clearCtlProperties);
        push_op(OP_XCHG);               //exchange <index>, <proc_addr>
        push_op(OP_ARG);                //push arg <index>
        push_constant(LIT_INTEGER);
        push_constant(CTP_TRACK);       //Track
        push_op(OP_ARG);                //push arg <level>
        push_constant(LIT_INTEGER);
        push_constant(2);               //nargs
        push_op(OP_CALL);               //clearCtlProperties(n, CTP_TRACK)
        push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("Track")) {
        push_op(OP_DUPL);               //duplicate <index>
        push_proc_addr(setCtlProperties);
        push_op(OP_XCHG);               //exchange <index>, <proc_addr>
        push_op(OP_ARG);                //push arg <index>
        push_constant(LIT_INTEGER);
        push_constant(CTP_TRACK);       //Track
        push_op(OP_ARG);                //push arg <level>
        push_constant(LIT_INTEGER);
        push_constant(2);               //nargs
        push_op(OP_CALL);               //setCtlProperties(n, CTP_TRACK)
        push_op(OP_POP);                //discard return value
    }
    else if (mdp_keywordi("Action")) {
        if (!mdp_char('=')) REJECT;
        push_op(OP_DUPL);               //duplicate <index>
        push_proc_addr(setCtlAction);
        push_op(OP_XCHG);               //exchange <index>, <proc_addr>
        push_op(OP_ARG);                //push arg <index>
        push_constant(LIT_INTEGER);
        if (mdp_keywordi("None")) {
            push_constant(CA_NONE);
        }
        else if (mdp_keywordi("Cancel")) {
            push_constant(CA_CANCEL);
        }
        else if (mdp_keywordi("Apply")) {
            push_constant(CA_APPLY);
        }
        else if (mdp_keywordi("Preview")) {
            push_constant(CA_PREVIEW);
        }
        else if (mdp_keywordi("Edit")) {
            push_constant(CA_EDIT);
        }
        else if (mdp_keywordi("About")) {
            push_constant(CA_ABOUT);
        }
        else if (mdp_keywordi("Reset")) {
            push_constant(CA_RESET);
        }
        else REJECT;
        push_op(OP_ARG);                //push arg <action>
        push_constant(LIT_INTEGER);
        push_constant(2);               //nargs
        push_op(OP_CALL);               //setCtlAction(n, <action>)
        push_op(OP_POP);                //discard return value
    }
/*
;   <control_image_specification>:
;               'Image' "=" <filename> ["/" <character_constant>]   //optional file type
;               'Bitmap' "=" <filename> |
;               'Metafile' "=" <filename> |
;               'Enhmetafile' "=" <filename> |
;               'Jpeg' "=" <filename> |
;               'Gif' "=" <filename> |
;               'Mig' "=" <filename> |
;               'Icon' "=" <filename> |
;               'Cursor' "=" <filename>
;
*/
    else if (mdp_keywordi("Image")) {
        if (!mdp_char('=')) REJECT;
        push_op(OP_DUPL);               //duplicate <index>
        push_proc_addr(setCtlImage);
        push_op(OP_XCHG);               //exchange <index>, <proc_addr>
        push_op(OP_ARG);                //push arg <index>
        if (!string_literal()) REJECT;
        push_op(OP_ARG);                //push arg <string>
        if (mdp_char('/')) {
            if (!character_constant()) REJECT;
        }
        else {
            push_constant(LIT_INTEGER);
            push_constant(0);           //<type> = 0 (unspecified)
        }
        push_op(OP_ARG);                //push arg <type>
        push_constant(LIT_INTEGER);
        push_constant(3);               //nargs
        push_op(OP_CALL);               //call setCtlImage(<index>, <string>, <type>)
        push_op(OP_POP);                //discard return value
    }
    else if ((iType = 'B', mdp_keywordi("Bitmap")) ||
             (iType = 'W', mdp_keywordi("Metafile")) ||
             (iType = 'E', mdp_keywordi("Enhmetafile")) ||
             (iType = 'J', mdp_keywordi("Jpeg")) ||
             (iType = 'G', mdp_keywordi("Gif")) ||
             (iType = 'M', mdp_keywordi("Mig")) ||
             (iType = 'I', mdp_keywordi("Icon")) ||
             (iType = 'C', mdp_keywordi("Cursor")))
    {
        if (!mdp_char('=')) REJECT;
        push_op(OP_DUPL);               //duplicate <index>
        push_proc_addr(setCtlImage);
        push_op(OP_XCHG);               //exchange <index>, <proc_addr>
        push_op(OP_ARG);                //push arg <index>
        if (!string_literal()) REJECT;
        push_op(OP_ARG);                //push arg <string>
        push_constant(LIT_INTEGER);
        push_param(iType);
        push_op(OP_ARG);                //push arg <type>
        push_constant(LIT_INTEGER);
        push_constant(3);               //nargs
        push_op(OP_CALL);               //call setCtlImage(<index>, <string>, <type>)
        push_op(OP_POP);                //discard return value
    }
    else REJECT;
    ACCEPT;
} /*control_subkey_item*/

/*************************** <region_expression> ************************
;
; Syntax:
;
;	<region_expression> ::=
;		<region_OR_expression>
;
;	<region_OR_expression> ::=
;		<region_XOR_expression>									|
;		<region_OR_expression "|" <region_XOR_expression>
;
;	<region_XOR_expression> ::=
;		<region_AND_expression>									|
;		<region_XOR_expression> "^" <region_AND_expression>
;
;	<region_AND_expression> ::=
;		<region_DIFF_expression>								|
;		<region_AND_expression> "&" <region_DIFF_expression>
;
;	<region_DIFF_expression> ::=
;		<region_primary_expression>								|
;		<region_DIFF_expression> "-" <region_primary_expression>
;
;	<region_primary_expression> ::=
;		"(" <region_expression> ")"									|
;		'RECT' "(" <left> "," <top> "," <right> "," <bottom> ")"	|
;		'RRECT' "(" <left> "," <top> "," <right> "," <bottom> "," <width> "," <height> ")"	|
;		'CIRCLE' "(" <left> "," <top> "," <diameter> ")"			|
;		'ELLIPSE' "(" <left> "," <top> "," <right> "," <bottom> ")"	|
;		'POLY' "(" <poly_fill_mode> "," <poly_points_list> ")"				|
;		'DIALOG' | 'TITLEBAR' | 'MENUBAR' | 'CLIENT' | 'STATUSBAR'
;
;	<left> ::= <integer_constant>
;
;	<top> ::= <integer_constant>
;
;	<right> ::= <integer_constant>
;
;	<bottom> ::= <integer_constant>
;
;	<width> ::= <integer_constant>
;
;	<height> ::= <integer_constant>
;
;	<diameter> ::= <integer_constant>
;
;	<poly_fill_mode> ::= 'ALTERNATE' | 'WINDING'
;
;	<poly_points_list> ::=
;		<poly_point> {"," <poly_point>}*
;
;	<poly_point> ::=
;		<integer_constant> "," <integer_constant>
;
*/
STATE(region_expression)
    MARK;
    if (!region_OR_expression()) REJECT;
    ACCEPT;
} /*region_expression*/

STATE(region_OR_expression)
    MARK;
    if (!region_XOR_expression()) REJECT;
    while (mdp_char('|')) {
        if (!region_XOR_expression()) REJECT;
        push_op(OP_IOR);
    }
    ACCEPT;
} /*region_OR_expression*/

STATE(region_XOR_expression)
    MARK;
    if (!region_AND_expression()) REJECT;
    while (mdp_char('^')) {
        if (!region_AND_expression()) REJECT;
        push_op(OP_XOR);
    }
    ACCEPT;
} /*region_XOR_expression*/

STATE(region_AND_expression)
    MARK;
    if (!region_DIFF_expression()) REJECT;
    while (mdp_char('&')) {
        if (!region_DIFF_expression()) REJECT;
        push_op(OP_AND);
    }
    ACCEPT;
} /*region_AND_expression*/

STATE(region_DIFF_expression)
    MARK;
    if (!region_primary_expression()) REJECT;
    while (mdp_char('-')) {
        if (!region_primary_expression()) REJECT;
        push_op(OP_SUB);
    }
    ACCEPT;
} /*region_DIFF_expression*/

STATE(region_primary_expression)
    MARK;
    if (mdp_char('(')) {
        if (!region_expression()) REJECT;
        if (!mdp_char(')')) REJECT;
    }
    else if (mdp_keywordi("RECT")) {
        push_proc_addr(createRectRgn);
        if (!mdp_char('(')) REJECT;
        if (!constant_expression()) REJECT;
        push_op(OP_ARG);                //left
        if (!mdp_char(',')) REJECT;
        if (!constant_expression()) REJECT;
        push_op(OP_ARG);                //top
        if (!mdp_char(',')) REJECT;
        if (!constant_expression()) REJECT;
        push_op(OP_ARG);                //right
        if (!mdp_char(',')) REJECT;
        if (!constant_expression()) REJECT;
        push_op(OP_ARG);                //bottom
        if (!mdp_char(')')) REJECT;
        push_constant(LIT_INTEGER);
        push_constant(4);               //nargs
        push_op(OP_CALL);
    }
    else if (mdp_keywordi("RRECT")) {
        push_proc_addr(createRoundRectRgn);
        if (!mdp_char('(')) REJECT;
        if (!constant_expression()) REJECT;
        push_op(OP_ARG);                //left
        if (!mdp_char(',')) REJECT;
        if (!constant_expression()) REJECT;
        push_op(OP_ARG);                //top
        if (!mdp_char(',')) REJECT;
        if (!constant_expression()) REJECT;
        push_op(OP_ARG);                //right
        if (!mdp_char(',')) REJECT;
        if (!constant_expression()) REJECT;
        push_op(OP_ARG);                //bottom
        if (!mdp_char(',')) REJECT;
        if (!constant_expression()) REJECT;
        push_op(OP_ARG);                //width
        if (!mdp_char(',')) REJECT;
        if (!constant_expression()) REJECT;
        push_op(OP_ARG);                //height
        if (!mdp_char(')')) REJECT;
        push_constant(LIT_INTEGER);
        push_constant(6);               //nargs
        push_op(OP_CALL);
    }
    else if (mdp_keywordi("CIRCLE")) {
        push_proc_addr(createCircularRgn);
        if (!mdp_char('(')) REJECT;
        if (!constant_expression()) REJECT;
        push_op(OP_ARG);                //left
        if (!mdp_char(',')) REJECT;
        if (!constant_expression()) REJECT;
        push_op(OP_ARG);                //top
        if (!mdp_char(',')) REJECT;
        if (!constant_expression()) REJECT;
        push_op(OP_ARG);                //diameter
        if (!mdp_char(')')) REJECT;
        push_constant(LIT_INTEGER);
        push_constant(3);               //nargs
        push_op(OP_CALL);
    }
    else if (mdp_keywordi("ELLIPSE")) {
        push_proc_addr(createEllipticRgn);
        if (!mdp_char('(')) REJECT;
        if (!constant_expression()) REJECT;
        push_op(OP_ARG);                //left
        if (!mdp_char(',')) REJECT;
        if (!constant_expression()) REJECT;
        push_op(OP_ARG);                //top
        if (!mdp_char(',')) REJECT;
        if (!constant_expression()) REJECT;
        push_op(OP_ARG);                //right
        if (!mdp_char(',')) REJECT;
        if (!constant_expression()) REJECT;
        push_op(OP_ARG);                //bottom
        if (!mdp_char(')')) REJECT;
        push_constant(LIT_INTEGER);
        push_constant(4);               //nargs
        push_op(OP_CALL);
    }
    else if (mdp_keywordi("POLY")) {
        int nPoints;
        UINT4 nCount_save;
        push_proc_addr(createPolyRgn);
        if (!mdp_char('(')) REJECT;
        if (!poly_fill_mode()) REJECT;
        push_op(OP_ARG);                //<fillMode>
        if (!mdp_char(',')) REJECT;
        push_constant(LIT_INTEGER);
        nCount_save = Ncount;
        push_constant(0);               //placeholder for <nPoints>
        push_op(OP_ARG);
        if (!(nPoints = poly_points_list())) REJECT;
        if (!mdp_char(')')) REJECT;
        if (nPoints < 3) REJECT;
        SDL[nCount_save] = nPoints;     //patch actual value of <nPoints>
        push_constant(LIT_INTEGER);
        push_param(nPoints*2+2);        //nargs
        push_op(OP_CALL);
    }
    else if (mdp_keywordi("DIALOG")) {
        push_proc_addr(createDialogRgn);
        push_constant(LIT_INTEGER);
        push_constant(0);               //nargs
        push_op(OP_CALL);
    }
    else if (mdp_keywordi("TITLEBAR")) {
        push_proc_addr(createTitlebarRgn);
        push_constant(LIT_INTEGER);
        push_constant(0);               //nargs
        push_op(OP_CALL);
    }
    else if (mdp_keywordi("MENUBAR")) {
        push_proc_addr(createMenubarRgn);
        push_constant(LIT_INTEGER);
        push_constant(0);               //nargs
        push_op(OP_CALL);
    }
    else if (mdp_keywordi("CLIENT")) {
        push_proc_addr(createClientRgn);
        push_constant(LIT_INTEGER);
        push_constant(0);               //nargs
        push_op(OP_CALL);
    }
    else if (mdp_keywordi("STATUSBAR")) {
        push_proc_addr(createStatusbarRgn);
        push_constant(LIT_INTEGER);
        push_constant(0);               //nargs
        push_op(OP_CALL);
    }
    else REJECT;
    ACCEPT;
} /*region_primary_expression*/

STATE(poly_fill_mode)
    MARK;
    if (mdp_keywordi("ALTERNATE")) {
        push_constant(LIT_INTEGER);
        push_constant(ALTERNATE);
    }
    else if (mdp_keywordi("WINDING")) {
        push_constant(LIT_INTEGER);
        push_constant(WINDING);
    }
    else REJECT;
    ACCEPT;
} /*poly_fill_mode*/

STATE(poly_points_list)
    int nPoints = 0;
    MARK;
    do {
        if (!poly_point()) REJECT;
        nPoints++;
    } while (mdp_char(','));
    ACCEPTX(nPoints);
} /*poly_points_list*/

STATE(poly_point)
    MARK;
    if (!constant_expression()) REJECT;
    push_op(OP_ARG);
    if (!mdp_char(',')) REJECT;
    if (!constant_expression()) REJECT;
    push_op(OP_ARG);
    ACCEPT;
} /*poly_point*/

#ifndef BETA

/*************************** <EI_booleanExpression> ************************
;
; Syntax:
;
; EnableInfo grammar
; ------------------
;
;	<booleanExpression>:
;		<conjunction> {"||" <conjunction>}*
;
;	<conjunction>:
;		<relation> {"&&" <relation>}*
;
;	<relation>:
;		<equality> {<relationOperator> <equality>}*
;
;	<equality>:
;		<simpleExpression> {<equalityOperator> <simpleExpression>}*
;
;	<simpleExpression>:
;		<term> {<addOperator> <term>}*
;
;	<term>:
;		<factor> {<mulOperator> <factor>}*
;
;	<factor>:
;		<integer> | <intrinsic> | <ident> | "(" <booleanExpression> ")" |
;		"+" <factor> | "-" <factor> | "!" <factor>
;
;	<integer>:
;		{digit}+
;
;	<intrinsic>:
;		<limitFunction> | <dimFunction> | <inFunction>
;
;	<limitFunction>:
;		{"min" | "max"} "(" <simpleExpression> "," <simpleExpression>
;		               {"," <simpleExpression>}* ")"
;
;	<dimFunction>:
;		"dim" "(" <simpleExpression> "," <simpleExpression> ")"
;
;	<inFunction>:
;		"in" "(" <simpleExpression> {"," <simpleExpression>}* ")"
;
;	<ident>:
;		{alpha | "_"} {alpha | digit | "_"}*
;
;	<mulOperator>:
;		"*" | "/"
;
;	<addOperator>:
;		"+" | "-"
;
;	<equalityOperator>:
;		"==" | "!="
;
;	<relationOperator>:
;		"<" | "<=" | ">=" | ">"
;
; EnableInfo constants:
;
; Constant			    Description
; ----------------	    ------------------------------------
; true 				    Boolean true
; false				    Boolean false
; BitmapMode			Bitmap mode
; GrayScaleMode		    Grayscale mode
; IndexedColorMode	    Indexed color mode
; RGBMode				RGB color mode
; CMYKMode			    CMYK color mode
; HSLMode				HSL color mode
; HSBMode				HSB color mode
; MultichannelMode	    Multichannel mode
; DuotoneMode			Duotone mode
; LabMode				Lab color mode
; Gray16Mode			Grayscale mode, 16 bits per channel
; RGB48Mode			    RGB color mode, 16 bits per channel
; Lab48Mode				Lab color mode, 16 bits per channel
; CMYK64Mode			CMYK color mode, 16 bits per channel
; DeepMultichannelMode	Multichannel mode, 16 bits per channel
; Duotone16Mode			Duotone mode, 16 bits per channel
;
; EnableInfo variables:
;
; Variable			        Description
; ----------------------	-------------------------------------------------
; PSHOP_ImageMode		    Image mode
; PSHOP_ImageDepth		    Image depth
; PSHOP_HasLayerMask	    Boolean for presence of layer mask
; PSHOP_HasSelectionMask	Boolean for presence of selection mask
; PSHOP_HasTransparencyMask Boolean for presence of transparency mask
; PSHOP_NumTargetChannels	Number of target channels
; PSHOP_NumTrueChannels	    Number of image channels
; PSHOP_IsTargetComposite	Boolean for whether flattened
; PSHOP_ImageWidth		    Width of the image
; PSHOP_ImageHeight		    Height of the image.
;
*/
STATE(EI_booleanExpression)
    MARK;
    do {
        if (!EI_conjunction()) REJECT;
    } while (logical_or_operator());
    ACCEPT;
} /*EI_booleanExpression*/

STATE(EI_conjunction)
    MARK;
    do {
        if (!EI_relation()) REJECT;
    } while (logical_and_operator());
    ACCEPT;
} /*EI_conjunction*/

STATE(EI_relation)
    MARK;
    do {
        if (!EI_equality()) REJECT;
    } while (le_operator()||ge_operator()||lt_operator()||gt_operator());
    ACCEPT;
} /*EI_relation*/

STATE(EI_equality)
    MARK;
    do {
        if (!EI_simpleExpression()) REJECT;
    } while (eq_operator()||ne_operator());
    ACCEPT;
} /*EI_equality*/

STATE(EI_simpleExpression)
    MARK;
    do {
        if (!EI_term()) REJECT;
    } while (mdp_char('+')||mdp_char('-'));
    ACCEPT;
} /*EI_simpleExpression*/

STATE(EI_term)
    MARK;
    do {
        if (!EI_factor()) REJECT;
    } while (mdp_char('*')||mdp_char('/'));
    ACCEPT;
} /*EI_term*/

STATE(EI_factor)
    MARK;
    while (mdp_char('+')||mdp_char('-')||mdp_char('!'))
        ;
    if (mdp_char('(')) {
        if (!EI_booleanExpression()) REJECT;
        if (!mdp_char(')')) REJECT;
        ACCEPT;
    }
    if (mdp_decimal()) ACCEPT;
    if (EI_intrinsic()) ACCEPT;
    if (EI_constant()) ACCEPT;
    if (EI_variable()) ACCEPT;
    REJECT;
} /*EI_factor*/

STATE(EI_intrinsic)
    MARK;
    if (mdp_keyword("min")||mdp_keyword("max")) {
        if (!mdp_char('(')) REJECT;
        if (!EI_simpleExpression()) REJECT;
        if (!mdp_char(',')) REJECT;
        do {
            if (!EI_simpleExpression()) REJECT;
        } while (mdp_char(','));
        if (!mdp_char(')')) REJECT;
    }
    else if (mdp_keyword("dim")) {
        if (!mdp_char('(')) REJECT;
        if (!EI_simpleExpression()) REJECT;
        if (!mdp_char(',')) REJECT;
        if (!EI_simpleExpression()) REJECT;
        if (!mdp_char(')')) REJECT;
    }
    else if (mdp_keyword("in")) {
        if (!mdp_char('(')) REJECT;
        do {
            if (!EI_simpleExpression()) REJECT;
        } while (mdp_char(','));
        if (!mdp_char(')')) REJECT;
    }
    else REJECT;
    ACCEPT;
} /*EI_intrinsic*/

STATE(EI_constant)
    MARK;
    if (mdp_keyword("true")) ACCEPT;
    if (mdp_keyword("false")) ACCEPT;
    if (mdp_keyword("BitmapMode")) ACCEPT;
    if (mdp_keyword("GrayScaleMode")) ACCEPT;
    if (mdp_keyword("IndexedColorMode")) ACCEPT;
    if (mdp_keyword("RGBMode")) ACCEPT;
    if (mdp_keyword("CMYKMode")) ACCEPT;
    if (mdp_keyword("HSLMode")) ACCEPT;
    if (mdp_keyword("HSBMode")) ACCEPT;
    if (mdp_keyword("MultichannelMode")) ACCEPT;
    if (mdp_keyword("DuotoneMode")) ACCEPT;
    if (mdp_keyword("LabMode")) ACCEPT;
    if (mdp_keyword("Gray16Mode")) ACCEPT;
    if (mdp_keyword("RGB48Mode")) ACCEPT;
	//Added by Harald Heim, December 6, 2002
	if (mdp_keyword("Lab48Mode")) ACCEPT;
    if (mdp_keyword("CMYK64Mode")) ACCEPT;
    if (mdp_keyword("DeepMultichannelMode")) ACCEPT;
    if (mdp_keyword("Duotone16Mode")) ACCEPT;

    REJECT;
} /*EI_constant*/

STATE(EI_variable)
    MARK;
    if (mdp_keyword("PSHOP_ImageMode")) ACCEPT;
    if (mdp_keyword("PSHOP_ImageDepth")) ACCEPT;
    if (mdp_keyword("PSHOP_HasLayerMask")) ACCEPT;
    if (mdp_keyword("PSHOP_HasSelectionMask")) ACCEPT;
    if (mdp_keyword("PSHOP_HasTransparencyMask")) ACCEPT;
    if (mdp_keyword("PSHOP_NumTargetChannels")) ACCEPT;
    if (mdp_keyword("PSHOP_NumTrueChannels")) ACCEPT;
    if (mdp_keyword("PSHOP_IsTargetComposite")) ACCEPT;
    if (mdp_keyword("PSHOP_ImageWidth")) ACCEPT;
    if (mdp_keyword("PSHOP_ImageHeight")) ACCEPT;
    REJECT;
} /*EI_variable*/


/*************************** <PiMI_resource> ************************
;
; Syntax:
;
;	<PiMI_resource>:
;		<PiMI_resource_header> <PiMI_resource_body>
;
;	<PiMI_resource_header>:
;		"resource" "'PiMI'" "("
;		<resourceID> ["," <resource_name>] ["," <resource_attribute_list>]
;		")"
;
;	<resource_name>:
;		<string>
;
;	<resource_attribute_list>:
;		<resource_attribute> {"|" <resource_attribute>}*
;
;	<resource attribute>:
;		"purgeable"			//Anything else?
;
;	<PiMI_resource_body>:
;		"{"
;		<PiMI_version_property> ","
;		<PiMI_subversion_property> ","
;		<PiMI_priority_property> ","
;		<PiMI_supported_modes_property> ","
;		<PiMI_required_host_property> ","
;		"{" [<PiMI_format_extension_field>] "}"
;		["," "{" <PiMI_filter_case_properties> "}"]
;		"}" ";"
;
;	<PiMI_version_property>:
;		<int16>	|			//The version number of the interface supported
;		"latestAcquireVersion" |
;		"latestExportVersion" |
;		"latestExtensionVersion" |
;		"latestFilterVersion" |
;		"latestFormatVersion" |
;		"latestParserVersion" |
;		"latestSelectionVersion" |
;		"latestPickerVersion"
;
;	<PiMI_subversion_property>:
;		<int16>	|			//The sub-version number of the interface
;		"latestAcquireSubVersion" |
;		"latestExportSubVersion" |
;		"latestExtensionSubVersion" |
;		"latestFilterSubVersion" |
;		"latestFormatSubVersion" |
;		"latestParserSubVersion" |
;		"latestSelectionSubVersion" |
;		"latestPickerSubVersion"
;
;	<PiMI_priority_property>:
;		<int16>				//The plug-in's priority
;
;	<PiMI_supported_modes_property>:			//Bitmask of supported image modes
;		<PiMI_supported_mode> {"+" <PiMI_supported_mode>}*
;
;	<PiMI_supported_mode>:
;		"supportsBitmap" |
;		"supportsGrayScale" |
;		"supportsIndexedColor" |
;		"supportsRGBColor" |
;		"supportsCMYKColor" |
;		"supportsHSLColor" |
;		"supportsHSBColor" |
;		"supportsMultichannel" |
;		"supportsDuotone" |
;		"supportsLABColor" |
;		"supportsGray16" |			//new
;		"supportsRGB48"				//new
;
;	<PiMI_required_host_property>:
;		"'     '"		|			//No required host
;		<OSType>
;
;	<PiMI_format_extension_field>:
;		("cannotRead" | "canRead") ","					//Can we read using this format?
;		("cannotReadAll" | "canReadAll") ","			//Can this plug-in read from all files?
;		("cannotWrite" | "canWrite") ","				//Can we write using this format?
;		("cannotWriteIfRead" | "canWriteIfRead") ","	//Can we write if we read using this format?
;		("doesntSaveResources" | "savesResources") ","	//Does this file format save resource data?
;		[("doesntSupportsFilterCalls" | "supportsFilterCalls") ","]	//Supports filter calls?
;		"{"							//Maximum # of channels for each image mode:
;			<max channels> ","		// for Bitmap mode
;			<max channels> ","		// for GrayScale mode
;			<max channels> ","		// for IndexColor mode
;			<max channels> ","		// for RGBColor mode
;			<max channels> ","		// for CMYKColor mode
;			<max channels> ","		// for HSLColor mode
;			<max channels> ","		// for HSBColor mode
;			<max channels> ","		// for Multichannel mode
;			<max channels> ","		// for Duotone mode
;			<max channels> ","		// for LABColor mode
;			<max channels> ","		// for Gray16 mode
;			<max channels> ","		// for RGB48 mode
;			<max channels> ","		// reserved
;			<max channels> ","		// reserved
;			<max channels> ","		// reserved
;			<max channels> 		    // reserved
;		"}" ","
;		<max rows> ","
;		<max columns> ","
;		<new file type> ","
;		<new creator type> ","
;		"{" [<PiMI_type_creator_pair_list>] "}" ","
;		"{" [<PiMI_supported_extensions_list>] "}"
;
;	<max channels>:
;		<int16>			//Maximum # of channels for this image mode
;
;	<max rows>:
;		<int16>			//Maximum rows allowed in document
;
;	<max columns>:
;		<int16>			//Maximum columns allowed in document
;
;	<new file type>:
;		<OSType>		//The file type if we create a file
;
;	<new creator type>:
;		<OSType>		//The creator type if we create a file
;
;	<PiMI_type_creator_pair_list>:	//List of type/creator pairs supported
;		<type_creator_pair> {"," <type_creator_pair>}*
;
;	<type_creator_pair>:
;		<OSType> ","	//File type
;		<OSType>		//File creator
;
;	<PiMI_supported_extensions_list>:	//List of file extensions supported
;		<OSType> {"," <OSType>}*
;
;	<PiMI_filter_case_properties>:
;		<filter_case_flag> {"+" <filter_case_flag>}*	//just guessing at this
;
;	<filter_case_flag>:
;		"dontCopySourceToDestination" |
;		"worksWithBlankData" |							//just guessing
;		"filterLayerMasks" |							//just guessing
;		"writeOutsideSelection"							//just guessing
;
*/
STATE(PiMI_resource)
    MARK;
    if (!PiMI_resource_header()) REJECT;
    if (!PiMI_resource_body()) REJECT;
    ACCEPT;
} /*PiMI_resource*/

STATE(PiMI_resource_header)
    MARK;
    if (!mdp_keywordi("resource")) REJECT;
    if (!mdp_keywordi("'PiMI'")) REJECT;
    if (!mdp_char('(')) REJECT;
    if (!constant_integer_expression()) REJECT; //<resourceID>
    optional_resource_name();
    optional_resource_attribute_list();
    if (!mdp_char(')')) REJECT;
    ACCEPT;
} /*PiMI_resource_header*/

STATE(optional_resource_name)
    MARK;
    if (!mdp_char(',')) REJECT;
    if (!string_literal()) REJECT;  //<resource_name>
    ACCEPT;
} /*optional_resource_name*/

STATE(optional_resource_attribute_list)
    MARK;
    if (!mdp_char(',')) REJECT;
    do {
        if (!resource_attribute()) REJECT;
    } while (mdp_char('|'));
    ACCEPT;
} /*resource_attribute_list*/

STATE(resource_attribute)
    MARK;
    if (mdp_keywordi("purgeable")) ACCEPT;
    REJECT;
} /*resource_attribute*/

STATE(PiMI_resource_body)
    MARK;
    if (!mdp_char('{')) REJECT;
    if (!PiMI_version_property()) REJECT;
    if (!mdp_char(',')) REJECT;
    if (!PiMI_subversion_property()) REJECT;
    if (!mdp_char(',')) REJECT;
    if (!PiMI_priority_property()) REJECT;
    if (!mdp_char(',')) REJECT;
    if (!PiMI_supported_modes_property()) REJECT;
    if (!mdp_char(',')) REJECT;
    if (!PiMI_required_host_property()) REJECT;
    if (!mdp_char(',')) REJECT;
    if (!mdp_char('{')) REJECT;
    PiMI_format_extension_field();      //optional
    if (!mdp_char('}')) REJECT;
    if (mdp_char(',')) {
        //new...therefore optional
        if (!mdp_char('{')) REJECT;
        if (!PiMI_filter_case_properties()) REJECT;
        if (!mdp_char('}')) REJECT;
    }
    if (!mdp_char('}')) REJECT;
    if (!mdp_char(';')) REJECT;
    ACCEPT;
} /*PiMI_resource_body*/

STATE(PiMI_version_property)
    MARK;
    if (mdp_keywordi("latestAcquireVersion")) ACCEPT;
    if (mdp_keywordi("latestExportVersion")) ACCEPT;
    if (mdp_keywordi("latestExtensionVersion")) ACCEPT;
    if (mdp_keywordi("latestFilterVersion")) ACCEPT;
    if (mdp_keywordi("latestFormatVersion")) ACCEPT;
    if (mdp_keywordi("latestParserVersion")) ACCEPT;
    if (mdp_keywordi("latestSelectionVersion")) ACCEPT;
    if (mdp_keywordi("latestPickerVersion")) ACCEPT;
    if (constant_integer_expression()) ACCEPT;
    REJECT;
} /*PiMI_version_property*/

STATE(PiMI_subversion_property)
    MARK;
    if (mdp_keywordi("latestAcquireSubVersion")) ACCEPT;
    if (mdp_keywordi("latestExportSubVersion")) ACCEPT;
    if (mdp_keywordi("latestExtensionSubVersion")) ACCEPT;
    if (mdp_keywordi("latestFilterSubVersion")) ACCEPT;
    if (mdp_keywordi("latestFormatSubVersion")) ACCEPT;
    if (mdp_keywordi("latestParserSubVersion")) ACCEPT;
    if (mdp_keywordi("latestSelectionSubVersion")) ACCEPT;
    if (mdp_keywordi("latestPickerSubVersion")) ACCEPT;
    if (constant_integer_expression()) ACCEPT;
    REJECT;
} /*PiMI_subversion_property*/

STATE(PiMI_priority_property)
    MARK;
    if (!constant_integer_expression()) REJECT;
    ACCEPT;
} /*PiMI_priority_property*/

STATE(PiMI_supported_modes_property)
    MARK;
    gParams->SupportedModes = 0;
    do {
        if (!PiMI_supported_mode()) REJECT;
    } while (mdp_char('+'));
    ACCEPT;
} /*PiMI_supported_modes_property*/

STATE(PiMI_supported_mode)
    MARK;
    if (mdp_keywordi("supportsBitmap")) {
        gParams->SupportedModes |= FLAG(plugInModeBitmap);
    }
    else if (mdp_keywordi("supportsGrayScale")) {
        gParams->SupportedModes |= FLAG(plugInModeGrayScale);
    }
    else if (mdp_keywordi("supportsIndexedColor")) {
        gParams->SupportedModes |= FLAG(plugInModeIndexedColor);
    }
    else if (mdp_keywordi("supportsRGBColor")) {
        gParams->SupportedModes |= FLAG(plugInModeRGBColor);
    }
    else if (mdp_keywordi("supportsCMYKColor")) {
        gParams->SupportedModes |= FLAG(plugInModeCMYKColor);
    }
    else if (mdp_keywordi("supportsHSLColor")) {
        gParams->SupportedModes |= FLAG(plugInModeHSLColor);
    }
    else if (mdp_keywordi("supportsHSBColor")) {
        gParams->SupportedModes |= FLAG(plugInModeHSBColor);
    }
    else if (mdp_keywordi("supportsMultichannel")) {
        gParams->SupportedModes |= FLAG(plugInModeMultichannel);
    }
    else if (mdp_keywordi("supportsDuotone")) {
        gParams->SupportedModes |= FLAG(plugInModeDuotone);
    }
    else if (mdp_keywordi("supportsLABColor")) {
        gParams->SupportedModes |= FLAG(plugInModeLabColor);
    }
    else if (mdp_keywordi("supportsGray16")) {
        gParams->SupportedModes |= FLAG(plugInModeGray16);
    }
    else if (mdp_keywordi("supportsRGB48")) {
        gParams->SupportedModes |= FLAG(plugInModeRGB48);

	}
	//Added by Harald Heim, December 6, 2002
	else if (mdp_keywordi("supportsLab48")) {
        gParams->SupportedModes |= FLAG(plugInModeLab48);
    }
    else if (mdp_keywordi("supportsCMYK64")) {
        gParams->SupportedModes |= FLAG(plugInModeCMYK64);
    }
    else if (mdp_keywordi("supportsDeepMultichannel")) {
        gParams->SupportedModes |= FLAG(plugInModeDeepMultichannel);
    }
    else if (mdp_keywordi("supportsDuotone16")) {
        gParams->SupportedModes |= FLAG(plugInModeDuotone16);


    }
    else REJECT;
    ACCEPT;
} /*PiMI_supported_mode*/

STATE(PiMI_required_host_property)
    MARK;
    if (!constant_integer_expression()) REJECT;
    ACCEPT;
} /*PiMI_required_host_property*/

STATE(PiMI_format_extension_field)
    int i;
    MARK;
    if (mdp_keywordi("cannotRead")) {
    }
    else if (mdp_keywordi("canRead")) {
    }
    else REJECT;
    if (!mdp_char(',')) REJECT;
    if (mdp_keywordi("cannotReadAll")) {
    }
    else if (mdp_keywordi("canReadAll")) {
    }
    else REJECT;
    if (!mdp_char(',')) REJECT;
    if (mdp_keywordi("cannotWrite")) {
    }
    else if (mdp_keywordi("canWrite")) {
    }
    else REJECT;
    if (!mdp_char(',')) REJECT;
    if (mdp_keywordi("cannotWriteIfRead")) {
    }
    else if (mdp_keywordi("canWriteIfRead")) {
    }
    else REJECT;
    if (!mdp_char(',')) REJECT;
    if (mdp_keywordi("doesntSaveResources")) {
    }
    else if (mdp_keywordi("savesResources")) {
    }
    else REJECT;
    if (!mdp_char(',')) REJECT;
    //following field is new and therefore optional
    if (mdp_keywordi("doesntSupportsFilterCalls") ||    //[sic!]
        mdp_keywordi("doesntSupportFilterCalls")) {     //to spell it correctly
        if (!mdp_char(',')) REJECT;
    }
    else if (mdp_keywordi("supportFilterCalls")) {
        if (!mdp_char(',')) REJECT;
    }
    if (!mdp_char('{')) REJECT;
    for (i=0; i < 16; i++) {
        if (!constant_integer_expression()) REJECT;     //<max_channels>
        if (i < 15 && !mdp_char(',')) REJECT;
    }
    if (!mdp_char('}')) REJECT;
    if (!mdp_char(',')) REJECT;
    if (!constant_integer_expression()) REJECT;     //<max_rows>
    if (!mdp_char(',')) REJECT;
    if (!constant_integer_expression()) REJECT;     //<max_columns>
    if (!mdp_char(',')) REJECT;
    if (!constant_integer_expression()) REJECT;     //<new_file_type>
    if (!mdp_char(',')) REJECT;
    if (!constant_integer_expression()) REJECT;     //<new_creator_type>
    if (!mdp_char(',')) REJECT;
    if (!mdp_char('{')) REJECT;
    PiMI_type_creator_pair_list();      //optional
    if (!mdp_char('}')) REJECT;
    if (!mdp_char(',')) REJECT;
    if (!mdp_char('{')) REJECT;
    PiMI_supported_extensions_list();   //optional
    if (!mdp_char('}')) REJECT;
    ACCEPT;
} /*PiMI_format_extension_field*/

STATE(PiMI_type_creator_pair_list)
    MARK;
    do {
        if (!constant_integer_expression()) REJECT;     //File type
        if (!mdp_char(',')) REJECT;
        if (!constant_integer_expression()) REJECT;     //File creator
    } while (mdp_char(','));
    ACCEPT;
} /*PiMI_type_creator_pair_list*/

STATE(PiMI_supported_extensions_list)
    MARK;
    do {
        if (!constant_integer_expression()) REJECT;     //File extension
    } while (mdp_char(','));
    ACCEPT;
} /*PiMI_supported_extensions_list*/

STATE(PiMI_filter_case_properties)
    MARK;
    do {
        if (mdp_keywordi("dontCopySourceToDestination")) {
        }
        else if (mdp_keywordi("worksWithBlankData")) {
        }
        else if (mdp_keywordi("filterLayerMasks")) {
        }
        else if (mdp_keywordi("writeOutsideSelection") ||
                 mdp_keywordi("writesOutsideSelection")) {  //have it your way...
        }
        else REJECT;
    } while (mdp_char('+') || mdp_char('|'));
    ACCEPT;
} /*PiMI_filter_case_properties*/


/*************************** <PiPL_resource> ************************
;
; Syntax:
;
;	<PiPL_resource>:
;		<PiPL_resource_header> <PiPL_resource_body>
;
;	<PiPL_resource_header>:
;		"resource" "'PiPL'" "("
;		<resourceID> ["," <resource_name>] ["," <resource_attribute_list>]
;		")"
;
;	<resource_name>:
;		<string>
;
;	<resource_attribute_list>:
;		<attribute> {"|" <attribute>}*
;
;	<attribute>:
;		"purgeable"				//Anything else?
;
;	<PiPL_resource_body>:
;		"{" "{"
;		[<PiPL_property_list>]
;		"}" "}" ";"
;
;	<PiPL_property_list>:
;		<PiPL_propery> {"," <PiPL_property>}*
;
;	<PiPL_property>:
;		<PiPL_kind_property> |
;		<PiPL_version_property> |
;		<PiPL_priority_property> |
;		<PiPL_required_host_property> |
;		<PiPL_name_property> |
;		<PiPL_category_property> |
;		<PiPL_68k_code_descriptor_property> |
;		<PiPL_68kFpu_code_descriptor_property> |			//new
;		<PiPL_powerpc_code_descriptor_property> |
;		<PiPL_win32_x86_code_property> |
;		<PiPL_supported_modes_property> |
;		<PiPL_enable_info_property> |					//new
;		<PiPL_filter_case_info_property> |
;		<PiPL_format_file_type_property> |
;		<PiPL_read_types_property> |
;		<PiPL_write_types_property> |
;		<PiPL_filtered_types_property> |
;		<PiPL_read_extensions_property> |
;		<PiPL_write_extensions_property> |
;		<PiPL_filtered_extensions_property> |
;		<PiPL_format_flags_property> |
;		<PiPL_format_maximum_size_property> |
;		<PiPL_format_maximum_channels_property> |
;		<PiPL_parsable_types_property> |
;		<PiPL_parsable_extensions_property> |
;		<PiPL_filtered_parsable_types_property> |
;		<PiPL_filtered_parsable_extensions_property> |
;		<PiPL_parsable_clipboard_types_property>
;
;	<PiPL_kind_property>:
;		"Kind" "{" <PiPL_kind_ID> "}"
;
;	<PiPL_kind_ID> := 
;		<OSType> |
;		"Filter" |
;		"Parser" |
;		"ImageFormat" |
;		"Extension" |
;		"Acquire" |
;		"Export"
;
;	<PiPL_version_property>:
;		"Version" "{" <PiPL_version_clause> "}"
;
;	<PiPL_version_clause>:
;		<int32> |
;		"(" <PiPL_wired_version_ID_high> "<<" "16" ")" "|"
;		"(" <PiPL_wired_version_ID_low> ")" |
;		<PiPL_wired_version_ID>
;
;	<PiPL_wired_version_ID>:
;		"FilterVersion" |
;		"ParserVersion" |
;		"ImageFormatVersion" |
;		"ExtensionVersion" |
;		"AcquireVersion" |
;		"ExportVersion" |
;		"SelectionVersion" |				//new
;		"PickerVersion"						//new
;
;	<PiPL_wired_version_ID_high>:
;		"latestFilterVersion" |
;		"latestParserVersion" |
;		"latestImageFormatVersion" |
;		"latestExtensionVersion" |
;		"latestAcquireVersion" |
;		"latestExportVersion" |
;		"latestSelectionVersion" |			//new
;		"latestPickerVersion"				//new
;
;	<PiPL_wired_version_ID_low>:
;		"latestFilterSubVersion" |
;		"latestParserSubVersion" |
;		"latestImageFormatSubVersion" |
;		"latestExtensionSubVersion" |
;		"latestAcquireSubVersion" |
;		"latestExportSubVersion" |
;		"latestSelectionSubVersion" |		//new
;		"latestPickerSubVersion"			//new
;
;	<PiPL_priority_property>:
;		"Priority" "{" <int16> "}"
;
;	<PiPL_required_host_property>:
;		"Host" "{" <OSType> "}"
;
;	<PiPL_name_property>:
;		"Name" "{" <string> "}"
;
;	<PiPL_category_property>:
;		"Category" "{" <string> "}"
;
;	<PiPL_68k_code_descriptor_property>:
;		"Code68k" "{" <OSType> "," <int16> "}"
;
;	<PiPL_68kFpu_code_descriptor_property>:
;		"Code68kFPU" "{" <OSType> "," <int16> "}"		//new
;
;	<PiPL_powerpc_code_descriptor_property>:
;		"CodePowerPC" "{" <int32> "," <int32> ["," <PiPL_entry_point_name>] "}"
;
;	<PiPL_win32_x86_code_property>:
;		"CodeWin32X86" "{" <PiPL_entry_point_name> "}
;
;	<PiPL_entry_point_name>:
;		<string>
;
;	<PiPL_bitmap_support>:
;		"noBitmap" | "doesSupportBitmap"
;
;	<PiPL_grayscale_support>:
;		"noGrayScale" | "doesSupportGrayScale"
;
;	<PiPL_indexed_support>:
;		"noIndexedColor" | "doesSupportIndexedColor"
;
;	<PiPL_RGB_support>:
;		"noRGBColor" | "doesSupportRGBColor"
;
;	<PiPL_CMYK_support>:
;		"noCMYKColor" | "doesSupportCMYKColor"
;
;	<PiPL_HSL_support>:
;		"noHSLColor" | "doesSupportHSLColor"
;
;	<PiPL_HSB_support>:
;		"noHSBColor" | "doesSupportHSBColor"
;
;	<PiPL_multichannel_support>:
;		"noMultichannel" | "doesSupportMultichannel"
;
;	<PiPL_duotone_support>:
;		"noDuotone" | "doesSupportDuotone"
;
;	<PiPL_LAB_support>:
;		"noLABColor" | "doesSupportLABColor"
;
;	<PiPL_Gray16_support>:
;		"noGray16" | "doesSupportGray16"		//new
;
;	<PiPL_RGB48_support>:
;		"noRGB48" | "doesSupportRGB48"			//new
;
;	<PiPL_supported_modes_property>:
;		"SupportedModes"
;		"{"
;		<PiPL_bitmap_support> ","
;		<PiPL_grayscale_support> ","
;		<PiPL_indexed_support> ","
;		<PiPL_RGB_support> ","
;		<PiPL_CMYK_support> ","
;		<PiPL_HSL_support> ","
;		<PiPL_HSB_support> ","
;		<PiPL_multichannel_support> ","
;		<PiPL_duotone_support> ","
;		<PiPL_LAB_support> [","
;		<PiPL_Gray16_support> [","				//new
;		<PiPL_RGB48_support>]]					//new
;		"}"
;
;	<PiPL_enable_info_property>:
;		"EnableInfo"
;		"{"
;		<EI_booleanExpression>			//see Appendix K
;		"}"
;
;	<PiPL_filter_case_info_property>:
;		"FilterCaseInfo"
;		"{"
;		"{"
;		<PiPL_filter_info_case> ","	// filterCaseFlatImageNoSelection
;		<PiPL_filter_info_case> ","	// filterCaseFlatImageWithSelection
;		<PiPL_filter_info_case> ","	// filterCaseFloatingSelection
;		<PiPL_filter_info_case> ","	// filterCaseEditableTransparencyNoSelection
;		<PiPL_filter_info_case> ","	// filterCaseEditableTransparencyWithSelection
;		<PiPL_filter_info_case> ","	// filterCaseProtectedTransparencyNoSelection
;		<PiPL_filter_info_case> 	// filterCaseProtectedTransparencyWithSelection
;		"}"
;		"}"
;
;	<PiPL_filter_info_case>:
;		<PiPL_input_matting> "," <PiPL_output_matting> ","
;		<PiPL_layer_mask_flag> "," <PiPL_blank_data_flag> "," <PiPL_copy_source_flag>
;		["," <PiPL_writes_outside_selection_flag>]				// new
;
;	<PiPL_input_matting>:
;		"inCantFilter" |
;		"inStraightData" |
;		"inBlackMat" |
;		"inGrayMat" |
;		"inWhiteMat" |
;		"inDefringe" |
;		"inBlackZap" |
;		"inGrayZap" |
;		"inWhiteZap" |
;		"inBackgroundZap" |
;		"inForegroundZap"
;
;	<PiPL_output_matting>:
;		"outCantFilter" |
;		"outStraightData" |
;		"outBlackMat" |
;		"outGrayMat" |
;		"outWhiteMat" |
;		"outFillMask"
;
;	<PiPL_layer_mask_flag>:
;		"doesNotFilterLayerMasks" | "filtersLayerMasks"
;
;	<PiPL_blank_data_flag>:
;		"doesNotWorkWithBlankData" | "worksWithBlankData"
;
;	<PiPL_copy_source_flag>:
;		"copySourceToDestination" | "doNotCopySourceToDestination"
;
;	<PiPL_writes_outside_selection_flag>:								//new
;		"doNotWriteOutsideSelection" | "writeOutsideSelection"
;
;	<PiPL_type_creator_pair>:
;		<OSType> "," <OSType>
;
;	<PiPL_format_file_type_property>:
;		"{"
;		<PiPL_type_creator_pair>
;		"}"
;
;	<PiPL_type_creator_pair_list>:
;		["{" <PiPL_type_creator_pair> "}" {"," "{" <PiPL_type_creator_pair> "}"}*]
;
;	<PiPL_read_types_property>:
;		"{"
;		<PiPL_type_creator_pair_list>
;		"}"
;
;	<PiPL_write_types_property>:
;		"{"
;		<PiPL_type_creator_pair_list>
;		"}"
;
;	<PiPL_filtered_types_property>:
;		"{"
;		<PiPL_type_creator_pair_list>
;		"}"
;
;	<PiPL_ostype_list>:
;		["{" <OSType> "}" {"," "{" <OSType> "}"}*]
;
;	<PiPL_read_extensions_property>:
;		"{"
;		<PiPL_ostype_list>
;		"}"
;
;	<PiPL_write_extensions_property>:
;		"{"
;		<PiPL_ostype_list>
;		"}"
;
;	<PiPL_filtered_extensions_property>:
;		"{"
;		<PiPL_ostype_list>
;		"}"
;
;	<PiPL_saves_image_resources_flag>:
;		"fmtDoesNotSaveImageResources" | "fmtSavesImageResources"
;
;	<PiPL_can_read_flag>:
;		"fmtCannotRead" | "fmtCanRead"
;
;	<PiPL_can_write_flag>:
;		"fmtCannotWrite" | "fmtCanWrite"
;
;	<PiPL_write_if_read_flag>:
;		"fmtWritesAll" | "fmtCanWriteIfRead"    //[sic??]
;
;	<PiPL_format_flags_property>:
;		"{"
;		<PiPL_saves_image_resources_flag> ","
;		<PiPL_can_read_flag> ","
;		<PiPL_can_write_flag> ","
;		<PiPL_write_if_read_flag>
;		"}"
;
;	<PiPL_format_maximum_size_property>:
;		"{"
;		<int16>, <int16>
;		"}"
;
;	<PiPL_format_maximum_channels_property>:    //TBD
;		"{"									//just guessing...
;		<int16> {"," <int16>}*				//max channels array, indexed by image mode
;		"}"
;
;	<PiPL_parsable_types_property>:
;		"{"
;		<PiPL_type_creator_pair_list>
;		"}"
;
;	<PiPL_parsable_extensions_property>:
;		"{"
;		<PiPL_ostype_list>
;		"}"
;
;	<PiPL_filtered_parsable_types_property>:
;		"{"
;		<PiPL_type_creator_pair_list>
;		"}"
;
;	<PiPL_filtered_parsable_extensions_property>:
;		"{"
;		<PiPL_ostype_list>
;		"}"
;
;	<PiPL_parsable_clipboard_types_property>:
;		"{"
;		<PiPL_ostype_list>
;		"}"
;
*/
STATE(PiPL_resource)
    MARK;
    if (!PiPL_resource_header()) REJECT;
    if (!PiPL_resource_body()) REJECT;
    ACCEPT;
} /*PiPL_resource*/

STATE(PiPL_resource_header)
    MARK;
    if (!mdp_keywordi("resource")) REJECT;
    if (!mdp_keywordi("'PiPL'")) REJECT;
    if (!mdp_char('(')) REJECT;
    if (!constant_integer_expression()) REJECT; //<resourceID>
    optional_resource_name();
    optional_resource_attribute_list();
    if (!mdp_char(')')) REJECT;
    ACCEPT;
} /*PiPL_resource_header*/

STATE(PiPL_resource_body)
    MARK;
    if (!mdp_char('{')) REJECT;
    if (!mdp_char('{')) REJECT;
    PiPL_property_list();       //optional
    if (!mdp_char('}')) REJECT;
    if (!mdp_char('}')) REJECT;
    if (!mdp_char(';')) REJECT;
    ACCEPT;
} /*PiPL_resource_body*/

STATE(PiPL_property_list)
    MARK;
    do {
        if (!PiPL_property()) REJECT;
    } while (mdp_char(','));
    ACCEPT;
} /*PiPL_property_list*/

STATE(PiPL_property)
    MARK;
    if (PiPL_kind_property()) ACCEPT;
    if (PiPL_version_property()) ACCEPT;
    if (PiPL_priority_property()) ACCEPT;
    if (PiPL_required_host_property()) ACCEPT;
    if (PiPL_name_property()) ACCEPT;
    if (PiPL_category_property()) ACCEPT;
    if (PiPL_68k_code_descriptor_property()) ACCEPT;
    if (PiPL_68kFpu_code_descriptor_property()) ACCEPT;
    if (PiPL_powerpc_code_descriptor_property()) ACCEPT;
    if (PiPL_win32_x86_code_property()) ACCEPT;
    if (PiPL_supported_modes_property()) ACCEPT;
    if (PiPL_enable_info_property()) ACCEPT;
    if (PiPL_filter_case_info_property()) ACCEPT;
    if (PiPL_format_file_type_property()) ACCEPT;
    if (PiPL_read_types_property()) ACCEPT;
    if (PiPL_write_types_property()) ACCEPT;
    if (PiPL_filtered_types_property()) ACCEPT;
    if (PiPL_read_extensions_property()) ACCEPT;
    if (PiPL_write_extensions_property()) ACCEPT;
    if (PiPL_filtered_extensions_property()) ACCEPT;
    if (PiPL_format_flags_property()) ACCEPT;
    if (PiPL_format_maximum_size_property()) ACCEPT;
    if (PiPL_format_maximum_channels_property()) ACCEPT;
    if (PiPL_parsable_types_property()) ACCEPT;
    if (PiPL_parsable_extensions_property()) ACCEPT;
    if (PiPL_filtered_parsable_types_property()) ACCEPT;
    if (PiPL_filtered_parsable_extensions_property()) ACCEPT;
    if (PiPL_parsable_clipboard_types_property()) ACCEPT;
    REJECT;
} /*PiPL_property*/

STATE(PiPL_kind_property)
    MARK;
    if (!mdp_keywordi("Kind")) REJECT;
    if (!mdp_char('{')) REJECT;
    if (!PiPL_kind_ID()) REJECT;
    if (!mdp_char('}')) REJECT;
    ACCEPT;
} /*PiPL_kind_property*/

STATE(PiPL_kind_ID)
    MARK;
    if (mdp_keywordi("Filter")) ACCEPT;
    if (mdp_keywordi("Parser")) ACCEPT;
    if (mdp_keywordi("ImageFormat")) ACCEPT;
    if (mdp_keywordi("Extension")) ACCEPT;
    if (mdp_keywordi("Acquire")) ACCEPT;
    if (mdp_keywordi("Export")) ACCEPT;
    if (constant_integer_expression()) ACCEPT;   //<OSType>
    REJECT;
} /*PiPL_kind_ID*/

STATE(PiPL_version_property)
    MARK;
    if (!mdp_keywordi("Version")) REJECT;
    if (!mdp_char('{')) REJECT;
    if (!PiPL_version_clause()) REJECT;
    if (!mdp_char('}')) REJECT;
    ACCEPT;
} /*PiPL_version_property*/

STATE(PiPL_version_clause)
    MARK;
    if (mdp_char('(')) {
        if (!PiPL_wired_version_ID_high()) REJECT;
        if (!left_shift_operator()) REJECT;
        if (!mdp_keywordi("16")) REJECT;
        if (!mdp_char(')')) REJECT;
        if (!or_operator()) REJECT;
        if (mdp_char('(')) {
            if (!PiPL_wired_version_ID_low()) REJECT;
            if (!mdp_char(')')) REJECT;
        }
        //some examples don't wrap ID_low in parens...
        else if (!PiPL_wired_version_ID_low()) REJECT;
    }
    else if (PiPL_wired_version_ID()) {
    }
    else if (constant_integer_expression()) {
    }
    else REJECT;
    ACCEPT;
} /*PiPL_version_clause*/

STATE(PiPL_wired_version_ID)
    MARK;
    if (mdp_keywordi("FilterVersion")) ACCEPT;
    if (mdp_keywordi("ParserVersion")) ACCEPT;
    if (mdp_keywordi("ImageFormatVersion")) ACCEPT;
    if (mdp_keywordi("ExtensionVersion")) ACCEPT;
    if (mdp_keywordi("AcquireVersion")) ACCEPT;
    if (mdp_keywordi("ExportVersion")) ACCEPT;
    if (mdp_keywordi("SelectionVersion")) ACCEPT;
    if (mdp_keywordi("PickerVersion")) ACCEPT;
    REJECT;
} /*PiPL_wired_version_ID*/

STATE(PiPL_wired_version_ID_high)
    MARK;
    if (mdp_keywordi("latestFilterVersion")) ACCEPT;
    if (mdp_keywordi("latestParserVersion")) ACCEPT;
    if (mdp_keywordi("latestImageFormatVersion")) ACCEPT;
    if (mdp_keywordi("latestExtensionVersion")) ACCEPT;
    if (mdp_keywordi("latestAcquireVersion")) ACCEPT;
    if (mdp_keywordi("latestExportVersion")) ACCEPT;
    if (mdp_keywordi("latestSelectionVersion")) ACCEPT;
    if (mdp_keywordi("latestPickerVersion")) ACCEPT;
    REJECT;
} /*PiPL_wired_version_ID_high*/

STATE(PiPL_wired_version_ID_low)
    MARK;
    if (mdp_keywordi("latestFilterSubVersion")) ACCEPT;
    if (mdp_keywordi("latestParserSubVersion")) ACCEPT;
    if (mdp_keywordi("latestImageFormatSubVersion")) ACCEPT;
    if (mdp_keywordi("latestExtensionSubVersion")) ACCEPT;
    if (mdp_keywordi("latestAcquireSubVersion")) ACCEPT;
    if (mdp_keywordi("latestExportSubVersion")) ACCEPT;
    if (mdp_keywordi("latestSelectionSubVersion")) ACCEPT;
    if (mdp_keywordi("latestPickerSubVersion")) ACCEPT;
    REJECT;
} /*PiPL_wired_version_ID_low*/

STATE(PiPL_priority_property)
    MARK;
    if (!mdp_keywordi("Priority")) REJECT;
    if (!mdp_char('{')) REJECT;
    if (!constant_integer_expression()) REJECT;
    if (!mdp_char('}')) REJECT;
    ACCEPT;
} /*PiPL_priority_property*/

STATE(PiPL_required_host_property)
    MARK;
    if (!mdp_keywordi("Host")) REJECT;
    if (!mdp_char('{')) REJECT;
    if (!constant_integer_expression()) REJECT;    //<OSType>
    if (!mdp_char('}')) REJECT;
    ACCEPT;
} /*PiPL_required_host_property*/

STATE(PiPL_name_property)
    MARK;
    if (!mdp_keywordi("Name")) REJECT;
    if (!mdp_char('{')) REJECT;
    if (!string_literal()) REJECT;
    if (!mdp_char('}')) REJECT;
    ACCEPT;
} /*PiPL_name_property*/

STATE(PiPL_category_property)
    MARK;
    if (!mdp_keywordi("Category")) REJECT;
    if (!mdp_char('{')) REJECT;
    if (!string_literal()) REJECT;
    if (!mdp_char('}')) REJECT;
    ACCEPT;
} /*PiPL_category_property*/

STATE(PiPL_68k_code_descriptor_property)
    MARK;
    if (!mdp_keywordi("Code68k")) REJECT;
    if (!mdp_char('{')) REJECT;
    if (!constant_integer_expression()) REJECT;     //<OSType>
    if (!mdp_char(',')) REJECT;
    if (!constant_integer_expression()) REJECT;     //<int16>
    if (!mdp_char('}')) REJECT;
    ACCEPT;
} /*PiPL_68k_code_descriptor_property*/

STATE(PiPL_68kFpu_code_descriptor_property)
    MARK;
    if (!mdp_keywordi("Code68kFPU")) REJECT;
    if (!mdp_char('{')) REJECT;
    if (!constant_integer_expression()) REJECT;     //<OSType>
    if (!mdp_char(',')) REJECT;
    if (!constant_integer_expression()) REJECT;     //<int16>
    if (!mdp_char('}')) REJECT;
    ACCEPT;
} /*PiPL_68kFpu_code_descriptor_property*/

STATE(PiPL_powerpc_code_descriptor_property)
    MARK;
    if (!mdp_keywordi("CodePowerPC")) REJECT;
    if (!mdp_char('{')) REJECT;
    if (!constant_integer_expression()) REJECT;     //<int32> Container Offset
    if (!mdp_char(',')) REJECT;
    if (!constant_integer_expression()) REJECT;     //<int32> Container Length
    if (mdp_char(',')) {
        //optional entry point name
        if (!PiPL_entry_point_name()) REJECT;
    }
    if (!mdp_char('}')) REJECT;
    ACCEPT;
} /*PiPL_powerpc_code_descriptor_property*/

STATE(PiPL_win32_x86_code_property)
    MARK;
    if (!mdp_keywordi("CodeWin32X86")) REJECT;
    if (!mdp_char('{')) REJECT;
    if (!PiPL_entry_point_name()) REJECT;
    if (!mdp_char('}')) REJECT;
    ACCEPT;
} /*PiPL_win32_x86_code_property*/

STATE(PiPL_entry_point_name)
    MARK;
    if (!string_literal()) REJECT;
    ACCEPT;
} /*PiPL_entry_point_name*/

STATE(PiPL_bitmap_support)
    MARK;
    if (mdp_keywordi("noBitmap")) ACCEPT;
    if (mdp_keywordi("doesSupportBitmap")) {
        gParams->SupportedModes |= FLAG(plugInModeBitmap);
        ACCEPT;
    }
    REJECT;
} /*PiPL_bitmap_support*/

STATE(PiPL_grayscale_support)
    MARK;
    if (mdp_keywordi("noGrayScale")) ACCEPT;
    if (mdp_keywordi("doesSupportGrayScale")) {
        gParams->SupportedModes |= FLAG(plugInModeGrayScale);
        ACCEPT;
    }
    REJECT;
} /*PiPL_grayscale_support*/

STATE(PiPL_indexed_support)
    MARK;
    if (mdp_keywordi("noIndexedColor")) ACCEPT;
    if (mdp_keywordi("doesSupportIndexedColor")) {
        gParams->SupportedModes |= FLAG(plugInModeIndexedColor);
        ACCEPT;
    }
    REJECT;
} /*PiPL_indexed_support*/

STATE(PiPL_RGB_support)
    MARK;
    if (mdp_keywordi("noRGBColor")) ACCEPT;
    if (mdp_keywordi("doesSupportRGBColor")) {
        gParams->SupportedModes |= FLAG(plugInModeRGBColor);
        ACCEPT;
    }
    REJECT;
} /*PiPL_RGB_support*/

STATE(PiPL_CMYK_support)
    MARK;
    if (mdp_keywordi("noCMYKColor")) ACCEPT;
    if (mdp_keywordi("doesSupportCMYKColor")) {
        gParams->SupportedModes |= FLAG(plugInModeCMYKColor);
        ACCEPT;
    }
    REJECT;
} /*PiPL_CMYK_support*/

STATE(PiPL_HSL_support)
    MARK;
    if (mdp_keywordi("noHSLColor")) ACCEPT;
    if (mdp_keywordi("doesSupportHSLColor")) {
        gParams->SupportedModes |= FLAG(plugInModeHSLColor);
        ACCEPT;
    }
    REJECT;
} /*PiPL_HSL_support*/

STATE(PiPL_HSB_support)
    MARK;
    if (mdp_keywordi("noHSBColor")) ACCEPT;
    if (mdp_keywordi("doesSupportHSBColor")) {
        gParams->SupportedModes |= FLAG(plugInModeHSBColor);
        ACCEPT;
    }
    REJECT;
} /*PiPL_HSB_support*/

STATE(PiPL_multichannel_support)
    MARK;
    if (mdp_keywordi("noMultichannel")) ACCEPT;
    if (mdp_keywordi("doesSupportMultichannel")) {
        gParams->SupportedModes |= FLAG(plugInModeMultichannel);
        ACCEPT;
    }
    REJECT;
} /*PiPL_multichannel_support*/

STATE(PiPL_duotone_support)
    MARK;
    if (mdp_keywordi("noDuotone")) ACCEPT;
    if (mdp_keywordi("doesSupportDuotone")) {
        gParams->SupportedModes |= FLAG(plugInModeDuotone);
        ACCEPT;
    }
    REJECT;
} /*PiPL_duotone_support*/

STATE(PiPL_LAB_support)
    MARK;
    if (mdp_keywordi("noLABColor")) ACCEPT;
    if (mdp_keywordi("doesSupportLABColor")) {
        gParams->SupportedModes |= FLAG(plugInModeLabColor);
        ACCEPT;
    }
    REJECT;
} /*PiPL_LAB_support*/

STATE(PiPL_Gray16_support)
    MARK;
    if (mdp_keywordi("noGray16")) ACCEPT;
    if (mdp_keywordi("doesSupportGray16")) {
        gParams->SupportedModes |= FLAG(plugInModeGray16);
        ACCEPT;
    }
    REJECT;
} /*PiPL_Gray16_support*/

STATE(PiPL_RGB48_support)
    MARK;
    if (mdp_keywordi("noRGB48")) ACCEPT;
    if (mdp_keywordi("doesSupportRGB48")) {
        gParams->SupportedModes |= FLAG(plugInModeRGB48);
        ACCEPT;
    }
    REJECT;
} /*PiPL_RGB48_support*/



//Added by Harald Heim, December 6, 2002
STATE(PiPL_LAB48_support)
    MARK;
    if (mdp_keywordi("noLAB48")) ACCEPT;
    if (mdp_keywordi("doesSupportLAB48")) {
        gParams->SupportedModes |= FLAG(plugInModeLab48);
        ACCEPT;
    }
    REJECT;
} /*PiPL_LAB48_support*/

STATE(PiPL_CMYK64_support)
    MARK;
    if (mdp_keywordi("noCMYK64")) ACCEPT;
    if (mdp_keywordi("doesSupportCMYK64")) {
        gParams->SupportedModes |= FLAG(plugInModeCMYK64);
        ACCEPT;
    }
    REJECT;
} /*PiPL_CMYK64_support*/

STATE(PiPL_deepmultichannel_support)
    MARK;
    if (mdp_keywordi("noDeepMultichannel")) ACCEPT;
    if (mdp_keywordi("doesDeepMultichannel")) {
        gParams->SupportedModes |= FLAG(plugInModeDeepMultichannel);
        ACCEPT;
    }
    REJECT;
} /*PiPL_deepmultichannel_support*/

STATE(PiPL_duotone16_support)
    MARK;
    if (mdp_keywordi("noDuotone16")) ACCEPT;
    if (mdp_keywordi("doesSupportDuotone16")) {
        gParams->SupportedModes |= FLAG(plugInModeDuotone16);
        ACCEPT;
    }
    REJECT;
} /*PiPL_duotone16_support*/



STATE(PiPL_supported_modes_property)
    MARK;
    if (!mdp_keywordi("SupportedModes")) REJECT;
    if (!mdp_char('{')) REJECT;
    gParams->SupportedModes= 0;
    if (!PiPL_bitmap_support()) REJECT;
    if (!mdp_char(',')) REJECT;
    if (!PiPL_grayscale_support()) REJECT;
    if (!mdp_char(',')) REJECT;
    if (!PiPL_indexed_support()) REJECT;
    if (!mdp_char(',')) REJECT;
    if (!PiPL_RGB_support()) REJECT;
    if (!mdp_char(',')) REJECT;
    if (!PiPL_CMYK_support()) REJECT;
    if (!mdp_char(',')) REJECT;
    if (!PiPL_HSL_support()) REJECT;
    if (!mdp_char(',')) REJECT;
    if (!PiPL_HSB_support()) REJECT;
    if (!mdp_char(',')) REJECT;
    if (!PiPL_multichannel_support()) REJECT;
    if (!mdp_char(',')) REJECT;
    if (!PiPL_duotone_support()) REJECT;
    if (!mdp_char(',')) REJECT;
    if (!PiPL_LAB_support()) REJECT;
    if (mdp_char(',')){
        //new, therefore optional
        if (!PiPL_Gray16_support()) REJECT;
        if (mdp_char(',')) {
            //new, therefore optional
            if (!PiPL_RGB48_support()) REJECT;

			//Added by Harald Heim, December 6, 2002
			if (mdp_char(',')) {
				//new, therefore optional
				if (!PiPL_LAB48_support()) REJECT;
				if (mdp_char(',')) {
					//new, therefore optional
					if (!PiPL_CMYK64_support()) REJECT;
					if (mdp_char(',')) {
						//new, therefore optional
						if (!PiPL_deepmultichannel_support()) REJECT;
						if (mdp_char(',')) {
							//new, therefore optional
							if (!PiPL_duotone16_support()) REJECT;
						}
					}
				}
			}
			//End Added

        }
    }
    if (!mdp_char('}')) REJECT;
    ACCEPT;
} /*PiPL_supported_modes_property*/

STATE(PiPL_enable_info_property)
    MARK;
    if (!mdp_keywordi("EnableInfo")) REJECT;
    if (!mdp_char('{')) REJECT;
    gParams->EI_start_index = (int)(inp - inp_start);
    if (!EI_booleanExpression()) REJECT;
    gParams->EI_count = (int)(inp - inp_start) - gParams->EI_start_index;
    gParams->EI_specified = TRUE;
    if (!mdp_char('}')) REJECT;
    ACCEPT;
} /*PiPL_enable_info_property*/

STATE(PiPL_filter_case_info_property)
    MARK;
    if (!mdp_keywordi("FilterCaseInfo")) REJECT;
    if (!mdp_char('{')) REJECT;
    if (!mdp_char('{')) REJECT;

    // filterCaseFlatImageNoSelection
    if (!PiPL_filter_info_case()) REJECT;
    if (!mdp_char(',')) REJECT;
    gParams->fici[filterCaseFlatImageNoSelection-1] = fici;

    // filterCaseFlatImageWithSelection
    if (!PiPL_filter_info_case()) REJECT;
    if (!mdp_char(',')) REJECT;
    gParams->fici[filterCaseFlatImageWithSelection-1] = fici;

    // filterCaseFloatingSelection
    if (!PiPL_filter_info_case()) REJECT;
    if (!mdp_char(',')) REJECT;
    gParams->fici[filterCaseFloatingSelection-1] = fici;

    // filterCaseEditableTransparencyNoSelection
    if (!PiPL_filter_info_case()) REJECT;
    if (!mdp_char(',')) REJECT;
    gParams->fici[filterCaseEditableTransparencyNoSelection-1] = fici;

    // filterCaseEditableTransparencyWithSelection
    if (!PiPL_filter_info_case()) REJECT;
    if (!mdp_char(',')) REJECT;
    gParams->fici[filterCaseEditableTransparencyWithSelection-1] = fici;

    // filterCaseProtectedTransparencyNoSelection
    if (!PiPL_filter_info_case()) REJECT;
    if (!mdp_char(',')) REJECT;
    gParams->fici[filterCaseProtectedTransparencyNoSelection-1] = fici;

    // filterCaseProtectedTransparencyWithSelection
    if (!PiPL_filter_info_case()) REJECT;
    gParams->fici[filterCaseProtectedTransparencyWithSelection-1] = fici;

    if (!mdp_char('}')) REJECT;
    if (!mdp_char('}')) REJECT;
    ACCEPT;
} /*PiPL_filter_case_info_property*/

STATE(PiPL_filter_info_case)
    MARK;
    fici.inputHandling = filterDataHandlingCantFilter;
    fici.outputHandling = filterDataHandlingCantFilter;
    fici.flags1 = 0;
    fici.flags2 = 0;    //reserved
    if (!PiPL_input_matting()) REJECT;
    if (!mdp_char(',')) REJECT;
    if (!PiPL_output_matting()) REJECT;
    if (!mdp_char(',')) REJECT;
    if (!PiPL_layer_mask_flag()) REJECT;
    if (!mdp_char(',')) REJECT;
    if (!PiPL_blank_data_flag()) REJECT;
    if (!mdp_char(',')) REJECT;
    if (!PiPL_copy_source_flag()) REJECT;
    //new, therefore optional...
    PiPL_optional_writes_outside_selection_flag();
    ACCEPT;
} /*PiPL_filter_info_case*/

STATE(PiPL_input_matting)
    MARK;
    if (mdp_keywordi("inCantFilter")) {
        fici.inputHandling = filterDataHandlingCantFilter;
    }
    else if (mdp_keywordi("inStraightData")) {
        fici.inputHandling = filterDataHandlingNone;
    }
    else if (mdp_keywordi("inBlackMat")) {
        fici.inputHandling = filterDataHandlingBlackMat;
    }
    else if (mdp_keywordi("inGrayMat")) {
        fici.inputHandling = filterDataHandlingGrayMat;
    }
    else if (mdp_keywordi("inWhiteMat")) {
        fici.inputHandling = filterDataHandlingWhiteMat;
    }
    else if (mdp_keywordi("inDefringe")) {
        fici.inputHandling = filterDataHandlingDefringe;
    }
    else if (mdp_keywordi("inBlackZap")) {
        fici.inputHandling = filterDataHandlingBlackZap;
    }
    else if (mdp_keywordi("inGrayZap")) {
        fici.inputHandling = filterDataHandlingGrayZap;
    }
    else if (mdp_keywordi("inWhiteZap")) {
        fici.inputHandling = filterDataHandlingWhiteZap;
    }
    else if (mdp_keywordi("inBackgroundZap")) {
        fici.inputHandling = filterDataHandlingBackgroundZap;
    }
    else if (mdp_keywordi("inForegroundZap")) {
        fici.inputHandling = filterDataHandlingForegroundZap;
    }
    else REJECT;
    ACCEPT;
} /*PiPL_input_matting*/

STATE(PiPL_output_matting)
    MARK;
    if (mdp_keywordi("outCantFilter")) {
        fici.outputHandling = filterDataHandlingCantFilter;
    }
    else if (mdp_keywordi("outStraightData")) {
        fici.outputHandling = filterDataHandlingNone;
    }
    else if (mdp_keywordi("outBlackMat")) {
        fici.outputHandling = filterDataHandlingBlackMat;
    }
    else if (mdp_keywordi("outGrayMat")) {
        fici.outputHandling = filterDataHandlingGrayMat;
    }
    else if (mdp_keywordi("outWhiteMat")) {
        fici.outputHandling = filterDataHandlingWhiteMat;
    }
    else if (mdp_keywordi("outFillMask")) {
        fici.outputHandling = filterDataHandlingFillMask;
    }
    else REJECT;
    ACCEPT;
} /*PiPL_output_matting*/

STATE(PiPL_layer_mask_flag)
    MARK;
    if (mdp_keywordi("doesNotFilterLayerMasks")) {
        fici.flags1 &= ~BIT(PIFilterFiltersLayerMaskBit);
    }
    else if (mdp_keywordi("filtersLayerMasks")) {
        fici.flags1 |= BIT(PIFilterFiltersLayerMaskBit);
    }
    else REJECT;
    ACCEPT;
} /*PiPL_layer_mask_flag*/

STATE(PiPL_blank_data_flag)
    MARK;
    if (mdp_keywordi("doesNotWorkWithBlankData")) {
        fici.flags1 &= ~BIT(PIFilterWorksWithBlankDataBit);
    }
    else if (mdp_keywordi("worksWithBlankData")) {
        fici.flags1 |= BIT(PIFilterWorksWithBlankDataBit);
    }
    else REJECT;
    ACCEPT;
} /*PiPL_blank_data_flag*/

STATE(PiPL_copy_source_flag)
    MARK;
    if (mdp_keywordi("copySourceToDestination")) {
        fici.flags1 &= ~BIT(PIFilterDontCopyToDestinationBit);
    }
    else if (mdp_keywordi("doNotCopySourceToDestination")) {
        fici.flags1 |= BIT(PIFilterDontCopyToDestinationBit);
    }
    else REJECT;
    ACCEPT;
} /*PiPL_copy_source_flag*/

STATE(PiPL_optional_writes_outside_selection_flag)
    MARK;
    if (!mdp_char(',')) REJECT;
    if (mdp_keywordi("doNotWriteOutsideSelection")) {
        fici.flags1 &= ~BIT(PIFilterWritesOutsideSelectionBit);
    }
    else if (mdp_keywordi("doesNotWriteOutsideSelection")) {
        fici.flags1 &= ~BIT(PIFilterWritesOutsideSelectionBit);
    }
    else if (mdp_keywordi("writeOutsideSelection")) {
        fici.flags1 |= BIT(PIFilterWritesOutsideSelectionBit);
    }
    else if (mdp_keywordi("writesOutsideSelection")) {
        fici.flags1 |= BIT(PIFilterWritesOutsideSelectionBit);
    }
    else REJECT;
    ACCEPT;
} /*PiPL_optional_writes_outside_selection_flag*/

STATE(PiPL_type_creator_pair)
    MARK;
    if (!constant_integer_expression()) REJECT;     //<OSType> File type
    if (!mdp_char(',')) REJECT;
    if (!constant_integer_expression()) REJECT;     //<OSType> File creator
    ACCEPT;
} /*PiPL_type_creator_pair*/

STATE(PiPL_format_file_type_property)
    MARK;
    if (!mdp_char('{')) REJECT;
    if (!PiPL_type_creator_pair()) REJECT;
    if (!mdp_char('}')) REJECT;
    ACCEPT;
} /*PiPL_format_file_type_property*/

STATE(PiPL_type_creator_pair_list)
    MARK;
    if (!mdp_char('{')) ACCEPT;     //empty list is okay
    if (!PiPL_type_creator_pair()) REJECT;
    if (!mdp_char('}')) REJECT;
    while (mdp_char(',')) {
        if (!mdp_char('{')) REJECT;
        if (!PiPL_type_creator_pair()) REJECT;
        if (!mdp_char('}')) REJECT;
    }
    ACCEPT;
} /*PiPL_type_creator_pair_list*/

STATE(PiPL_read_types_property)
    MARK;
    if (!mdp_char('{')) REJECT;
    if (!PiPL_type_creator_pair_list()) REJECT;
    if (mdp_char('}')) REJECT;
    ACCEPT;
} /*PiPL_read_types_property*/

STATE(PiPL_write_types_property)
    MARK;
    if (!mdp_char('{')) REJECT;
    if (!PiPL_type_creator_pair_list()) REJECT;
    if (mdp_char('}')) REJECT;
    ACCEPT;
} /*PiPL_write_types_property*/

STATE(PiPL_filtered_types_property)
    MARK;
    if (!mdp_char('{')) REJECT;
    if (!PiPL_type_creator_pair_list()) REJECT;
    if (mdp_char('}')) REJECT;
    ACCEPT;
} /*PiPL_filtered_types_property*/

STATE(PiPL_ostype_list)
    MARK;
    if (!mdp_char('{')) ACCEPT;     //empty list is okay
    if (!constant_integer_expression()) REJECT;     //<OSType>
    if (!mdp_char('}')) REJECT;
    while (mdp_char(',')) {
        if (!mdp_char('{')) REJECT;
        if (!constant_integer_expression()) REJECT; //<OSType>
        if (!mdp_char('}')) REJECT;
    }
    ACCEPT;
} /*PiPL_ostype_list*/

STATE(PiPL_read_extensions_property)
    MARK;
    if (!mdp_char('{')) REJECT;
    if (!PiPL_ostype_list()) REJECT;
    if (mdp_char('}')) REJECT;
    ACCEPT;
} /*PiPL_read_extensions_property*/

STATE(PiPL_write_extensions_property)
    MARK;
    if (!mdp_char('{')) REJECT;
    if (!PiPL_ostype_list()) REJECT;
    if (mdp_char('}')) REJECT;
    ACCEPT;
} /*PiPL_write_extensions_property*/

STATE(PiPL_filtered_extensions_property)
    MARK;
    if (!mdp_char('{')) REJECT;
    if (!PiPL_ostype_list()) REJECT;
    if (mdp_char('}')) REJECT;
    ACCEPT;
} /*PiPL_filtered_extensions_property*/

STATE(PiPL_saves_image_resources_flag)
    MARK;
    if (mdp_keywordi("fmtDoesNotSaveImageResources")) ACCEPT;
    if (mdp_keywordi("fmtSavesImageResources")) ACCEPT;
    REJECT;
} /*PiPL_saves_image_resources_flag*/

STATE(PiPL_can_read_flag)
    MARK;
    if (mdp_keywordi("fmtCannotRead")) ACCEPT;
    if (mdp_keywordi("fmtCanRead")) ACCEPT;
    REJECT;
} /*PiPL_can_read_flag*/

STATE(PiPL_can_write_flag)
    MARK;
    if (mdp_keywordi("fmtCannotWrite")) ACCEPT;
    if (mdp_keywordi("fmtCanWrite")) ACCEPT;
    REJECT;
} /*PiPL_can_write_flag*/

STATE(PiPL_write_if_read_flag)
    MARK;
    if (mdp_keywordi("fmtWritesAll")) ACCEPT;           //sic???
    if (mdp_keywordi("fmtCanWriteIfRead")) ACCEPT;
    REJECT;
} /*PiPL_write_if_read_flag*/

STATE(PiPL_format_flags_property)
    MARK;
    if (!mdp_char('{')) REJECT;
    if (!PiPL_saves_image_resources_flag()) REJECT;
    if (!mdp_char(',')) REJECT;
    if (!PiPL_can_read_flag()) REJECT;
    if (!mdp_char(',')) REJECT;
    if (!PiPL_can_write_flag()) REJECT;
    if (!mdp_char(',')) REJECT;
    if (!PiPL_write_if_read_flag()) REJECT;
    if (!mdp_char('}')) REJECT;
    ACCEPT;
} /*PiPL_format_flags_property*/

STATE(PiPL_format_maximum_size_property)
    MARK;
    if (!mdp_char('{')) REJECT;
    if (!constant_integer_expression()) REJECT;     //<int16> max rows??
    if (!mdp_char(',')) REJECT;
    if (!constant_integer_expression()) REJECT;     //<int16> max cols??
    if (!mdp_char('}')) REJECT;
    ACCEPT;
} /*PiPL_format_maximum_size_property*/

STATE(PiPL_format_maximum_channels_property)
    MARK;
    if (!mdp_char('{')) REJECT;
    do {
        if (!constant_integer_expression()) REJECT; //max channels for this image mode
    } while (mdp_char(','));
    if (!mdp_char('}')) REJECT;
    ACCEPT;
} /*PiPL_format_maximum_channels_property*/

STATE(PiPL_parsable_types_property)
    MARK;
    if (!mdp_char('{')) REJECT;
    if (!PiPL_type_creator_pair_list()) REJECT;
    if (mdp_char('}')) REJECT;
    ACCEPT;
} /*PiPL_parsable_types_property*/

STATE(PiPL_parsable_extensions_property)
    MARK;
    if (!mdp_char('{')) REJECT;
    if (!PiPL_ostype_list()) REJECT;
    if (mdp_char('}')) REJECT;
    ACCEPT;
} /*PiPL_parsable_extensions_property*/

STATE(PiPL_filtered_parsable_types_property)
    MARK;
    if (!mdp_char('{')) REJECT;
    if (!PiPL_type_creator_pair_list()) REJECT;
    if (mdp_char('}')) REJECT;
    ACCEPT;
} /*PiPL_filtered_parsable_types_property*/

STATE(PiPL_filtered_parsable_extensions_property)
    MARK;
    if (!mdp_char('{')) REJECT;
    if (!PiPL_ostype_list()) REJECT;
    if (mdp_char('}')) REJECT;
    ACCEPT;
} /*PiPL_filtered_parsable_extensions_property*/

STATE(PiPL_parsable_clipboard_types_property)
    MARK;
    if (!mdp_char('{')) REJECT;
    if (!PiPL_ostype_list()) REJECT;
    if (mdp_char('}')) REJECT;
    ACCEPT;
} /*PiPL_parsable_clipboard_types_property*/

#endif //ifndef BETA

/*************************** <color> ************************
;
; Syntax:
;
;   <color> ::=
;       'RGB' "(" <integer_constant> "," <integer_constant> "," 
;                                        <integer_constant> ")"     | //RGB(r,g,b)
;       "#" <hexadecimal_string>                                    | //#rrggbb
;       <hexadecimal_constant>                                      | //0xbbggrr
;       <X11_color_name>                                            |
;       <HTML_color_name>                                           |
;       <JAVA_color_name>                                           |
;       <NS_color_name>                                             |
;       <Windows_static_color_name>                                 |
;       <Windows_UI_color_name>
;
;   <X11_color_name> ::= ['X11' "."] {'aliceblue' | ... | 'yellowgreen'}
;
;   <HTML_color_name> ::= ['HTML' "."] {'Aquamarine' | ... | 'YellowGreen'}
;
;   <JAVA_color_name> ::= ['JAVA" "."] {'black' | ... | 'yellow'}
;
;   <NS_color_name> ::= ['NS' "."] {'black' | ... | 'yellow'}
;
;   <Windows_static_color_name> ::= ['Win' "."] {'black' | ... | 'yellow'}
;
;   <Windows_UI_color_name> ::= ['UI' "."] {COLOR_*}
;
*/
  
// NOTE: RGBQuad has Blue in lsb, but RGB macro puts Red in lsb.
//       Need to byteswap so Red is in LSB for Win32 API...
INT4 RGBswap(INT4 rgb) {
    //reverse the order of the low 3 bytes of an RGB colorref
    //(which really means just swapping the RR and BB bytes)
    return ((rgb & 0xff) << 16) | (rgb & 0xff00) | ((rgb >> 16) & 0xff);
} /*RGBswap*/

/*--------------------------------------------------------------------------*/
/*                                                                          */
/*  X11 color names:                                                        */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static struct {
    char *name;
    UINT4 value;
} X11_color[] =
{
 "aliceblue"         ,0xF0F8FF,
 "antiquewhite"      ,0xFAEBD7,
 "aqua"              ,0x00FFFF,
 "aquamarine"        ,0x7FFFD4,
 "azure"             ,0xF0FFFF,
 "beige"             ,0xF5F5DC,
 "bisque"            ,0xFFE4C4,
 "black"             ,0x000000,
 "blanchedalmond"    ,0xFFEBCD,
 "blue"              ,0x0000FF,
 "blueviolet"        ,0x8A2BE2,
 "brown"             ,0xA52A2A,
 "burlywood"         ,0xDEB887,
 "cadetblue"         ,0x5F9EA0,
 "chartreuse"        ,0x7FFF00,
 "chocolate"         ,0xD2691E,
 "coral"             ,0xFF7F50,
 "cornflowerblue"    ,0x6495ED,
 "cornsilk"          ,0xFFF8DC,
 "crimson"           ,0xDC143C,
 "cyan"              ,0x00FFFF,
 "darkblue"          ,0x00008B,
 "darkcyan"          ,0x008B8B,
 "darkgoldenrod"     ,0xB8860B,
 "darkgray"          ,0xA9A9A9,
 "darkgreen"         ,0x006400,
 "darkkhaki"         ,0xBDB76B,
 "darkmagenta"       ,0x8B008B,
 "darkolivegreen"    ,0x556B2F,
 "darkorange"        ,0xFF8C00,
 "darkorchid"        ,0x9932CC,
 "darkred"           ,0x8B0000,
 "darksalmon"        ,0xE9967A,
 "darkseagreen"      ,0x8FBC8F,
 "darkslateblue"     ,0x483D8B,
 "darkslategray"     ,0x2F4F4F,
 "darkturquoise"     ,0x00CED1,
 "darkviolet"        ,0x9400D3,
 "deeppink"          ,0xFF1493,
 "deepskyblue"       ,0x00BFFF,
 "dimgray"           ,0x696969,
 "dodgerblue"        ,0x1E90FF,
 "firebrick"         ,0xB22222,
 "floralwhite"       ,0xFFFAF0,
 "forestgreen"       ,0x228B22,
 "fuchsia"           ,0xFF00FF,
 "gainsboro"         ,0xDCDCDC,
 "ghostwhite"        ,0xF8F8FF,
 "gold"              ,0xFFD700,
 "goldenrod"         ,0xDAA520,
 "gray"              ,0x808080,
 "green"             ,0x008000,
 "greenyellow"       ,0xADFF2F,
 "honeydew"          ,0xF0FFF0,
 "hotpink"           ,0xFF69B4,
 "indianred"         ,0xCD5C5C,
 "indigo"            ,0x4B0082,
 "ivory"             ,0xFFFFF0,
 "khaki"             ,0xF0E68C,
 "lavender"          ,0xE6E6FA,
 "lavenderblush"     ,0xFFF0F5,
 "lawngreen"         ,0x7CFC00,
 "lemonchiffon"      ,0xFFFACD,
 "lightblue"         ,0xADD8E6,
 "lightcoral"        ,0xF08080,
 "lightcyan"         ,0xE0FFFF,
 "lightgoldenrodyellow" ,0xFAFAD2,
 "lightgreen"        ,0x90EE90,
 "lightgrey"         ,0xD3D3D3,
 "lightpink"         ,0xFFB6C1,
 "lightsalmon"       ,0xFFA07A,
 "lightseagreen"     ,0x20B2AA,
 "lightskyblue"      ,0x87CEFA,
 "lightslategray"    ,0x778899,
 "lightsteelblue"    ,0xB0C4DE,
 "lightyellow"       ,0xFFFFE0,
 "lime"              ,0x00FF00,
 "limegreen"         ,0x32CD32,
 "linen"             ,0xFAF0E6,
 "magenta"           ,0xFF00FF,
 "maroon"            ,0x800000,
 "mediumaquamarine"  ,0x66CDAA,
 "mediumblue"        ,0x0000CD,
 "mediumorchid"      ,0xBA55D3,
 "mediumpurple"      ,0x9370DB,
 "mediumseagreen"    ,0x3CB371,
 "mediumslateblue"   ,0x7B68EE,
 "mediumspringgreen" ,0x00FA9A,
 "mediumturquoise"   ,0x48D1CC,
 "mediumvioletred"   ,0xC71585,
 "midnightblue"      ,0x191970,
 "mintcream"         ,0xF5FFFA,
 "mistyrose"         ,0xFFE4E1,
 "moccasin"          ,0xFFE4B5,
 "navajowhite"       ,0xFFDEAD,
 "navy"              ,0x000080,
 "oldlace"           ,0xFDF5E6,
 "olive"             ,0x808000,
 "olivedrab"         ,0x6B8E23,
 "orange"            ,0xFFA500,
 "orangered"         ,0xFF4500,
 "orchid"            ,0xDA70D6,
 "palegoldenrod"     ,0xEEE8AA,
 "palegreen"         ,0x98FB98,
 "paleturquoise"     ,0xAFEEEE,
 "palevioletred"     ,0xDB7093,
 "papayawhip"        ,0xFFEFD5,
 "peachpuff"         ,0xFFDAB9,
 "peru"              ,0xCD853F,
 "pink"              ,0xFFC0CB,
 "plum"              ,0xDDA0DD,
 "powderblue"        ,0xB0E0E6,
 "purple"            ,0x800080,
 "red"               ,0xFF0000,
 "rosybrown"         ,0xBC8F8F,
 "royalblue"         ,0x4169E1,
 "saddlebrown"       ,0x8B4513,
 "salmon"            ,0xFA8072,
 "sandybrown"        ,0xF4A460,
 "seagreen"          ,0x2E8B57,
 "seashell"          ,0xFFF5EE,
 "sienna"            ,0xA0522D,
 "silver"            ,0xC0C0C0,
 "skyblue"           ,0x87CEEB,
 "slateblue"         ,0x6A5ACD,
 "slategray"         ,0x708090,
 "snow"              ,0xFFFAFA,
 "springgreen"       ,0x00FF7F,
 "steelblue"         ,0x4682B4,
 "tan"               ,0xD2B48C,
 "teal"              ,0x008080,
 "thistle"           ,0xD8BFD8,
 "tomato"            ,0xFF6347,
 "turquoise"         ,0x40E0D0,
 "violet"            ,0xEE82EE,
 "wheat"             ,0xF5DEB3,
 "white"             ,0xFFFFFF,
 "whitesmoke"        ,0xF5F5F5,
 "yellow"            ,0xFFFF00,
 "yellowgreen"       ,0x9ACD32
};

STATE(X11_color_name)
    int i;
    MARK;
    if (mdp_keywordi("X11")) {
        //optional 'X11.'
        if (!mdp_char('.')) REJECT;
    }
    // Lookup in X11 color table.
    for (i=0; i < sizeof(X11_color)/sizeof(X11_color[0]); i++) {
        if (mdp_keywordi(X11_color[i].name)) {
            push_param(LIT_INTEGER);
            push_param(RGBswap(X11_color[i].value));
            ACCEPT;
        }
    } /*for*/
    REJECT;
} /*X11_color_name*/

/*--------------------------------------------------------------------------*/
/*                                                                          */
/*  HTML color names:                                                       */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static struct {
    char *name;
    UINT4 value;
} HTML_color[] =
{
//=========================================================
//                                  Notes:
//                                  = same as X11
//                                  ! conflict with X11
//Color name             0xRRGGBB   * Netscape "safe" color
//--------------------   ----------------------------------
 "Aquamarine"			,0x70DB93,  //	!
 "BakersChocolate"		,0x5C3317,  //
 "BlueViolet"			,0x9F5F9F,  //	!
 "Black"				,0x000000,  //	=	*
 "Blue"				    ,0x0000FF,  //	=	*
 "Brass"				,0xB5A642,  //
 "BrightGold"			,0xD9D919,  //
 "Brown"				,0xA62A2A,  //	!
 "Bronze"				,0x8C7853,  //
 "BronzeII"			    ,0xA67D3D,  //
 "CadetBlue"			,0x5F9F9F,  //	!
 "CoolCopper"			,0xD98719,  //
 "Copper"				,0xB87333,  //
 "Coral"				,0xFF7F00,  //	!
 "CornFlowerBlue"		,0x42426F,  //	!
 "Cyan"				    ,0x00FFFF,  //	=	*
 "DarkBrown"			,0x5C4033,  //
 "DarkGreen"			,0x2F4F2F,  //	!
 "DarkGreenCopper"		,0x4A766E,  //
 "DarkOliveGreen"		,0x4F4F2F,  //	!
 "DarkOrchid"			,0x9932CD,  //	!
 "DarkPurple"			,0x871F78,  //
 "DarkSlateBlue"		,0x6B238E,  //	!
 "DarkSlateGray"		,0x2F4F4F,  //	=
 "DarkTan"				,0x97694F,  //
 "DarkTurquoise"		,0x7093DB,  //	!
 "DarkWood"			    ,0x855E42,  //
 "DimGray"				,0x545454,  //	!
 "DustyRose"			,0x856363,  //
 "Feldspar"			    ,0xD19275,  //
 "Firebrick"			,0x8E2323,  //	!
 "ForestGreen"			,0x238E23,  //	!
 "Gold"				    ,0xCD7F32,  //	!
 "Goldenrod"			,0xDBDB70,  //	!
 "Gray"				    ,0xC0C0C0,  //
 "Green"				,0x00FF00,  //	!	*
 "GreenCopper"			,0x527F76,  //
 "GreenYellow"			,0x93DB70,  //	!
 "HunterGreen"			,0x215E21,  //
 "IndianRed"			,0x4E2F2F,  //
 "Khaki"				,0x9F9F5F,  //	!
 "LightBlue"			,0xC0D9D9,  //	!
 "LightGray"			,0xA8A8A8,  //
 "LightSteelBlue"		,0x8F8FBD,  //	!
 "LightWood"			,0xE9C2A6,  //
 "LimeGreen"			,0x32CD32,  //	=
 "Magenta"				,0xFF00FF,  //	=	*
 "MandarinOrange"		,0xE47833,  //
 "Maroon"				,0x8E236B,  //	!
 "MediumAquamarine"	    ,0x32CD99,  //	!
 "MediumBlue"			,0x3232CD,  //	!
 "MediumForestGreen"	,0x6B8E23,  //
 "MediumGoldenrod"		,0xEAEAAE,  //
 "MediumOrchid"		    ,0x9370DB,  //	!
 "MediumSeaGreen"		,0x426F42,  //	!
 "MediumSlateBlue"		,0x7F00FF,  //	!
 "MediumSpringGreen"	,0x7FFF00,  //	!
 "MediumTurquoise"		,0x70DBDB,  //	!
 "MediumVioletRed"		,0xDB7093,  //	!
 "MediumWood"			,0xA68064,  //
 "MidnightBlue"		    ,0x2F2F4F,  //	!
 "NavyBlue"			    ,0x23238E,  //
 "NeonBlue"			    ,0x4D4DFF,  //
 "NeonPink"			    ,0xFF6EC7,  //
 "NewMidnightBlue"		,0x00009C,  //
 "NewTan"				,0xEBC79E,  //
 "OldGold"				,0xCFB53B,  //
 "Orange"				,0xFF7F00,  //	!
 "OrangeRed"			,0xFF2400,  //	!
 "Orchid"				,0xDB70DB,  //	!
 "PaleGreen"			,0x8FBC8F,  //	!
 "Pink"				    ,0xBC8F8F,  //	!
 "Plum"				    ,0xEAADEA,  //	!
 "Quartz"				,0xD9D9F3,  //
 "Red"					,0xFF0000,  //	=	*
 "RichBlue"			    ,0x5959AB,  //
 "Salmon"				,0x6F4242,  //	!
 "Scarlet"				,0x8C1717,  //
 "SeaGreen"			    ,0x238E68,  //	!
 "SemiSweetChocolate"	,0x6B4226,  //
 "Sienna"				,0x8E6B23,  //	!
 "Silver"				,0xE6E8FA,  //	!
 "SkyBlue"				,0x3299CC,  //	!
 "SlateBlue"			,0x007FFF,  //	!
 "SpicyPink"			,0xFF1CAE,  //
 "SpringGreen"			,0x00FF7F,  //	=
 "SteelBlue"			,0x236B8E,  //	!
 "SummerSky"			,0x38B0DE,  //
 "Tan"					,0xDB9370,  //	!
 "Thistle"				,0xD8BFD8,  //	=
 "Turquoise"			,0xADEAEA,  //	!
 "VeryDarkBrown"		,0x5C4033,  //
 "VeryLightGray"		,0xCDCDCD,  //
 "Violet"				,0x4F2F4F,  //	!
 "VioletRed"			,0xCC3299,  //
 "Wheat"				,0xD8D8BF,  //	!
 "White"				,0xFFFFFF,  //	=	*
 "Yellow"				,0xFFFF00,  //		*
 "YellowGreen"			,0x99CC32   //	!
};

STATE(HTML_color_name)
    int i;
    MARK;
    if (mdp_keywordi("HTML")) {
        //optional 'HTML.'
        if (!mdp_char('.')) REJECT;
    }
    // Lookup in HTML color table.
    for (i=0; i < sizeof(HTML_color)/sizeof(HTML_color[0]); i++) {
        if (mdp_keywordi(HTML_color[i].name)) {
            push_param(LIT_INTEGER);
            push_param(RGBswap(HTML_color[i].value));
            ACCEPT;
        }
    } /*for*/
    REJECT;
} /*HTML_color_name*/

/*--------------------------------------------------------------------------*/
/*                                                                          */
/*  JAVA color names:                                                       */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static struct {
    char *name;
    UINT4 value;
} JAVA_color[] =
{
 // I'm just guessing at these values ... they're
 // probably platform-dependent .. we'll just set them
 // to the 'X11' colors for now.  Someday we should
 // check these values against an actual win95 JVM.
 "black"             ,0x000000,
 "blue"              ,0x0000FF,
 "cyan"              ,0x00FFFF,
 "darkGray"          ,0xA9A9A9,
 "gray"              ,0x808080,
 "green"             ,0x008000,
 "lightGray"         ,0xD3D3D3, //=X11 "lightgrey'
 "magenta"           ,0xFF00FF,
 "orange"            ,0xFFA500,
 "pink"              ,0xFFC0CB,
 "red"               ,0xFF0000,
 "white"             ,0xFFFFFF,
 "yellow"            ,0xFFFF00
};

STATE(JAVA_color_name)
    int i;
    MARK;
    if (mdp_keywordi("JAVA")) {
        //optional 'JAVA.'
        if (!mdp_char('.')) REJECT;
    }
    // Lookup in JAVA color table.
    for (i=0; i < sizeof(JAVA_color)/sizeof(JAVA_color[0]); i++) {
        if (mdp_keywordi(JAVA_color[i].name)) {
            push_param(LIT_INTEGER);
            push_param(RGBswap(JAVA_color[i].value));
            ACCEPT;
        }
    } /*for*/
    REJECT;
} /*JAVA_color_name*/

/*--------------------------------------------------------------------------*/
/*                                                                          */
/*  Netscape "safe" color names:                                            */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static struct {
    char *name;
    UINT4 value;
} NS_color[] =
{
//=========================================================
//                                  Notes:
//                                  = same as X11
//                                  ! conflict with X11
//Color name             0xRRGGBB   % conflict with HTML
//--------------------   ----------------------------------
 "Black"				,0x000000,  //	=
 "Blue"				    ,0x0000CC,  //	!%
 "BrightBlue"			,0x0000FF,  //	
 "BrightCyan"			,0x00FFFF,  //	
 "BrightGreen"			,0x00FF00,  //
 "BrightMagenta"		,0xFF00FF,  //
 "BrightRed"			,0xFF0000,  //
 "Cyan"				    ,0x00CCCC,  //	!%
 "DarkBlue"			    ,0x000066,  //	!
 "DarkCyan"			    ,0x006666,  //	!
 "DarkGray"			    ,0x666666,  //	!
 "DarkGreen"			,0x006600,  //	!%
 "DarkMagenta"			,0x660066,  //	!
 "DarkRed"				,0x660000,  //	!
 "Green"				,0x00CC00,  //	!%
 "Magenta"				,0xCC00CC,  //	!%
 "MediumBlue"			,0x000099,  //	!%
 "MediumCyan"			,0x009999,  //
 "MediumGray"			,0x999999,  //
 "MediumGreen"			,0x009900,  //
 "MediumMagenta"		,0x990099,  //
 "MediumRed"			,0x990000,  //
 "NetscapeGray"		    ,0xCCCCCC,  //
 "Red"					,0xCC0000,  //	!%
 "VeryDarkBlue"		    ,0x000033,  //
 "VeryDarkCyan"		    ,0x003333,  //
 "VeryDarkGray"		    ,0x333333,  //
 "VeryDarkGreen"		,0x003300,  //
 "VeryDarkMagenta"		,0x330033,  //
 "VeryDarkRed"			,0x330000,  //
 "White"				,0xFFFFFF,  //	=
 "Yellow"				,0xFFFF00   //	=
};

STATE(NS_color_name)
    int i;
    MARK;
    if (mdp_keywordi("NS")) {
        //optional 'NS.'
        if (!mdp_char('.')) REJECT;
    }
    // Lookup in NS color table.
    for (i=0; i < sizeof(NS_color)/sizeof(NS_color[0]); i++) {
        if (mdp_keywordi(NS_color[i].name)) {
            push_param(LIT_INTEGER);
            push_param(RGBswap(NS_color[i].value));
            ACCEPT;
        }
    } /*for*/
    REJECT;
} /*NS_color_name*/

/*--------------------------------------------------------------------------*/
/*                                                                          */
/*  Windows static color names:                                             */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static struct {
    char *name;
    UINT4 value;
} Win_color[] =
{
//=========================================================
//                                  Notes:
//                                  = same as X11
//                                  ! conflict with X11
//                                  % conflict with HTML
//Color name             0xRRGGBB   * Netscape "safe" color
//--------------------   ----------------------------------
 "Black"				,0x000000,  //	=	*
 "Blue"				    ,0x0000FF,  //	=	*
 "Cream"				,0xFFFBF0,  //
 "Cyan"				    ,0x00FFFF,  //	=	*
 "DarkBlue"			    ,0x000080,  //	!
 "DarkCyan"			    ,0x008080,  //	!
 "DarkGray"			    ,0x808080,  //	!
 "DarkGreen"			,0x008000,  //	!%
 "DarkMagenta"			,0x800080,  //	!
 "DarkRed"				,0x800000,  //	!
 "DarkYellow"			,0x808000,  //
 "Green"				,0x00FF00,  //	!	*
 "LightGray"			,0xC0C0C0,  //	 %
 "Magenta"				,0xFF00FF,  //	=	*
 "MediumGray"			,0xA0A0A4,  //
 "MoneyGreen"			,0xC0DCC0,  //
 "Red"					,0xFF0000,  //	=	*
 "SkyBlue"				,0xA6CAF0,  //	!%
 "White"				,0xFFFFFF,  //	=	*
 "Yellow"				,0xFFFF00   //	=	*
};

STATE(Windows_static_color_name)
    int i;
    MARK;
    if (mdp_keywordi("Win")) {
        //optional 'Win.'
        if (!mdp_char('.')) REJECT;
    }
    // Lookup in Windows static color table.
    for (i=0; i < sizeof(Win_color)/sizeof(Win_color[0]); i++) {
        if (mdp_keywordi(Win_color[i].name)) {
            push_param(LIT_INTEGER);
            push_param(RGBswap(Win_color[i].value));
            ACCEPT;
        }
    } /*for*/
    REJECT;
} /*Windows_static_color_name*/


#define UI(name) #name, name

static struct {
    char *name;
    UINT4 value;
} UI_color[] =
{
 UI(COLOR_3DDKSHADOW),			//Dark shadow for three-dimensional display elements.
 UI(COLOR_3DFACE), 				//Face color for three-dimensional display elements.
 UI(COLOR_BTNFACE),
 UI(COLOR_3DHILIGHT), 			//Highlight color for three-dimensional display elements 
 UI(COLOR_3DHIGHLIGHT),			//(for edges facing the light source.)
 UI(COLOR_BTNHILIGHT), 
 UI(COLOR_BTNHIGHLIGHT),
 UI(COLOR_3DLIGHT),				//Light color for three-dimensional display elements 
							    //(for edges facing the light source.)
 UI(COLOR_3DSHADOW), 			//Shadow color for three-dimensional display elements 
 UI(COLOR_BTNSHADOW),			//(for edges facing away from the light source).
 UI(COLOR_ACTIVEBORDER),		//Active window border.
 UI(COLOR_ACTIVECAPTION),		//Active window caption.
 UI(COLOR_APPWORKSPACE),		//Background color of multiple document interface (MDI) 
							    //applications.
 UI(COLOR_BACKGROUND),			//Desktop.
 UI(COLOR_DESKTOP),
 UI(COLOR_BTNTEXT),				//Text on push buttons.
 UI(COLOR_CAPTIONTEXT),			//Text in caption, size box, and scroll bar arrow box.
 UI(COLOR_GRAYTEXT),			//Grayed (disabled) text. This color is set to 0 if the 
							    //current display driver does not support a solid gray 
							    //color.
 UI(COLOR_HIGHLIGHT),			//Item(s) selected in a control.
 UI(COLOR_HIGHLIGHTTEXT),		//Text of item(s) selected in a control.
 UI(COLOR_INACTIVEBORDER),		//Inactive window border.
 UI(COLOR_INACTIVECAPTION),		//Inactive window caption.
 UI(COLOR_INACTIVECAPTIONTEXT),	//Color of text in an inactive caption.
 UI(COLOR_INFOBK),				//Background color for tooltip controls.
 UI(COLOR_INFOTEXT),			//Text color for tooltip controls.
 UI(COLOR_MENU),				//Menu background.
 UI(COLOR_MENUTEXT),			//Text in menus.
 UI(COLOR_SCROLLBAR),			//Scroll bar gray area.
 UI(COLOR_WINDOW),				//Window background.
 UI(COLOR_WINDOWFRAME),			//Window frame.
 UI(COLOR_WINDOWTEXT)			//Text in windows.
};

#undef UI

STATE(Windows_UI_color_name)
    int i;
    MARK;
    if (mdp_keywordi("UI")) {
        //optional 'UI.'
        if (!mdp_char('.')) REJECT;
    }
    // Lookup in Windows UI color table.
    for (i=0; i < sizeof(UI_color)/sizeof(UI_color[0]); i++) {
        if (mdp_keywordi(UI_color[i].name)) {
            push_param(LIT_INTEGER);
            push_param(GetSysColor(UI_color[i].value));
            ACCEPT;
        }
    } /*for*/
    REJECT;
} /*Windows_UI_color_name*/

STATE(color_name_or_constant)
    MARK;
    if (mdp_keywordi("RGB")) {
        INT4 rgb;
        if (!mdp_char('(')) REJECT;
        if (!integer_constant()) REJECT;
        pop_param();    //discard the LIT_INTEGER descriptor
        pop_param();
        rgb = (INT4)mdp_number & 0xff;  //red component
        if (!mdp_char(',')) REJECT;
        if (!integer_constant()) REJECT;
        pop_param();    //discard the LIT_INTEGER descriptor
        pop_param();
        rgb |= (mdp_number & 0xff) << 8;  //green component
        if (!mdp_char(',')) REJECT;
        if (!integer_constant()) REJECT;
        pop_param();    //discard the LIT_INTEGER descriptor
        pop_param();
        rgb |= (mdp_number & 0xff) << 16;   //blue component
        if (!mdp_char(')')) REJECT;
        push_param(LIT_INTEGER);
        push_param(rgb);
    }
    else if (mdp_char('#')) {
        // HTML-style hex constant
        //OOPS!!! The PP grabs the '#'!!!!!
        if (!mdp_hex_digits()) REJECT;
        //Need to swap R into lsb...
        mdp_number = RGBswap((INT4)mdp_number);
        push_value(LIT_INTEGER);
    }
    else if (hexadecimal_constant()) {
        //C-style hex value -- RGB is already in correct order
        //in the LIT_INTEGER on the semantic stack.
    }
    else if (X11_color_name()) {
        //done!
    }
    else if (HTML_color_name()) {
        //done!
    }
    else if (JAVA_color_name()) {
        //done!
    }
    else if (NS_color_name()) {
        //done!
    }
    else if (Windows_static_color_name()) {
        //done!
    }
    else if (Windows_UI_color_name()) {
        //done!
    }
    else REJECT;
    ACCEPT;
} /*color_name_or_constant*/

STATE(color)
    MARK;
    if (color_name_or_constant()) ACCEPT;
    if (constant_integer_expression()) ACCEPT;
    REJECT;
} /*color*/

#ifndef BETA
/*************************** <FML_program> ************************
;
; Syntax:
;
;	<FML_program> ::= [ '%FML' [<program_revision_number>] ]	//optional header
;                     {<global_declaration>}+
;                     <end_of_program>
;
;	<global_declaration> ::= <function_definition>  |
;                            <declaration>          |
;                            <FFP_key_value_pair>   | // Overkill: <FFP_key_value_pair> includes 
;                            <statement>              // some elements parsed by <statement>,
;                                                     // but that's harmless.  More importantly,
;                                                     // parse <FFP_key_value_pair> *before*
;                                                     // parsing <statement> so <labeled_statement>
;                                                     // won't gobble certain FFP_key_value_pair's.
;
*/
STATE(FML_program)
    MARK;
    prog_type = PROG_FML;
	prog_rev_major = INT_MAX;	//max available
	prog_rev_minor = INT_MAX;	//max available
	//optional program header
	if (mdp_keywordi("%FML")) {
		//optional revision number
		program_revision_number();
	}
	//push an SD_PROG item onto the SDL
	push_constant(SD_PROG);
	push_param(prog_type);
	push_param(prog_rev_major);
	push_param(prog_rev_minor);

    if (!global_declaration()) REJECT;  //at least one!
    while (global_declaration())
        ;
    if (end_of_program()) ACCEPT;
    REJECT;
}

STATE(global_declaration)
    MARK;
    // parse FFP_key_value_pair first, before statement() eats what might look like a label...
    if (function_definition() || declaration() || FFP_key_value_pair() || statement()) ACCEPT;
    REJECT;
}

#if 0   // Full-blown C-compatible <function_defintion>:
/*************************** <function_definition> ************************
;
; Syntax:
;
;	<function_definition> ::=
;		[<declaration_specifiers>] <declarator> <compound_statement>
;
*/
STATE(function_definition)
    MARK;
#if 0
    declaration_specifiers();   //optional                      // MAYBE SHOULD *REQUIRE* AT LEAST ONE???
#else
    if (!declaration_specifiers()) REJECT;  //at least one declaration specifier for now...
#endif
    if (declarator() && compound_statement()) ACCEPT;
    REJECT;
}
#else   // For now, we use a greatly simplified <function_definition>:
/*************************** <function_definition> ************************
;
; Syntax:
;
;	<function_definition> ::=
;		<function_specifiers> <function_declarator> "(" <parameter_list> ")" <compound_statement>
;
;   <function_specifiers> ::= {<function_specifier>}+
;
;   <function_specifier> ::= <type_specifier> | <storage_class_specifier>
;
;   <function_declarator> ::= [<pointer>] <function_name>   //NOT REALLY CORRECT
;
;   <function_name> ::= <identifier> | "(" <function_name> ")"
;
;   <parameter_list> ::= <parameter_decl> {"," <parameter_decl>}* ["," "..."] |
;                        "void"
;
;   <parameter_decl> ::= <param_specifiers> <param_declarator>
;
;   <param_specifiers> ::= {<param_specifier>}+
;
:   <param_specifier> ::= <type_specifier> | <storage_class_specifier> | <type_qualifier>
;
;   <param_declarator> ::= ["*"] <param_declarator> |          // NOT TOTALLY CORRECT FOR NOW!!!
;                         <identifier> ["[" "]"] |      // DICEY!!!!!
;                         "(" <param_declarator> ")"
;
*/
STATE(function_definition)
    MARK;
    push_op(OP_USER_FUNCTION);
#if 1 // temporary test
    if (!mdp_keyword("USERFUNC")) REJECT;
#else
    if (!function_specifiers()) REJECT;  //at least one function specifier for now...
    if (!function_declarator()) REJECT;
#endif
    if (!mdp_char('(')) REJECT;
    if (!parameter_list()) REJECT;
    if (!mdp_char(')')) REJECT;
    if (!compound_statement()) REJECT;
    push_op(OP_ON_HANDLER_END);     //no need to set/restore ebx???
    ACCEPT;
}

STATE(function_specifiers)
    MARK;
    if (!function_specifier()) REJECT;
    while (function_specifier())
        ;
    ACCEPT;
}

STATE(function_specifier)
    MARK;
    if (type_specifier() || storage_class_specifier()) ACCEPT;
    REJECT;
}

STATE(function_declarator)
    MARK;
    pointer();  //optional
    if (function_name()) ACCEPT;
    REJECT;
}

STATE(function_name)
    MARK;
    if (mdp_char('(')) {
        if (!function_name()) REJECT;
        if (!mdp_char(')')) REJECT;
    }
    else if (identifier()) ACCEPT;
    REJECT;
}

STATE(parameter_list)
    MARK;
    if (mdp_keyword("void")) {
        ///push_op(DCL_VOID);
        ACCEPT;
    }
    if (!parameter_decl()) REJECT;
    ///push_op(DCL_PARAM);
    while (mdp_char(',')) {
        if (ellipsis()) {
            push_op(DCL_ELLIPSIS);
            ACCEPT;
        }
        if (!parameter_decl()) REJECT;
        ///push_op(DCL_PARAM);
    }
    ACCEPT;
}

STATE(parameter_decl)
    MARK;
    if (!param_specifiers()) REJECT;
    if (!param_declarator()) REJECT;
    ACCEPT;
}

STATE(param_specifiers)
    MARK;
    if (!param_specifier()) REJECT;
    while (param_specifier())
        ;
    ACCEPT;
}

STATE(param_specifier)
    MARK;
    if (type_specifier() || storage_class_specifier() || type_qualifier()) ACCEPT;
    REJECT;
}

STATE(param_declarator)
    MARK;
    if (mdp_char('(')) {
        if (!param_declarator()) REJECT;
        if (!mdp_char(')')) REJECT;
        ACCEPT;
    }
    if (mdp_char('*')) {
        if (!param_declarator()) REJECT;
        ///push_op(DCL_POINTER);
        ACCEPT;
    }
    if (!identifier()) REJECT;
    if (mdp_char('[')) {
        if (!mdp_char(']')) REJECT;
        ///push_op(DCL_ARRAYDIMNUL);
    }
    ACCEPT;
}


#endif

#endif //ifndef BETA

/*************************** Parser Driver Routines ********************/

int Mdp_parse(char *in, UINT4 flags, UINT4 *sdlp, UINT4 nsdl)
{
    int ok;

#ifndef STAND_ALONE_PARSER
    if (setjmp(Mde_error_exit_jb)) {
        //someone longjmp'ed back to us...
        gParams->fValidParse = FALSE;    //redundant???
        return FALSE;
    }
#endif

    mdp_flags = flags;
    SDL = sdlp;
    Nsdl = nsdl;
    inp = in;
    inp_start = in;
    inp_high_water = in;
    inp_high_water_warn = in - 1;
    skip_whitespace = TRUE;
    check_for_paragraph = FALSE;
    processing_directive = FALSE;
    processing_asm_statement = FALSE;
    pp_if_level = 0;
    pp_skipping = FALSE;
	current_language = LANG_C;
	prog_type = PROG_UNKNOWN;
	prog_rev_major = INT_MAX;	//max available
	prog_rev_minor = INT_MAX;	//max available
    Ncount = 0;

#ifndef BETA
    // Initialize default PiPL contents...
    FmInitPiPLContents();
#endif

    ok = FM_program();
    if (!ok) {
        // Parse failed.  Scan the first unmatched token
        // for use in error messages.
        char *save_looking_for = MDP_looking_for;
        char save_state_name_char = state_name[1]; //for REJECT_CHAR in mdp_char and mdp_char_upper
        inp = inp_high_water;
        skip_whitespace = TRUE;
        check_for_paragraph = FALSE;
        processing_directive = FALSE;
        processing_asm_statement = FALSE;   //???
        pp_if_level = 0;
        pp_skipping = FALSE;
        Ncount = 0;
        error_token();
        push_constant(-1);
        state_name[1] = save_state_name_char;
        MDP_looking_for = save_looking_for;
        return MD386__SYNTAX;
    }
    return ok;
} /*Mdp_parse*/


#if defined(STAND_ALONE_PARSER) || defined(MDP_TEST)

    /* Routine to dump contents of the SDL for debugging purposes. */

// WARNING: Mdp_parse must have been called previously to set
// the values for SDL, Nsdl, Ncount, etc.

#ifndef BETA

void Mdp_dumpSDL(const char line[])
{
    UINT4  i;
    int lineSize = (int)strlen(line);

#if 0
    //debugging...
    PRINTF2(" Ncount = %d\r\n", Ncount);
    PRINTF2("&xyzzy = %8X\r\n", (UINT4)"xyzzy");
    PRINTF2("&xyzzy = %8X\r\n", (UINT4)"xyzzy");
    PRINTF2("&xyzzy - 4096 = %8X\r\n", (UINT4)"xyzzy" - 4096);
    PRINTF2("&xyzzy + 4096 = %8X\r\n", (UINT4)"xyzzy" + 4096);
#endif

    for (i = 0; i < Ncount; i++) {
        PRINTF2("%8X ", SDL[i]);
        /* Very rough heuristic to see if this might be a string pointer... */
        if (i >= 1 && 
#ifdef _WIN64
			i+1 < Ncount &&											/* WIN64 ptr uses 2 SDL slots */
#endif
            (*(UINT_PTR *)&SDL[i] >= (UINT_PTR) line &&				/* points into input string */
             *(UINT_PTR *)&SDL[i] <  (UINT_PTR) &line[lineSize] ||
             *(UINT_PTR *)&SDL[i] >= (UINT_PTR)"xyzzy" - 0x8000 &&	/* or points somewhere near */
             *(UINT_PTR *)&SDL[i] <  (UINT_PTR)"xyzzy" + 0x8000) &&	/* the string literal pool  */
            SDL[i-1] < 80)											/* and is reasonable size	*/
		{
            PRINTF3 ("\"%.*s\"", SDL[i-1], *(char **)&SDL[i]);
#ifdef _Win64
			i++;	//WIN64 ptr consumes 1 extra SDL slot.
#endif
        }
#if 0   //Now done inside switch below...
        else if (i >= 2 && SDL[i-2] == LIT_REAL) {
            PRINTF2 ("%#g", *(double *)&SDL[i-1]);
        }
#endif
        else {
            switch (SDL[i]) {
            case SD_ID:         PRINTF("SD_ID"); break;
			case SD_PROG:		assert(i+3 < Ncount);
								PRINTF("SD_PROG\r\n");
								PRINTF2("%8X \r\n", SDL[i+1]);
								PRINTF2("%8X \r\n", SDL[i+2]);
								PRINTF2("%8X ", SDL[i+3]);
								i += 3;
								break;
            case LIT_INTEGER:   PRINTF("LIT_INTEGER"); break;
            case LIT_LOGICAL:   PRINTF("LIT_LOGICAL"); break;
            case LIT_REAL:      assert(i+2 < Ncount);
                                PRINTF("LIT_REAL\r\n");
                                PRINTF2("%8X \r\n", SDL[i+1]);
                                PRINTF2("%8X ", SDL[i+2]);
                                PRINTF2 ("%#g", *(double *)&SDL[i+1]);
                                i += 2;
                                break;
			case LIT_REGION:	PRINTF("LIT_REGION"); break;	/* literal region (4 byte handle) */
			case LIT_UINT32:	PRINTF("LIT_UINT32"); break;	/* literal uint32 value */
			case LIT_INT64:		PRINTF("LIT_INT64"); break;		/* literal integer*8 value */
			case LIT_UINT64:	PRINTF("LIT_UINT64"); break;	/* literal uint64 value */
            case OP_ADD:        PRINTF("OP_ADD"); break;
            case OP_SUB:        PRINTF("OP_SUB"); break;
            case OP_NEG:        PRINTF("OP_NEG"); break;
            case OP_LOAD:       PRINTF("OP_LOAD"); break;
            case OP_STORE:      PRINTF("OP_STORE"); break;
            case OP_QUAL:       PRINTF("OP_QUAL"); break;
            case OP_LOOKUP:     PRINTF("OP_LOOKUP"); break;
            case OP_INDEX:      PRINTF("OP_INDEX"); break;
            case OP_MEMBER:     PRINTF("OP_MEMBER"); break;
            case OP_CONCAT:     PRINTF("OP_CONCAT"); break;
            case OP_BLDPTR:     PRINTF("OP_BLDPTR"); break;
            case OP_DUPL:       PRINTF("OP_DUPL"); break;
            case OP_PHYS:       PRINTF("OP_PHYS"); break;
            case OP_MARK:       PRINTF("OP_MARK"); break;
            case OP_VAR2TYPE:   PRINTF("OP_VAR2TYPE"); break;
            case OP_LSH:        PRINTF("OP_LSH"); break;
            case OP_RSH:        PRINTF("OP_RSH"); break;
            case OP_MUL:        PRINTF("OP_MUL"); break;
            case OP_DIV:        PRINTF("OP_DIV"); break;
            case OP_REM:        PRINTF("OP_REM"); break;
            case OP_ASS_ADD:    PRINTF("OP_ASS_ADD"); break;
            case OP_ASS_SUB:    PRINTF("OP_ASS_SUB"); break;
            case OP_ASS_MUL:    PRINTF("OP_ASS_MUL"); break;
            case OP_ASS_DIV:    PRINTF("OP_ASS_DIV"); break;
            case OP_ASS_REM:    PRINTF("OP_ASS_REM"); break;
            case OP_ASS_AND:    PRINTF("OP_ASS_AND"); break;
            case OP_ASS_XOR:    PRINTF("OP_ASS_XOR"); break;
            case OP_ASS_IOR:    PRINTF("OP_ASS_IOR"); break;
            case OP_ASS_LSH:    PRINTF("OP_ASS_LSH"); break;
            case OP_ASS_RSH:    PRINTF("OP_ASS_RSH"); break;
            case OP_LOG_AND:    PRINTF("OP_LOG_AND"); break;
            case OP_LOG_OR:     PRINTF("OP_LOG_OR"); break;
            case OP_LOG_NOT:    PRINTF("OP_LOG_NOT"); break;
            case OP_IF:         PRINTF("OP_IF"); break;
            case OP_AND:        PRINTF("OP_AND"); break;
            case OP_IOR:        PRINTF("OP_IOR"); break;
            case OP_XOR:        PRINTF("OP_XOR"); break;
            case OP_NOT:        PRINTF("OP_NOT"); break;
            case OP_PREINC:     PRINTF("OP_PREINC"); break;
            case OP_PREDEC:     PRINTF("OP_PREDEC"); break;
            case OP_POSTINC:    PRINTF("OP_POSTINC"); break;
            case OP_POSTDEC:    PRINTF("OP_POSTDEC"); break;
            case OP_ARG:        PRINTF("OP_ARG"); break;
            case OP_CALL:       PRINTF("OP_CALL"); break;
            case OP_SIZEOF:     PRINTF("OP_SIZEOF"); break;
            case OP_ADDR:       PRINTF("OP_ADDR"); break;
            case OP_NOP:        PRINTF("OP_NOP"); break;
            case OP_EQ:         PRINTF("OP_EQ"); break;
            case OP_NE:         PRINTF("OP_NE"); break;
            case OP_LT:         PRINTF("OP_LT"); break;
            case OP_GT:         PRINTF("OP_GT"); break;
            case OP_LE:         PRINTF("OP_LE"); break;
            case OP_GE:         PRINTF("OP_GE"); break;
            case OP_CAST:       PRINTF("OP_CAST"); break;
            case OP_THEN:       PRINTF("OP_THEN"); break;
            case OP_ELSE:       PRINTF("OP_ELSE"); break;
            case OP_POP:        PRINTF("OP_POP"); break;
            case OP_XCHG:       PRINTF("OP_XCHG"); break;
            case OP_LOOP:       PRINTF("OP_LOOP"); break;
            case OP_ENDLOOP:    PRINTF("OP_ENDLOOP"); break;
            case OP_BREAK:      PRINTF("OP_BREAK"); break;
            case OP_CONTINUE:   PRINTF("OP_CONTINUE"); break;
            case OP_NEXT:       PRINTF("OP_NEXT"); break;
            case OP_GOTO:       PRINTF("OP_GOTO"); break;
            case OP_LABEL:      PRINTF("OP_LABEL"); break;
            case OP_RETURN:     PRINTF("OP_RETURN"); break;
            case OP_SWITCH:     PRINTF("OP_SWITCH"); break;
            case OP_CASE:       PRINTF("OP_CASE"); break;
            case OP_CASETEST:   PRINTF("OP_CASETEST"); break;
            case OP_DEFAULT:    PRINTF("OP_DEFAULT"); break;
            case OP_ENDSW:      PRINTF("OP_ENDSW"); break;
            case OP_BREAKF:     PRINTF("OP_BREAKF"); break;
            case OP_FOR:        PRINTF("OP_FOR"); break;
            case OP_FOR1:       PRINTF("OP_FOR1"); break;
            case OP_FOR2:       PRINTF("OP_FOR2"); break;
            case OP_FOR3:       PRINTF("OP_FOR3"); break;
            case OP_FOR4:       PRINTF("OP_FOR4"); break;
            case OP_ENDFOR:     PRINTF("OP_ENDFOR"); break;
            case OP_DO1:        PRINTF("OP_DO1"); break;
            case OP_DO2:        PRINTF("OP_DO2"); break;
            case OP_DO3:        PRINTF("OP_DO3"); break;
            case OP_REPEAT:     PRINTF("OP_REPEAT"); break;
            case OP_SKIP:       PRINTF("OP_SKIP"); break;
            case OP_ENDSKIP:    PRINTF("OP_ENDSKIP"); break;
            case OP_DECLARE:    PRINTF("OP_DECLARE"); break;
            case OP_ENTER:      PRINTF("OP_ENTER"); break;
            case OP_SETCHAN_R:  PRINTF("OP_SETCHAN_R"); break;
            case OP_SETCHAN_G:  PRINTF("OP_SETCHAN_G"); break;
            case OP_SETCHAN_B:  PRINTF("OP_SETCHAN_B"); break;
            case OP_SETCHAN_A:  PRINTF("OP_SETCHAN_A"); break;
            case OP_SETCHAN_S:  PRINTF("OP_SETCHAN_S"); break;
            case OP_SETVAL_R:   PRINTF("OP_SETVAL_R"); break;
            case OP_SETVAL_G:   PRINTF("OP_SETVAL_G"); break;
            case OP_SETVAL_B:   PRINTF("OP_SETVAL_B"); break;
            case OP_SETVAL_A:   PRINTF("OP_SETVAL_A"); break;
            case OP_SETVAL_S:   PRINTF("OP_SETVAL_S"); break;
#ifndef NO_FFP_INLINES
            case OP_MIN:        PRINTF("OP_MIN"); break;
            case OP_MAX:        PRINTF("OP_MAX"); break;
            case OP_SQR:        PRINTF("OP_SQR"); break;
            case OP_ABS:        PRINTF("OP_ABS"); break;
            case OP_DIF:        PRINTF("OP_DIF"); break;
            case OP_CTL:        PRINTF("OP_CTL"); break;
            case OP_VAL:        PRINTF("OP_VAL"); break;
            case OP_SIN:        PRINTF("OP_SIN"); break;
            case OP_COS:        PRINTF("OP_COS"); break;
            case OP_TAN:        PRINTF("OP_TAN"); break;
            case OP_GET:        PRINTF("OP_GET"); break;
            case OP_PUT:        PRINTF("OP_PUT"); break;
            case OP_CNVX:       PRINTF("OP_CNVX"); break;
            case OP_CNVY:       PRINTF("OP_CNVY"); break;
            case OP_GAMMA:      PRINTF("OP_GAMMA"); break;
            case OP_FTOL:       PRINTF("OP_FTOL"); break;
            case OP_FLOOR:      PRINTF("OP_FLOOR"); break;
            case OP_CEIL:       PRINTF("OP_CEIL"); break;
            case OP_ROUND:      PRINTF("OP_ROUND"); break;
            case OP_CHOP:       PRINTF("OP_CHOP"); break;
            case OP_IFLOOR:     PRINTF("OP_IFLOOR"); break;
            case OP_ICEIL:      PRINTF("OP_ICEIL"); break;
            case OP_IROUND:     PRINTF("OP_IROUND"); break;
            case OP_ICHOP:      PRINTF("OP_ICHOP"); break;
            case OP_POW:        PRINTF("OP_POW"); break;
#endif //!NO_FFP_INLINES

            case OP_ON_END:             PRINTF("ON_OP_END"); break;
            case OP_ON_INIT_DIALOG:     PRINTF("OP_ON_INIT_DIALOG"); break;
            case OP_ON_CTL:             PRINTF("OP_ON_CTL"); break;
            case OP_ON_ZERO_DIVIDE:     PRINTF("OP_ON_ZERO_DIVIDE"); break;
            case OP_ON_DIVIDE_OVERFLOW: PRINTF("OP_ON_DIVIDE_OVERFLOW"); break;
            case OP_FOR_EVERY_PIXEL:    PRINTF("OP_FOR_EVERY_PIXEL"); break;
            case OP_FOR_EVERY_PIXEL_END:      PRINTF("OP_FOR_EVERY_PIXEL_END"); break;
            case OP_FOR_EVERY_TILE:     PRINTF("OP_FOR_EVERY_TILE"); break;
            case OP_FOR_EVERY_END:      PRINTF("OP_FOR_EVERY_END"); break;
            case OP_ON_HANDLER_END:     PRINTF("OP_ON_HANDLER_END"); break;
            case OP_ON_FILTER_START:    PRINTF("OP_ON_FILTER_START"); break;
            case OP_ON_FILTER_END:      PRINTF("OP_ON_FILTER_END"); break;
            case OP_USER_FUNCTION:      PRINTF("OP_USER_FUNCTION"); break;

            case OP_SAVE_CHANNEL_CODE: PRINTF("OP_SAVE_CHANNEL_CODE"); break;
            case OP_END_CHANNEL_CODE:  PRINTF("OP_END_CHANNEL_CODE"); break;
            case OP_DUMP_CHANNEL_CODE: PRINTF("OP_DUMP_CHANNEL_CODE"); break;

            case OP_COMPOSE_TYPE:   PRINTF("OP_COMPOSE_TYPE"); break;
            case OP_COMPOSE_DECL:   PRINTF("OP_COMPOSE_DECL"); break;
            case OP_END_DECL:       PRINTF("OP_END_DECL"); break;
            case OP_BEGIN_SCOPE:    PRINTF("OP_BEGIN_SCOPE"); break;
            case OP_END_SCOPE:      PRINTF("OP_END_SCOPE"); break;
            case OP_BEGIN_INIT:     PRINTF("OP_BEGIN_INIT"); break;
            case OP_END_INIT:       PRINTF("OP_END_INIT"); break;

            case OP_SHORT:          PRINTF("OP_SHORT"); break;
            case OP_INT:            PRINTF("OP_INT"); break;
            case OP_LONG:           PRINTF("OP_LONG"); break;
            case OP_BOOL:           PRINTF("OP_BOOL"); break;
            case OP_FLOAT:          PRINTF("OP_FLOAT"); break;
            case OP_DOUBLE:         PRINTF("OP_DOUBLE"); break;
            case OP_TEMPREAL:       PRINTF("OP_TEMPREAL"); break;
            case OP_BCD:	        PRINTF("OP_BCD"); break;
			case OP_VOID:			PRINTF("OP_VOID"); break;
#ifdef X64
            case OP_INT64:          PRINTF("OP_INT64"); break;
            case OP_QWORD:          PRINTF("OP_QWORD"); break;
			case OP_INT_PTR:        PRINTF("OP_INT_PTR"); break;
			case OP_UINT_PTR:       PRINTF("OP_UINT_PTR"); break;
#endif //X64
			case OP_HALF:			PRINTF("OP_HALF"); break;
			case OP_BYTE:			PRINTF("OP_BYTE"); break;
			case OP_SIGNED:			PRINTF("OP_SIGNED"); break;
			case OP_UNSIGNED:		PRINTF("OP_UNSIGNED"); break;
			case OP_CHAR:			PRINTF("OP_CHAR"); break;
			case OP_WORD:			PRINTF("OP_WORD"); break;
			case OP_DWORD:			PRINTF("OP_DWORD"); break;
			
			case OP_DEC_STACK:      PRINTF("OP_DEC_STACK"); break;
            case OP_ARGLIST:        PRINTF("OP_ARGLIST"); break;

            case OP_EMBED_BITMAP:   PRINTF("OP_EMBED_BITMAP"); break;
            case OP_EMBED_ICON:     PRINTF("OP_EMBED_ICON"); break;
            case OP_EMBED_METAFILE: PRINTF("OP_EMBED_METAFILE"); break;
            case OP_EMBED_WAVE:     PRINTF("OP_EMBED_WAVE"); break;
			case OP_EMBED_OTHER:    PRINTF("OP_EMBED_OTHER"); break;
            case OP_EMBED_CURSOR:   PRINTF("OP_EMBED_CURSOR"); break;

            case OP_ABOUT:          PRINTF("OP_ABOUT"); break;

            case TYP_BYTE:      PRINTF("TYP_BYTE"); break;
            case TYP_INTEGER:   PRINTF("TYP_INTEGER"); break;
            case TYP_LOGICAL:   PRINTF("TYP_LOGICAL"); break;
            case TYP_REAL:      PRINTF("TYP_REAL"); break;
            case TYP_CHAR:      PRINTF("TYP_CHAR"); break;
            case TYP_INSTR:     PRINTF("TYP_INSTR"); break;
            case TYP_PTR:       PRINTF("TYP_PTR"); break;
            case TYP_SELECTOR:  PRINTF("TYP_SELECTOR"); break;
			case TYP_INT64:		PRINTF("TYP_INT64"); break;
            case TYP_ARRAY:     PRINTF("TYP_ARRAY"); break;
            case TYP_STRUCTURE: PRINTF("TYP_STRUCTURE"); break;
            case TYP_LIST:      PRINTF("TYP_LIST"); break;
            case TYP_ASCIC:     PRINTF("TYP_ASCIC"); break;
            case TYP_ASCIZ:     PRINTF("TYP_ASCIZ"); break;
            case TYP_NULL:      PRINTF("TYP_NULL"); break;
            case SD_LINE:       PRINTF("SD_LINE"); break;
            case SD_LABEL:      PRINTF("SD_LABEL"); break;
            case SD_SEGMENT:    PRINTF("SD_SEGMENT"); break;
            case SD_GROUP:      PRINTF("SD_GROUP"); break;
            case SD_COMMON:     PRINTF("SD_COMMON"); break;
            case SD_RADIX:      PRINTF("SD_RADIX"); break;
            case SD_MODE:       PRINTF("SD_MODE"); break;
            case SD_FILE:       PRINTF("SD_FILE"); break;
            case SD_PC:         PRINTF("SD_PC"); break;
            case SD_LAST:       PRINTF("SD_LAST"); break;
            case SD_NEXT:       PRINTF("SD_NEXT"); break;
            case SD_TIME:       PRINTF("SD_TIME"); break;

            case DCL_TYPEDEF:   PRINTF("DCL_TYPEDEF"); break;
            case DCL_EXTERN:    PRINTF("DCL_EXTERN"); break;
            case DCL_STATIC:    PRINTF("DCL_STATIC"); break;
            case DCL_AUTO:      PRINTF("DCL_AUTO"); break;
            case DCL_REGISTER:  PRINTF("DCL_REGISTER"); break;
            case DCL_VOID:      PRINTF("DCL_VOID"); break;
            case DCL_CHAR:      PRINTF("DCL_CHAR"); break;
            case DCL_SHORT:     PRINTF("DCL_SHORT"); break;
            case DCL_INT:       PRINTF("DCL_INT"); break;
            case DCL_LONG:      PRINTF("DCL_LONG"); break;
            case DCL_BOOL:      PRINTF("DCL_BOOL"); break;
            case DCL_BYTE:      PRINTF("DCL_BYTE"); break;
            case DCL_WORD:      PRINTF("DCL_WORD"); break;
            case DCL_DWORD:     PRINTF("DCL_DWORD"); break;
            case DCL_FLOAT:     PRINTF("DCL_FLOAT"); break;
            case DCL_DOUBLE:    PRINTF("DCL_DOUBLE"); break;
            case DCL_SIGNED:    PRINTF("DCL_SIGNED"); break;
            case DCL_UNSIGNED:  PRINTF("DCL_UNSIGNED"); break;
            case DCL_SATURATED: PRINTF("DCL_SATURATED"); break;
            case DCL_STRUCT:    PRINTF("DCL_STRUCT"); break;
            case DCL_UNION:     PRINTF("DCL_UNION"); break;
            case DCL_ENUM:      PRINTF("DCL_ENUM"); break;
            case DCL_CONST:     PRINTF("DCL_CONST"); break;
            case DCL_VOLATILE:  PRINTF("DCL_VOLATILE"); break;
            case DCL_POINTER:   PRINTF("DCL_POINTER"); break;
            case DCL_ARRAYDIM:  PRINTF("DCL_ARRAYDIM"); break;
            case DCL_PARAM:     PRINTF("DCL_PARAM"); break;
            case DCL_MEMBER:    PRINTF("DCL_MEMBER"); break;
            case DCL_TEMPREAL:  PRINTF("DCL_TEMPREAL"); break;
            case DCL_ARRAYDIMNUL: PRINTF("DCL_ARRAYDIMNUL"); break;
            case DCL_FUNCTION:  PRINTF("DCL_FUNCTION"); break;
            case DCL_ELLIPSIS:  PRINTF("DCL_ELLIPSIS"); break;
#ifdef X64
            case DCL_INT64:     PRINTF("DCL_INT64"); break;
            case DCL_QWORD:     PRINTF("DCL_QWORD"); break;
			case DCL_INT_PTR:	PRINTF("DCL_INT_PTR"); break;
			case DCL_UINT_PTR:	PRINTF("DCL_UINT_PTR"); break;
#endif //X64
			case DCL_HALF:		PRINTF("DCL_HALF"); break;

            default:
                if (SDTYPEKIND(SDL[i]) == LIT_CHAR ||
                    SDTYPEKIND(SDL[i]) == SD_IDLIT) {
                    // Print nicely-formatted LIT_CHAR/SD_IDLIT string literal...
                    int bc = SDBYTES(SDL[i]);   //byte count
                    int j;  //running char index
                    const char *dnamep;
					if (SDTYPEKIND(SDL[i]) == LIT_CHAR)
                        dnamep = "LIT_CHAR";
                    else
                        dnamep = "SD_IDLIT";

                    ///assert(bc > 0);	// 0 byte LIT_CHAR is now allowed
#if 0
                    PRINTF3("%s (%d)\r\n", dnamep, bc);
#else
					if (bc > 0)
						PRINTF2("%s\r\n", dnamep);
					else
						PRINTF2("%s", dnamep);	//avoid blank line
#endif
                    for (j=0; j < bc; j++) {
                        char ch = (SDL[i+1+j/4] >> 8*(j%4)) & 0xFF; //next char from lit
                        if (j%4 == 0) PRINTF2("%8X \"", SDL[i+1+j/4]);  //opening quote
                        if (isprint(ch & 0xff)) {	//avoid asserting if ch is negative
                            //handle necessary character escape codes
                            switch (ch) {
                            case '\"':   PRINTF("\""); break;
                            case '\\':   PRINTF("\\"); break;
                            default:     PRINTF2("%c", ch);   //char verbatim
                            }
                        }
                        else if (j == bc-1) {
                            //escape at end of string needs no 0-padding;
                            //in fact, don't print a terminating NUL at all
                            //unless it is on a line by itself...
                            if (ch || j%4 == 0) PRINTF2("\\%o", ch);
                        }
                        else PRINTF2("\\%03o", ch & 0xff); //other escapes may need 0-padding to 3 places to avoid ambiguity
                        if (j%4 == 3) { //last char on this line...
                            PRINTF("\"");   //quote after every 4 chars
                            if (j != bc-1) { //new line unless very last char
                                PRINTF("\r\n");
                            }
                        }
                    } //for j
                    if (bc%4 != 0) PRINTF("\"");    //very final quote if fewer than 4 chars on last line
                    i += (bc + 3)/4;
                }
                else if ((SDL[i] & 0xFFFF00FF) == SD_REG)
                                PRINTF("SD_REG");
                break;
            } //switch
        } //if
        PRINTF("\r\n");
    } //for i
} /*Mdp_dumpSDL*/

#endif //ifndef BETA

    /* Display a syntax error message (debug version). */

// WARNING: Mdp_parse must have been called previously to set
// the values for SDL, Nsdl, Ncount, etc.

char *Mdp_syntaxErr(void)
{
#if 1
    char *p = *(char **)&SDL[2];

    assert(SDL[0] == SD_ID);
	//should check for reasonable string length at SDL[1}...
	if (SDL[1] <= 32) {
	    p[SDL[1]] = 0;  //NUL-terminate the error token.
	}
	else {
		p[32] = 0;	// truncate token to 32 chars.
	}
#else
    char *p = inp_high_water;
    static const char whitespace[] = " \t\r\n";

    /* Truncate the "near" token at the first whitespace (if */
    /* any) following the first (possibly blank) char.       */
    /* (But don't let the "near" token exceed 32 chars in    */
    /* any case, to avoid overflowing string buffers else-   */
    /* where.)                                               */

    // Scan over leading whitespace...
    p += strspn(p, whitespace);
    // Find next whitespace, if any...
    p = strpbrk(p, whitespace);
    if (p) *p = 0; // truncate at trailing whitespace.
    p = inp_high_water;
    // optionally, scan off leading whitespace once more...
    p += strspn(p, whitespace);
    if (strlen(p) > 32) p[32] = 0; // truncate token to 32 chars.
#endif
    /* If the "near" token is null, we probably ran into the */
    /* end of source prematurely.                            */
    PRINTF3("Error while looking for %s, near \"%s\"\r\n",
        MDP_looking_for, *p ? p : "end_of_source");
#if 0
    msgBox(MB_ICONERROR, "FilterMeister Syntax Error",
        "Error while looking for %s\nnear \"%s\"",
        MDP_looking_for, *p ? p : "end_of_source");
#endif
    return p;
} /*Mdp_syntaxErr*/

#endif

/*********************************************************************************/

#ifdef STAND_ALONE_PARSER

#define STAND_ALONE_SDL_SIZE 5000

    /* Driver for stand-alone (debug) version of the parser. */

void main( void )
{
    UINT4 SDL[STAND_ALONE_SDL_SIZE];
    char line[1024];
    UINT4  iss;
    UINT4  i;

    do {
        PRINTF( "Parser> " );
#ifdef USE_GETS
        gets( line );  /* gets() seems to have a builtin limit of ~254 chars under Win95 */
#else
        /* Well, it seems fgets() also has a builtin limit of ~254 chars under Win95 -- */
        /* probably a limit built in to the lower-level console i/o... */
        /* Also, ^Z just seems to leave line unchanged and does NOT return NULL from */
        /* fgets() ??? */
        if( fgets( line, sizeof line, stdin ) == NULL)
             PRINTF( "fgets error\r\n" );
        for (i = 0; i < sizeof(line) && line[i] != 0; ++i)
        {   /* Replace newlines with blanks */
            if (line[i] == '\n') line[i] = ' ';
        }
#endif
        /* PRINTF2( "The line entered was: %s\r\n", line ); */
        iss = Mdp_parse(line, 0, SDL, STAND_ALONE_SDL_SIZE);
        if (iss & 1) {
            Mdp_dumpSDL(line);
        }
        else if (iss == MD386__SYNTAX) {
            PRINTF2(" iss = %8X\r\n", iss);
            Mdp_syntaxErr();
        }
    } while (!(Ncount == 2 && SDL[0] == -2));  /* terminate on quit command */
    exit(0);
} //main

#elif !defined(MDP_TEST)


        /************************************************/
        /*                                              */
        /*               MDP_PARSE_COMMAND              */
        /*                                              */
        /*  Returns TRUE if more to be parsed, else     */
        /*  returns FALSE.  Syntax errors cause an      */
        /*  abort exit to be taken.                     */
        /*                                              */
        /************************************************/

#define MD386_SDL_SIZE 5000

 UINT4 SDL[MD386_SDL_SIZE];

 /**procedure**/
 UINT4 Mdp_parse_command(STRINGDESCRIPTOR *commanddp, 
                         SEMANTICLISTSTRUCTURE **semanticlistpp)
 {
     #define COMMANDD (*(STRINGDESCRIPTOR *) commanddp)
     /* line MUST be static since we return pointers to tokens embedded in line!!! */
     static char    line[256];
     char           *p;
     UINT4          iss;

     if (COMMANDD.len >= sizeof(line))
         Mde_abort_command(MD386__CMDTOOLONG, sizeof(line) - 1, 0, 0);
     
     memcpy (line, COMMANDD.ptr, COMMANDD.len);
     line[COMMANDD.len] = 0;   /* NUL terminator */

     iss = Mdp_parse(line, 0, SDL, MD386_SDL_SIZE);

     if (iss == MD386__SYNTAX)
     {   /* Truncate the "near" token at the first blank (if any) */
         /* following the first (possibly blank) char.            */
         p = inp_high_water;
         if (*p == ' ') p++;
         p = strchr(p, ' ');
         if (p) *p = 0;

         /* If the "near" token is null, we probably ran into the */
         /* end of line prematurely.                              */
         p = inp_high_water;
         if (*p == 0) p = "end_of_line";

         /* Error while parsing %s, near "%s" */
         Mde_abort_command(MD386__CMDSYNTAX, (UINT4) MDP_looking_for, 
                           (UINT4) p, 0);
     }

     /* Update command line descriptor to point to */
     /* remaining unparsed command(s).             */

     COMMANDD.len -= (inp - line);
     COMMANDD.ptr += (inp - line);

     *semanticlistpp = (SEMANTICLISTSTRUCTURE *) SDL;

     return More;
     
     #undef COMMANDD
 } /*Mdp_parse_command*/


        /************************************************/
        /*                                              */
        /*                 MDP_INITIALIZE               */
        /*                                              */
        /************************************************/

 /**procedure**/
 UINT4 Mdp_initialize(void)
 {
     return SS$_NORMAL;
 } /*Mdp_initialize*/


        /************************************************/
        /*                                              */
        /*                 MDP_TERMINATE                */
        /*                                              */
        /************************************************/

 /**procedure**/
 UINT4 Mdp_terminate(void)
 {
     return SS$_NORMAL;
 } /*Mdp_terminate*/

#endif
